package wind.android.bussiness.guide.activity

import android.content.Intent
import android.os.Bundle
import android.os.Message
import android.view.View
import base.BaseActivity
import cn.sharesdk.framework.Platform
import cn.sharesdk.framework.PlatformActionListener
import cn.sharesdk.framework.ShareSDK
import cn.sharesdk.wechat.friends.Wechat
import database.orm.CommDao
import database.orm.model.BaseOrmModel
import database.orm.model.KeyValue
import database.orm.model.LoginUserInfoModel
import kotlinx.android.synthetic.main.boot_login_view.*
import net.network.SkyProcessor
import net.network.sky.data.LoginConfig
import session.F5Session
import util.LoggerManager
import util.PackageManagerUtil
import util.SharedPreferenceService
import wind.android.R
import wind.android.bussiness.login.acitvity.LoginActivity
import wind.android.bussiness.login.listener.ISmsListener
import wind.android.bussiness.login.manager.WindStockSmsManager
import wind.android.bussiness.login.model.ThirdPartyBindState
import wind.android.bussiness.login.model.ThirdPartyLogin
import wind.android.bussiness.probe.activity.UserInfoEditActivity
import wind.android.common.StockConstants
import wind.android.session.Session
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      BootLoginActivity.java
 * # Function:       引导用户登录
 * # @author         ylrong
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ylrong     2019/5/29    16:33
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class BootLoginActivity : BaseActivity(), View.OnClickListener {

    private val LOGIN_SUCCESS = 0

    /**
     * 判断微信是否登录成功
     */
    private var hasAuthorize = false
    private var reConnectCount = 0
    private var thirdPartyName: String? = null
    private var thirdPartyHeadImage: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        hideNavigationBar(true)
        setContentView(R.layout.boot_login_view)
        // 固定风格沉浸式状态栏
        setStatusBarColor(resources.getColor(R.color.white), resources.getColor(R.color.windstock_toolbar_bg))
        close.setOnClickListener(this)
        wxlogin.setOnClickListener(this)
        phonelogin.setOnClickListener(this)
    }

    override fun onClick(v: View) {
        val id = v.id
        when (id) {
            R.id.close -> {
                // 关闭按钮
                onBack()
            }
            R.id.wxlogin -> {
                // 微信登录
                loginWX()
            }
            R.id.phonelogin -> {
                // 手机号登录
                startActivityForResult(Intent(this, LoginActivity::class.java),
                        WelcomeFragment.BOOT_LOGIN_REQUESTCODE)
            }
            else -> {

            }
        }
    }

    override fun onBack() {
        // 关闭或者实体返回键，直接退出引导登录
        setResult(WelcomeFragment.BOOT_LOGIN_RESULTERRORCODE)
        super.onBack()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == WelcomeFragment.BOOT_LOGIN_REQUESTCODE && F5Session.getInstance().isLoginReal()) {
            // 在手机号登录页面或者验证码登录页面登录成功，通过返回欢迎页跳转主页面
            setResult(WelcomeFragment.BOOT_LOGIN_RESULTSUCCESSCODE)
            finish()
        }
    }


    /**
     * 微信登录
     */
    private fun loginWX() {
        if (!PackageManagerUtil.isAvilible(this, "com.tencent.mm")) {
            showAlertView("请先下载微信")
            return
        }
        showProgressMum()
        authorize(ShareSDK.getPlatform(Wechat.NAME))
    }

    /**
     * 微信认证并获取微信信息
     */
    private fun authorize(plat: Platform?) {
        if (plat == null) {
            hideProgressMum()
            return
        }

        plat.platformActionListener = object : PlatformActionListener {

            override fun onError(arg0: Platform, arg1: Int, arg2: Throwable) {
                hideProgressMum()
            }

            override fun onComplete(arg0: Platform, arg1: Int, arg2: HashMap<String, Any>) {
                val id = arg2["unionid"] as String
                thirdPartyName = arg2["nickname"] as String
                thirdPartyHeadImage = arg2["headimgurl"] as String
                LoggerManager.Write("LoginActivity", "login", "id = $id")
                LoggerManager.Write("LoginActivity", "login", "name = $thirdPartyName")
                LoggerManager.Write("LoginActivity", "login", "icon = $thirdPartyHeadImage")
                WindStockSmsManager.getInstance().thirdPartyRegister(id, "1", mISmsListener)
            }

            override fun onCancel(arg0: Platform, arg1: Int) {
                runOnUiThread { showAlertView("取消登录") }

                hideProgressMum()
            }
        }
        //获取用户资料
        plat.showUser(null)
    }

    internal var mISmsListener: ISmsListener<ThirdPartyLogin> = ISmsListener { resultCode, _, t ->
        if (t != null) {
            hasAuthorize = true
            runOnUiThread {
                val msg = Message.obtain()
                msg.what = LoginActivity.GET_ACCOUNT_BY_WXID
                msg.obj = t
                sendMessage(msg)
            }

        } else {
            hasAuthorize = false
            runOnUiThread { showAlertView("登录失败:$resultCode") }

            hideProgressMum()
        }
    }

    /**
     * 检查是否绑定第三方账号回调
     */
    private val checkThirdPartyBindStateListener = ISmsListener<ThirdPartyBindState> { _, _, t ->
        if (t != null) {
            //记录该账号是否绑定微信  我的万得页面需要使用
            val authData = SkyProcessor.getInstance().authData
            if (authData != null) {
                SharedPreferenceService.getInstance().put(UserInfoEditActivity.ACCOUNT_BIND_STATE_WX + authData.UserID,
                        t.isBindThirdPartyAccount)
            }
        }
    }

    override fun handleMessage(msg: Message) {
        if (msg.what == LOGIN_SUCCESS) {
            hideProgressMum()
            setResult(WelcomeFragment.BOOT_LOGIN_RESULTSUCCESSCODE)
            finish()
            val authData = SkyProcessor.getInstance().authData
            if (authData != null) {
                SharedPreferenceService.getInstance().put(UserInfoEditActivity.NICK_NAME_WX + authData.UserID, thirdPartyName)
                SharedPreferenceService.getInstance().put(UserInfoEditActivity.PHONE_NAME_WX + authData.UserID, thirdPartyHeadImage)
                //如果登录成功发送检查第三方账号绑定情况请求
                WindStockSmsManager.getInstance().checkThirdPartyBindState(authData.UserID.toString(), "1", checkThirdPartyBindStateListener)
            }

        } else if (msg.what == LoginActivity.GET_ACCOUNT_BY_WXID) {
            val t = msg.obj as ThirdPartyLogin
            autoLoginWX(t)
        } else if (msg.what == 1) {
            //微信登录后账号不存在 hasAuthorize为true；重新连接三次。三次后重置hasAuthorize = false;
            if (reConnectCount >= 3) {
                hasAuthorize = false
            }
            if (hasAuthorize) {
                reConnectCount++
                val tmpMsg = Message.obtain()
                tmpMsg.what = LoginActivity.GET_ACCOUNT_BY_WXID
                tmpMsg.obj = msg.obj
                sendMessageDelayed(tmpMsg, 1000)
            } else {
                reConnectCount = 0
                hideProgressMum()
                showAlertView(LoginConfig.getLoginReturnMessage(msg.what))
            }
        } else {
            hideProgressMum()
            showAlertView("账号登录失败")
        }
    }

    private fun autoLoginWX(t: ThirdPartyLogin) {
        Thread(Runnable {
            val ret = SkyProcessor.getInstance()
                    .safeLogin(t.userID, t.userPass,
                            false)
            val msg = Message.obtain()
            msg.what = ret
            msg.obj = t
            if (ret == 0) {
                Session.loginUserName = t.userID
                Session.loginPassword = t.userPass
                try {
                    updateAutoLoginValue(t.userID, t.userPass)
                } catch (e: Exception) {
                    e.printStackTrace()
                }

            }
            sendMessage(msg)
        }).start()
    }

    /**
     * 更新用户名密码
     */
    private fun updateAutoLoginValue(userId: String, userPass: String) {
        val loginUserInfo = LoginUserInfoModel()
        loginUserInfo.setA_loginUserName(userId)
        loginUserInfo.setB_loginUserPass(userPass)
        val dao = CommDao.getInstance()
        val key1 = KeyValue()
        key1.sign_key = StockConstants.AUTO_LOGIN
        key1.sign_value = "true"

        val key2 = KeyValue()
        key2.sign_key = StockConstants.AUTO_LOGIN_USERNAME
        key2.sign_value = userId

        val key3 = KeyValue()
        key3.sign_key = StockConstants.AUTO_LOGIN_PASSWORD
        key3.sign_value = userPass

        val list = ArrayList<BaseOrmModel>()
        list.add(key1)
        list.add(key2)
        list.add(key3)

        dao.createOrUpdateAll(list, KeyValue::class.java)
        dao.createOrUpdateByKey(loginUserInfo, LoginUserInfoModel::class.java)
    }
}
