package wind.android.bussiness.probe.activity

import android.content.pm.ActivityInfo
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.os.Bundle
import android.os.Message
import android.view.*
import android.widget.FrameLayout
import base.BaseActivity
import base.BaseNetHandler
import cn.sharesdk.framework.Platform
import cn.sharesdk.wechat.friends.Wechat
import com.windshare.IShare
import com.windshare.SharePopupView
import com.windshare.ShareStateEnum
import com.windshare.ShareTypeEnum
import kotlinx.android.synthetic.main.activity_global_market_map.*
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import util.BitMapUtil
import util.log.WindLog
import wind.android.R
import wind.android.bussiness.login.utils.JsonUtils
import wind.android.bussiness.probe.manager.GlobalMapManager
import wind.android.bussiness.probe.model.GlobalMarketModel
import wind.android.bussiness.probe.view.IndexMarketView
import wind.android.f5.snapshot.BitmapUtils
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      LandConfigSettingActivity$
 * # Function:       横屏的web页面
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/3/17$    14:17$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class GlobalMarketMapActivity : BaseActivity() {

    companion object {
        const val REALLY_MAP_WIDTH = 1334
        const val REALLY_MAP_HEIGHT = 750
        const val MARKET_ITEM_WIDTH = 142
        const val MARKET_ITEM_HEIGHT = 88

        const val MSG_SET_DATA = 0x1001
        const val MSG_UPDATE_DATA = 0x1002

        val INDEX_INDICATOR = intArrayOf(Indicator.DI_SHORTNAMECHS, Indicator.DI_NEWPRICE, Indicator.DI_CHANGE, Indicator.DI_CHANGERANGE, Indicator.DI_PRICEUNIT)
    }

    /**
     * X轴偏移量
     */
    private var mOffsetX = 0f

    /**
     * Y轴偏移量
     */
    private var mOffsetY = 0f

    /**
     * 缩放比例
     */
    private var mScale = 1.0f

    private var mItemWidth = 0

    private var mItemHeight = 0

    private val mData = mutableListOf<GlobalMarketModel>()

    private var mWindCode = mutableListOf<String>()

    private val mMapManager by lazy { GlobalMapManager() }

    private val mViewMap = mutableMapOf<String, IndexMarketView>()

    /**
     * 用户手动退出该界面
     */
    private var isBackByUser = false

    override fun onCreate(savedInstanceState: Bundle?) {
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_global_market_map)
        initData()
        initView()
    }

    private fun initData() {
        readLocalData()
        mMapManager.addCallback(object : GlobalMapManager.IRequestSpeedCallback {
            override fun setData(data: Vector<RealQuoteItem>) {
                val msg = Message.obtain()
                msg.what = MSG_SET_DATA
                msg.obj = data
                sendMessage(msg)
            }

            override fun updateData(data: RealQuoteItem) {
                val msg = Message.obtain()
                msg.what = MSG_UPDATE_DATA
                msg.obj = data
                sendMessage(msg)
            }
        })
    }

    override fun onResume() {
        super.onResume()
        mMapManager.subMarket(mWindCode.toTypedArray(), INDEX_INDICATOR)
    }

    override fun onPause() {
        super.onPause()
        if (requestedOrientation != ActivityInfo.SCREEN_ORIENTATION_PORTRAIT && isBackByUser) {
            requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
        }
        mMapManager.unSubMarket(mWindCode.toTypedArray(), INDEX_INDICATOR)
    }

    private fun readLocalData() {
        var inputStream: InputStream? = null
        var inputStreamReader: InputStreamReader? = null
        var reader: BufferedReader? = null
        val builder = StringBuilder()
        var line: String?
        try {
            inputStream = assets.open("global_map.json")
            inputStreamReader = InputStreamReader(inputStream)
            reader = BufferedReader(inputStreamReader)
            do {
                line = reader.readLine()
                if (line.isNullOrEmpty()) {
                    break
                } else {
                    builder.append(line)
                }
            } while (true)
        } catch (e: IOException) {
            e.printStackTrace()
        } finally {
            try {
                reader?.close()
            } catch (e: IOException) {
                e.printStackTrace()
            }
            try {
                inputStreamReader?.close()
            } catch (e: IOException) {
                e.printStackTrace()
            }
            try {
                inputStream?.close()
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        val source = builder.toString()
        val parseArray = JsonUtils.parseArray(source, GlobalMarketModel::class.java) ?: null
        parseArray?.let {
            mData.addAll(it)
        }
        mData.forEach {
            if (!it.windCode.isEmpty()) {
                mWindCode.add(it.windCode)
            }
        }
    }

    private fun initView() {
        hideNavigationBar(true)
        global_map.post {
            val width = global_map.width
            val height = global_map.height
            if (width == 0 || height == 0) {
                return@post
            }
            val reallyScale = REALLY_MAP_WIDTH.toFloat() / REALLY_MAP_HEIGHT.toFloat()
            val imageScale = width.toFloat() / height.toFloat()
            when {
                reallyScale > imageScale -> {
                    mOffsetY = (height - (width.toFloat() / reallyScale)) / 2
                    mScale = REALLY_MAP_WIDTH.toFloat() / width.toFloat()
                }
                reallyScale < imageScale -> {
                    mOffsetX = (width - (height * reallyScale)) / 2
                    mScale = REALLY_MAP_HEIGHT.toFloat() / height.toFloat()
                }
                else -> mScale = REALLY_MAP_HEIGHT.toFloat() / height.toFloat()
            }
            val waterMarketParams = water_mark_img.layoutParams
            if (waterMarketParams is FrameLayout.LayoutParams) {
                waterMarketParams.width = width / 3
                waterMarketParams.height = ViewGroup.LayoutParams.WRAP_CONTENT
                waterMarketParams.setMargins(mOffsetX.toInt() + 10, 0, 0 ,mOffsetY.toInt() + 10)
            }
            water_mark_img.visibility = View.VISIBLE
            if (mScale == 0f) {
                return@post
            }
            mItemWidth = (MARKET_ITEM_WIDTH / mScale).toInt()
            mItemHeight = (MARKET_ITEM_HEIGHT / mScale).toInt()
            val layoutParam = share_btn.layoutParams
            if (layoutParam is FrameLayout.LayoutParams) {
                layoutParam.rightMargin = layoutParam.rightMargin + mOffsetX.toInt()
                layoutParam.bottomMargin = layoutParam.bottomMargin + mOffsetY.toInt()
                share_btn.layoutParams = layoutParam
            }
        }
        share_btn.setOnClickListener {
            doShare()
        }
        close_btn.setOnClickListener {
            onBack()
        }
    }

    private fun doShare() {
        SharePopupView(this, object : IShare {
            override fun getShareViewType(): Array<ShareTypeEnum> {
                return arrayOf(ShareTypeEnum.SHARE_WECHAT, ShareTypeEnum.SHARE_WECHAT_MOMENTS)
            }

            override fun getShareParams(type: ShareTypeEnum?): Platform.ShareParams {
                val takePhoto = takePhoto()
                val params = Platform.ShareParams()
                if (type == ShareTypeEnum.SHARE_WECHAT || type == ShareTypeEnum.SHARE_WECHAT_MOMENTS) {
                    params.imagePath = takePhoto
                    params.shareType = Wechat.SHARE_IMAGE
                }
                return params
            }

            override fun shareResult(type: ShareTypeEnum?, state: ShareStateEnum?) {
            }
        }).show()
    }

    private fun takePhoto(): String? {
        if (global_map.width == 0 || global_map.height == 0) {
            return ""
        }
        val picPath = getPicPath()
        val bitmap = Bitmap.createBitmap(global_map.width,
                global_map.height, Bitmap.Config.ARGB_8888)
        val canvas = Canvas(bitmap)
        canvas.drawColor(Color.WHITE)
        global_map_main_view.draw(canvas)
        val rightBitmap = (resources.getDrawable(R.drawable.land_share_img) as? BitmapDrawable)?.bitmap
//        val watermarkBitmap = (resources.getDrawable(R.drawable.watermark_wind) as? BitmapDrawable)?.bitmap
        val shareBitmap = BitmapUtils.mergeBitmap_LR(bitmap, rightBitmap, false)
//        if (watermarkBitmap != null && !watermarkBitmap.isRecycled()) {
//            // 绘制水印
//            val watermarkWidth = ((global_map.width - mOffsetX * 2) / 3).toInt()
//            if (watermarkBitmap.width > 0f) {
//                val scaledBitmap = Bitmap.createScaledBitmap(watermarkBitmap, watermarkWidth,
//                        (watermarkBitmap.getHeight() * 1f / watermarkBitmap.getWidth() * watermarkWidth).toInt(), false)
//                val waterCanvas = Canvas(shareBitmap)
//                waterCanvas.translate(mOffsetX, global_map.height - mOffsetY - scaledBitmap.height)
//                waterCanvas.drawBitmap(scaledBitmap, 0f, 0f, null)
//                watermarkBitmap.recycle()
//            }
//        }
        bitmap?.recycle()
        BitMapUtil.savePic(shareBitmap, picPath)
        return picPath
    }

    /**
     * 获取 深度资料 图表 保存的文件路径
     */
    private fun getPicPath(): String {
        return (WindLog.getCachePath() + "/image/share/global_map.png")
    }

    private fun addView(model: GlobalMarketModel) {
        if (mViewMap.containsKey(model.windCode)) {
            // 已经添加过该view，不再添加
            return
        }
        if (mItemWidth == 0 || mItemHeight == 0) {
            return
        }
        if (model.positionX == 0 || model.positionY == 0) {
            return
        }
        val layoutParam = FrameLayout.LayoutParams(mItemWidth, mItemHeight)
        layoutParam.gravity = Gravity.START or Gravity.TOP
        layoutParam.setMargins((model.positionX / mScale + mOffsetX).toInt(),
                (model.positionY / mScale + mOffsetY).toInt(), 0, 0)
        val indexView = IndexMarketView(this)
        indexView.setData(model)
        indexView.layoutParams = layoutParam
        global_map_main_view.addView(indexView)
        mViewMap[model.windCode] = indexView
    }

    override fun onSpeedStatusChange(status: Int, detailMsg: String?) {
        super.onSpeedStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            mMapManager.unSubMarket(mWindCode.toTypedArray(), INDEX_INDICATOR)
            mMapManager.subMarket(mWindCode.toTypedArray(), INDEX_INDICATOR)
        }
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when (msg?.what) {
            MSG_SET_DATA -> {
                msg.obj?.let {
                    val data = it as Vector<RealQuoteItem>
                    mMapManager.parseData(mData, data)
                    mData.forEach { model ->
                        addView(model)
                    }
                }
            }
            MSG_UPDATE_DATA -> {
                msg.obj?.let {
                    val data = it as RealQuoteItem
                    val itemData = mMapManager.parseItemData(mData, data)
                    itemData?.let { model ->
                        val entries = mViewMap.entries
                        entries.forEach { item ->
                            if (item.key == model.windCode) {
                                item.value.setData(model)
                                item.value.updateBg()
                                return@forEach
                            }
                        }
                    }

                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        mMapManager.addCallback(null)
    }

    override fun finish() {
        isBackByUser = true
        super.finish()
    }
}