package wind.android.bussiness.probe.manager

import android.os.Build
import log.LogHelper
import net.bussiness.Stock
import net.datamodel.network.CommonFunc
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import net.datamodel.network.SpeedConst
import net.listener.ITcpDataReceiver
import net.network.model.SpeedErrorModel
import net.network.speed.TcpProcessor
import net.util.Assist
import wind.android.bussiness.login.utils.JsonUtils
import wind.android.bussiness.probe.model.GlobalMarketModel
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      ConfigSettingManager
 * # Function:       WebView页面请求管理器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/3/17$    16:09$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class GlobalMapManager {

    interface IRequestSpeedCallback {
        fun setData(data: Vector<RealQuoteItem>)

        fun updateData(data: RealQuoteItem)
    }

    private var mIndicators: IntArray? = null

    private var mWindCode: Array<String>? = null

    private val mStockMap = mutableMapOf<String, MutableMap<String, String>>()

    private var mCallback: IRequestSpeedCallback? = null

    private val mReceiver = object : ITcpDataReceiver{
        override fun onSubDataRecived(items: Any?) {
            if (items is Vector<*>) {
                val element = items.elementAtOrNull(0)
                if (element is RealQuoteItem) {
                    mCallback?.updateData(element)
                }
            }
        }

        override fun onMaskDataReceived(requestID: Int, data: Any?, type: Int): Boolean {
            if (type == SpeedConst.REQ_MASKED_SUBUNSUB.toInt() && data is Vector<*>) {
                if (data.size == 0 || data[0] !is RealQuoteItem) {
                    return false
                }
                val list = data as Vector<RealQuoteItem>
                mCallback?.setData(list)
                return true
            }
            return false
        }

        override fun onErrorReceived(model: SpeedErrorModel?, type: Int) {
        }

        override fun getTimeOutValue(): Int {
            return 0
        }
    }

    private fun parseItem(item: RealQuoteItem, indicators: IntArray?) {
        val windCode = item.WindCode
        if (windCode.isNullOrBlank()) {
            return
        }
        val itemIndicators = item.indicators ?: null
        val size = itemIndicators?.size ?: 0
        val map = mStockMap[windCode] ?: mutableMapOf()
        val stockName = item.StockName ?: ""
        var unit = 0
        var newPrice = 0.0
        var change = 0.0
        var changeRange = 0.0
        for (i in 0 until size) {
            when (itemIndicators?.get(i)) {
                Indicator.DI_NEWPRICE ->  newPrice = item.value?.get(i)?.toDouble() ?: 0.0
                Indicator.DI_CHANGE -> change = item.value?.get(i)?.toDouble() ?: 0.0
                Indicator.DI_CHANGERANGE -> changeRange = item.value?.get(i)?.toDouble() ?:0.0
                Indicator.DI_PRICEUNIT -> unit = item.value?.get(i)?.toInt() ?: 0
            }
        }
        if (unit == 0) {
            unit = map[Indicator.DI_PRICEUNIT.toString()]?.toInt() ?: 0
        }
        indicators?.forEach {
            if (itemIndicators?.contains(it) == true) {
                when(it) {
                    Indicator.DI_SHORTNAMECHS -> map.put(it.toString(), stockName)
                    Indicator.DI_NEWPRICE ->  map.put(it.toString(), CommonFunc.doubleFormat(newPrice, unit))
                    Indicator.DI_CHANGE -> map.put(it.toString(), CommonFunc.doubleFormat(change, unit))
                    Indicator.DI_CHANGERANGE -> map.put(it.toString(), CommonFunc.doubleFormat(changeRange, unit) + "%")
                    Indicator.DI_PRICEUNIT -> map.put(it.toString(), unit.toString())
                }
            }
        }
        mStockMap.put(windCode, map)
    }

    fun addCallback(callback: IRequestSpeedCallback?) {
        mCallback = callback
    }

    fun subMarket(windCodes: Array<String>, indicators: IntArray) {
        mIndicators = indicators.copyOf()
        mWindCode = windCodes.copyOf()
        TcpProcessor.getInstance().registerPushDataReceiver(mReceiver)
        Stock.SubscribeRequest(windCodes, null, indicators, mReceiver)
    }

    fun unSubMarket(windCodes: Array<String>, indicators: IntArray) {
        TcpProcessor.getInstance().unregisterPushDataReceiver(mReceiver)
        Stock.SubscribeRequest(null, windCodes, indicators, mReceiver)
    }

    fun parseData(data: MutableList<GlobalMarketModel>, list: Vector<RealQuoteItem>) {
        list.forEach {
            parseItemData(data, it)
        }
    }

    fun parseItemData(data: MutableList<GlobalMarketModel>, item: RealQuoteItem): GlobalMarketModel? {
        data.forEach {
            if (it.windCode.isNotEmpty() && it.windCode == item.WindCode) {
                val indicators = item.indicators ?: null
                val size = indicators?.size ?: 0
                if (!item.StockName.isNullOrEmpty()) {
                    it.stockName = item.StockName
                }
                for (i in 0 until size) {
                    when (indicators?.get(i)) {
                        Indicator.DI_NEWPRICE ->  it.price = item.value?.get(i)?.toDouble() ?: 0.0
                        Indicator.DI_CHANGE -> it.change = item.value?.get(i)?.toDouble() ?: 0.0
                        Indicator.DI_CHANGERANGE -> it.changeRange = item.value?.get(i)?.toDouble() ?:0.0
                        Indicator.DI_PRICEUNIT -> it.unit = item.value?.get(i)?.toInt() ?: 0
                    }
                }
                return it
            }
        }
        return null
    }
}