package wind.android.bussiness.probe.view

import android.content.Context
import android.graphics.Color
import android.os.Handler
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.LinearLayout
import kotlinx.android.synthetic.main.item_index_market.view.*
import net.datamodel.network.CommonFunc
import wind.android.R
import wind.android.bussiness.probe.model.GlobalMarketModel
import wind.android.news2.util.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      IndexMarketView$
 * # Function:       市场行情视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/3/19$    14:03$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class IndexMarketView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        LinearLayout(context, attrs, defStyleAttr) {

    companion object {
        const val MSG_RESTORE_BG = 0x1001
        const val DELAY_TIME = 200L
    }

    private val mHandler = Handler {
        if (it.what == MSG_RESTORE_BG) {
            index_market_layout.setBackgroundColor(Color.TRANSPARENT)
        }
        false
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.item_index_market, this)
        this.orientation = LinearLayout.VERTICAL
    }

    fun setData(model: GlobalMarketModel) {
        index_market_name.text = model.stockName
        val textColor = getTextColor(model.change)
        index_market_price.setTextColor(textColor)
        index_market_change.setTextColor(textColor)
        index_change_market_range.setTextColor(textColor)
        index_market_price.text = CommonFunc.doubleFormat(model.price, model.unit)
        index_market_change.text = CommonFunc.doubleFormat(model.change, model.unit)
        index_change_market_range.text = (CommonFunc.doubleFormat(model.changeRange, model.unit) + "%")
    }

    private fun getTextColor(value: Double): Int {
        return StockUtil.getChangeColor(value.toFloat())
    }

    fun updateBg() {
        index_market_layout.setBackgroundResource(R.color.f5_sector_item_bg_red_5)
        if (mHandler.hasMessages(MSG_RESTORE_BG)) {
            mHandler.removeMessages(MSG_RESTORE_BG)
        }
        mHandler.sendEmptyMessageDelayed(MSG_RESTORE_BG, DELAY_TIME)
    }
}