package wind.android.bussiness.strategy.limitupseizeup

import android.support.annotation.IntDef
import database.orm.CommDao
import database.value.KeyValueEnum
import util.JsonUtils

/**
 * # **********************************************************************************************
 * # ClassName:      LimitConfig
 * # Description:    涨停捉妖配置项
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/12/24    19:25
 * # Modifications:  initial
 * # **********************************************************************************************
 */

/**
 * functionId:0:默认，1:涨停，2：快涨停，3：集合竞价开板，4：涨速：5：主力：6开板
 */
const val LIMIT_UP_FUN_ID = 1
const val LIMIT_UP_SOON_FUN_ID = 2
const val LIMIT_UP_MAIN_POWER = 5
const val LIMIT_UP_START_BOARD_FUN_ID = 6

@IntDef(LIMIT_UP_FUN_ID, LIMIT_UP_SOON_FUN_ID, LIMIT_UP_START_BOARD_FUN_ID)
annotation class LimitFunId

/**
 * 字段排序（负数为本地排序）
 *
 * 0默认，1股票名称，2涨停时间、 3状态(连板情况)、 4股票热度(概念、大涨原因)、 5涨停封单额，6价格，7主力流入(废弃)，
 * 8涨幅，9开板时间，10换手率，11开板次数，12流通市值,13涨停价委买,
 * 14: 5分钟涨跌幅,15:1分钟涨跌幅，16:5分钟涨速，17:1分钟涨速
 * 18主力强度，19主力成本，20主力浮盈，21主力流入，22主力分歧，
 * 23主力时间，24主力撤买，25主力撤卖，26主力买入（次)，27主力卖出（次）
 *
 * -1超级单差
 */
const val SORT_DEFAULT_ID = 0 // 0默认
const val SORT_NAME_ID = 1 // 1股票名称
const val SORT_TIME_ID = 2 // 2涨停时间
const val SORT_STATUS_ID = 3 // 3状态(连板情况)
const val SORT_CAUSE_ID = 4 // 4股票热度(概念、大涨原因)
const val SORT_ORDERTOTAL_ID = 5 // 5涨停封单额
const val SORT_PRICE_ID = 6 // 6价格
const val SORT_VOLUME_NETIN_ID = 7 // 7主力流入(废弃)
const val SORT_CHANGE_RATE_ID = 8 // 8涨幅
const val SORT_OPEN_TIME_ID = 9 // 9开板时间
const val SORT_TURNOVER_RATE_ID = 10 // 10换手率
const val SORT_OPENT_TIMES_ID = 11 // 11开板次数
const val SORT_LISTED_STOCK_ID = 12 // 12流通市值
const val SORT_MAIN_STRENGTH_ID = 18 // 18主力强度
const val SORT_MAIN_COST_ID = 19 // 19主力成本
const val SORT_MAIN_PROFIT_ID = 20 // 20主力浮盈
const val SORT_MAIN_IN_ID = 21 // 21主力流入
const val SORT_MAIN_DIVERGENCE_ID = 22 // 22主力分歧
const val SORT_MAIN_TIME_ID = 23 // 23主力时间
const val SORT_MAIN_CANCEL_BUY_ID = 24 // 24主力撤买
const val SORT_MAIN_CANCEL_SELL_ID = 25 // 25主力撤卖
const val SORT_MAIN_BUY_TIMES_ID = 26 // 26主力买入（次)
const val SORT_MAIN_SELL_TIMES_ID = 27 // 27主力卖出（次）
/** 本地排序 定义为负值 **/
const val SORT_MAIN_SUPER_DIFF_ID = -1 // -1超级单差
const val SORT_TODAY_CHANGE = -2 // 最新涨幅-2
const val SORT_UP_TO_CHANGE = -3 // 至今涨幅-3

/** 当日走势101 **/
const val SORT_LIMIE_TREND = 101

@IntDef(SORT_DEFAULT_ID, SORT_NAME_ID, SORT_TIME_ID, SORT_STATUS_ID, SORT_CAUSE_ID, SORT_ORDERTOTAL_ID, SORT_PRICE_ID,
        SORT_VOLUME_NETIN_ID, SORT_CHANGE_RATE_ID, SORT_OPEN_TIME_ID, SORT_TURNOVER_RATE_ID, SORT_OPENT_TIMES_ID, SORT_LISTED_STOCK_ID,
        SORT_MAIN_STRENGTH_ID, SORT_MAIN_COST_ID, SORT_MAIN_PROFIT_ID, SORT_MAIN_IN_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_TIME_ID,
        SORT_MAIN_CANCEL_BUY_ID, SORT_MAIN_CANCEL_SELL_ID, SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID, SORT_MAIN_SUPER_DIFF_ID)
annotation class SortFieldId

@IntDef(SORT_DEFAULT_ID, SORT_NAME_ID, SORT_TIME_ID, SORT_STATUS_ID, SORT_CAUSE_ID, SORT_ORDERTOTAL_ID, SORT_PRICE_ID,
        SORT_VOLUME_NETIN_ID, SORT_CHANGE_RATE_ID, SORT_OPEN_TIME_ID, SORT_TURNOVER_RATE_ID, SORT_OPENT_TIMES_ID, SORT_LISTED_STOCK_ID,
        SORT_MAIN_STRENGTH_ID, SORT_MAIN_COST_ID, SORT_MAIN_PROFIT_ID, SORT_MAIN_IN_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_TIME_ID,
        SORT_MAIN_CANCEL_BUY_ID, SORT_MAIN_CANCEL_SELL_ID, SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID, SORT_MAIN_SUPER_DIFF_ID,
        SORT_LIMIE_TREND, SORT_UP_TO_CHANGE, SORT_TODAY_CHANGE)
annotation class LimitFieldIndex

/**
 * 列排序
 * @param sortId 接口字段排序
 * @param hasTip 表头是否存在问号
 * @param isFix 是否固定，不支持列排序
 */
enum class LimitSort(@LimitFieldIndex val index: Int, var value: String, @SortFieldId val sortId: Int, val hasTip: Boolean = false, val isFix: Boolean = false) {
    STOCK_NAME(SORT_NAME_ID, "股票名称", SORT_DEFAULT_ID, isFix = true),
    /**
     * 快涨停
     * 股票走势、涨幅、主力强度、大涨原因、
     * 连板情况、主力浮盈、主力流入、主力分歧、超级单差、
     * 主力成本、主力买入、主力卖出、主力撤买、主力撤卖
     * 流通市值、价格
     */
    TREND(SORT_LIMIE_TREND, "股票走势", SORT_DEFAULT_ID),
    CHANGE(SORT_CHANGE_RATE_ID, "涨幅", SORT_CHANGE_RATE_ID),
    MAIN_POWER(SORT_MAIN_STRENGTH_ID, "主力强度", SORT_MAIN_STRENGTH_ID, true),
    HIGH_REASON(SORT_CAUSE_ID, "大涨原因", SORT_CAUSE_ID),
    STATE(SORT_STATUS_ID, "连板情况", SORT_STATUS_ID),
    MAIN_GAIN(SORT_MAIN_PROFIT_ID, "主力浮盈", SORT_MAIN_PROFIT_ID, true),
    MAIN_INFLOW(SORT_MAIN_IN_ID, "主力流入", SORT_MAIN_IN_ID, true),
    MAIN_DIFF(SORT_MAIN_DIVERGENCE_ID, "主力分歧", SORT_MAIN_DIVERGENCE_ID, true),
    SUPER_DIFF(SORT_MAIN_SUPER_DIFF_ID, "超级单差", SORT_MAIN_SUPER_DIFF_ID, true),
    MAIN_COST(SORT_MAIN_COST_ID, "主力成本", SORT_MAIN_COST_ID, true),
    MAIN_BUY(SORT_MAIN_BUY_TIMES_ID, "主力买入(次)", SORT_MAIN_BUY_TIMES_ID, true),
    MAIN_SELL(SORT_MAIN_SELL_TIMES_ID, "主力卖出(次)", SORT_MAIN_SELL_TIMES_ID, true),
    MAIN_BUY_CANCEL(SORT_MAIN_CANCEL_BUY_ID, "主力撤买", SORT_MAIN_CANCEL_BUY_ID, true),
    MAIN_SELL_CANCEL(SORT_MAIN_CANCEL_SELL_ID, "主力撤卖", SORT_MAIN_CANCEL_SELL_ID, true),
    MARKET_VALUE(SORT_LISTED_STOCK_ID, "流通市值", SORT_LISTED_STOCK_ID),
    PRICE(SORT_PRICE_ID, "价格", SORT_PRICE_ID),

    /**
     * 涨停
     * （今日涨幅、至今涨幅）、股票走势、涨停原因、
     * 主力强度、主力分歧、超级单差、状态、涨停封单额、
     * 主力浮盈、主力流入、涨停时间、主力成本、
     * 主力买入、主力卖出、主力撤买、主力撤卖、流通市值、价格
     */
    TODAY_CHANGE(SORT_TODAY_CHANGE, "今日涨幅", SORT_DEFAULT_ID, isFix = true),
    UP_TO_CHANGE(SORT_UP_TO_CHANGE, "至今涨幅", SORT_DEFAULT_ID, isFix = true),
    ORDERTOTAL(SORT_ORDERTOTAL_ID, "涨停封单额", SORT_ORDERTOTAL_ID),
    TIME(SORT_TIME_ID, "涨停时间", SORT_TIME_ID),

    /**
     * 主力
     * 股票走势、主力强度、涨跌幅、主力分歧、超级单差、
     * 主力流入、主力浮盈、主力时间、概念、连板情况、
     * 价格、主力成本、主力买入、主力卖出、主力撤买、主力撤卖、
     * 流通市值、换手率
     */
    MAIN_TIME(SORT_MAIN_TIME_ID, "主力时间", SORT_MAIN_TIME_ID, true),
    HAND_RATE(SORT_TURNOVER_RATE_ID, "换手率", SORT_TURNOVER_RATE_ID),

    /**
     * 开板
     * 股票走势、涨幅、主力强度、概念、连板情况、
     * 主力浮盈、主力流入、主力分歧、超级单差、主力成本、
     * 主力买入、主力卖出、主力撤买、主力撤卖、换手率、流通市值、价格
     * 开板次数、开板时间
     */
    OPEN_NUMBER(SORT_OPENT_TIMES_ID, "开板次数", SORT_OPENT_TIMES_ID),
    OPEN_TIME(SORT_OPEN_TIME_ID, "开板时间", SORT_OPEN_TIME_ID)
}

/**
 * 主力
 * 股票走势、主力强度、涨跌幅、主力分歧、超级单差、
 * 主力流入、主力浮盈、主力时间、概念、连板情况、
 * 价格、主力成本、主力买入、主力卖出、主力撤买、主力撤卖、
 * 流通市值、换手率
 */
val defaultMainPower = listOf(/*SORT_TODAY_CHANGE, SORT_UP_TO_CHANGE, */SORT_LIMIE_TREND, SORT_MAIN_STRENGTH_ID,
        SORT_CHANGE_RATE_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_SUPER_DIFF_ID, SORT_MAIN_IN_ID,
        SORT_MAIN_PROFIT_ID, SORT_MAIN_TIME_ID, SORT_CAUSE_ID, SORT_STATUS_ID,
        SORT_PRICE_ID, SORT_MAIN_COST_ID, SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID,
        SORT_MAIN_CANCEL_BUY_ID, SORT_MAIN_CANCEL_SELL_ID, SORT_LISTED_STOCK_ID, SORT_TURNOVER_RATE_ID)

/**
 * 开板
 * 股票走势、涨幅、主力强度、概念、连板情况、
 * 主力浮盈、主力流入、主力分歧、超级单差、主力成本、
 * 主力买入、主力卖出、主力撤买、主力撤卖、换手率、流通市值、价格
 * 开板次数、开板时间
 */
val defaultOpenBoard = listOf(/*SORT_TODAY_CHANGE, SORT_UP_TO_CHANGE, */SORT_LIMIE_TREND, SORT_CHANGE_RATE_ID,
        SORT_MAIN_STRENGTH_ID, SORT_CAUSE_ID, SORT_STATUS_ID, SORT_MAIN_PROFIT_ID,
        SORT_MAIN_IN_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_SUPER_DIFF_ID, SORT_MAIN_COST_ID,
        SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID, SORT_MAIN_CANCEL_BUY_ID, SORT_MAIN_CANCEL_SELL_ID,
        SORT_TURNOVER_RATE_ID, SORT_LISTED_STOCK_ID, SORT_PRICE_ID, SORT_OPENT_TIMES_ID,
        SORT_OPEN_TIME_ID)

/**
 * 涨停
 * （今日涨幅、至今涨幅）、股票走势、涨停原因、
 * 主力强度、主力分歧、超级单差、状态、涨停封单额、
 * 主力浮盈、主力流入、涨停时间、主力成本、
 * 主力买入、主力卖出、主力撤买、主力撤卖、流通市值、价格
 */
val defaultUp = listOf(/*SORT_TODAY_CHANGE, SORT_UP_TO_CHANGE, */SORT_LIMIE_TREND, SORT_CAUSE_ID,
        SORT_MAIN_STRENGTH_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_SUPER_DIFF_ID, SORT_STATUS_ID,
        SORT_ORDERTOTAL_ID, SORT_MAIN_PROFIT_ID, SORT_MAIN_IN_ID, SORT_TIME_ID,
        SORT_MAIN_COST_ID, SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID, SORT_MAIN_CANCEL_BUY_ID,
        SORT_MAIN_CANCEL_SELL_ID, SORT_LISTED_STOCK_ID, SORT_PRICE_ID)

/**
 * 快涨停
 * 股票走势、涨幅、主力强度、大涨原因、
 * 连板情况、主力浮盈、主力流入、主力分歧、超级单差、
 * 主力成本、主力买入、主力卖出、主力撤买、主力撤卖
 * 流通市值、价格
 */
val defaultUpSoon = listOf(/*SORT_TODAY_CHANGE, SORT_UP_TO_CHANGE, */SORT_LIMIE_TREND, SORT_CHANGE_RATE_ID,
        SORT_MAIN_STRENGTH_ID, SORT_CAUSE_ID, SORT_STATUS_ID, SORT_MAIN_PROFIT_ID,
        SORT_MAIN_IN_ID, SORT_MAIN_DIVERGENCE_ID, SORT_MAIN_SUPER_DIFF_ID, SORT_MAIN_COST_ID,
        SORT_MAIN_BUY_TIMES_ID, SORT_MAIN_SELL_TIMES_ID, SORT_MAIN_CANCEL_BUY_ID, SORT_MAIN_CANCEL_SELL_ID,
        SORT_LISTED_STOCK_ID, SORT_PRICE_ID)

fun getTabIndexs(@LimitFunId id: Int): List<Int> {
    return when (id) {
        LIMIT_UP_FUN_ID -> {
            val dbList = JsonUtils.parseArray(CommDao.getInstance()
                    .getValueByKey(KeyValueEnum.KEY_SORT_LIMIT_UP), Int::class.java)
            if (dbList != null && dbList.size == defaultUp.size)
                dbList else defaultUp
        }
        LIMIT_UP_SOON_FUN_ID -> {
            val dbList = JsonUtils.parseArray(CommDao.getInstance()
                    .getValueByKey(KeyValueEnum.KEY_SORT_LIMIT_SOON), Int::class.java)
            if (dbList != null && dbList.size == defaultUpSoon.size)
                dbList else defaultUpSoon
        }
        LIMIT_UP_MAIN_POWER -> {
            val dbList = JsonUtils.parseArray(CommDao.getInstance()
                    .getValueByKey(KeyValueEnum.KEY_SORT_LIMIT_MAIN), Int::class.java)
            if (dbList != null && dbList.size == defaultMainPower.size)
                dbList else defaultMainPower
        }
        LIMIT_UP_START_BOARD_FUN_ID -> {
            val dbList = JsonUtils.parseArray(CommDao.getInstance()
                    .getValueByKey(KeyValueEnum.KEY_SORT_LIMIT_OPEN_BOARD), Int::class.java)
            if (dbList != null && dbList.size == defaultOpenBoard.size)
                dbList else defaultOpenBoard
        }
        else -> defaultUp
    }
}

fun getTabLimits(@LimitFunId id: Int): MutableList<LimitSort> {
    val sortIndex = getTabIndexs(id)
    val sortList = mutableListOf<LimitSort>()
    for (item in sortIndex) {
        sortList.add(getLimitByIndex(item, id))
    }
    return sortList
}

fun getTabDefaultLimits(@LimitFunId id: Int): MutableList<LimitSort> {
    val sortIndex = when (id) {
        LIMIT_UP_FUN_ID -> defaultUp
        LIMIT_UP_SOON_FUN_ID -> defaultUpSoon
        LIMIT_UP_MAIN_POWER -> defaultMainPower
        LIMIT_UP_START_BOARD_FUN_ID -> defaultOpenBoard
        else -> null
    }
    val sortList = mutableListOf<LimitSort>()
    sortIndex?.forEach {
        sortList.add(getLimitByIndex(it, id))
    }
    return sortList
}

fun getLimitByIndex(@LimitFieldIndex index: Int, @LimitFunId id: Int): LimitSort {
    return when (index) {
        SORT_LIMIE_TREND -> LimitSort.TREND
        SORT_CHANGE_RATE_ID -> LimitSort.CHANGE
        SORT_MAIN_STRENGTH_ID -> LimitSort.MAIN_POWER
        SORT_CAUSE_ID -> {
            when (id) {
                // 涨停tab
                LIMIT_UP_FUN_ID -> {
                    LimitSort.HIGH_REASON.value = "涨停原因"
                    LimitSort.HIGH_REASON
                }
                // 开板tab
                LIMIT_UP_START_BOARD_FUN_ID,
                    // 主力tab
                LIMIT_UP_MAIN_POWER -> {
                    LimitSort.HIGH_REASON.value = "概念"
                    LimitSort.HIGH_REASON
                }
                // 大涨原因
                else -> {
                    LimitSort.HIGH_REASON.value = "大涨原因"
                    LimitSort.HIGH_REASON
                }
            }
        }
        SORT_STATUS_ID -> {
            when (id) {
                // 涨停tab
                LIMIT_UP_FUN_ID -> {
                    LimitSort.STATE.value = "状态"
                    LimitSort.STATE
                }
                // 连板情况
                else -> {
                    LimitSort.STATE.value = "连板情况"
                    LimitSort.STATE
                }
            }
        }
        SORT_MAIN_PROFIT_ID -> LimitSort.MAIN_GAIN
        SORT_MAIN_IN_ID -> LimitSort.MAIN_INFLOW
        SORT_MAIN_DIVERGENCE_ID -> LimitSort.MAIN_DIFF
        SORT_MAIN_SUPER_DIFF_ID -> LimitSort.SUPER_DIFF
        SORT_MAIN_COST_ID -> LimitSort.MAIN_COST
        SORT_MAIN_BUY_TIMES_ID -> LimitSort.MAIN_BUY
        SORT_MAIN_SELL_TIMES_ID -> LimitSort.MAIN_SELL
        SORT_MAIN_CANCEL_BUY_ID -> LimitSort.MAIN_BUY_CANCEL
        SORT_MAIN_CANCEL_SELL_ID -> LimitSort.MAIN_SELL_CANCEL
        SORT_LISTED_STOCK_ID -> LimitSort.MARKET_VALUE
        SORT_PRICE_ID -> LimitSort.PRICE
        SORT_TODAY_CHANGE -> LimitSort.TODAY_CHANGE
        SORT_UP_TO_CHANGE -> LimitSort.UP_TO_CHANGE
        SORT_ORDERTOTAL_ID -> LimitSort.ORDERTOTAL
        SORT_TIME_ID -> LimitSort.TIME
        SORT_MAIN_TIME_ID -> LimitSort.MAIN_TIME
        SORT_TURNOVER_RATE_ID -> LimitSort.HAND_RATE
        SORT_OPENT_TIMES_ID -> LimitSort.OPEN_NUMBER
        SORT_OPEN_TIME_ID -> LimitSort.OPEN_TIME
        else -> LimitSort.CHANGE
    }
}