package wind.android.bussiness.strategy.limitupseizeup

import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.os.Message
import android.support.design.widget.TabLayout
import android.util.SparseArray
import android.view.MotionEvent
import android.view.View
import android.widget.ImageView
import base.BaseNetHandler
import base.RootFragment
import base.screenshot.ScreenShotManager
import base.screenshot.view.SnapPopupWindow
import database.orm.CommDao
import database.value.KeyValueEnum
import datamodel.speed.MarketDBModel
import eventbus.EventBusManager
import eventinterface.TouchEventListener
import kotlinx.android.synthetic.main.activity_limit_up_seize.*
import kotlinx.android.synthetic.main.activity_limit_up_seize_head.*
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import session.F5Session
import ui.UINavigationBar
import useraction.UserActionReader
import util.*
import wind.android.R
import wind.android.base.StockBaseActivity
import wind.android.bussiness.optionalstock.manager.OptionalUtil
import wind.android.bussiness.optionalstock.util.UserActionFunctionId.*
import wind.android.bussiness.strategy.limitupseizeup.LimitUpSettingActivity.Companion.SAVE_LIMIT_SHOW_TAG
import wind.android.bussiness.strategy.limitupseizeup.LimitUpSettingActivity.Companion.SAVE_LIMIT_SORT
import wind.android.bussiness.strategy.limitupseizeup.LimitUpSettingActivity.Companion.SHOW_TAG_KEY
import wind.android.bussiness.strategy.limitupseizeup.bean.*
import wind.android.bussiness.strategy.limitupseizeup.fragment.LimitMainPowerFrag
import wind.android.bussiness.strategy.limitupseizeup.fragment.LimitOpenBoardFrag
import wind.android.bussiness.strategy.limitupseizeup.fragment.LimitUpFrag
import wind.android.bussiness.strategy.limitupseizeup.fragment.LimitUpSoonFrag
import wind.android.bussiness.strategy.limitupseizeup.presenter.LimitUpActPresenter
import wind.android.bussiness.strategy.limitupseizeup.presenter.LimitUpActPresenter.TOPIC_CLEAR_ALL_ID
import wind.android.bussiness.strategy.limitupseizeup.view.ILimitActivityView
import wind.android.bussiness.strategy.view.StockThemeTagLayout
import wind.android.common.StockThemeUtils
import wind.android.f5.activity.SpeedDetailActivity
import wind.android.f5.snapshot.BitmapUtils
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      LimitUpSeizeActivity
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/9/24    19:25
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class LimitUpSeizeActivity : StockBaseActivity(), TouchEventListener, StockThemeTagLayout.OnTagViewClickListener,
        View.OnClickListener, ILimitActivityView, TabLayout.BaseOnTabSelectedListener<TabLayout.Tab> {

    private var mCurrentFragment: RootFragment? = null
    private val mFragments: SparseArray<RootFragment> = SparseArray()
    private lateinit var mPresenter: LimitUpActPresenter
    private val mScreenShotManager: ScreenShotManager by lazy { ScreenShotManager(this) }
    private var mScreenShotLastPath: String? = null
    /**
     * 是否展示题材标签
     */
    private var showTagValue: Boolean = true

    override fun onCreate(savedInstanceState: Bundle?) {
        StockThemeUtils.setTheme(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_limit_up_seize)
        mPresenter = LimitUpActPresenter(this)
        mScreenShotManager.setCallback {
            if (!it.isNullOrBlank() && it != mScreenShotLastPath) {
                mScreenShotLastPath = it
                runOnUiThread {
                    takeScreenShot()
                    UserActionReader.subMitUserAction(LIMIT_UP_SCREEN_SHOT)
                }
            }
        }
        initView()
        EventBusManager.getInstance().register(this)
        EventBusManager.getInstance().postSticky(LimitCurrentDateEvent(true))
        mPresenter.reqTradeCalendar()
        val tabIdStr = intent?.getStringExtra(TAB_ID)
        val tabIndex = StringUtils.valueOfInt(tabIdStr)
        tab_layout.setSelectTab(tabIndex)
        //蒙层引导
        showMaskLabel(intArrayOf(0, 0, 0, 0), intArrayOf(R.drawable.limit_up_mask1, R.drawable.limit_up_mask2),
                "limit_up_help", null, null)
        UserActionReader.subMitUserAction(LIMIT_UP_OPEN)
    }

    override fun onResume() {
        super.onResume()
        mPresenter.subIndex()
        mScreenShotManager.register()
    }

    override fun onPause() {
        super.onPause()
        mPresenter.unSubIndex()
        mScreenShotManager.unregister()
    }

    override fun onDestroy() {
        val limitReqParamsEvent = EventBusManager.getInstance().getStickyEvent(LimitReqParamsEvent::class.java)
        limitReqParamsEvent?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        val limitCurrentDateEvent = EventBusManager.getInstance().getStickyEvent(LimitCurrentDateEvent::class.java)
        limitCurrentDateEvent?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        val limitTodayDateEvent = EventBusManager.getInstance().getStickyEvent(LimitTodayDateEvent::class.java)
        limitTodayDateEvent?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        val limitVerticalToHorizontalEvent = EventBusManager.getInstance().getStickyEvent(LimitVerticalToHorizontalEvent::class.java)
        limitVerticalToHorizontalEvent?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        EventBusManager.getInstance().unregister(this)
        mFragments.clear()
        mPresenter.detachView()
        mScreenShotManager.onDestroy()
        tab_layout.clearOnTabSelectedListeners()
        super.onDestroy()
    }

    override fun onSpeedStatusChange(status: Int, detailMsg: String?) {
        super.onSpeedStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            mPresenter.reqTradeCalendar()
            mPresenter.subIndex()
        }
    }

    /**
     * 初始化view
     */
    private fun initView() {
        navigationBar.setTitle(getString(R.string.limit_up_seize))
        initShareBtn()
        navigationBar.setListener(this)
        //左上角大盘指数
        val color = StockUtil.RISE_RED_COLOR
        index_price.setTextColor(color)
        index_change.setTextColor(color)
        index_change_rate.setTextColor(color)
        index_part.setOnClickListener(this)
        tag_scroll_layout.setOnTagViewClickListener(this)
        //底部条件按钮
        only_low_price.setOnClickListener(this)
        only_small_plate.setOnClickListener(this)
        main_power_in.setOnClickListener(this)
        only_optional.setOnClickListener(this)
        // 横屏按钮
        limit_rotate.setOnClickListener(this)
        //日历
        calendar_view.setOnDateSelectListener { dateStr, isLast ->
            EventBusManager.getInstance().post(LimitDateMsgEvent(dateStr))
            EventBusManager.getInstance().postSticky(LimitCurrentDateEvent(isLast))
        }
        //frag_container
        if (CommonValue.styleType == CommonValue.StyleType.STYLE_WHITE) {
            tab_constraint.setBackgroundResource(R.color.bg_color_normal_white)
            ThemeUtils.setWhiteBg(frag_container, resources)
        } else {
            tab_constraint.setBackgroundResource(R.color.bg_color_normal_black)
            ThemeUtils.setBlackBg(frag_container, resources)
        }
        val tabTiles = resources.getStringArray(R.array.limit_up_seize_titles)
        tab_layout.setTabItems(tabTiles, tabTitleTextSize = 15)
        tab_layout.addOnTabSelectedListener(this)
        showTagValue = CommDao.getInstance().getValueByKey(KeyValueEnum.KEY_LIMIT_TAG_SHOW)
                ?.toBoolean() ?: true
        tag_scroll_layout.visibility = if (showTagValue) View.VISIBLE else View.GONE
    }

    private fun initShareBtn() {
        val shareBtn = ImageView(this)
        shareBtn.setImageResource(R.drawable.icon_share_press)
        val width = UINavigationBar.DEF_BAR_HEIGHT / 2
        navigationBar.setRightView(shareBtn, width)

        val settingBtn = ImageView(this)
        settingBtn.setImageResource(R.drawable.icon_setting_w)
        navigationBar.setRightSecondView(settingBtn, UINavigationBar.DEF_BAR_HEIGHT * 2 / 3)
    }

    /**
     * 展示/隐藏对应的Fragment
     *
     * @param index tab索引
     */
    private fun showTab(index: Int) {
        var fragment = mFragments.get(index)
        if (mCurrentFragment != null && mCurrentFragment === fragment) {
            return
        }
        val transaction = supportFragmentManager.beginTransaction()
        mCurrentFragment?.let { transaction.hide(it) }
        if (fragment == null) {
            fragment = when (index) {
                LIMIT_UP_FRAG_INDEX -> LimitUpFrag()
                LIMIT_SOON_FRAG_INDEX -> LimitUpSoonFrag()
                LIMIT_MAIN_POWER_FRAG_INDEX -> LimitMainPowerFrag()
                LIMIT_OPEN_BOARD_FRAG -> LimitOpenBoardFrag()
                else -> null
            }
            fragment?.let {
                val simpleName = it::class.java.simpleName
                transaction.add(R.id.frag_container, it, simpleName)
                mCurrentFragment = it
                mFragments.put(index, it)
                val date = calendar_view.currentDateStr
                val isChecked = only_optional.isChecked
                val stockFilterIds = mPresenter.stockFilterIds
                val topicIds = mPresenter.topicIds
                EventBusManager.getInstance().postSticky(LimitReqParamsEvent(it.hashCode(), date,
                        isChecked, stockFilterIds, topicIds))
            }
        } else {
            transaction.show(fragment)
            mCurrentFragment = fragment
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1 && isDestroyed) {
            return
        }
        transaction.commitAllowingStateLoss()
        //功能点
        when (index) {
            LIMIT_UP_FRAG_INDEX -> UserActionReader.subMitUserAction(LIMIT_UP_LIMIT_UP_TAB)
            LIMIT_SOON_FRAG_INDEX -> UserActionReader.subMitUserAction(LIMIT_UP_SOON_LIMIT_UP_TAB)
            LIMIT_MAIN_POWER_FRAG_INDEX -> UserActionReader.subMitUserAction(LIMIT_UP_MAIN_TAB)
            LIMIT_OPEN_BOARD_FRAG -> UserActionReader.subMitUserAction(LIMIT_UP_START_BOARD_TAB)
        }
    }

    /**
     * 截图
     */
    private fun takeScreenShot() {
        val bodyView = mainBodyView
        if (navigationBar == null || bodyView == null || isFinishing) {
            return
        }
        val height = navigationBar.height + bodyView.height
        val width = navigationBar.width
        if (width > 0 && height > 0) {
            val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
            val canvas = Canvas(bitmap)
            if (CommonValue.styleType == CommonValue.StyleType.STYLE_BLACK) {
                canvas.drawColor(Color.BLACK)
            } else {
                canvas.drawColor(Color.WHITE)
            }
            navigationBar.draw(canvas)
            canvas.translate(0f, navigationBar.height.toFloat())
            bodyView.draw(canvas)
            val shareBitmap = BitmapUtils.mergeBitmap_TB(bitmap, null, null, this)
            BitMapUtil.savePic(shareBitmap, ScreenShotManager.SCREEN_SHOT_TMP_PATH)
            val snapPopupWindow = SnapPopupWindow(this, ScreenShotManager.SCREEN_SHOT_TMP_PATH)
            snapPopupWindow.showDialog()
        }
    }

    @Subscribe
    fun onFragMessage(messageEvent: LimitMessageEvent) {
        if (messageEvent.getmMessageId() == LimitMessageEvent.MESSAGE_LIMIT_TAG_LIST) {
            val tagList = messageEvent.agrObj as? MutableList<LimitTopicModel>
            if (!mPresenter.isSameTopicCompareToCur(tagList)) {
                onUpdateTagView(tagList)
                mPresenter.setCurTopicModels(tagList)
            }
        } else if (messageEvent.getmMessageId() == LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT) {
            val tabStockCount = messageEvent.agrObj as? MutableMap<Int, Int?>
            onUpdateTabCount(tabStockCount)
        } else if (messageEvent.getmMessageId() == LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT_POS) {
            val tabPos = when (messageEvent.arg1) {
                LIMIT_UP_FUN_ID -> LIMIT_UP_FRAG_INDEX
                LIMIT_UP_SOON_FUN_ID -> LIMIT_SOON_FRAG_INDEX
                LIMIT_UP_MAIN_POWER -> LIMIT_MAIN_POWER_FRAG_INDEX
                LIMIT_UP_START_BOARD_FUN_ID -> LIMIT_OPEN_BOARD_FRAG
                else -> -1
            }
            onUpdateTabCount(tabPos, messageEvent.arg2)
        }
    }

    @Subscribe(sticky = true, threadMode = ThreadMode.MAIN)
    fun onFilterMsg(msgEvent: LimitVerticalToHorizontalEvent) {
        // 更新底部按钮状态
        msgEvent.filterIdList?.let {
            only_low_price.isChecked = it.contains(1)
            only_small_plate.isChecked = it.contains(2)
            main_power_in.isChecked = it.contains(3)
            mPresenter.stockFilterIds = msgEvent.filterIdList
        }
        only_optional.isChecked = msgEvent.isChecked
        // 标签筛选
        mPresenter.topicIds = msgEvent.topicIds
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.index_part -> {
                val intent = Intent(this, SpeedDetailActivity::class.java)
                intent.putExtra(F5Session.POSITION, 0)
                F5Session.getInstance().f5WindCodes = arrayOf("000001.SH")
                startActivity(intent)
            }
            R.id.only_low_price -> {
                only_low_price.isChecked = !only_low_price.isChecked
                mPresenter.setStockFilterIds(1)
            }
            R.id.only_small_plate -> {
                only_small_plate.isChecked = !only_small_plate.isChecked
                mPresenter.setStockFilterIds(2)
            }
            R.id.main_power_in -> {
                main_power_in.isChecked = !main_power_in.isChecked
                mPresenter.setStockFilterIds(3)
            }
            R.id.only_optional -> {
                val checked = !only_optional.isChecked
                only_optional.isChecked = checked
                mPresenter.setOptionalFilter(checked)
            }
            R.id.limit_rotate -> {
                val intent = Intent(this, LimitUpLandActivity::class.java)
                intent.putExtra(TAB_ID, tab_layout.selectedTabPosition)
                startActivityForResult(intent, TO_VERTICAL_ACTIVITY)
                val stockFilterIds = mPresenter.stockFilterIds
                val topicIds = mPresenter.topicIds
                val isChecked = only_optional.isChecked
                EventBusManager.getInstance().postSticky(LimitHorizontalToVerticalEvent(
                        calendar_view.currentDateStr, stockFilterIds, topicIds, isChecked))
            }
        }
    }

    override fun handleMessage(msg: Message?) {
        when (msg?.what) {
            INDEX_MSG -> {
                val model = msg.obj as MarketDBModel
                var color = OptionalUtil.setElementValue(index_change, model.change,
                        false, true, 2)
                if (color == Color.WHITE || color == Color.BLACK) {
                    color = 0xFFDD2637.toInt()
                }
                OptionalUtil.setElementValue(index_change_rate, model.changeRate,
                        true, false, 2)
                index_change.setTextColor(color)
                index_change_rate.setTextColor(color)
                index_price.text = model.newPrice ?: "0.00"
                index_price.setTextColor(color)
                index_name.setTextColor(color)
            }
            TOPIC_TAG_MSG -> {
                val topicModelList = msg.obj as List<LimitTopicModel>
                val hasSelected = mPresenter.setTopicListSelecedItem(topicModelList)
                tag_scroll_layout.setThemeTagViewData(topicModelList, hasSelected)
                if (topicModelList.isEmpty()) {
                    mPresenter.showCountdown()
                }
            }
            STOCK_NUMS_MSG -> {
                val numMap = msg.obj as MutableMap<Int, Int?>
                val tab0Count = numMap[LIMIT_UP_FUN_ID]
                tab_layout.setTabMsgCount(LIMIT_UP_FRAG_INDEX, tab0Count ?: 0)
                val tab1Count = numMap[LIMIT_UP_SOON_FUN_ID]
                tab_layout.setTabMsgCount(LIMIT_SOON_FRAG_INDEX, tab1Count ?: 0)
                val tab2Count = numMap[LIMIT_UP_MAIN_POWER]
                tab_layout.setTabMsgCount(LIMIT_MAIN_POWER_FRAG_INDEX, tab2Count ?: 0)
                val tab3Count = numMap[LIMIT_UP_START_BOARD_FUN_ID]
                tab_layout.setTabMsgCount(LIMIT_OPEN_BOARD_FRAG, tab3Count ?: 0)
            }
            STOCK_NUMS_POS_MSG -> tab_layout.setTabMsgCount(msg.arg1, msg.arg2)
            TOPIC_TAG_ERROR_MSG -> ToastTool.showToast(msg.obj as String)
            TRADE_CALENDER_MSG -> {
                val tradeCalenderList = msg.obj as List<String>
                if (tradeCalenderList.isNotEmpty()) {
                    calendar_view.setCalenderDateRes(tradeCalenderList)
                    EventBusManager.getInstance().postSticky(LimitTodayDateEvent(tradeCalenderList.last()))
                }
            }
            OPEN_TIME_MSG -> {
                //倒计时结束，触发重新发送涨停捉妖列表请求
                EventBus.getDefault().post(LimitDateMsgEvent(calendar_view.currentDateStr))
                tag_scroll_layout.setTagTipText("暂无标签数据")
            }
        }
    }

    override fun touchEvent(sender: View?, event: MotionEvent?) {
        val viewTag = sender?.tag as? Int
        if (viewTag == UINavigationBar.RIGHT_BUTTON) {
            takeScreenShot()
            UserActionReader.subMitUserAction(LIMIT_UP_SHARE)
        } else if (viewTag == UINavigationBar.RIGHT_SECOND_BUTTON) {
            val intent = Intent(this, LimitUpSettingActivity::class.java)
            intent.putExtra(SHOW_TAG_KEY, showTagValue)
            startActivityForResult(intent, TO_SETTING_ACTIVITY)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == TO_SETTING_ACTIVITY && resultCode == Activity.RESULT_OK) {
            data?.let {
                val isShowTag = it.getBooleanExtra(SAVE_LIMIT_SHOW_TAG, showTagValue)
                showTagValue = isShowTag
                tag_scroll_layout.visibility = if (isShowTag) View.VISIBLE else View.GONE
                val needRefreshSort = it.getBooleanExtra(SAVE_LIMIT_SORT, false)
                if (needRefreshSort) {
                    val count = tab_layout.tabCount
                    for (index in 0 until count) {
                        val fragment = mFragments.get(index)
                        fragment?.let { rf ->
                            if (rf === mCurrentFragment) {
                                // 通知刷新表头
                                EventBus.getDefault().post(LimitMessageEvent(LimitMessageEvent.MESSAGE_LIMIT_LIST_TITLE, null))
                            } else {
                                val transaction = supportFragmentManager.beginTransaction()
                                mFragments.remove(index)
                                transaction.remove(rf)
                                transaction.commitAllowingStateLoss()
                            }
                        }
                    }
                }
            }
        } else if (requestCode == TO_VERTICAL_ACTIVITY && resultCode == Activity.RESULT_OK) {
            data?.let {
                val tabId = it.getIntExtra(TAB_ID, tab_layout.selectedTabPosition)
                tab_layout.setSelectTab(tabId)
            }
        }
    }

    override fun onTagViewClick(tagId: Int, tagName: String?) {
        mPresenter.setTopicIds(tagId, tagName)
    }

    override fun onTagSelectedClear() {
        mPresenter.setTopicIds(TOPIC_CLEAR_ALL_ID, null)
    }

    override fun onUpdateIndexView(indexModel: MarketDBModel?) {
        indexModel?.let {
            val msg = Message.obtain()
            msg.what = INDEX_MSG
            msg.obj = it
            sendMessage(msg)
        }
    }

    override fun onUpdateTagView(tagList: MutableList<LimitTopicModel>?) {
        val list = mutableListOf<LimitTopicModel>()
        tagList?.let { list.addAll(it) }
        val msg = Message.obtain()
        msg.what = TOPIC_TAG_MSG
        msg.obj = list
        sendMessage(msg)
    }

    override fun onUpdateTabCount(tabStockCount: MutableMap<Int, Int?>?) {
        val map = mutableMapOf<Int, Int?>()
        tabStockCount?.let { map.putAll(it) }
        val msg = Message.obtain()
        msg.what = STOCK_NUMS_MSG
        msg.obj = map
        sendMessage(msg)
    }

    /**
     * 根据tab position 更新指定tab的右上角数字
     *
     * @param position tab index
     * @param count 右上角数字
     */
    private fun onUpdateTabCount(position: Int, count: Int) {
        val msg = Message.obtain()
        msg.what = STOCK_NUMS_POS_MSG
        msg.arg1 = position
        msg.arg2 = count
        sendMessage(msg)
    }

    override fun onUpdateTradeCalendar(tradeList: MutableList<String>?) {
        val list = mutableListOf<String>()
        tradeList?.let { list.addAll(it) }
        val message = Message.obtain()
        message.what = TRADE_CALENDER_MSG
        message.obj = list
        sendMessage(message)
    }

    override fun onUpdateCountdown(text: CharSequence?, finish: Boolean) {
        if (finish) {
            //倒计时结束，延后几秒重新发送请求
            tag_scroll_layout.setTagTipText("数据正在处理中,请稍等～")
            sendEmptyMessageDelayed(OPEN_TIME_MSG, 5000)
        } else {
            tag_scroll_layout.setTagTipText(text)
        }
    }

    override fun onError(errorType: Int, errorMsg: String?) {

    }

    override fun onTabReselected(p0: TabLayout.Tab?) {
    }

    override fun onTabUnselected(p0: TabLayout.Tab?) {
    }

    override fun onTabSelected(p0: TabLayout.Tab?) {
        val position = p0?.position ?: 0
        showTab(position)
    }

    companion object ConstField {
        const val TAB_ID = "tabId"
        /**
         * tab index
         */
        private const val LIMIT_UP_FRAG_INDEX = 0
        private const val LIMIT_SOON_FRAG_INDEX = 1
        private const val LIMIT_MAIN_POWER_FRAG_INDEX = 2
        private const val LIMIT_OPEN_BOARD_FRAG = 3
        /**
         * message消息码
         */
        private const val TOPIC_TAG_MSG = 0x1
        private const val TOPIC_TAG_ERROR_MSG = 0x2
        private const val STOCK_NUMS_MSG = 0x3
        private const val INDEX_MSG = 0x4
        private const val TRADE_CALENDER_MSG = 0x5
        private const val OPEN_TIME_MSG = 0x6
        private const val STOCK_NUMS_POS_MSG = 0x7
        private const val TO_SETTING_ACTIVITY = 0x123
        private const val TO_VERTICAL_ACTIVITY = 0x124
    }
}
