package wind.android.bussiness.strategy.limitupseizeup

import android.app.Activity
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.support.design.widget.TabLayout
import android.util.SparseArray
import android.util.TypedValue
import android.view.Gravity
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import base.BaseActivity
import database.orm.CommDao
import database.value.KeyValueEnum
import eventinterface.TouchEventListener
import kotlinx.android.synthetic.main.activity_limit_up_setting.*
import kotlinx.android.synthetic.main.activity_limit_up_setting_head.*
import ui.UINavigationBar
import useraction.SkyUserAction
import useraction.UserActionReader
import wind.android.R
import wind.android.bussiness.optionalstock.util.UserActionFunctionId.*
import wind.android.bussiness.strategy.limitupseizeup.view.LimitSortSettingView
import wind.android.common.StockThemeUtils

/**
 * # **********************************************************************************************
 * # ClassName:      LimitUpSettingActivity
 * # Description:    涨停捉妖设置页面
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/12/24    14:05
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class LimitUpSettingActivity : BaseActivity(), TabLayout.BaseOnTabSelectedListener<TabLayout.Tab>, TouchEventListener, View.OnClickListener {
    /**
     * view缓存
     */
    private val viewCache: SparseArray<LimitSortSettingView> by lazy { SparseArray<LimitSortSettingView>() }
    /**
     * 当前列表view
     */
    private var currentView: LimitSortSettingView? = null
    /**
     * 显示题材/隐藏题材
     */
    private var isShowTag = true
    override fun onCreate(savedInstanceState: Bundle?) {
        StockThemeUtils.setTheme(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_limit_up_setting)
        initView()
        val showTagValue = intent?.getBooleanExtra(SHOW_TAG_KEY, true) ?: true
        show_tag_view.isSelected = showTagValue
        show_tag_title.isSelected = showTagValue
        hide_tag_view.isSelected = !showTagValue
        hide_tag_title.isSelected = !showTagValue
        isShowTag = showTagValue
        tab_layout.setSelectTab(TAB0)
    }

    /**
     * 初始化
     */
    private fun initView() {
        navigationBar.setTitle(resources.getString(R.string.limit_setting_title))
        val completeView = TextView(this)
        completeView.text = "完成"
        completeView.gravity = Gravity.CENTER_VERTICAL
        completeView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 15f)
        completeView.setTextColor(Color.WHITE)
        navigationBar.setRightView(completeView)
        val resetView = TextView(this)
        resetView.text = "重置"
        resetView.gravity = Gravity.CENTER_VERTICAL
        resetView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 15f)
        resetView.setTextColor(Color.WHITE)
        navigationBar.setRightSecondView(resetView, UINavigationBar.DEF_BAR_HEIGHT * 4 / 5)
        navigationBar.setListener(this)

        show_tag_view.setOnClickListener(this)
        show_tag_title.setOnClickListener(this)

        hide_tag_view.setOnClickListener(this)
        hide_tag_title.setOnClickListener(this)

        val tabTiles = resources.getStringArray(R.array.limit_up_seize_titles)
        tab_layout.setTabItems(tabTiles, tabTitleTextSize = 16)
        tab_layout.addOnTabSelectedListener(this)
    }

    override fun onTabReselected(p0: TabLayout.Tab?) {
    }

    override fun onTabUnselected(p0: TabLayout.Tab?) {
    }

    override fun onTabSelected(p0: TabLayout.Tab?) {
        val position = p0?.position ?: 0
        showTab(position)
    }

    override fun touchEvent(sender: View?, event: MotionEvent?) {
        val viewTag = sender?.tag as? Int
        if (viewTag == UINavigationBar.RIGHT_BUTTON) {
            val size = viewCache.size()
            var isNeedUpdateSort = false
            for (i in 0 until size) {
                val sortSettingView = viewCache[i]
                if (!isNeedUpdateSort) {
                    isNeedUpdateSort = sortSettingView?.isDragAndNoSave() ?: false
                }
                val saveOrder = sortSettingView?.saveOrder() ?: false
                if (saveOrder) {
                    val paramItem = when (i) {
                        TAB0 -> SkyUserAction.ParamItem("tab", "zt")
                        TAB1 -> SkyUserAction.ParamItem("tab", "kzt")
                        TAB2 -> SkyUserAction.ParamItem("tab", "zl")
                        TAB3 -> SkyUserAction.ParamItem("tab", "kb")
                        else -> null
                    }
                    paramItem?.let { UserActionReader.subMitUserAction(LIMIT_SETTING_COMPLETE_BUTTON, paramItem) }
                }
            }
            CommDao.getInstance().updateKeyValue(KeyValueEnum.KEY_LIMIT_TAG_SHOW, isShowTag.toString())
            val tagValue = if (isShowTag) "显示题材" else "隐藏题材"
            UserActionReader.subMitUserAction(LIMIT_SETTING_IS_SHOW_TAG, SkyUserAction.ParamItem("tag", tagValue))
            val intent = Intent()
            intent.putExtra(SAVE_LIMIT_SORT, isNeedUpdateSort)
            intent.putExtra(SAVE_LIMIT_SHOW_TAG, isShowTag)
            setResult(Activity.RESULT_OK, intent)
            finish()
        } else if (viewTag == UINavigationBar.RIGHT_SECOND_BUTTON) {
            currentView?.resetOrder()
        }
    }

    /**
     * 切换展示tab
     *
     * @param index tab index
     */
    private fun showTab(index: Int) {
        view_container.removeAllViews()
        val view = viewCache.get(index) ?: when (index) {
            TAB0 -> {
                val view = LimitSortSettingView(this, tabId = LIMIT_UP_FUN_ID)
                viewCache.put(TAB0, view)
                view
            }
            TAB1 -> {
                val view = LimitSortSettingView(this, tabId = LIMIT_UP_SOON_FUN_ID)
                viewCache.put(TAB1, view)
                view
            }
            TAB2 -> {
                val view = LimitSortSettingView(this, tabId = LIMIT_UP_MAIN_POWER)
                viewCache.put(TAB2, view)
                view
            }
            TAB3 -> {
                val view = LimitSortSettingView(this, tabId = LIMIT_UP_START_BOARD_FUN_ID)
                viewCache.put(TAB3, view)
                view
            }
            else -> null
        }
        view?.let {
            val layoutParams = FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT
                    , ViewGroup.LayoutParams.MATCH_PARENT)
            view_container.addView(it, layoutParams)
            currentView = it
        }

    }


    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.show_tag_view,
            R.id.show_tag_title -> {
                if (!isShowTag) {
                    show_tag_view.isSelected = true
                    show_tag_title.isSelected = true
                    hide_tag_view.isSelected = false
                    hide_tag_title.isSelected = false
                    isShowTag = true
                }
                UserActionReader.subMitUserAction(LIMIT_SETTING_SHOW_TAG)
            }
            R.id.hide_tag_view,
            R.id.hide_tag_title -> {
                if (isShowTag) {
                    show_tag_view.isSelected = false
                    show_tag_title.isSelected = false
                    hide_tag_view.isSelected = true
                    hide_tag_title.isSelected = true
                    isShowTag = false
                }
                UserActionReader.subMitUserAction(LIMIT_SETTING_HIDE_TAG)
            }
        }
    }

    companion object {
        private const val TAB0 = 0
        private const val TAB1 = 1
        private const val TAB2 = 2
        private const val TAB3 = 3
        const val SAVE_LIMIT_SORT = "save_limit_sort"
        const val SAVE_LIMIT_SHOW_TAG = "save_limit_show_tag"
        const val SHOW_TAG_KEY = "show_tag_key"
    }
}