package wind.android.bussiness.strategy.limitupseizeup.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.*
import android.view.GestureDetector.SimpleOnGestureListener
import android.widget.ImageView
import android.widget.TextView
import util.CommonValue
import wind.android.R
import wind.android.bussiness.optionalstock.listener.ISortAdapter
import wind.android.bussiness.strategy.limitupseizeup.LimitSort
import java.util.*

/**
 * # **********************************************************************************************
 * # ClassName:      SortSettingAdapter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/12/25    14:25
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class LimitSortSettingAdapter(private val context: Context) : RecyclerView.Adapter<RecyclerView.ViewHolder>(), ISortAdapter {
    /**
     * 数据源
     */
    val data: ArrayList<LimitSort> by lazy { arrayListOf<LimitSort>() }
    /**
     * 拖动排序监听事件
     */
    var dragSortListener: ((RecyclerView.ViewHolder) -> Unit)? = null
    /**
     * 固定顺序个数
     */
    private var fixListSize: Int = 0
    /**
     * 拖动排序但未保存
     */
    var isDragNoSave = false

    /**
     * 设置数据
     *
     * @param list
     */
    fun setData(list: List<LimitSort>?) {
        data.clear()
        list?.let {
            val filterFix = it.filter { item -> item.isFix }
            fixListSize = filterFix.size
            val filterNotFix = it.filter { item -> !item.isFix }
            data.addAll(filterFix)
            data.addAll(filterNotFix)
        }
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.limit_setting_sort_item, viewGroup, false)
        return SortSettingViewHolder(view, dragSortListener)
    }

    override fun getItemCount(): Int = data.size

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        val viewHolder = holder as SortSettingViewHolder
        val item = data.elementAtOrNull(position)
        item?.run {
            viewHolder.title.text = value
            viewHolder.dragImg.visibility = if (isFix) View.GONE else View.VISIBLE
        }
    }

    override fun onItemMove(fromPosition: Int, toPosition: Int) {
        val realToPosition = if (toPosition < fixListSize)
            fixListSize else toPosition
        //交换位置
        when {
            fromPosition == realToPosition || fromPosition < 0 || realToPosition < 0 -> {
                return
            }
            fromPosition < realToPosition && realToPosition < data.size -> {
                for (i in fromPosition until realToPosition) {
                    Collections.swap(data, i, i + 1)
                }
            }
            fromPosition > realToPosition -> {
                for (i in fromPosition downTo realToPosition + 1) {
                    Collections.swap(data, i, i - 1)
                }
            }
        }
        notifyItemMoved(fromPosition, realToPosition)
        isDragNoSave = true
    }

    @SuppressLint("ClickableViewAccessibility")
    class SortSettingViewHolder(itemView: View, dragSortListener: ((RecyclerView.ViewHolder) -> Unit)?) : RecyclerView.ViewHolder(itemView) {
        val title: TextView by lazy { itemView.findViewById<TextView>(R.id.title) }
        val dragImg: ImageView by lazy { itemView.findViewById<ImageView>(R.id.drag_img) }

        init {
            val simpleOnGestureListener: GestureDetector.OnGestureListener = object : SimpleOnGestureListener() {
                override fun onShowPress(e: MotionEvent) {
                    dragSortListener?.invoke(this@SortSettingViewHolder)
                    super.onShowPress(e)
                }
            }
            val imgResId = if (CommonValue.styleType == CommonValue.StyleType.STYLE_BLACK) {
                R.drawable.ico_drag_black
            } else {
                R.drawable.ico_drag_white
            }
            dragImg.setImageResource(imgResId)
            val gestureDetector = GestureDetector(itemView.context, simpleOnGestureListener)
            dragImg.setOnTouchListener { _, event ->
                gestureDetector.onTouchEvent(event)
                true
            }
        }
    }
}