package wind.android.bussiness.strategy.limitupseizeup.fragment

import android.content.Intent
import android.os.Bundle
import android.os.Message
import android.support.v7.app.AlertDialog
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import base.RootFragment
import eventbus.EventBusManager
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import session.F5Session
import util.StringUtils
import util.ToastTool
import wind.android.R
import wind.android.bussiness.login.acitvity.LoginActivity
import wind.android.bussiness.probe.activity.ConfigSettingActivity
import wind.android.bussiness.strategy.limitupseizeup.LimitSort
import wind.android.bussiness.strategy.limitupseizeup.adapter.BaseLimitUpAdapter
import wind.android.bussiness.strategy.limitupseizeup.bean.*
import wind.android.bussiness.strategy.limitupseizeup.presenter.BaseLimitPresenter
import wind.android.bussiness.strategy.limitupseizeup.view.ILimitFragView
import wind.android.common.HttpUrlUtils_windstock

/**
 * # **********************************************************************************************
 * # ClassName:      AbsLimitUpFrag
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/9/26    19:18
 * # Modifications:  initial
 * # **********************************************************************************************
 */

abstract class BaseLimitUpFrag<T : BaseLimitPresenter<ILimitFragView>> : RootFragment(), ILimitFragView, BaseLimitUpAdapter.TrendItemListener {
    /**
     * 页面是否可见
     */
    protected var mFragVisible = false
    /**
     * presenter
     */
    protected val mPresenter by lazy { createPresenter() }

    @Volatile
    protected open var mPermission = false

    protected var mLastDate = true

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(getLayoutId(), null)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        mFragVisible = true
        init()
        if (isRegisterEventBus()) {
            EventBusManager.getInstance().register(this)
        }
    }

    override fun onResume() {
        super.onResume()
        if (mFragVisible) {
            onFragShow(true)
        }
    }

    override fun onPause() {
        super.onPause()
        if (mFragVisible) {
            onFragHide(true)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        mPresenter.detachView()
        if (isRegisterEventBus()) {
            EventBusManager.getInstance().unregister(this)
        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        if (hidden) {
            mFragVisible = false
            onFragHide(false)
        } else {
            mFragVisible = true
            onFragShow(false)
        }
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        if (!mFragVisible) return
    }

    override fun onUpdatePermission(isPermission: Boolean) {
        mPermission = isPermission
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onActMessage(messageEvent: LimitDateMsgEvent) {
        onDateChange(messageEvent.date)
    }

    @Subscribe(threadMode = ThreadMode.MAIN, sticky = true)
    fun onLastMessage(messageEvent: LimitCurrentDateEvent) {
        onLastDate(messageEvent.isLastDate)
        mLastDate = messageEvent.isLastDate
    }

    @Subscribe(threadMode = ThreadMode.MAIN, sticky = true)
    fun onTodayMessage(messageEvent: LimitTodayDateEvent) {
        messageEvent.todayDate?.let {
            mPresenter.initLittleTrendManager(StringUtils.valueOfInt(it, 0))
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onActMessage(messageEvent: LimitStockFilterMsgEvent) {
        onStockFilterChange(messageEvent.filterIdList)
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onActMessage(messageEvent: LimitTopicMsgEvent) {
        onTopicIdsChange(messageEvent.topicIdList)
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onActMessage(messageEvent: LimitOptionalFilterMsgEvent) {
        onFilterOptional(messageEvent.isFilterOptional)
    }

    @Subscribe(threadMode = ThreadMode.MAIN, sticky = true)
    fun onActMessage(messageEvent: LimitReqParamsEvent) {
        if (messageEvent.tag == this.hashCode()) {
            onInitParamsReceive(messageEvent.date, messageEvent.isFilterOptional,
                    messageEvent.selectedFilterIds, messageEvent.selectedTopicIds)
            val event = EventBusManager.getInstance().getStickyEvent(LimitReqParamsEvent::class.java)
            event?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onActMessage(messageEvent: LimitMessageEvent) {
        onActNoStickyMsg(messageEvent)
    }

    /**
     * 弹框提示
     */
    protected fun showAlertDialog(title: String, message: String) {
        activity?.let {
            val builder = AlertDialog.Builder(it)
                    .setTitle(title)
                    .setMessage(message)
                    .setPositiveButton(getString(R.string.alert_dialog_know), null)
                    .create()
            builder.show()
        }
    }

    /**
     * 前往权限购买页面
     */
    protected fun jumpToUnlockBuy() {
        if (F5Session.getInstance().isLoginReal) {
            val intent = Intent(activity, ConfigSettingActivity::class.java)
            intent.putExtra(ConfigSettingActivity.STOCK_COMMOM_URL, HttpUrlUtils_windstock.limitBuyurl)
            startActivityForResult(intent, TO_UNLOCK_BUY)
        } else {
            ToastTool.showToast(resources.getString(R.string.dialog_phone_alert))
            startActivity(Intent(activity, LoginActivity::class.java))
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == TO_UNLOCK_BUY && F5Session.getInstance().return2RefreshTag) {
            onLimitPermissionBuy()
            F5Session.getInstance().return2RefreshTag = false
        }
    }

    override fun getTrend(windCode: String?) {
        windCode?.let { mPresenter.subTrend(it) }
    }

    protected fun clickTip(v: View?) {
        val tag = v?.tag
        if (tag is LimitSort) {
            when (tag) {
                LimitSort.MAIN_POWER -> showAlertDialog(getString(R.string.limit_main_strength_explain_title),
                        getString(R.string.limit_main_strength_explain))
                LimitSort.MAIN_GAIN -> showAlertDialog(getString(R.string.limit_main_profit_explain_title),
                        getString(R.string.limit_main_profit_explain))
                LimitSort.MAIN_INFLOW -> showAlertDialog(getString(R.string.limit_main_in_explain_title),
                        getString(R.string.limit_main_in_explain))
                LimitSort.MAIN_DIFF -> showAlertDialog(getString(R.string.limit_main_divergence_explain_title),
                        getString(R.string.limit_main_divergence_explain))
                LimitSort.SUPER_DIFF -> showAlertDialog(getString(R.string.limit_main_super_diff_explain_title),
                        getString(R.string.limit_main_super_diff_explain))
                LimitSort.MAIN_COST -> showAlertDialog(getString(R.string.limit_main_cost_explain_title),
                        getString(R.string.limit_main_cost_explain))
                LimitSort.MAIN_BUY -> showAlertDialog(getString(R.string.limit_main_buy_times_explain_title),
                        getString(R.string.limit_main_buy_times_explain))
                LimitSort.MAIN_SELL -> showAlertDialog(getString(R.string.limit_main_sell_times_explain_title),
                        getString(R.string.limit_main_sell_times_explain))
                LimitSort.MAIN_BUY_CANCEL -> showAlertDialog(getString(R.string.limit_main_cancel_buy_explain_title),
                        getString(R.string.limit_main_cancel_buy_explain))
                LimitSort.MAIN_SELL_CANCEL -> showAlertDialog(getString(R.string.limit_main_cancel_sell_explain_title),
                        getString(R.string.limit_main_cancel_sell_explain))
                LimitSort.MAIN_TIME -> showAlertDialog(getString(R.string.limit_main_time_explain_title),
                        getString(R.string.limit_main_time_explain))
            }
        }
    }

    /**
     * 完成购买涨停捉妖权限
     */
    protected open fun onLimitPermissionBuy() {}

    /**
     * 布局文件
     */
    abstract fun getLayoutId(): Int

    abstract fun createPresenter(): T

    /**
     * 初始化
     */
    abstract fun init()

    /**
     * 是否要注册eventBus
     */
    abstract fun isRegisterEventBus(): Boolean

    /**
     * 页面隐藏
     */
    abstract fun onFragHide(byOnPause: Boolean)

    /**
     * 页面显示
     */
    abstract fun onFragShow(byOnResume: Boolean)

    /**
     * 日期变化
     */
    abstract fun onDateChange(date: String?)

    /**
     * 切换历史/最新日期
     */
    abstract fun onLastDate(isLast: Boolean)

    /**
     * 条件过滤（只看低价、只看小盘、主力流入）
     */
    abstract fun onStockFilterChange(filterIdList: MutableList<Int>)

    /**
     * 标签过滤条件
     */
    abstract fun onTopicIdsChange(topicIdList: MutableList<Int>)

    /**
     * 是否过滤自选
     */
    abstract fun onFilterOptional(filterOptional: Boolean)

    abstract fun onInitParamsReceive(date: String, isFilterOptional: Boolean,
                                     filterIds: MutableList<Int>?, topicIds: MutableList<Int>?)

    /**
     * Activity传过的非eventbus粘性消息
     */
    abstract fun onActNoStickyMsg(messageEvent: LimitMessageEvent)

    companion object {
        private const val TO_UNLOCK_BUY = 0x1
    }

}