package wind.android.bussiness.strategy.limitupseizeup.presenter

import android.content.Context
import android.text.TextUtils
import network.NetManager
import org.greenrobot.eventbus.EventBus
import wind.android.bussiness.optionalstock.listener.OnEditOptionalStockListener
import wind.android.bussiness.optionalstock.manager.OptionalOperate
import wind.android.bussiness.optionalstock.util.OptionalSession
import wind.android.bussiness.strategy.limitupseizeup.*
import wind.android.bussiness.strategy.limitupseizeup.bean.LimitMessageEvent
import wind.android.bussiness.strategy.limitupseizeup.bean.LimitTopicModel
import wind.android.bussiness.strategy.limitupseizeup.bean.LimitUpDetailModel
import wind.android.bussiness.strategy.limitupseizeup.bean.LimitUpInfoModel
import wind.android.bussiness.strategy.limitupseizeup.view.ILimitFragView
import wind.android.f5.model.LittleTrendModel

/**
 * # **********************************************************************************************
 * # ClassName:      LimitUpFragPresenter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/9/27    17:56
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class LimitUpFragPresenter(fragView: ILimitFragView, @LimitFunId val mFuntionId: Int) : BaseLimitPresenter<ILimitFragView>(fragView) {

    /**
     * 数据请求model层
     */
    private val mManager: LimitUpSeizeManager = LimitUpSeizeManager()
    /**
     * 过滤条件（只看低价、只看小盘、主力流入）
     */
    private var mStockFilterIds: MutableList<Int>? = null
    /**
     * 题材标签条件
     */
    private var mTopicIds: MutableList<Int>? = null
    /**
     * 日期
     */
    private var mDate: String? = null
    /**
     * 排序指标id
     */
    @SortFieldId
    private var mSortFiedId: Int = 0
    /**
     * 排序类型
     */
    private var mSortType: Int = 0
    /**
     * 列表item position
     */
    private var mPosition: Int = 0
    /**
     * 缓存题材标签列表
     */
    private val mTopicModelList: MutableList<LimitTopicModel> = mutableListOf()
    /**
     * 是否过滤自选股
     */
    @Volatile
    private var mIsFilterOptional: Boolean = false

    private val mResultCallBack: OnResultCallBack<LimitUpInfoModel> = object : OnResultCallBack<LimitUpInfoModel> {
        override fun onResultCallBack(result: LimitUpInfoModel?) {
            val view = getView()
            view?.onHideProgressMum()
            if (result == null) {
                view?.onUpdateListAll(null)
                EventBus.getDefault().post(LimitMessageEvent(LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT, null))
                EventBus.getDefault().post(LimitMessageEvent(LimitMessageEvent.MESSAGE_LIMIT_TAG_LIST, null))
            } else {
                val stockDetailList = result.stockDetail
                val permission = result.isPermission
                if (stockDetailList != null && !stockDetailList.isEmpty()) {
                    val iterator = stockDetailList.iterator()
                    while (iterator.hasNext()) {
                        val item = iterator.next()
                        item.isPermission = permission
                        if (!markOptional(item) && mIsFilterOptional) {
                            iterator.remove()
                        }
                    }
                }
                val stockNums = result.stockNums
                val topicMap = result.topicList
                var topicModelList: MutableList<LimitTopicModel>? = null
                if (topicMap != null) {
                    topicModelList = mutableListOf()
                    topicMap.forEach { topicModelList.add(it.value) }
                    topicModelList.sort()
                    mTopicModelList.clear()
                    mTopicModelList.addAll(topicModelList)
                }
                view?.onUpdatePermission(permission)
                view?.onUpdateListAll(stockDetailList)
                EventBus.getDefault().post(LimitMessageEvent(LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT,
                        stockNums))
                if (mIsFilterOptional) {
                    var count = 0
                    if (stockDetailList != null) {
                        count = stockDetailList.size
                    }
                    EventBus.getDefault().post(LimitMessageEvent(
                            LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT_POS, mFuntionId, count))
                }
                EventBus.getDefault().post(LimitMessageEvent(
                        LimitMessageEvent.MESSAGE_LIMIT_TAG_LIST, topicModelList))
            }
        }

        override fun onError(errorCode: Int, errorMsg: String?) {
            val view = getView()
            view?.let {
                EventBus.getDefault().post(LimitMessageEvent(
                        LimitMessageEvent.MESSAGE_LIMIT_TAG_LIST, mTopicModelList))
                it.onHideProgressMum()
                it.onError(errorCode, errorMsg)
            }
        }
    }

    /**
     * 设置股票过滤条件id参数
     *
     * @param stockFilterIds 股票过滤条件
     */
    fun setStockFilterIds(stockFilterIds: MutableList<Int>?) {
        mStockFilterIds = stockFilterIds
    }

    /**
     * 设置题材标签id列表参数
     *
     * @param topicIds 题材标签id
     */
    fun setTopicIds(topicIds: MutableList<Int>?) {
        mTopicIds = topicIds
    }

    /**
     * 设置日期参数
     *
     * @param date 日期
     */
    fun setDate(date: String?) {
        mDate = date
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param date           日期
     * @param stockFilterIds 股票过滤列表（只看低价、只看小盘、主力流入）
     * @param functionId     功能选项
     * @param topicIds       题材标签ID列表
     * @param sortFiedId     排序字段ID
     * @param sortType       排序方式
     */
    private fun requestStockDetail(date: String?, stockFilterIds: MutableList<Int>?, functionId: Int,
                                   topicIds: MutableList<Int>?, sortFiedId: Int, sortType: Int) {
        mManager.requestStockDetailInfo(date, stockFilterIds, functionId, topicIds,
                sortFiedId, sortType, mResultCallBack)
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param date           日期
     * @param stockFilterIds 股票过滤条件（只看低价、只看小盘、主力流入）
     * @param topicIds       题材标签ID
     * @param sortFiedId     排序字段ID
     * @param sortType       排序方式
     */
    fun reqLimitDetail(date: String, stockFilterIds: MutableList<Int>?, topicIds: MutableList<Int>?,
                       sortFiedId: Int = mSortFiedId, sortType: Int = mSortType) {
        mDate = date
        setStockFilterIds(stockFilterIds)
        setTopicIds(topicIds)
        mSortFiedId = sortFiedId
        mSortType = sortType
        val view = getView() ?: return
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
        } else {
            view.onShowProgressMum()
            requestStockDetail(date, mStockFilterIds, mFuntionId, mTopicIds, sortFiedId, sortType)
        }
    }

    /**
     * 请求涨停捉妖信息列表
     */
    fun reqLimitDetail() {
        val view = getView() ?: return
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
            EventBus.getDefault().post(LimitMessageEvent(
                    LimitMessageEvent.MESSAGE_LIMIT_TAG_LIST, mTopicModelList))
        } else {
            view.onShowProgressMum()
            requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
        }
    }

    fun reqLimitDetailByPullRefresh() {
        val view = getView() ?: return
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
        } else {
            requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
        }
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param date 日期
     */
    fun reqLimitDetailByDate(date: String?) {
        val view = getView() ?: return
        mDate = date
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
        } else {
            view.onShowProgressMum()
            requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
        }
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param stockFilterIds 股票过滤条件
     */
    fun reqLimitDetailByFilterId(stockFilterIds: MutableList<Int>?) {
        val view = getView() ?: return
        setStockFilterIds(stockFilterIds)
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
        } else {
            view.onShowProgressMum()
            requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
        }
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param topicIds 题材标签id
     */
    fun reqLimitDetailByTopicId(topicIds: MutableList<Int>?) {
        val view = getView() ?: return
        setTopicIds(topicIds)
        if (!NetManager.isNetworkAvailable()) {
            view.onShowTip("请检查网络连接")
        } else {
            view.onShowProgressMum()
            requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
        }
    }

    /**
     * 请求涨停捉妖信息列表
     *
     * @param sortFiedId 排序字段ID
     * @param sortType   排序方式
     */
    fun reqLimitDetailBySort(@SortFieldId sortFiedId: Int, sortType: Int, listData: MutableList<LimitUpDetailModel?>) {
        val view = getView() ?: return
        if (sortFiedId >= 0) {
            mSortFiedId = sortFiedId
            mSortType = sortType
            if (!NetManager.isNetworkAvailable()) {
                view.onShowTip("请检查网络连接")
            } else {
                view.onShowProgressMum()
                requestStockDetail(mDate, mStockFilterIds, mFuntionId, mTopicIds, mSortFiedId, mSortType)
            }
        } else {
            if (listData.size > 0) {
                // 本地排序
                when (sortFiedId) {
                    SORT_MAIN_SUPER_DIFF_ID -> {
                        val list = listData.sortedBy {
                            it?.let { item ->
                                if (sortType == 2) item.buyAvgAmount - item.sellAvgAmount
                                else item.sellAvgAmount - item.buyAvgAmount
                            }
                        }.toMutableList()
                        view.onUpdateListAll(list)
                    }
                    SORT_TODAY_CHANGE -> {
                    }
                    SORT_UP_TO_CHANGE -> {
                    }
                }
            }
        }
    }

    /**
     * 设置删除/添加自选回调
     */
    fun setOptionalOperateListener() {
        OptionalOperate.setOnEditOptionalStockListener(object : OnEditOptionalStockListener {
            override fun onAddStock(addSuccess: Boolean) {
                val view = getView() ?: return
                if (addSuccess) {
                    view.onShowTip("添加自选成功")
                    view.onUpdateListByPosition(mPosition, ILimitFragView.ADD_OPT_STOCK)
                } else {
                    view.onShowTip("添加自选失败")
                }
            }

            override fun onDelStock(delSuccess: Boolean) {
                val view = getView() ?: return
                if (delSuccess) {
                    view.onShowTip("删除自选成功")
                    view.onUpdateListByPosition(mPosition, ILimitFragView.DEL_OPT_STOCK)
                } else {
                    view.onShowTip("删除自选失败")
                }
            }
        })
    }

    /**
     * 释放删除/添加自选回调
     */
    fun releaseOptionalOperateListener() {
        OptionalOperate.setOnEditOptionalStockListener(null)
    }

    /**
     * 删除添加自选
     */
    fun operateToOptional(context: Context?, item: LimitUpDetailModel?, position: Int) {
        if (context == null || item == null) {
            return
        }
        mPosition = position
        val windCode = item.windCode
        if (TextUtils.isEmpty(windCode)) {
            return
        }
        if (OptionalSession.getInstance().isOptionalExist(windCode)) {
            OptionalOperate.delFromOptional(windCode)
        } else {
            OptionalOperate.addToOptional(windCode, item.name, context)
        }
    }

    /**
     * 设置过滤自选股标志位
     *
     * @param isFilterOptional true过滤  false 不过滤
     */
    fun setFilterOptional(isFilterOptional: Boolean) {
        mIsFilterOptional = isFilterOptional
    }

    /**
     * 自选股过滤操作
     *
     * @param isFilterOptional true过滤  false 不过滤
     */
    fun filterOptional(isFilterOptional: Boolean, listData: MutableList<LimitUpDetailModel?>?) {
        setFilterOptional(isFilterOptional)
        if (isFilterOptional && !listData.isNullOrEmpty()) {
            val list = listData.filter { it?.isOptionalStock ?: false }.toMutableList()
            val view = getView()
            if (view != null) {
                view.onUpdateListAll(list)
                EventBus.getDefault().post(LimitMessageEvent(
                        LimitMessageEvent.MESSAGE_LIMIT_TAB_COUNT_POS, mFuntionId, list.size))
            }
        } else if (!isFilterOptional) {
            reqLimitDetail()
        }
    }

    /**
     * 获取列表的windCode数组
     *
     * @param data 列表数据
     * @return 列表的windCode数组
     */
    fun getWindCodes(data: List<LimitUpDetailModel?>): Array<String?> {
        return data.map { it?.windCode }.toTypedArray()
    }

    /**
     * 标记是否在自选股列表中
     *
     * @param item 涨停捉妖item
     * @return true 该item对应的股票在自选股列表中 false 不在自选股列表中
     */
    private fun markOptional(item: LimitUpDetailModel?): Boolean {
        val list = OptionalSession.getInstance().selfStockTreeGroupModel
        if (item != null && list != null) {
            //全部自选股组
            val curPosition = OptionalSession.getInstance().optionalPosition
            for ((index, value) in list.withIndex()) {
                if (value != null && value.records != null && value.records.size > 0) {
                    for (model in value.records) {
                        if (!item.windCode.isNullOrEmpty() && item.windCode == model.windCode) {
                            item.isOptionalStock = true
                            if (index == curPosition) {
                                item.isCurGroupOptStock = true
                            }
                            return true
                        }
                    }
                }
            }
        }
        return false
    }

    override fun onRefresh(model: LittleTrendModel?) {
        val view = getView() ?: return
        view.onUpdateLittleTrend(model)
    }
}