package wind.android.bussiness.strategy.limitupseizeup.view

import android.content.Context
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.helper.ItemTouchHelper
import android.util.AttributeSet
import database.orm.CommDao
import database.value.KeyValueEnum
import util.JsonUtils
import wind.android.bussiness.optionalstock.callback.DragSortTouchCallback
import wind.android.bussiness.strategy.limitupseizeup.*
import wind.android.bussiness.strategy.limitupseizeup.adapter.LimitSortSettingAdapter

/**
 * # **********************************************************************************************
 * # ClassName:      LimitSortSettingView
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/12/25    13:14
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class LimitSortSettingView @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0,
        @LimitFunId val tabId: Int = LIMIT_UP_FUN_ID
) : RecyclerView(context, attrs, defStyleAttr) {

    private val adapter: LimitSortSettingAdapter by lazy { LimitSortSettingAdapter(context) }

    init {
        layoutManager = LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        val dragSortTouchCallback = DragSortTouchCallback(adapter)
        val itemTouchHelper = ItemTouchHelper(dragSortTouchCallback)
        itemTouchHelper.attachToRecyclerView(this)
        adapter.dragSortListener = {
            itemTouchHelper.startDrag(it)
        }
        val tabLimits = getTabLimits(tabId)
        tabLimits.add(0, LimitSort.STOCK_NAME)
        adapter.setData(tabLimits)
        setAdapter(adapter)
    }

    /**
     * 保存顺序
     */
    fun saveOrder(): Boolean {
        if (!adapter.isDragNoSave) return false
        val list = arrayListOf<LimitSort>()
        list.addAll(adapter.data)
        val sortList = mutableListOf<Int>()
        list.forEach {
            if (it != LimitSort.STOCK_NAME) {
                sortList.add(it.index)
            }
        }
        when (tabId) {
            LIMIT_UP_FUN_ID -> CommDao.getInstance().updateKeyValue(KeyValueEnum.KEY_SORT_LIMIT_UP,
                    JsonUtils.toJson(sortList))
            LIMIT_UP_SOON_FUN_ID -> CommDao.getInstance().updateKeyValue(KeyValueEnum.KEY_SORT_LIMIT_SOON,
                    JsonUtils.toJson(sortList))
            LIMIT_UP_MAIN_POWER -> CommDao.getInstance().updateKeyValue(KeyValueEnum.KEY_SORT_LIMIT_MAIN,
                    JsonUtils.toJson(sortList))
            LIMIT_UP_START_BOARD_FUN_ID -> CommDao.getInstance().updateKeyValue(KeyValueEnum.KEY_SORT_LIMIT_OPEN_BOARD,
                    JsonUtils.toJson(sortList))
        }
        adapter.isDragNoSave = false
        return true
    }

    fun resetOrder(){
        val tabDefaultLimits = getTabDefaultLimits(tabId)
        tabDefaultLimits.add(0, LimitSort.STOCK_NAME)
        adapter.setData(tabDefaultLimits)
        adapter.notifyDataSetChanged()
        adapter.isDragNoSave = true
    }

    fun isDragAndNoSave(): Boolean {
        return adapter.isDragNoSave
    }

}