package wind.android.bussiness.strategy.limitupseizeup.view

import android.content.Context
import android.content.res.ColorStateList
import android.content.res.TypedArray
import android.graphics.Color
import android.support.annotation.ColorInt
import android.support.annotation.ColorRes
import android.support.v4.content.res.ResourcesCompat
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import util.CommonValue
import util.StringUtils
import wind.android.R
import wind.android.news2.view.tablayout.MsgView

/**
 * # **********************************************************************************************
 * # ClassName:      TitleMsgView
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/10/10    14:02
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class TitleMsgView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : ViewGroup(context, attrs, defStyleAttr) {

    /**
     * y轴方向偏移比例
     */
    private var yOffsetRadio: Float
    /**
     * 标题文字大小
     */
    private var titleTextSize: Int
    /**
     * 标题文字颜色
     */
    private var titleTextColor: ColorStateList?
    /**
     * 底部指示线颜色
     */
    private var indicatorColor: Int

    init {
        val ta: TypedArray = context.obtainStyledAttributes(attrs, R.styleable.TitleMsgView)
        yOffsetRadio = ta.getFloat(R.styleable.TitleMsgView_y_offset_radio, 0.3f)
        titleTextSize = ta.getInt(R.styleable.TitleMsgView_title_text_size, 15)
        titleTextColor = ta.getColorStateList(R.styleable.TitleMsgView_title_text_color)
        indicatorColor = ta.getColor(R.styleable.TitleMsgView_indicator_color, Color.RED)
        ta.recycle()
        if (titleTextColor == null) {
            titleTextColor = if (CommonValue.styleType == CommonValue.StyleType.STYLE_BLACK)
                ResourcesCompat.getColorStateList(resources, R.color.title_tab_txt_selector_b, null) else
                ResourcesCompat.getColorStateList(resources, R.color.title_tab_txt_selector_w, null)
        }
        addViews(context)
    }

    /**
     * 添加view
     *
     * @param context 上下文
     */
    private fun addViews(context: Context) {
        val titleText = TextView(context)
        titleText.setTextSize(TypedValue.COMPLEX_UNIT_DIP, titleTextSize.toFloat())
        titleText.setTextColor(titleTextColor)
        addView(titleText, 0)
        val msgText = MsgView(context)
        msgText.setHideBackground(true)
        msgText.backgroundColor = 0xFFFD481F.toInt()
        msgText.setIsRadiusHalfHeight(true)
        msgText.strokeColor = 0xFFFD481F.toInt()
        msgText.strokeWidth = 1
        msgText.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 10f)
        msgText.setTextColor(0xFFFF0000.toInt())
        msgText.maxLines = 1
        msgText.gravity = Gravity.START or Gravity.TOP
        val padding = StringUtils.dipToPx(2f)
        msgText.setPadding(padding, padding, padding, padding)
        addView(msgText, 1)
        val bottomLine = View(context)
        bottomLine.setBackgroundColor(indicatorColor)
        val bottomLineLp = MarginLayoutParams(BOTTOM_LINE_WIDTH, BOTTOM_LINE_HEIGHT)
        bottomLine.layoutParams = bottomLineLp
        addView(bottomLine, 2)
    }

    /**
     * 设置标题文字内容
     *
     * @param text 标题内容
     */
    fun setTitleText(text: String?) {
        val titleTv: TextView? = getChildAt(0) as? TextView
        titleTv?.text = text
    }

    /**
     * 设置标题文字大小
     *
     * @param size 标题文字大小，单位dp
     */
    fun setTitleTextSize(size: Int) {
        titleTextSize = size
        val titleTv: TextView? = getChildAt(0) as? TextView
        titleTv?.setTextSize(TypedValue.COMPLEX_UNIT_DIP, size.toFloat())
    }

    /**
     * 设置标题文字颜色
     *
     * @param colorRes 颜色资源id
     */
    fun setTitleTextColor(@ColorRes colorRes: Int) {
        titleTextColor = ResourcesCompat.getColorStateList(resources, colorRes, null)
        val titleTv: TextView? = getChildAt(0) as? TextView
        titleTv?.setTextColor(titleTextColor)
    }

    /**
     * 设置右上角文字内容
     *
     * @param text 右上角文字内容
     */
    fun setMsgText(text: String?) {
        val msgView: MsgView? = getChildAt(1) as? MsgView
        msgView?.text = text
    }

    /**
     * 设置右上角文字是否可见
     *
     * @param visible 是否可见
     */
    fun setMsgTextVisible(visible: Boolean) {
        val msgView: MsgView? = getChildAt(1) as? MsgView
        msgView?.visibility = if (visible) View.VISIBLE else View.GONE
    }

    /**
     * 设置底部指示线颜色
     *
     * @param color 颜色值
     */
    fun setBottomLineColor(@ColorInt color: Int) {
        indicatorColor = color
        val indicatorView: View? = getChildAt(2)
        indicatorView?.setBackgroundColor(color)
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val sizeWidth = MeasureSpec.getSize(widthMeasureSpec)
        val sizeHeight = MeasureSpec.getSize(heightMeasureSpec)
        val modeWidth = MeasureSpec.getMode(widthMeasureSpec)
        val modeHeight = MeasureSpec.getMode(heightMeasureSpec)
        var width = 0
        var height = 0
        measureChildren(widthMeasureSpec, heightMeasureSpec)
        //childCount = 3
        for (i in 0 until childCount) {
            val childView = getChildAt(i)
            childView?.let {
                val lp = layoutParams as MarginLayoutParams
                if (i < 2) {
                    width += it.measuredWidth + lp.leftMargin + lp.rightMargin
                }
                val childHeight = it.measuredHeight + lp.topMargin + lp.bottomMargin
                if (i == 1) {
                    height += (childHeight * yOffsetRadio).toInt()
                } else {
                    height += childHeight
                }
            }
        }
        width = if (modeWidth == MeasureSpec.EXACTLY) sizeWidth else width
        height = if (modeHeight == MeasureSpec.EXACTLY) sizeHeight else
            height + BOTTOM_LINE_MARGIN_TOP
        setMeasuredDimension(width, height)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        var view0Left = 0
        var view0Top = 0
        var view0Right = 0
        var view0Bottom = 0
        val isExactly = layoutParams?.width != LayoutParams.WRAP_CONTENT
                && layoutParams?.height != LayoutParams.WRAP_CONTENT
        val childView0 = getChildAt(0)
        val childView1 = getChildAt(1)
        val childView2 = getChildAt(2)
        if (isExactly) {
            if (childView0 != null && childView0.visibility != View.GONE) {
                view0Left = measuredWidth / 2 - childView0.measuredWidth / 2
                view0Top = measuredHeight / 2 - childView0.measuredHeight / 2
                view0Right = measuredWidth / 2 + childView0.measuredWidth / 2
                view0Bottom = measuredHeight / 2 + childView0.measuredHeight / 2
                childView0.layout(view0Left, view0Top, view0Right, view0Bottom)
            }
        } else {
            if (childView0 != null && childView0.visibility != View.GONE) {
                view0Left = 0
                view0Top = measuredHeight - childView0.measuredHeight - (childView2?.measuredHeight
                        ?: 0) - BOTTOM_LINE_MARGIN_TOP
                view0Right = childView0.measuredWidth
                view0Bottom = view0Top + childView0.measuredHeight
                childView0.layout(view0Left, view0Top, view0Right, view0Bottom)
            }
        }
        if (childView1 != null && childView1.visibility != View.GONE) {
            val view1Top = view0Top - (yOffsetRadio * childView1.measuredHeight).toInt()
            childView1.layout(view0Right, if (view1Top > 0) view1Top else 0, measuredWidth,
                    measuredHeight / 2)
        }
        if (childView2 != null && childView2.visibility != View.GONE) {
            childView2.layout((view0Left + view0Right) / 2 - childView2.measuredWidth / 2,
                    view0Bottom + BOTTOM_LINE_MARGIN_TOP,
                    (view0Left + view0Right) / 2 + childView2.measuredWidth / 2,
                    view0Bottom + BOTTOM_LINE_MARGIN_TOP + childView2.measuredHeight)
        }
    }

    override fun setSelected(selected: Boolean) {
        val bottomLine: View? = getChildAt(2)
        bottomLine?.visibility = if (selected) View.VISIBLE else View.GONE
        super.setSelected(selected)
    }

    override fun generateLayoutParams(attrs: AttributeSet?): LayoutParams {
        return MarginLayoutParams(context, attrs)
    }

    override fun generateLayoutParams(p: LayoutParams?): LayoutParams {
        return MarginLayoutParams(p)
    }

    override fun generateDefaultLayoutParams(): LayoutParams {
        return MarginLayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
    }

    companion object ConstValue {
        val BOTTOM_LINE_MARGIN_TOP = StringUtils.dipToPx(4f)
        val BOTTOM_LINE_WIDTH = StringUtils.dipToPx(25f)
        val BOTTOM_LINE_HEIGHT = StringUtils.dipToPx(2f)
    }
}