package wind.android.bussiness.trade.abstocktrade

import cn.com.hh.trade.data.*
import net.datamodel.network.CommonFunc
import net.util.Assist
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.model.TradeDelegationModel
import wind.android.bussiness.trade.model.TradeDoneModel
import wind.android.bussiness.trade.view.TradeCommListView
import java.text.DecimalFormat

/**
 * # **********************************************************************************************
 * # ClassName:      AStockUtil
 * # Description:    交易工具类
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/22    16:04
 * # Modifications:  initial
 * # **********************************************************************************************
 */

/**
 * String扩展函数，增加千分位，注意:只能是Number转型而来
 * @type 0:Int  1:Double 后续可增加其他类型
 */
fun String.addThousandth(type: Int): String {
    return try {
        when (type) {
            0 -> {
                val format = DecimalFormat(INT_FORMAT)
                format.format(this.toInt())
            }
            1 -> {
                val format = DecimalFormat(DOUBLE_FORMAT)
                format.format(this.toDouble())
            }
            else -> this
        }
    } catch (e: Exception) {
        this
    }
}

/**
 * 扩展函数，增加千分位
 */
fun Double.addThousandth(): String {
    val format = DecimalFormat(DOUBLE_FORMAT)
    return try {
        format.format(this)
    } catch (e: ArithmeticException) {
        this.toString()
    }
}

/**
 * 扩展函数，增加千分位
 */
fun Int.addThousandth(): String {
    val format = DecimalFormat(INT_FORMAT)
    return try {
        format.format(this)
    } catch (e: IllegalArgumentException) {
        this.toString()
    }
}

fun Double.priceDecimalFormat(decimalCount: Int): String {
    val patternStr = if (decimalCount <= 0) "#" else {
        val str = StringBuilder("0.")
        for (i in 0 until decimalCount) {
            str.append("0")
        }
        str.toString()
    }
    return try {
        DecimalFormat(patternStr).format(this)
    } catch (e: ArithmeticException) {
        this.toString()
    }
}

fun transferMode(fun1024: TagAns_Fun1024): TradeDelegationModel {
    val model = TradeDelegationModel()
    model.windCode = TradeConstantData.getWindCodeEnd(fun1024.stockCode, fun1024.marketType) // "600009.sh";
    model.stockName = fun1024.stockName // "上海机场";
    model.delegateDate = Assist.getDateByFormat2(fun1024.nOrderDate.toString() + "", TradeCommListView.SRC_FMT, TradeCommListView.SHOW_FMT) //委托日期
    model.delegateTime = CommonFunc.fixTimeWithSecond(fun1024.nOrderTime) // "14:20:21";
    model.direction = fun1024.chBSFlag
    model.count = fun1024.nOrderVolume.toString()
    model.clinchCount = fun1024.nMadeVolume.toString()
    model.nMadePrice = CommonFunc.fixTextTreadeEx(fun1024.nOrderPrice / 10000.0, 3) //委托价格
    model.nOrderPrice = CommonFunc.fixTextTreadeEx(fun1024.nMadePrice / 10000.0, 3) //成交价格
    model.cost = CommonFunc.fixTextTreadeEx(fun1024.nOrderVolume * (fun1024.nOrderPrice / 10000.0), 3)
    model.nMadeVolume = fun1024.nMadeVolume
    model.nOrderVolume = fun1024.nOrderVolume
    model.nCancelVolume = fun1024.nCancelVolume
    model.chOrderStatus = fun1024.chOrderStatus.toInt()
    model.remark = fun1024.buildStringFromBArr(fun1024.chRemark)
    model.consignCode = fun1024.buildStringFromBArr(fun1024.chOrderNumber) // 委托编号
    model.madeAmount = fun1024.dMadeAmt // 成交金额
    model.chExtflag1 = fun1024.chExtFlag1
    model.remark = fun1024.buildStringFromBArr(fun1024.chRemark)
    return model
}

fun transferMode(fun1026: TagAns_Fun1026): TradeDoneModel {
    val model = TradeDoneModel()
    model.windCode = TradeConstantData.getWindCodeEnd(fun1026.stockCode, fun1026.marketType) // windCode
    model.setStockName(fun1026.stockName) // 证券名称
    model.tradeStatus = fun1026.chMadeStatus //交易状态
    if (fun1026.nMadePrice != 0) {
        model.tradePrice = CommonFunc.fixTextTreade(fun1026.nMadePrice / 10000.0, 3) //成交价格
    }
    model.orderPrice = CommonFunc.fixTextTreade(fun1026.nOrderPrice / 10000.0, 3) //委托价格
    model.orderVolume = "" + fun1026.nOrderVolume // 委托数量
    model.tradeVolume = "" + fun1026.nMadeVolume // 成交数量
    model.tradeDate = Assist.getDateByFormat2(fun1026.nMadeDate.toString() + "", TradeCommListView.SRC_FMT, TradeCommListView.SHOW_FMT) //成交日期
    model.tradeTime = CommonFunc.fixTimeWithSecond(fun1026.nMadeTime) // 成交时间
    model.direction = fun1026.chBSFlag // 方向
    model.consignCode = fun1026.buildStringFromBArr(fun1026.chOrderNumber) // 委托编号
    model.tradeCode = fun1026.buildStringFromBArr(fun1026.chMadeNumber) // 成交编号
    model.paperUser = fun1026.buildStringFromBArr(fun1026.chAccountCode) // 证券账户
    model.madeAmount = fun1026.dMadeAmt // 成交 金额
    model.chExtflag1 = fun1026.chExtFlag1
    model.remark = fun1026.buildStringFromBArr(fun1026.chRemark)
    return model
}

fun transferMode(fun1030: TagAns_Fun1030): TradeDelegationModel {
    val model = TradeDelegationModel()
    model.windCode = TradeConstantData.getWindCodeEnd(fun1030.stockCode, fun1030.marketType) // "600009.sh";
    model.stockName = fun1030.stockName // "上海机场";
    model.delegateDate = Assist.getDateByFormat2(fun1030.nOrderDate.toString() + "", TradeCommListView.SRC_FMT, "MM/dd") //委托日期
    model.delegateTime = CommonFunc.fixTimeWithSecond(fun1030.nOrderTime).substring(0, 5) // "14:20:21";
    model.direction = fun1030.chBSFlag
    model.count = fun1030.nOrderVolume.toString()
    model.clinchCount = fun1030.nMadeVolume.toString()
    model.nMadePrice = CommonFunc.fixTextTreadeEx(fun1030.nMadePrice / 10000.0, 3) //委托价格
    model.nOrderPrice = CommonFunc.fixTextTreadeEx(fun1030.nOrderPrice / 10000.0, 3) //成交价格
    model.cost = CommonFunc.fixTextTreadeEx(fun1030.nOrderVolume * (fun1030.nOrderPrice / 10000.0), 3)
    model.nMadeVolume = fun1030.nMadeVolume
    model.nOrderVolume = fun1030.nOrderVolume
    model.nCancelVolume = fun1030.nCancelVolume
    model.chOrderStatus = fun1030.chOrderStatus.toInt()
    model.remark = fun1030.buildStringFromBArr(fun1030.chRemark)
    model.consignCode = fun1030.buildStringFromBArr(fun1030.chOrderNumber) // 委托编号
    model.madeAmount = fun1030.dMadeAmt // 成交 金额
    model.chExtflag1 = fun1030.chExtFlag1
    model.remark = fun1030.buildStringFromBArr(fun1030.chRemark)
    return model
}

fun transferMode(fun1031: TagAns_Fun1031): TradeDoneModel {
    val model = TradeDoneModel()
    model.windCode = TradeConstantData.getWindCodeEnd(fun1031.stockCode, fun1031.marketType) // 代码
    model.tradeStatus = fun1031.chMadeStatus //交易状态
    model.stockName = fun1031.stockName // 证券名称
    model.tradePrice = CommonFunc.fixTextTreade(fun1031.nMadePrice / 10000.0, 3) //成交价格
    model.orderPrice = CommonFunc.fixTextTreade(fun1031.nOrderPrice / 10000.0, 3) //委托价格
    model.orderVolume = "" + fun1031.nOrderVolume // 委托数量
    model.tradeVolume = "" + fun1031.nMadeVolume // 成交数量
    model.tradeDate = Assist.getDateByFormat2(fun1031.nMadeDate.toString() + "", TradeCommListView.SRC_FMT, "MM/dd") //成交日期
    model.tradeTime = CommonFunc.fixTimeWithSecond(fun1031.nMadeTime).substring(0, 5) // 成交时间
    model.direction = fun1031.chBSFlag // 方向
    model.consignCode = fun1031.buildStringFromBArr(fun1031.chOrderNumber) // 委托编号
    model.tradeCode = fun1031.buildStringFromBArr(fun1031.chMadeNumber) // 成交编号
    model.paperUser = fun1031.buildStringFromBArr(fun1031.chAccountCode) // 证券账户
    model.madeAmount = fun1031.dMadeAmt // 成交 金额
    model.chExtflag1 = fun1031.chExtFlag1
    model.remark = fun1031.buildStringFromBArr(fun1031.chRemark)
    return model
}

fun transferMode1095(fun1095: Any?): TradeDelegationModel {
    val model = TradeDelegationModel()
    if (fun1095 is TagAns_Fun1095) {
        model.windCode = TradeConstantData.getWindCodeEnd(fun1095.stockCode, fun1095.marketType) // "600009.sh";
        model.stockName = fun1095.stockName // "上海机场";
        model.delegateDate = Assist.getDateByFormat2(fun1095.nOrderDate.toString() + "", TradeCommListView.SRC_FMT, TradeCommListView.SHOW_FMT) //委托日期
        model.delegateTime = CommonFunc.fixTimeWithSecond(fun1095.nOrderTime) // "14:20:21";
        model.direction = fun1095.chBSFlag
        model.count = fun1095.nOrderVolume.toString()
        model.clinchCount = fun1095.nMadeVolume.toString()
        model.nMadePrice = CommonFunc.fixTextTreadeEx(fun1095.nOrderPrice / 10000.0, 3) //委托价格
        model.nOrderPrice = CommonFunc.fixTextTreadeEx(fun1095.nMadePrice / 10000.0, 3) //成交价格
        model.cost = CommonFunc.fixTextTreadeEx(fun1095.nOrderVolume * (fun1095.nOrderPrice / 10000.0), 3)
        model.nMadeVolume = fun1095.nMadeVolume
        model.nOrderVolume = fun1095.nOrderVolume
        model.nCancelVolume = fun1095.nCancelVolume
        model.chOrderStatus = fun1095.chOrderStatus.toInt()
        model.remark = fun1095.buildStringFromBArr(fun1095.chRemark)
        model.consignCode = fun1095.buildStringFromBArr(fun1095.chOrderNumber) // 委托编号
        model.madeAmount = fun1095.dMadeAmt // 成交 金额
        model.commType = 1
        model.chExtflag1 = fun1095.chExtFlag1
        model.remark = fun1095.buildStringFromBArr(fun1095.chRemark)
    }
    return model
}

/**
 * model转换
 */
fun transferMode1096(fun1096: Any?): TagAns_Fun1025 {
    val model = TagAns_Fun1025()
    if (fun1096 is TagAns_Fun1096) {
        model.chStockCode = fun1096.chStockCode // 代码
        model.chStockName = fun1096.chStockName // 证券名称
        model.nOrderPrice = fun1096.nOrderPrice //委托价格
        model.nMadePrice = fun1096.nMadePrice
        model.nOrderVolume = fun1096.nOrderVolume // 委托数量
        model.nMadeVolume = fun1096.nMadeVolume // 成交数量
        model.nOrderDate = fun1096.nOrderDate //委托日期
        model.nOrderTime = fun1096.nOrderTime // 委托时间
        model.chBSFlag = fun1096.chBSFlag // 方向
        model.chOrderNumber = fun1096.chOrderNumber // 委托编号
        model.chAccountCode = fun1096.chAccountCode // 证券账户
        model.chRemark = fun1096.chRemark // 说明
        model.iResv2 = 1096.toLong() // 代表预约委托单
        model.chExtFlag1 = fun1096.chExtFlag1
    }
    return model
}

fun transformToFun1010(item: TagAns_Fun1012): TagAns_Fun1010 {
    val fun1010 = TagAns_Fun1010()
    fun1010.nStockAvl = item.nStockAvl
    fun1010.dFundAvl = item.dFundAvl
    fun1010.chMarketType = item.chMarketType
    fun1010.chBSFlag = item.chBSFlag
    fun1010.chExtFlag1 = item.chExtFlag1
    fun1010.chStockCode = item.chStockCode
    fun1010.nOrderVolume = item.nOrderVolume
    fun1010.nOrderPrice = item.nOrderPrice
    fun1010.chResv = item.chResv
    fun1010.iOrderVolume = item.iOrderVolume
    return fun1010
}

