package wind.android.bussiness.trade.abstocktrade.activity

import android.os.Build
import android.os.Bundle
import android.support.design.widget.TabLayout
import android.text.TextUtils
import android.util.SparseArray
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.widget.AdapterView
import android.widget.FrameLayout
import android.widget.ListView
import android.widget.PopupWindow
import cn.com.hh.trade.data.NSDPROCAPI
import eventbus.EventBusManager
import eventinterface.TouchEventListener
import kotlinx.android.synthetic.main.activity_a_stock_trade.*
import kotlinx.android.synthetic.main.trade_activity_header.*
import net.datamodel.speed.SecType2
import net.datamodel.speed.WindCodeType
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import ui.UINavigationBar
import ui.screen.UIScreen
import util.StringUtils
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.GGT_TYPE_KEY
import wind.android.bussiness.trade.abstocktrade.NON_GGT_TYPE
import wind.android.bussiness.trade.abstocktrade.eventbus.StockTradeMsgEvent
import wind.android.bussiness.trade.abstocktrade.eventbus.UserCodeMsgEvent
import wind.android.bussiness.trade.abstocktrade.fragment.*
import wind.android.bussiness.trade.activity.BaseTradeActivity
import wind.android.bussiness.trade.adapter.TradeTitleAdapter
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.constant.TradeUserAction
import wind.android.bussiness.trade.model.TradeJumpConstants.*
import wind.android.bussiness.trade.util.TradeAccountManager
import wind.android.common.StockThemeUtils

/**
 * # **********************************************************************************************
 * # ClassName:      AStockTradeActivity
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/18    15:23
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockTradeActivity : BaseTradeActivity(), TabLayout.BaseOnTabSelectedListener<TabLayout.Tab>, TouchEventListener {

    /**
     * Fragment列表
     */
    private val fragments = SparseArray<BaseATradeFrag<*, *>>()
    /**
     * 当前Fragment
     */
    private var currentFragment: BaseATradeFrag<*, *>? = null

    private val accountPopWindow: PopupWindow by lazy { PopupWindow() }
    private val accountListView: ListView by lazy { ListView(this) }
    private val accountAdapter: TradeTitleAdapter by lazy { TradeTitleAdapter(this) }

    /**
     * 账号选择监听
     */
    private val mOnItemClickListener = AdapterView.OnItemClickListener { _, _, position, _ ->
        trade_account.text = accountAdapter.getItem(position)
        accountAdapter.setSelection(position)
        accountAdapter.notifyDataSetChanged()
        accountListView.setSelection(position)
        accountPopWindow.dismiss()
        EventBusManager.getInstance().postSticky(UserCodeMsgEvent(getCurrentUserCode()))
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        StockThemeUtils.setTheme(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_a_stock_trade)
        initNav()
        initAccount()
        val tabs = resources.getStringArray(R.array.a_stock_trade_tabs)
        val tabValue = intent?.getIntExtra(KEY_BSTYPE, 0) ?: 0
        val tabIndex = when (tabValue) {
            TAB_TRADE_DEAL,
            TAB_TRADE_DELEGATE,
            TAB_TRADE_QUERY -> QUERY_TAB
            TAB_TRADE_TRANSFER -> TRANSFER_ACCOUNT_TAB
            else -> tabValue
        }
        tab_layout.setTabItems(tabs, tabTitleTextSize = 16)
        tab_layout.addOnTabSelectedListener(this)
        tab_layout.setSelectTab(tabIndex)
        EventBusManager.getInstance().register(this)
    }

    override fun onDestroy() {
        super.onDestroy()
        tab_layout.clearOnTabSelectedListeners()
        fragments.clear()
        currentFragment = null
        val tradeMsgEvent = EventBusManager.getInstance().getStickyEvent(UserCodeMsgEvent::class.java)
        tradeMsgEvent?.let { EventBusManager.getInstance().removeStickyEvent(it) }
        EventBusManager.getInstance().unregister(this)
    }

    override fun onTabReselected(p0: TabLayout.Tab?) {
    }

    override fun onTabUnselected(p0: TabLayout.Tab?) {
    }

    override fun onTabSelected(p0: TabLayout.Tab?) {
        val position = p0?.position ?: 0
        showTab(position)
    }

    private fun initNav() {
        navigationBar.setTitle("交易")
        if (TradeConstantData.selectedItem?.type != TradeConstantData.TYPE_HKSTOCK && getGgtType() == NON_GGT_TYPE) {
            val titleView = LayoutInflater.from(this).inflate(R.layout.trade_activity_header, null)
            navigationBar.setTitleView(titleView)
            navigationBar.setListener(this)
            if (!TextUtils.isEmpty(TradeConstantData.selectedItem?.name)) {
                trade_name.text = TradeConstantData.selectedItem.name
            }
        }
    }

    private fun initAccount() {
        accountListView.layoutParams = FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT)
        accountListView.dividerHeight = 0
        accountListView.setPadding(StringUtils.dipToPx(15f),
                StringUtils.dipToPx(15f), StringUtils.dipToPx(15f),
                StringUtils.dipToPx(10f))
        accountListView.isVerticalScrollBarEnabled = false
        accountListView.selector = resources.getDrawable(
                R.drawable.transparent)
        accountListView.onItemClickListener = mOnItemClickListener
        accountListView.cacheColorHint = 0
        accountListView.setFadingEdgeLength(0)
        accountListView.adapter = accountAdapter

        //初始化弹出框
        accountPopWindow.contentView = accountListView
        accountPopWindow.width = UIScreen.screenWidth / 2
        accountPopWindow.isTouchable = true
        accountPopWindow.isFocusable = true
        accountPopWindow.isClippingEnabled = false
        accountPopWindow.setBackgroundDrawable(resources.getDrawable(
                R.drawable.optional_group_bg))
        accountPopWindow.animationStyle = android.R.style.Animation_Toast
        accountPopWindow.setOnDismissListener { }

        doFilterUserCode(null, intent?.getStringExtra(KEY_WINDCODE))
    }

    /**
     * 展示/隐藏对应的Fragment
     *
     * @param index tab索引
     */
    private fun showTab(index: Int) {
        var fragment = fragments.get(index)
        if (currentFragment != null && currentFragment === fragment) {
            return
        }
        val transaction = supportFragmentManager.beginTransaction()
        currentFragment?.let { transaction.hide(it) }
        if (fragment == null) {
            fragment = when (index) {
                BUY_TAB -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.ACTION_TRADE_BUY)
                    AStockBuyFrag()
                }
                SELL_TAB -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.ACTION_TRADE_SELL)
                    AStockSellFrag()
                }
                CANCEL_TAB -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.ACTION_TRADE_CANCELLABLE_RECORD)
                    AStockCancelFrag()
                }
                QUERY_TAB -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.ACTION_TRADE_QUERY_FRAG)
                    AStockQueryFrag()
                }
                TRANSFER_ACCOUNT_TAB -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.ACTION_TRADE_BANK)
                    AStockTransferFrag()
                }
                else -> null
            }
            fragment?.let {
                val bundle = Bundle()
                bundle.putInt(GGT_TYPE_KEY, getGgtType())
                bundle.putInt(KEY_BSTYPE, intent?.getIntExtra(KEY_BSTYPE, 0) ?: 0)
                bundle.putString(KEY_WINDCODE, intent?.getStringExtra(KEY_WINDCODE))
                it.setArguments(bundle)
                transaction.add(R.id.frag_container, it, it::class.java.simpleName)
                currentFragment = it
                fragments.put(index, it)
            }
        } else {
            transaction.show(fragment)
            currentFragment = fragment
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1 && isDestroyed) {
            return
        }
        transaction.commitAllowingStateLoss()
    }

    override fun touchEvent(sender: View?, event: MotionEvent?) {
        if (sender?.tag == UINavigationBar.CENTER_BUTTON) {
            // 股东账号选择框
            if (!accountPopWindow.isShowing) {
                accountPopWindow.showAsDropDown(navigationBar, UIScreen.screenWidth / 4, 0)
            }
        }
    }

    /**
     * 根据windCode或指定的userCode过滤股东账号
     */
    private fun filterUserCode(tmpUserCode: String?, tmpWindCode: String?): List<String?>? {
        val stockHolders = TradeAccountManager.getInstance().tradeAccount?.shareTradeAccountData?.stockHolders
        if (stockHolders != null && stockHolders.isNotEmpty()) {
            val accountId = TradeAccountManager.getInstance().tradeAccount.accountId
            val tmpFilter = stockHolders.filter {
                when {
                    tmpUserCode != null && tmpWindCode != null -> {
                        val marketType = TradeConstantData.getMarketType(tmpWindCode).toByte()
                        val accountType = TradeConstantData.getAccountType(tmpWindCode, marketType)
                        it.getString(it.chUserCode) == tmpUserCode && it.getString(it.chAccountCode) == accountId
                                && marketType == it.chMarketType && accountType == it.chAccountType
                    }
                    tmpUserCode != null -> {
                        it.getString(it.chUserCode) == tmpUserCode && it.getString(it.chAccountCode) == accountId
                    }
                    tmpWindCode != null -> {
                        val marketType = TradeConstantData.getMarketType(tmpWindCode).toByte()
                        val accountType = TradeConstantData.getAccountType(tmpWindCode, marketType)
                        it.getString(it.chAccountCode) == accountId && marketType == it.chMarketType
                                && accountType == it.chAccountType

                    }
                    else -> {
                        it.getString(it.chAccountCode) == accountId
                    }
                }
            }
            return tmpFilter.map { changeUserCode(tmpWindCode, it.chMarketType, it.getString(it.chUserCode)) }
        }
        return null
    }

    /**
     * 获取当前筛选的userCode
     */
    private fun getCurrentUserCode(): String {
        val account = trade_account.text.toString()
        val userCode = account.split(" ")
        return when {
            userCode.size >= 2 -> {
                userCode[1]
            }
            userCode.size == 1 -> {
                userCode[0]
            }
            else -> {
                account
            }
        }
    }

    /**
     * 根据市场类型 将userCode转换为可显示字符串
     */
    private fun changeUserCode(tmpWindCode: String?, marketType: Byte, userCode: String): String? {
        return when (WindCodeType.getWindSecType(tmpWindCode, null).toByte()) {
            SecType2.BaseHSBStock ->
                if (marketType.toInt() == NSDPROCAPI.MARKET_SH) {
                    "沪B $userCode" // 上海B股
                } else {
                    "深B $userCode" // 深圳B股
                }
            SecType2.BaseHKStock -> "港 $userCode"
            SecType2.Sanban -> "三板 $userCode" // 三板区分400与420开头的票  420开头的为B股
            else ->
                if (marketType.toInt() == NSDPROCAPI.MARKET_SH) {
                    "沪A $userCode" // 上海A股
                } else {
                    "深A $userCode" // 深圳A股
                }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun update(messageEvent: StockTradeMsgEvent) {
        if (messageEvent.type == StockTradeMsgEvent.USERCODE_FRAGMENT_ACTIVITY) {
            doFilterUserCode(messageEvent.userCode, messageEvent.windCode)
        }
    }

    private fun doFilterUserCode(tmpUserCode: String?, tmpWindCode: String?) {
        val filter = filterUserCode(tmpUserCode, tmpWindCode)
        if (!filter.isNullOrEmpty()) {
            accountPopWindow.height = filter.size * StringUtils.dipToPx(30f) + StringUtils.dipToPx(25f)
            accountAdapter.setList(filter)
            accountAdapter.setSelection(0)
            trade_account.text = filter.firstOrNull() ?: ""
            // 通知fragment更新
            EventBusManager.getInstance().postSticky(UserCodeMsgEvent(getCurrentUserCode()))
        }
    }

    companion object {
        const val BUY_TAB = 0
        const val SELL_TAB = 1
        const val CANCEL_TAB = 2
        const val QUERY_TAB = 3
        const val TRANSFER_ACCOUNT_TAB = 4
    }
}