package wind.android.bussiness.trade.abstocktrade.adapter

import android.content.Context
import android.support.v7.widget.AppCompatTextView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.home.model.JSHoldStockValue
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      AStockBuyAdapter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockBuyAdapter(private val context: Context) : BaseAdapter() {
    /**
     * 数据源
     */
    private val data: ArrayList<JSHoldStockValue> by lazy { ArrayList<JSHoldStockValue>() }

    fun setData(list: List<JSHoldStockValue>?) {
        data.clear()
        list?.let { data.addAll(it) }
        notifyDataSetChanged()
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val viewHolder: ViewHolder
        var view = convertView
        if (view == null) {
            view = LayoutInflater.from(context).inflate(R.layout.a_stock_buy_item, null)
            viewHolder = ViewHolder(view)
            view.tag = viewHolder
        } else {
            viewHolder = view.tag as ViewHolder
        }
        val item = getItem(position)
        item?.apply {
            viewHolder.stockName.text = stockName
            viewHolder.stockCode.text = windCode
            val profitValue = try {
                dIncome.toDouble()
            } catch (e: Exception) {
                0.0
            }
            val color = if (profitValue >= 0.0) StockUtil.getChangeColor(1f) else StockUtil.getChangeColor(-1f)
            viewHolder.marketValue.text = dMktVal.addThousandth(1)
            viewHolder.marketValue.setTextColor(color)
            viewHolder.profit.text = dIncome.addThousandth(1)
            viewHolder.profit.setTextColor(color)
            viewHolder.holdAmount.text = nStockQty.addThousandth(0)
            viewHolder.holdAmount.setTextColor(color)
            viewHolder.salableAmount.text = nStockAvl.addThousandth(0)
            viewHolder.salableAmount.setTextColor(color)
            viewHolder.cost.text = nCostPrice
            viewHolder.cost.setTextColor(color)
            viewHolder.price.text = nNewPrice
            viewHolder.price.setTextColor(color)
        }
        return view!!

    }

    override fun getItem(position: Int): JSHoldStockValue? {
        return if (position < data.size) data[position] else null
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return data.size
    }

    /**
     * ViewHolder
     */
    private class ViewHolder(itemView: View) {
        val stockName: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.stock_name) }
        val stockCode: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.stock_code) }
        val marketValue: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.market_value) }
        val profit: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.profit) }
        val holdAmount: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.hold_amount) }
        val salableAmount: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.salable_amount) }
        val cost: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.cost) }
        val price: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.price) }
    }
}