package wind.android.bussiness.trade.abstocktrade.adapter

import android.content.Context
import android.support.v7.widget.AppCompatTextView
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import cn.com.hh.trade.data.TagAns_Fun1025
import net.datamodel.network.CommonFunc
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.adapter.TradeOverAdapter
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      AStockCancelAdapter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/20    9:56
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockCancelAdapter(val context: Context) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    /**
     * adapter数据源
     */
    private var data: ArrayList<TagAns_Fun1025?> = arrayListOf()
    /**
     * 撤单点击事件
     */
    var cancelClick: ((Int) -> Unit)? = null

    /**
     * 设置数据源
     *
     * @param list
     */
    fun setData(list: List<TagAns_Fun1025?>?) {
        data.clear()
        list?.let { data.addAll(list) }
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.a_trade_cancel_item, viewGroup, false)
        return TradeCancelViewHolder(view, cancelClick)
    }

    override fun getItemCount(): Int {
        return data.size
    }

    override fun onBindViewHolder(vh: RecyclerView.ViewHolder, position: Int) {
        val viewHolder = vh as TradeCancelViewHolder
        val item = if (position < data.size) data[position] else null
        item?.run {
            val showText = TradeOverAdapter.getDirectionTextToShow(chBSFlag, if (iResv2 == 1096.toLong()) 1 else 0, buildStringFromBArr(chRemark))
            val color = if (showText[1].contains(TradeOverAdapter.BUY)) StockUtil.getChangeColor(1f) else StockUtil.getChangeColor(-1f)
            viewHolder.stockName.text = stockName.toString()
            viewHolder.trustPrice.text = CommonFunc.fixTextTreade(nOrderPrice / 10000.0, 3)
            viewHolder.trustPrice.setTextColor(color)
            viewHolder.trustStatus.text = getTrustStatus(this, showText[0])
            viewHolder.trustStatus.setTextColor(color)
            viewHolder.statusTime.text = CommonFunc.fixTimeWithSecond(nOrderTime)
            viewHolder.statusTime.setTextColor(color)
            viewHolder.trustAmount.text = nOrderVolume.addThousandth()
            viewHolder.trustAmount.setTextColor(color)
            viewHolder.dealAmount.text = nMadeVolume.addThousandth()
            viewHolder.dealAmount.setTextColor(color)
        }
    }

    /**
     * 获取买卖状态展示字符串
     */
    private fun getTrustStatus(item: TagAns_Fun1025, showText: String): String {
        val str = StringBuilder()
        str.append(showText)
        str.append("|")
        // 已报状态下 检查remark字段
        val remark = item.buildStringFromBArr(item.chRemark)
        val statusStr = if (item.nMadeVolume != 0) "部成" else if (TextUtils.isEmpty(remark)) "已报" else remark
        str.append(statusStr)
        return str.toString()
    }

    /**
     * ViewHolder
     */
    class TradeCancelViewHolder(itemView: View, private val clickListener: ((Int) -> Unit)?) : RecyclerView.ViewHolder(itemView) {
        /**
         * 股票名称
         */
        val stockName: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.stock_name) }
        /**
         * 委托价格
         */
        val trustPrice: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.trust_price) }
        /**
         * 买卖状态
         */
        val trustStatus: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.trust_status) }
        /**
         * 状态时间
         */
        val statusTime: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.status_time) }
        /**
         * 委托数量
         */
        val trustAmount: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.trust_amount) }
        /**
         * 成交梳理
         */
        val dealAmount: AppCompatTextView by lazy { itemView.findViewById<AppCompatTextView>(R.id.deal_amount) }
        private val cancelBtn: Button by lazy { itemView.findViewById<Button>(R.id.cancel_btn) }

        init {
            cancelBtn.setOnClickListener {
                clickListener?.invoke(adapterPosition)
            }
        }
    }
}