package wind.android.bussiness.trade.abstocktrade.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import ui.bell.DinTextView
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.adapter.TradeOverAdapter
import wind.android.bussiness.trade.model.TradeDelegationModel
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      AStockQueryDelegateAdapter
 * # Description:    当日委托
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockQueryDelegateAdapter(private val context: Context) : BaseAdapter() {
    /**
     * 数据源
     */
    private val data: ArrayList<TradeDelegationModel> by lazy { ArrayList<TradeDelegationModel>() }

    private var isHisory = false

    fun setData(list: Array<TradeDelegationModel>?) {
        data.clear()
        list?.let { data.addAll(it) }
        notifyDataSetChanged()
    }

    fun addData(list: Array<TradeDelegationModel>?) {
        list?.let { data.addAll(it) }
        notifyDataSetChanged()
    }

    fun setHistory(isHisory: Boolean) {
        this.isHisory = isHisory
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val viewHolder: ViewHolder
        var view = convertView
        if (view == null) {
            view = LayoutInflater.from(context).inflate(R.layout.a_stock_query_delegate_item, null)
            viewHolder = ViewHolder(view)
            view.tag = viewHolder
        } else {
            viewHolder = view.tag as ViewHolder
        }
        val item = getItem(position)
        item?.apply {
            val color = StockUtil.getChangeColor(0f)
            val colorRed = StockUtil.getChangeColor(1f)
            val colorGreen = StockUtil.getChangeColor(-1f)
            viewHolder.name.text = stockName
            viewHolder.code.text = windCode
            val showText = TradeOverAdapter.getDirectionTextToShow(direction, commType, remark)
            viewHolder.bs.text = showText[0]
            if (isHisory) {
                val time = "$delegateDate $delegateTime"
                viewHolder.time.text = time
            } else {
                viewHolder.time.text = delegateTime
            }
            viewHolder.delegateNumber.text = count.addThousandth(0)
            viewHolder.transactionNumber.text = clinchCount.addThousandth(0)
            viewHolder.price.text = nMadePrice
            viewHolder.state.text = TradeOverAdapter.getTradeStatusImgRes(this)
            viewHolder.name.setTextColor(color)
            viewHolder.code.setTextColor(color)
            viewHolder.bs.setTextColor(color)
            viewHolder.time.setTextColor(color)
            viewHolder.delegateNumber.setTextColor(color)
            viewHolder.transactionNumber.setTextColor(color)
            viewHolder.price.setTextColor(color)
            viewHolder.state.setTextColor(color)
            if (showText[1].contains(TradeOverAdapter.BUY)) {
                viewHolder.bs.setTextColor(colorRed)
                viewHolder.time.setTextColor(colorRed)
                viewHolder.delegateNumber.setTextColor(colorRed)
                viewHolder.transactionNumber.setTextColor(colorRed)
                viewHolder.price.setTextColor(colorRed)
                viewHolder.state.setTextColor(colorRed)
            } else if (showText[1].contains(TradeOverAdapter.SELL)) {
                viewHolder.bs.setTextColor(colorGreen)
                viewHolder.time.setTextColor(colorGreen)
                viewHolder.delegateNumber.setTextColor(colorGreen)
                viewHolder.transactionNumber.setTextColor(colorGreen)
                viewHolder.price.setTextColor(colorGreen)
                viewHolder.state.setTextColor(colorGreen)
            }


        }
        return view!!

    }

    override fun getItem(position: Int): TradeDelegationModel? {
        return if (position < data.size) data[position] else null
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return data.size
    }

    /**
     * ViewHolder
     */
    private class ViewHolder(itemView: View) {
        val name: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.name) }
        val code: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.code) }
        val bs: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.bs) }
        val time: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.time) }
        val delegateNumber: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.delegateNumber) }
        val transactionNumber: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.transactionNumber) }
        val price: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.price) }
        val state: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.state) }
    }
}