package wind.android.bussiness.trade.abstocktrade.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import net.datamodel.network.CommonFunc
import ui.bell.DinTextView
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.adapter.TradeOverAdapter
import wind.android.bussiness.trade.model.TradeDoneModel
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      AStockQueryTransactionAdapter
 * # Description:    当日成交
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockQueryTransactionAdapter(private val context: Context) : BaseAdapter() {
    /**
     * 数据源
     */
    private val data: ArrayList<TradeDoneModel> by lazy { ArrayList<TradeDoneModel>() }

    private var isHisory = false

    fun setData(list: Array<TradeDoneModel>?) {
        data.clear()
        list?.let { data.addAll(it) }
        notifyDataSetChanged()
    }

    fun setHistory(isHisory: Boolean) {
        this.isHisory = isHisory
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val viewHolder: ViewHolder
        var view = convertView
        if (view == null) {
            view = LayoutInflater.from(context).inflate(R.layout.a_stock_query_transaction_item, null)
            viewHolder = ViewHolder(view)
            view.tag = viewHolder
        } else {
            viewHolder = view.tag as ViewHolder
        }
        val item = getItem(position)
        item?.apply {
            val color = StockUtil.getChangeColor(0f)
            val colorRed = StockUtil.getChangeColor(1f)
            val colorGreen = StockUtil.getChangeColor(-1f)
            viewHolder.name.text = stockName
            if (isHisory) {
                val time = "$tradeDate $tradeTime"
                viewHolder.time.text = time
            } else {
                viewHolder.time.text = tradeTime
            }
            viewHolder.price.text = tradePrice
            viewHolder.number.text = tradeVolume.addThousandth(0)
            val showText = TradeOverAdapter.getDirectionTextToShow(direction, commType, remark)
            viewHolder.bs.text = showText[0]
            viewHolder.money.text = madeAmount.addThousandth()
            viewHolder.name.setTextColor(color)
            viewHolder.time.setTextColor(color)
            viewHolder.price.setTextColor(color)
            viewHolder.number.setTextColor(color)
            viewHolder.bs.setTextColor(color)
            viewHolder.money.setTextColor(color)
            if (showText[1].contains(TradeOverAdapter.BUY)) {
                viewHolder.price.setTextColor(colorRed)
                viewHolder.number.setTextColor(colorRed)
                viewHolder.bs.setTextColor(colorRed)
                viewHolder.money.setTextColor(colorRed)
            } else if (showText[1].contains(TradeOverAdapter.SELL)) {
                viewHolder.price.setTextColor(colorGreen)
                viewHolder.number.setTextColor(colorGreen)
                viewHolder.bs.setTextColor(colorGreen)
                viewHolder.money.setTextColor(colorGreen)
            }


        }
        return view!!

    }

    override fun getItem(position: Int): TradeDoneModel? {
        return if (position < data.size) data[position] else null
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return data.size
    }

    /**
     * ViewHolder
     */
    private class ViewHolder(itemView: View) {
        val name: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.name) }
        val time: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.time) }
        val price: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.price) }
        val number: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.number) }
        val bs: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.bs) }
        val money: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.money) }
    }
}