package wind.android.bussiness.trade.abstocktrade.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import cn.com.hh.trade.data.TagAns_Fun1044
import ui.bell.DinTextView
import util.DateUtil
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.news.anews.StockUtil
import java.nio.charset.Charset

/**
 * # **********************************************************************************************
 * # ClassName:      AStockTransferAdapter
 * # Description:    银证转账查询adapter
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockTransferAdapter(private val context: Context) : BaseAdapter() {
    /**
     * 数据源
     */
    private val data: ArrayList<TagAns_Fun1044> by lazy { ArrayList<TagAns_Fun1044>() }

    fun setData(list: Array<TagAns_Fun1044>?) {
        data.clear()
        list?.let {
            it.sortWith(compareBy({ item ->
                item.nOperDate
            }, { item ->
                item.nOperTime
            }
            ))
            it.reverse()
            data.addAll(it)
        }
        notifyDataSetChanged()
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val viewHolder: ViewHolder
        var view = convertView
        if (view == null) {
            view = LayoutInflater.from(context).inflate(R.layout.a_stock_transfer_item, null)
            viewHolder = ViewHolder(view)
            view.tag = viewHolder
        } else {
            viewHolder = view.tag as ViewHolder
        }
        val item = getItem(position)
        item?.apply {
            viewHolder.date.text = DateUtil.changeFormat(nOperDate.toString(), "yyyyMMdd", "yyyy-MM-dd")
            val time = if (nOperTime.toString().length == 5) "0$nOperTime" else nOperTime.toString()
            viewHolder.time.text = DateUtil.changeFormat(time, "HHmmss", "HH:mm")
            val tran = when (chTranType) {
                'I'.toByte() -> {
                    "银行转存"
                }
                'O'.toByte() -> {
                    "银行转取"
                }
                'C'.toByte() -> {
                    "查银行余额"
                }
                else -> {
                    "未知[$chTranType]"
                }
            }
            viewHolder.type.text = tran
            val money =
                    if (chTranType == 'C'.toByte()) {
                        // 显示银行余额
                        dFundBal.addThousandth()
                    } else {
                        dFundEffect.addThousandth()
                    }
            viewHolder.money.text = money
            viewHolder.money.setTextColor(StockUtil.getChangeColor(1f))
            //币种:'0'-RMB;'1'-HKD;'2'-USD;
            val moneyType =
                    when (chMoneyType) {
                        '0'.toByte() -> "人民币"
                        '1'.toByte() -> "港币"
                        '2'.toByte() -> "美元"
                        else -> "未知"
                    }
            viewHolder.moneyType.text = moneyType
            viewHolder.state.text = String(chStatus, Charset.forName("GBK"))
        }
        return view!!

    }

    override fun getItem(position: Int): TagAns_Fun1044? {
        return if (position < data.size) data[position] else null
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return data.size
    }

    /**
     * ViewHolder
     */
    private class ViewHolder(itemView: View) {
        val date: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.date) }
        val time: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.time) }
        val type: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.type) }
        val money: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.money) }
        val moneyType: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.moneyType) }
        val state: DinTextView by lazy { itemView.findViewById<DinTextView>(R.id.state) }
    }
}