package wind.android.bussiness.trade.abstocktrade.fragment

import android.os.Message
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import cn.com.hh.trade.data.TagAns_Fun1025
import kotlinextend.viewGone
import kotlinextend.viewVisible
import kotlinx.android.synthetic.main.frag_a_stock_cancel.*
import util.ThemeUtils
import wind.android.R
import wind.android.bussiness.optionalstock.preopt.view.RecyclerViewDivider
import wind.android.bussiness.trade.abstocktrade.adapter.AStockCancelAdapter
import wind.android.bussiness.trade.abstocktrade.presenter.AStockCancelPresenter
import wind.android.bussiness.trade.abstocktrade.view.IAStockCancelView

/**
 * # **********************************************************************************************
 * # ClassName:      AStockCancelFrag
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/18    16:58
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockCancelFrag : BaseATradeFrag<IAStockCancelView<TagAns_Fun1025>, AStockCancelPresenter>(), IAStockCancelView<TagAns_Fun1025>, View.OnClickListener {
    /**
     * 列表adapter
     */
    private val adapter: AStockCancelAdapter by lazy { AStockCancelAdapter(activity!!) }
    /**
     * 列表数据
     */
    private val data: ArrayList<TagAns_Fun1025> by lazy { arrayListOf<TagAns_Fun1025>() }

    override fun getLayoutId(): Int {
        return R.layout.frag_a_stock_cancel
    }

    override fun isRegisterEventBus(): Boolean {
        return false
    }

    override fun init() {
        list_view.layoutManager = LinearLayoutManager(activity!!, LinearLayoutManager.VERTICAL, false)
        list_view.addItemDecoration(RecyclerViewDivider(activity, LinearLayoutManager.HORIZONTAL,
                1, ThemeUtils.getColor(resources, R.color.color_line_setting_black,
                R.color.color_line_setting_white), true))
        adapter.cancelClick = ::onCancelClick
        list_view.adapter = adapter
        cancel_all.setOnClickListener(this)
        cancel_all_layout.visibility = View.GONE
    }

    override fun onFragHide(byOnPause: Boolean) {
        presenter.unregisterListener()
    }

    override fun onFragShow(byOnResume: Boolean) {
        presenter.registerListener()
        presenter.reqCancelableData(ggType)
    }

    override fun createPresenter(): AStockCancelPresenter {
        return AStockCancelPresenter(this)
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when (msg?.what) {
            MSG_UPDATE_LIST_ALL -> {
                val list = msg.obj as? List<TagAns_Fun1025>
                data.clear()
                if (!list.isNullOrEmpty()) {
                    data.addAll(list)
                    tip_info.viewGone()
                    cancel_all_layout.viewVisible()
                } else {
                    tip_info.viewVisible()
                    cancel_all_layout.viewGone()
                }
                adapter.setData(data)
            }
            MSG_ADD_UPDATE_LIST_ALL -> {
                val list = msg.obj as? List<TagAns_Fun1025>
                if (!list.isNullOrEmpty()) {
                    data.addAll(list)
                    adapter.setData(data)
                    tip_info.viewGone()
                    cancel_all_layout.viewVisible()
                }
            }
            MSG_SHOW_ERROR -> {
                val errorMsg = msg.obj as? String
                if (!errorMsg.isNullOrEmpty()) {
                    showDialog(message = errorMsg)
                }
            }
        }
    }

    override fun onUpdateList(list: List<TagAns_Fun1025>?) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_LIST_ALL
        msg.obj = list
        sendMessage(msg)
    }

    override fun onAddUpdateList(list: List<TagAns_Fun1025>?) {
        val msg = Message.obtain()
        msg.what = MSG_ADD_UPDATE_LIST_ALL
        msg.obj = list
        sendMessage(msg)
    }

    override fun showLoading() {
        showProgressMum()
    }

    override fun hideLoading() {
        hideProgressMum()
    }

    override fun errorMsg(errorType: Int, msg: String?) {
        val message = Message.obtain()
        message.what = MSG_SHOW_ERROR
        message.arg1 = errorType
        message.obj = msg
        sendMessage(message)
    }

    private fun onCancelClick(position: Int) {
        val tradeCancelModel = if (position >= 0 && position < data.size) data[position] else null
        tradeCancelModel?.let { presenter.cancelOrder(it) }
    }

    override fun onClick(v: View?) {
        if (v?.id == R.id.cancel_all) {
            if (data.isNotEmpty()) {
                showDialog(message = getString(R.string.trade_cancel_all_tip), showNeg = true,
                        posListener = View.OnClickListener { presenter.cancelOrderAll(data) })
            }
        }
    }

    companion object {
        private const val MSG_UPDATE_LIST_ALL = 0x1
        private const val MSG_SHOW_ERROR = 0x2
        private const val MSG_ADD_UPDATE_LIST_ALL = 0x3
    }

}