package wind.android.bussiness.trade.abstocktrade.fragment

import android.support.design.widget.TabLayout
import android.util.SparseArray
import android.view.View
import android.view.ViewGroup
import kotlinx.android.synthetic.main.frag_a_stock_transfer.*
import ui.bell.DinTextView
import ui.screen.UIScreen
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.presenter.AStockTransferPresenter
import wind.android.bussiness.trade.abstocktrade.view.*
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.constant.TradeUserAction
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      AStockTransferFrag
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/18    17:00
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockTransferFrag : BaseATradeFrag<IAStockTransferView, AStockTransferPresenter>(), IAStockTransferView {

    private lateinit var mTabNames: Array<String>

    private var mBaseAStockTradeViews = SparseArray<BaseAStockTradeView>()

    private val mBaseOnTabSelectedListener by lazy {
        object : TabLayout.BaseOnTabSelectedListener<TabLayout.Tab> {
            override fun onTabReselected(p0: TabLayout.Tab?) {

            }

            override fun onTabUnselected(p0: TabLayout.Tab?) {
                p0?.let {
                    val view = it.customView
                    if (view is DinTextView) {
                        view.setBackgroundResource(R.drawable.tab_astock_query_item_bg)
                        view.setTextColor(StockUtil.getChangeColor(0f))
                    }
                    val index = it.tag.toString().toInt()
                    val child = mBaseAStockTradeViews.get(index)
                    child?.hide()
                }
            }

            override fun onTabSelected(p0: TabLayout.Tab?) {
                p0?.let {
                    val view = it.customView
                    if (view is DinTextView) {
                        view.setBackgroundResource(R.drawable.tab_astock_query_item_select_bg)
                        view.setTextColor(StockUtil.getChangeColor(1f))
                    }
                    val index = it.tag.toString().toInt()
                    var child = mBaseAStockTradeViews.get(index)
                    if (child == null) {
                        child = when (it.text) {
                            mTabNames[0] -> {
                                TradeConstantData.submitUserActionEx(TradeUserAction.BUTTON_TRADE_BANKTOPAPER)
                                BankToSecurityView(tabLayout.context)
                            }
                            mTabNames[1] -> {
                                TradeConstantData.submitUserActionEx(TradeUserAction.BUTTON_TRADE_PAPERTOBANK)
                                SecurityToBankView(tabLayout.context)
                            }
                            mTabNames[2] -> TransferQueryView(tabLayout.context)
                            else -> BankToSecurityView(tabLayout.context)
                        }
                        mBaseAStockTradeViews.put(index, child)
                        child.initView()
                        child.ggType = ggType
                    }
                    linearLayout.removeAllViews()
                    linearLayout.addView(child)
                    child.show()
                }
            }
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.frag_a_stock_transfer
    }

    override fun isRegisterEventBus(): Boolean {
        return false
    }

    override fun init() {
        mTabNames = arrayOf(getString(R.string.bankToSecurity),
                getString(R.string.securityToBank), getString(R.string.query))
        tabLayout.addOnTabSelectedListener(mBaseOnTabSelectedListener)
        val layoutParams = ViewGroup.LayoutParams(UIScreen.screenWidth / (mTabNames.size + 1), ViewGroup.LayoutParams.MATCH_PARENT)
        for (index in mTabNames.indices) {
            val tab = tabLayout.newTab().setTag(index).setText(mTabNames[index])
            val view = View.inflate(context, R.layout.tab_transfer_tablayout, null)
            val tabTv = view.findViewById<DinTextView>(R.id.tabTv)
            tabTv.text = tab.text
            tabTv.tag = index
            tabTv.layoutParams = layoutParams
            tabTv.setTextColor(StockUtil.getChangeColor(0f))
            tab.customView = tabTv
            tabLayout.addTab(tab, false)
        }
    }

    override fun onFragHide(byOnPause: Boolean) {
        for (index in mTabNames.indices) {
            mBaseAStockTradeViews.get(index)?.hide()
        }
        mBaseAStockTradeViews.clear()
    }

    override fun onFragShow(byOnResume: Boolean) {
        tabLayout.getTabAt(0)?.select()
    }

    override fun createPresenter(): AStockTransferPresenter {
        return AStockTransferPresenter(this)
    }

    override fun showLoading() {
    }

    override fun hideLoading() {
    }

    override fun errorMsg(errorType: Int, msg: String?) {
    }
}