package wind.android.bussiness.trade.abstocktrade.fragment

import android.os.Bundle
import android.os.Message
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import base.RootFragment
import eventbus.EventBusManager
import ui.AlertDialog
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.GGT_TYPE_KEY
import wind.android.bussiness.trade.abstocktrade.NON_GGT_TYPE
import wind.android.bussiness.trade.abstocktrade.presenter.BaseTradePresenter
import wind.android.bussiness.trade.abstocktrade.view.IBaseTradeView

/**
 * # **********************************************************************************************
 * # ClassName:      BaseATradeFragment
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/18    16:41
 * # Modifications:  initial
 * # **********************************************************************************************
 */

abstract class BaseATradeFrag<T : IBaseTradeView, P : BaseTradePresenter<T>> : RootFragment() {
    /**
     * 页面是否可见
     */
    protected var fragVisible = false
    /**
     * presenter
     */
    protected val presenter by lazy { createPresenter() }

    protected val ggType: Int by lazy { arguments?.getInt(GGT_TYPE_KEY, NON_GGT_TYPE) ?: NON_GGT_TYPE }


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(getLayoutId(), container, false)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        if (isRegisterEventBus()) {
            EventBusManager.getInstance().register(this)
        }
        fragVisible = true
        init()
    }

    override fun onResume() {
        super.onResume()
        if (fragVisible) {
            onFragShow(true)
        }
    }

    override fun onPause() {
        super.onPause()
        if (fragVisible) {
            onFragHide(true)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        if (isRegisterEventBus()) {
            EventBusManager.getInstance().unregister(this)
        }
        presenter.detachView()
    }

    override fun onHiddenChanged(hidden: Boolean) {
        if (hidden) {
            onFragHide(false)
        } else {
            onFragShow(false)
        }
        fragVisible = !hidden
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        if (!fragVisible) return
    }

    /**
     * 提示框展示
     *
     * @param title
     * @param message
     * @param posListener
     * @param negListener
     */
    @JvmOverloads
    protected fun showDialog(title: String = getString(R.string.dialog_phone_title),
                             message: String, showNeg: Boolean = false, posListener: View.OnClickListener? = null,
                             negListener: View.OnClickListener? = null) {
        activity?.let {
            val dlg = AlertDialog(it).builder()
            dlg.setTitle(title).setMsg(message, Gravity.CENTER)
                    .setCancelable(true).setPositiveButton(getString(R.string.pickerview_submit), posListener)
            if (showNeg) {
                dlg.setNegativeButton(getString(R.string.pickerview_cancel), negListener)
            }
            dlg.show()
        }
    }

    /**
     * 布局
     *
     * @return layout id
     */
    abstract fun getLayoutId(): Int

    abstract fun isRegisterEventBus(): Boolean

    /**
     * 初始化
     */
    abstract fun init()

    abstract fun createPresenter(): P

    /**
     * 页面隐藏
     */
    abstract fun onFragHide(byOnPause: Boolean)

    /**
     * 页面显示
     */
    abstract fun onFragShow(byOnResume: Boolean)
}