package wind.android.bussiness.trade.abstocktrade.manager

import android.text.TextUtils
import cn.com.hh.trade.data.*
import wind.android.bussiness.trade.abstocktrade.transformToFun1010
import wind.android.bussiness.trade.activity.TradeNet
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.hmg.manager.TradeError
import wind.android.bussiness.trade.listener.*
import wind.android.bussiness.trade.util.TradeAccountManager
import wind.android.bussiness.trade.view.ITradeReqCallBack
import java.math.BigDecimal

/**
 * # ***********************************************************************************************
 * # ClassName:       AbTradeManager.kt
 * # Description:     AB股交易
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class AbTradeManager<T> {
    /**
     * 监听管理器
     */
    private var mTradeListener = ArrayList<OnAbResultListener<T>>()

    private var mReq1094: TagReq_Fun1094? = null

    /******************************TradeNet监听 ****************/
    private val mTradeHoldAmountListener: TradeHoldAmountListener by lazy {
        object : TradeHoldAmountListener {
            @Suppress("UNCHECKED_CAST")
            override fun onHoldAmountBack(data: Any?) {
                if (data != null && data is Array<*>) {
                    val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                    for (item in tmpListener) {
                        if (item is OnAbAssetInfoResultListener) {
                            item.onAssetInfo(data[0] as? Array<TagAns_Fun1023_0>)
                            item.onHoldStock(data[1] as? Array<TagAns_Fun1023_1>)
                        }
                    }
                }
            }

            override fun onHoldAmountBackError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    item.onError(TradeError.TRADE_ERROR_ASSETINFO, error)
                }
            }
        }
    }
    private val mTradeCallback: ITradeCallback<T> by lazy {
        object : ITradeCallback<T> {
            override fun callback(data: T?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    try {
                        item.onCallback(data)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
            }

            override fun onError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    item.onError(TradeError.TRADE_ERROR_COMMON, error)
                }
            }
        }
    }
    private val mTradeReqCallBack: ITradeReqCallBack<T> by lazy {
        object : ITradeReqCallBack<T> {
            override fun onTradeDataBack(obj: T?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    try {
                        item.onCallback(obj)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
            }

            override fun updateCDWT(obj: Any?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbCancelResultListener) {
                        item.onCancelEntrust(obj)
                    }
                }
            }

            override fun onError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    item.onError(TradeError.TRADE_ERROR_COMMON, error)
                }
            }

        }
    }
    private val mTradeBusinessNetListener: TradeBusinessNetListener<T> by lazy {
        object : TradeBusinessNetListener<T> {
            override fun onTradeDoneDataBack(obj: T?) {
                if (obj != null && obj is Array<*>) {
                    val ansFun1010s = obj.map {
                        if (it is TagAns_Fun1012) transformToFun1010(it) else it
                    }.filterIsInstance<TagAns_Fun1010>().toTypedArray()
                    val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                    for (item in tmpListener) {
                        try {
                            item.onCallback(ansFun1010s as? T)
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
            }

            override fun onTradeDoneDataBackError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    item.onError(TradeError.TRADE_ERROR_COMMON, error)
                }
            }

            override fun onMaxDataBack(obj: Any?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBuySellResultListener) {
                        item.onMaxCallback(obj)
                    }
                }
            }

            override fun onMaxDataBackError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBuySellResultListener) {
                        item.onError(TradeError.TRADE_ERROR_MAX, error)
                    }
                }
            }
        }
    }

    private val mTradeAdventrustListener by lazy {
        object : TradeAdventrustListener<T> {
            override fun onAdventrustBack(functionID: Int, callback: Any?) {
                if (functionID == NSDPROCAPI.FUNID_CLIENTRIGHTCX) {
                    if (callback is Array<*> && callback.size == 1 && callback[0] is TagAns_Fun1028) {
                        val ans1028 = callback[0] as TagAns_Fun1028
                        val clientRight =
                                ans1028.chClientRights?.firstOrNull { it == 'd'.toByte() } != null
                        if (clientRight) {
                            bsAdventrust(mReq1094)
                        } else {
                            clientRightSet()
                        }
                    }
                } else if (functionID == NSDPROCAPI.FUNID_CLIENTRIGHTSET) {
                    if (callback is Array<*> && callback.size == 1 && callback[0] is TagAns_Fun1029) {
                        val ans1029 = callback[0] as TagAns_Fun1029
                        val clientRight =
                                ans1029.chClientRights?.firstOrNull { it == 'd'.toByte() } != null
                        if (clientRight) {
                            bsAdventrust(mReq1094)
                        }
                    }
                } else {
                    val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                    for (item in tmpListener) {
                        if (item is OnAbCancelResultListener) {
                            item.onAdventrustBack(functionID, callback)
                        } else if (item is OnAbBuySellResultListener) {
                            item.onAdventrustBack(functionID, callback)
                        }
                    }
                }
            }

            override fun onAdventrustError(funtionID: Int, error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBuySellResultListener) {
                        item.onError(TradeError.TRADE_ERROR_COMMON, error)
                    }
                }
            }
        }
    }

    private val mSwitchBankCallback: SwitchBankCallback by lazy {
        object : SwitchBankCallback {
            override fun onBankMoney(obj: Any?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBankResultListener) {
                        item.onBankMoney(obj)
                    }
                }
            }

            override fun onBankInfo(obj: Any?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBankResultListener) {
                        item.onBankInfo(obj)
                    }
                }
            }

            override fun onBankTransfer(obj: Any?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBankResultListener) {
                        item.onBankTransfer(obj)
                    }
                }
            }

            override fun onError(error: String?) {
                val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
                for (item in tmpListener) {
                    if (item is OnAbBankResultListener) {
                        item.onError(TradeError.TRADE_ERROR_COMMON, error)
                    }
                }
            }
        }
    }

    /**
     * 注册监听：业务层需要设置监听回调处理
     *
     * @param listener
     */
    fun registerListener(listener: OnAbResultListener<T>) {
        if (!mTradeListener.contains(listener)) {
            mTradeListener.add(listener)
        }
    }

    /**
     * 注销监听：业务层使用完毕需要注销
     */
    fun unregisterListener(listener: OnAbResultListener<T>) {
        mTradeListener.remove(listener)
    }

    /**
     * 获取交易账号
     *
     * @return
     */
    private fun getTradeNet(accountId: String? = null): TradeNet? {
        //获取当前账号
        val tradeNet: TradeNet? = if (TextUtils.isEmpty(accountId)) {
            TradeAccountManager.getInstance().getChildTradeAccount(null)
        } else {
            TradeAccountManager.getInstance().getTradeNet(accountId, false)
        }
        return tradeNet?.apply {
            tradeHoldAmountListener = mTradeHoldAmountListener
            setTradeCallback(mTradeCallback)
            // 撤单
            setTradeDoneView(mTradeReqCallBack)
            // 买卖
            tradeBusinessNetListener = mTradeBusinessNetListener
            setTradeAdventrustListener(mTradeAdventrustListener)
            setSwitchBankView(mSwitchBankCallback)
        } ?: with(tradeNet) {
            val tmpListener = ArrayList<OnAbResultListener<T>>(mTradeListener)
            for (item in tmpListener) {
                item.onError(TradeError.TRADE_ERROR_COMMON, "网络异常，请重新登录")
            }
            null
        }
    }

    /**
     * 持仓查询
     * @param ggtType 港股通标志位
     */
    fun queryHoldList(ggtType: Int) {
        val reqFun1023 = TagReq_Fun1023()
        reqFun1023.chQryType = '0'.toByte()
        getTradeNet()?.tradeClient?.CCCXRequest(reqFun1023, ggtType)
    }

    /**
     * 最大可买/卖量查询
     *
     * @param nLotSize 每手股数  speed指标 Indicator.DI_LotSize=72
     * @param userCode 股东代码
     * @param ggtType 港股通类型
     * @param chExtFlag1 委托方式
     *
     */
    fun queryMaxVolume(windCode: String, chBSFlag: Byte, nOrderPrice: Double,
                       nLotSize: Float, userCode: String, ggtType: Int, chExtFlag1: Byte) {
        val chMarketType = TradeConstantData.getMarketType(windCode).toByte()
        val chStockCode = getCode(windCode)
        val chAccountType = TradeConstantData.getAccountType(windCode, chMarketType)
        getTradeNet()?.tradeClient?.MaxVolRequest(chMarketType, chStockCode, chBSFlag,
                nOrderPrice, nLotSize, userCode, chAccountType, ggtType, chExtFlag1)
    }

    /**
     * 买卖委托
     * @param chExtFlag1 委托方式
     * @param chExtFlag3 港股用(港股扩展买卖类型)
     * @param userCode 股东代码
     * @param ggtType 港股通标志位
     */
    fun bsDelegate(windCode: String, chBSFlag: Byte, orderPrice: Double,
                   nOrderVolume: Int, chExtFlag1: Byte, chExtFlag3: Byte,
                   userCode: String, ggtType: Int) {
        val chMarketType = TradeConstantData.getMarketType(windCode).toByte()
        val chStockCode = getCode(windCode)
        val chAccountType = TradeConstantData.getAccountType(windCode, chMarketType)
        val nOrderPrice = BigDecimal(orderPrice * 10000).setScale(0, BigDecimal.ROUND_HALF_UP).toInt()
        if (chExtFlag1 != 0.toByte() && chExtFlag1 != '*'.toByte()) {
            // 市价委托
            val reqFun1012 = TagReq_Fun1012()
            reqFun1012.chMarketType = chMarketType
            reqFun1012.setChStockCode(chStockCode.toByteArray(charset("GBK")))
            reqFun1012.nOrderPrice = nOrderPrice
            reqFun1012.nOrderVolume = nOrderVolume
            reqFun1012.chBSFlag = chBSFlag
            reqFun1012.chExtFlag1 = chExtFlag1
            reqFun1012.chExtFlag3 = chExtFlag3
            getTradeNet()?.tradeClient?.requestSJWT(reqFun1012)
        } else {
            getTradeNet()?.tradeClient?.MMWTRequest(chMarketType, chStockCode, nOrderPrice,
                    nOrderVolume, chBSFlag, chExtFlag3, chAccountType, userCode, ggtType, chExtFlag1)
        }
    }

    /**
     * 预约委托
     */
    fun bsAdventrust(chMarketType: Byte, chUserCode: String, chStockCode: String,
                     nOrderVolume: Int, dOrderPrice: Double, chBSFlag: Byte,
                     chExtFlag3: Byte, chAccountType: Byte) {
        mReq1094 = TagReq_Fun1094()
        mReq1094?.let {
            it.chMarketType = chMarketType
            it.setChUserCode(chUserCode.toByteArray(charset("GBK")))
            it.setChStockCode(chStockCode.toByteArray(charset("GBK")))
            it.nOrderVolume = nOrderVolume
            it.nOrderPrice = BigDecimal(dOrderPrice * 10000).setScale(0, BigDecimal.ROUND_HALF_UP).toInt()
            it.chBSFlag = chBSFlag
            it.chExtFlag3 = chExtFlag3
            it.chAdventrustType = '1'.toByte()
            it.chAccountType = chAccountType
            it.nStageNumber = 1
        }
        clientRightCX()
    }

    /**
     * 预约委托
     */
    private fun bsAdventrust(req1094: TagReq_Fun1094?) {
        req1094?.let { getTradeNet()?.tradeClient?.adventrust(it) }
    }

    /**
     * 客户权限信息查询
     */
    private fun clientRightCX() {
        getTradeNet()?.tradeClient?.clientRightCX()
    }

    /**
     * 客户权限信息查询
     */
    private fun clientRightSet() {
        val byteArray = ByteArray(1)
        byteArray[0] = 'd'.toByte()
        getTradeNet()?.tradeClient?.clientRightSet('1'.toByte(), byteArray)
    }

    /**
     * 可撤单查询
     * @param ggtType 港股通标志位
     */
    fun queryCancel(ggtType: Int) {
        val reqFun1025 = TagReq_Fun1025()
        reqFun1025.chQryType = '0'.toByte()
        getTradeNet()?.tradeClient?.KCDCXRequest(reqFun1025, ggtType)
    }

    /**
     * 撤单
     * @param ggtType 港股通标志位
     */
    fun cancelEntrust(fun1025: TagAns_Fun1025, ggtType: Int) {
        if (fun1025.iResv2 == 1096.toLong()) {
            cancelAdventrust(fun1025.chOrderNumber)
        } else {
            val reqFun1011 = TagReq_Fun1011()
            reqFun1011.chStockCode = fun1025.chStockCode
            reqFun1011.setChOrderNumber(fun1025.chOrderNumber)
            reqFun1011.setChOrderGroup(fun1025.chOrderGroup)
            reqFun1011.nOrderDate = fun1025.nOrderDate
            reqFun1011.nOrderTime = fun1025.nOrderTime
            reqFun1011.chMarketType = fun1025.chMarketType
            reqFun1011.chBSFlag = fun1025.chBSFlag
            reqFun1011.chExtFlag2 = fun1025.chExtFlag2
            reqFun1011.nOrderVolume = fun1025.nOrderVolume
            reqFun1011.nOrderPrice = fun1025.nOrderPrice
            reqFun1011.chExtFlag1 = fun1025.chExtFlag1
            reqFun1011.setChUserCode(fun1025.chUserCode)
            getTradeNet()?.tradeClient?.CXWTRequest(reqFun1011, ggtType)
        }
    }

    /**
     * 全部撤单
     */
    fun cancelEntrusts(regFun1011: List<TagReq_Fun1011>) {
        getTradeNet()?.tradeClient?.CXWTRequest(regFun1011)
    }

    /**
     * 委托查询
     * @param ggtType 港股通标志位
     */
    fun queryDelegate(ggtType: Int) {
        val reqFun1024 = TagReq_Fun1024()
        reqFun1024.chQryType = '0'.toByte()
        getTradeNet()?.tradeClient?.WTCX(reqFun1024, ggtType)
    }

    /**
     * 历史委托查询
     * @param ggtType 港股通标志位
     */
    fun queryDelegateHis(beginDate: Int, endDate: Int, ggtType: Int) {
        val reqFun1030 = TagReq_Fun1030()
        reqFun1030.chQryType = '0'.toByte()
        reqFun1030.nBeginDate = beginDate
        reqFun1030.nEndDate = endDate
        getTradeNet()?.tradeClient?.LSWTCXRequest(reqFun1030, ggtType)
    }

    /**
     * 成交查询
     * @param ggtType 港股通标志位
     */
    fun queryTransaction(ggtType: Int) {
        val reqFun1026 = TagReq_Fun1026()
        reqFun1026.chQryType = '0'.toByte()
        getTradeNet()?.tradeClient?.CJCXRequest(reqFun1026, ggtType)
    }

    /**
     * 历史成交查询
     * @param ggtType 港股通标志位
     */
    fun queryTransactionHis(beginDate: Int, endDate: Int, ggtType: Int) {
        val reqFun1031 = TagReq_Fun1031()
        reqFun1031.chQryType = '0'.toByte()
        reqFun1031.nBeginDate = beginDate
        reqFun1031.nEndDate = endDate
        getTradeNet()?.tradeClient?.LSCJCXRequest(reqFun1031, ggtType)
    }

    /**
     * 银行信息查询
     */
    fun queryBankInfo() {
        getTradeNet()?.tradeClient?.YHXXCXRequest()
    }

    /**
     * 银行余额查询
     */
    fun queryBankMoney(selectBank: TagAns_Fun1040, pwd: String) {
        val reqFun1043 = TagReq_Fun1043()
        reqFun1043.chMoneyType = if (TradeConstantData.TYPE_HKSTOCK == TradeConstantData.selectedItem.type) {
            TradeConstantData.CURRENCY_HKD.toByte()
        } else {
            TradeConstantData.CURRENCY_RMB.toByte()
        }
        reqFun1043.setChSpecBank(selectBank.chBankCode)
        reqFun1043.setChBankAccount(selectBank.chBankAccount)
        reqFun1043.setChBankPwd(pwd.toByteArray(charset("GBK")))
        getTradeNet()?.tradeClient?.YHYERequest(reqFun1043)
    }

    /**
     * 银证转账
     * @param tranType true-银行转证券  false-证券转银行
     * @param tranAmt 转账金额
     */
    fun bankTransfer(tranType: Boolean, selectBank: TagAns_Fun1040, pwd: String, tranAmt: Double) {
        val reqFun1042 = TagReq_Fun1042()
        reqFun1042.chMoneyType = if (TradeConstantData.TYPE_HKSTOCK == TradeConstantData.selectedItem.type) {
            TradeConstantData.CURRENCY_HKD.toByte()
        } else {
            TradeConstantData.CURRENCY_RMB.toByte()
        }
        if (tranType) {
            reqFun1042.chTranType = 'I'.toByte()
            reqFun1042.setChBankPwd(pwd.toByteArray(charset("GBK")))
        } else {
            reqFun1042.chTranType = 'O'.toByte()
            reqFun1042.setChFundPwd(pwd.toByteArray(charset("GBK")))
        }
        reqFun1042.setChSpecBank(selectBank.chBankCode)
        reqFun1042.setChBankAccount(selectBank.chBankAccount)

        reqFun1042.dTranAmt = tranAmt
        getTradeNet()?.tradeClient?.YZZZRequest(reqFun1042)
    }

    /**
     * 查询转账流水
     */
    fun queryBankFlow(beginDate: Int, endDate: Int) {
        val reqFun1044 = TagReq_Fun1044()
        reqFun1044.chQryFlag = '0'.toByte()
        reqFun1044.nBeginDate = beginDate
        reqFun1044.nEndDate = endDate
        getTradeNet()?.tradeClient?.YHZZCXRequest(reqFun1044)
    }

    /**
     * 查询预约委托委托
     */
    fun queryAdventrustDelegate() {
        // 湘财支持预约委托 且在预约委托时间段
        getTradeNet()?.tradeClient?.adventrust
    }

    /**
     * 查询预约委托可撤单
     */
    fun queryAdventrustCancel() {
        // 湘财支持预约委托 且在预约委托时间段
        getTradeNet()?.tradeClient?.adventrustWithDraw
    }

    /**
     * 预约委托撤销委托
     */
    fun cancelAdventrust(chOrderNumber: ByteArray) {
        getTradeNet()?.tradeClient?.adventrustWithDraw(chOrderNumber)
    }

    /**
     * 获取不带后缀的code，给交易用
     */
    private fun getCode(windCode: String): String {
        return windCode.takeWhile { it != '.' }
    }

}

