package wind.android.bussiness.trade.abstocktrade.manager

import net.datamodel.network.RealQuoteItem
import net.datamodel.speed.*
import wind.android.bussiness.trade.abstocktrade.manager.integrate.KlineIntegrate
import wind.android.bussiness.trade.abstocktrade.manager.integrate.TrendIntegrate
import wind.android.f5.level2.manager.ILevel2
import wind.android.f5.level2.manager.Level2Manager
import wind.android.f5.net.base.SpeedConnection
import wind.android.f5.view.element.trend.FQFactorNet
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:       TradeTrendKlineControl.kt
 * # Description:     交易分时k线请求
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class TradeTrendKlineControl {

    /**
     * Level2Manager
     */
    private lateinit var mTrendManager: Level2Manager
    private lateinit var mKlineManager: Level2Manager

    /**
     * 分时K线请求数据整合
     */
    private val mTrendIntegrate by lazy { TrendIntegrate() }
    private val mKlineIntegrate by lazy { KlineIntegrate() }
    /**
     * Level2Manager回调
     */
    private val mTrendListener by lazy {
        object : Level2Manager.ILevel2Listener {
            override fun addData(item: RealQuoteItem?) {
                if (item != null) {
                    mTrendIntegrate.updateData(item)
                }
            }

            override fun onError() {

            }

            override fun setData(type: Int, data: ILevel2Data?) {
                if (type == ILevel2.TODAY_TREND && data != null && data is NewTrendData) {
                    mTrendIntegrate.setHistoryData(data)

                    if (SecType2.isHighTech(WindCodeType.getWindSecType(data.windCode, null))) {
                        mTrendManager.sub(ILevel2.AFTERMARKET_TREND, 0)
                    }
                } else if (type == ILevel2.AFTERMARKET_TREND && data != null && data is BidTrendData) {
                    mTrendIntegrate.setAfterMarket(data)
                }
            }

            override fun setData(type: Int, list: Vector<*>?) {

            }
        }
    }
    /**
     * Level2Manager回调
     */
    private val mKlineListener by lazy {
        object : Level2Manager.ILevel2Listener {
            override fun addData(item: RealQuoteItem?) {
                if (item != null) {
                    mKlineIntegrate.updateData(item)
                }
            }

            override fun onError() {

            }

            override fun setData(type: Int, data: ILevel2Data?) {
                if (type == ILevel2.KLINE && data != null && data is NewKLineData) {
                    mKlineIntegrate.setHistoryData(data)
                }
            }

            override fun setData(type: Int, list: Vector<*>?) {

            }
        }
    }

    fun setTradeTrendKlineCallback(callback: TradeTrendKlineCallback?) {
        mTrendIntegrate.mTradeTrendKlineCallback = callback
        mKlineIntegrate.mTradeTrendKlineCallback = callback
    }

    /**
     * 分时请求+订阅
     */
    fun subTrend(windCode: String) {
        if (!this::mTrendManager.isInitialized) {
            mTrendManager = Level2Manager(windCode, mTrendListener)
        } else {
            mTrendManager.setWindCode(windCode)
        }
        mTrendManager.sub(ILevel2.TODAY_TREND, 0)
    }

    /**
     * k线请求+订阅
     */
    fun subKline(windCode: String) {
        if (!this::mKlineManager.isInitialized) {
            mKlineManager = Level2Manager(windCode, mKlineListener)
        } else {
            mKlineManager.setWindCode(windCode)
        }
        getFqFactorNet(windCode)
        mKlineManager.sub(ILevel2.KLINE, KlineIntegrate.KLINE_COUNT)
    }

    private fun getFqFactorNet(windCode: String) {
        var fqFactorNet = SpeedConnection.getInstance().fqFactorNetMap[windCode]
        if (fqFactorNet == null) {
            fqFactorNet = FQFactorNet(windCode)
            fqFactorNet.request()
            fqFactorNet.setFQFactorListener {
                mKlineIntegrate.refreshData()
            }
        }
    }

    /**
     * 分时反订阅
     */
    fun unsubTrend() {
        if (this::mTrendManager.isInitialized) {
            mTrendManager.unsub(ILevel2.TODAY_TREND)
        }
    }

    /**
     * k线反订阅
     */
    fun unsubKline() {
        if (this::mKlineManager.isInitialized) {
            mKlineManager.unsub(ILevel2.KLINE)
        }
    }

    fun clean() {
        mTrendIntegrate.clean()
        mKlineIntegrate.clean()
    }

    interface TradeTrendKlineCallback {
        fun onTrendCallback(marketTime: List<Float>, convertTime: List<Float>, timeDifference: Float,
                            priceDifference: Float, preClose: Float, priceHigh: Float, priceLow: Float,
                            maxVolume: Float, maxAmount: Float, precision: Int, isHighTech: Boolean,
                            currentDataVec: Vector<Any>)

        fun onKlineCallback(priceVolume: List<Float>, dateList: List<Int>, list: Vector<Any>, precision: Int)
    }

}