package wind.android.bussiness.trade.abstocktrade.manager.integrate

import net.datamodel.network.CommonFunc
import net.datamodel.network.Indicator
import net.datamodel.network.KlineParamentCycle
import net.datamodel.network.RealQuoteItem
import net.datamodel.speed.NewKLineData
import net.datamodel.speed.NewKLineDataItem
import net.datamodel.speed.SecType2
import net.datamodel.speed.WindCodeType
import wind.android.bussiness.trade.abstocktrade.manager.TradeTrendKlineControl
import wind.android.f5.net.base.SpeedConnection
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:       KlineIntegrate.kt
 * # Description:     k线请求数据整合
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class KlineIntegrate {
    companion object {
        const val KLINE_COUNT = 50
    }

    var mTradeTrendKlineCallback: TradeTrendKlineControl.TradeTrendKlineCallback? = null

    private lateinit var mWindCode: String
    private var mType: Int = SecType2.Unknown.toInt()

    private var mHistoryData: NewKLineData? = null
    private var mRealQuoteItem: RealQuoteItem? = null
    private var mNewKLineDataItem = NewKLineDataItem()

    private var isRefresh = false

    fun setHistoryData(data: NewKLineData) {
        isRefresh = true
        mHistoryData = data
        val item = mRealQuoteItem
        if (item != null) {
            addData(item)
        }
    }

    fun updateData(data: RealQuoteItem) {
        if (data.TodayDate > 0) {
            mRealQuoteItem = data
            getDiff(data)
        }
        addData(data)
    }

    private fun getDiff(data: RealQuoteItem) {
        mWindCode = data.WindCode
        mType = WindCodeType.getWindSecType(mWindCode, null)
    }

    @Synchronized
    fun addData(data: RealQuoteItem) {
        for (index in data.indicators.indices) {
            when (data.indicators[index]) {
                Indicator.DI_TRADETIME -> mNewKLineDataItem.time = data.value[index].toInt()
                Indicator.DI_NEWPRICE -> mNewKLineDataItem.close = data.value[index]
                Indicator.DI_OPENPRICE -> mNewKLineDataItem.open = data.value[index]
                Indicator.DI_TODAYHIGH -> mNewKLineDataItem.high = data.value[index]
                Indicator.DI_TODAYLOW -> mNewKLineDataItem.low = data.value[index]
                Indicator.DI_VOLUMN -> mNewKLineDataItem.volume = data.value[index]
                Indicator.DI_TOTALAMOUNT -> mNewKLineDataItem.amount = data.value[index]
            }
        }
        if (data.TodayDate > 0) {
            mNewKLineDataItem.data = data.TodayDate
        }
        refreshData()
    }

    fun refreshData() {
        if (mNewKLineDataItem.close > 0f) {
            if (mNewKLineDataItem.high == 0f) {
                mNewKLineDataItem.high = mNewKLineDataItem.close
            }
            if (mNewKLineDataItem.low == 0f) {
                mNewKLineDataItem.low = mNewKLineDataItem.close
            }
            if (mNewKLineDataItem.open == 0f) {
                mNewKLineDataItem.open = mNewKLineDataItem.close
            }
            // 合并历史与实时数据
            val newItem: NewKLineDataItem = mNewKLineDataItem.copy()
//            var unit = SpeedTrendUtil.getExange(type, newKLineData.getWindCode(), true)
//            item.volume /= unit
//            unit = SpeedTrendUtil.getExange(type, newKLineData.getWindCode(), false)
//            item.volume *= unit
            // K线列表是否包含当日K线
            var list = mHistoryData?.klineList
            val lastItem = list?.lastElement()
            var isTheSameDate = false
            if (lastItem is NewKLineDataItem) {
                isTheSameDate = lastItem.data == newItem.data
            }
            if (!isTheSameDate && isRefresh) {
                val fqFactorNet = SpeedConnection.getInstance().fqFactorNetMap[mWindCode]
                if (fqFactorNet != null && fqFactorNet.hqbfFactor > 0 && list != null && list.size > 0) {
                    isRefresh = false
                    for (listItem in list) {
                        if (listItem is NewKLineDataItem) {
                            listItem.fqParameter = 1 / fqFactorNet.hqbfFactor
                            listItem.open = listItem.fqPriceOpen
                            listItem.close = listItem.fqPriceClose
                            listItem.high = listItem.fqPriceHigh
                            listItem.low = listItem.fqPriceLow
                            listItem.settle = listItem.fqPriceSettle
                            listItem.avarage5 = listItem.fqAvarage5
                            listItem.avarage10 = listItem.fqAvarage10
                            listItem.avarage20 = listItem.fqAvarage20
                            listItem.avarage30 = listItem.fqAvarage30
                            listItem.avarage60 = listItem.fqAvarage60
                            listItem.avarage120 = listItem.fqAvarage120
                            listItem.avarage180 = listItem.fqAvarage180
                            listItem.avarage240 = listItem.fqAvarage240
                        }
                    }
                }
            }

            if (list == null) {
                list = Vector<Any>()
                list.add(newItem)
            } else if (list.size == 0) {
                list.add(newItem)
            } else {
                if (mHistoryData?.klineType == KlineParamentCycle.Day) {
                    if (newItem.data > 0) {
                        if (!isTheSameDate) {
                            list.add(newItem)
                        } else {
                            list[list.size - 1] = newItem
                        }
                    }
                }
            }

            var isFull = false
            var startIndex = 0
            if (list.size >= KLINE_COUNT) {
                startIndex = list.size - KLINE_COUNT
                isFull = true
            }
            // priceHigh-priceMiddle-priceLow-maxVolume
            val priceVolume = mutableListOf(Float.MIN_VALUE, 0f,
                    Float.MAX_VALUE, Float.MIN_VALUE)
            val dateList = mutableListOf(0, 0, 0)
            for (i in startIndex until list.size) {
                val item = list[i]
                if (item is NewKLineDataItem) {
                    priceVolume[3] = item.volume.coerceAtLeast(priceVolume[3])
                    priceVolume[0] = item.high.coerceAtLeast(priceVolume[0])
                    priceVolume[2] = item.low.coerceAtMost(priceVolume[2])
                    if (i == startIndex) {
                        dateList[0] = item.data
                    }
                    if (isFull && i == list.size - 1) {
                        dateList[2] = item.data
                    }
                    if (isFull && i == (list.size + startIndex) / 2) {
                        dateList[1] = item.data
                    }
                }
            }
            priceVolume[1] = (priceVolume[0] + priceVolume[2]) / 2f
            val precision = CommonFunc.getRadixPointFactor(mWindCode)
            mTradeTrendKlineCallback?.onKlineCallback(priceVolume, dateList, list, precision)
        }
    }

    fun clean() {
        mHistoryData = null
        mRealQuoteItem = null
        mNewKLineDataItem = NewKLineDataItem()
    }
}