package wind.android.bussiness.trade.abstocktrade.manager.integrate

import log.LogHelper
import net.datamodel.network.CommonFunc
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import net.datamodel.speed.*
import util.StringUtils
import wind.android.bussiness.trade.abstocktrade.manager.TradeTrendKlineControl
import wind.android.f5.tradetimeconfig.DefaultTradeManager
import wind.android.f5.tradetimeconfig.ITradeTime
import wind.android.f5.tradetimeconfig.model.TradeDateInfo
import wind.android.f5.view.element.SpeedTrendUtil
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:       TrendIntegrate.kt
 * # Description:     分时请求数据整合
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class TrendIntegrate : ITradeTime {

    var mTradeTrendKlineCallback: TradeTrendKlineControl.TradeTrendKlineCallback? = null

    /**
     * 市场时间-open-Suspend-Resume-Suspend1-Resume1-Suspend2-Resume2-close
     * @see net.datamodel.network.Indicator.DI_MARKETOPENTIME
     * @see net.datamodel.network.Indicator.DI_MARKETSUSPENDTIME
     * @see net.datamodel.network.Indicator.DI_MARKETRESUMETIME
     * @see net.datamodel.network.Indicator.DI_MARKETCLOSETIME
     */
    private var marketTime = mutableListOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
    private var convertTime = mutableListOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)

    private var stopStock = 0
    private var mWindCode: String? = null
    private var mType: Int = SecType2.Unknown.toInt()
    private var mTdi: TradeDateInfo? = null
    private var mHistoryData: NewTrendData? = null
    private var mAfterData: BidTrendData? = null

    private var priceHigh = 0f
    private var priceLow = 0f
    private var priceClose = 0f
    private var maxVolume = 0f
    private var maxAmount = 0f

    private var timeDifference = 0f
    private var priceDifference = 0f

    private var mRealQuoteItem: RealQuoteItem? = null

    /**
     * 是否科创板
     */
    private var isHighTech = false

    override fun getWindCode(): String? {
        return mWindCode
    }

    override fun setTradeDateInfo(tdi: TradeDateInfo) {
        mTdi = tdi
    }

    override fun getTradeDateInfo(): TradeDateInfo? {
        return mTdi
    }

    override fun setMarketOpen(marketOpen: Int) {
        this.marketTime[0] = marketOpen.toFloat()
    }

    override fun setMarketSuspend(marketSuspend: Int) {
        this.marketTime[1] = marketSuspend.toFloat()
    }

    override fun setMarketResume(marketResume: Int) {
        this.marketTime[2] = marketResume.toFloat()
    }

    override fun setMarketSuspend1(marketSuspend1: Int) {
        this.marketTime[3] = marketSuspend1.toFloat()
    }

    override fun setMarketResume1(marketResume1: Int) {
        this.marketTime[4] = marketResume1.toFloat()
    }

    override fun setMarketSuspend2(marketSuspend2: Int) {
        this.marketTime[5] = marketSuspend2.toFloat()
    }

    override fun setMarketResume2(marketResume2: Int) {
        this.marketTime[6] = marketResume2.toFloat()
    }

    override fun setMarketClose(marketClose: Int) {
        this.marketTime[7] = marketClose.toFloat()
    }

    override fun getMarketOpen(): Int {
        return marketTime[0].toInt()
    }

    override fun getMarketSuspend(): Int {
        return marketTime[1].toInt()
    }

    override fun getMarketResume(): Int {
        return marketTime[2].toInt()
    }

    override fun getMarketSuspend1(): Int {
        return marketTime[3].toInt()
    }

    override fun getMarketResume1(): Int {
        return marketTime[4].toInt()
    }

    override fun getMarketSuspend2(): Int {
        return marketTime[5].toInt()
    }

    override fun getMarketResume2(): Int {
        return marketTime[6].toInt()
    }

    override fun getMarketClose(): Int {
        return marketTime[7].toInt()
    }

    override fun setConvert_suspend() {
        convertTime[1] += 1440f
    }

    override fun setConvert_resume() {
        convertTime[2] += 1440f
    }

    override fun setConvert_suspend1() {
        convertTime[3] += 1440f
    }

    override fun setConvert_resume1() {
        convertTime[4] += 1440f
    }

    override fun setConvert_suspend2() {
        convertTime[5] += 1440f
    }

    override fun setConvert_resume2() {
        convertTime[6] += 1440f
    }

    fun setHistoryData(data: NewTrendData) {
        mHistoryData = data
        val item = mRealQuoteItem
        if (item != null) {
            addData(item)
        }
    }

    fun updateData(data: RealQuoteItem) {
        if (data.TodayDate > 0) {
            mRealQuoteItem = data
            getDiff(data)
        }
        addData(data)
    }

    fun setAfterMarket(data: BidTrendData) {
        mAfterData = data
        addData(null)
    }

    private fun getDiff(data: RealQuoteItem) {
        mWindCode = data.WindCode
        mType = WindCodeType.getWindSecType(mWindCode, null)

        for (index in data.indicators.indices) {
            when (data.indicators[index]) {
                Indicator.DI_MARKETOPENTIME ->
                    marketTime[0] = if (data.value[index] > 1440) 0f else data.value[index]
                Indicator.DI_MARKETSUSPENDTIME ->
                    marketTime[1] = if (data.value[index] > 1440) 0f else data.value[index]
                Indicator.DI_MARKETRESUMETIME ->
                    marketTime[2] = if (data.value[index] > 1440) 0f else data.value[index]
                Indicator.DI_MARKETCLOSETIME ->
                    marketTime[7] = if (data.value[index] > 1440) 0f else data.value[index]
                Indicator.DI_STOPSTOCK ->
                    stopStock = SpeedTrendUtil.getSuspensionFlag(data.value[index], data.TodayDate)
            }
        }
        DefaultTradeManager.initLocalConfig(this)

        if (mTdi == null) {
            if ((data.SecType == SecType2.WaresFutures.toInt() || data.SecType == SecType2.StockFutures.toInt())
                    && marketTime[1] == 615f && marketTime[2] == 630f) {
                //  10:15-10:30   11:30-13:30
                // 上海期货、大连商品、郑州商品、全球商品为83
                marketTime[3] = 690f
                marketTime[4] = 810f
            } else if ((data.SecType == SecType2.WaresFutures.toInt() || data.SecType == SecType2.StockFutures.toInt())
                    && (marketTime[1] == 150f || marketTime[1] == 60f || marketTime[1] == 1380f || marketTime[1] == 1410f)
                    && marketTime[2] == 540f) {
                // open21:00  2:30(1:00/23:30)-9:00 10:15-10:30  11:30-13:30  close15:00
                // 现货AU999,(AU.SHF AG.SHF)
                if (data.WindCode.endsWith(".SGE")) {
                    marketTime[3] = 690f
                    marketTime[4] = 810f
                } else {
                    marketTime[3] = 615f
                    marketTime[4] = 630f
                    marketTime[5] = 690f
                    marketTime[6] = 810f
                }
            } else if (data.SecType == SecType2.StockFutures.toInt() && marketTime[0] == 1020f
                    && marketTime[1] == 1380f && marketTime[2] == 555f && marketTime[7] == 975f
                    && data.WindCode.endsWith(".HK")) {
                // open17:00  23:00-9:15 12:00-13:00 close16:15
                // 港股期货
                marketTime[3] = 720f
                marketTime[4] = 780f
            } else if (SecType2.Sanban.toInt() == data.SecType) {
                // open9:30  11:30-13:00  close15:00
                // 解决bug：三板股票分时页面的走势，时间刻度有误（是交易双方私下约定的价格，随时交易，交易后才有数据）(时间区间和李超确认为9:30到15:00)
                marketTime[0] = 570f
                marketTime[1] = 690f
                marketTime[2] = 780f
                marketTime[7] = 900f
            } else if (data.WindCode.endsWith(".CSI") && SecType2.isIndexType(data.SecType)) {
                // 沪港通指数
            } else if (SecType2.BaseFx.toInt() == data.SecType) {
                //外汇
            } else if (data.WindCode.endsWith(".WI") && data.WindCode.startsWith("CGB")) {
                marketTime[0] = 540f
                marketTime[1] = 720f
                marketTime[2] = 810f
                marketTime[7] = 990f
            } else if (data.WindCode.endsWith(".WI") && marketTime[1] == 615f && marketTime[2] == 630f) {
                //AFI.WI
                marketTime[3] = 690f
                marketTime[4] = 810f
            } else if (SecType2.BaseBankRate.toInt() == mType && marketTime[0] == 0f && marketTime[7] == 0f) {
                // 利率
                marketTime[0] = 540f
                marketTime[1] = 720f
                marketTime[2] = 810f
                marketTime[7] = 990f
            }
        }

        convertTime[1] = marketTime[1]
        convertTime[2] = marketTime[2]
        convertTime[3] = marketTime[3]
        convertTime[4] = marketTime[4]
        convertTime[5] = marketTime[5]
        convertTime[6] = marketTime[6]

        // 开盘 休市 复市 休市1 复市1 休市2 复市2 闭盘
        // 实现逻辑，如果前比后大，则后面的加24小时
        DefaultTradeManager.initLocalMinutes(this)
        if (mTdi == null) {
            if (marketTime[1] != 0f) {
                if (marketTime[3] == 0f) { // 只有一个休市
                    if (marketTime[0] > marketTime[1]) {
                        convertTime[1] += 1440f
                        convertTime[2] += 1440f
                    } else if (marketTime[1] > marketTime[2]) {
                        convertTime[2] += 1440f
                    }
                } else {
                    if (marketTime[5] == 0f) { // 有两个休市
                        when {
                            marketTime[0] > marketTime[1] -> {
                                convertTime[1] += 1440f
                                convertTime[2] += 1440f
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                            }
                            marketTime[1] > marketTime[2] -> {
                                convertTime[2] += 1440f
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                            }
                            marketTime[2] > marketTime[3] -> {
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                            }
                            marketTime[3] > marketTime[4] -> {
                                convertTime[4] += 1440f
                            }
                        }
                    } else { // 有三个休市  比如AG.SHF
                        when {
                            marketTime[0] > marketTime[1] -> {
                                convertTime[1] += 1440f
                                convertTime[2] += 1440f
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                                convertTime[5] += 1440f
                                convertTime[6] += 1440f
                            }
                            marketTime[1] > marketTime[2] -> {
                                convertTime[2] += 1440f
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                                convertTime[5] += 1440f
                                convertTime[6] += 1440f
                            }
                            marketTime[2] > marketTime[3] -> {
                                convertTime[3] += 1440f
                                convertTime[4] += 1440f
                                convertTime[5] += 1440f
                                convertTime[6] += 1440f
                            }
                            marketTime[3] > marketTime[4] -> {
                                convertTime[4] += 1440f
                                convertTime[5] += 1440f
                                convertTime[6] += 1440f
                            }
                            marketTime[4] > marketTime[5] -> {
                                convertTime[5] += 1440f
                                convertTime[6] += 1440f
                            }
                            marketTime[5] > marketTime[6] -> {
                                convertTime[6] += 1440f
                            }
                        }
                    }
                }
            }
        }

        isHighTech = SecType2.isHighTech(mType)
        if (isHighTech) {
            marketSuspend1 = 900
            marketResume1 = 905
            marketClose = 15 * 60 + 30
        }
        // 计算实际交易时间
        timeDifference = marketTime[7] - marketTime[0] - (marketTime[2] - marketTime[1]) -
                (marketTime[4] - marketTime[3]) - (marketTime[6] - marketTime[5])
        //1440 = 24*60 处理全天交易的情况
        if (timeDifference <= 0) timeDifference += 1440f
    }

    @Synchronized
    private fun addData(data: RealQuoteItem?) {
        if (data == null) {
            addAfterData()
            return
        }
        val newTrendDataItem = NewTrendDataItem()
        var todayHigh = 0f
        var todayLow = 0f
        for (index in data.indicators.indices) {
            when (data.indicators[index]) {
                Indicator.DI_TODAYHIGH -> todayHigh = data.value[index]
                Indicator.DI_TODAYLOW -> todayLow = data.value[index]
                Indicator.DI_PREVCLOSE -> priceClose = data.value[index]
                Indicator.DI_TRADETIME -> newTrendDataItem.tradeTime = data.value[index].toInt()
                Indicator.DI_NEWPRICE -> newTrendDataItem.newValue = data.value[index]
                Indicator.DI_DELTAAMOUNT -> newTrendDataItem.deltaAmount = data.value[index]
                Indicator.DI_DELTAVOLUMN -> newTrendDataItem.deltaVolum = data.value[index].toLong()
                Indicator.DI_AVGPRICE -> newTrendDataItem.averagePrice = data.value[index]
            }
        }

        if (todayHigh != 0f) {
            priceHigh = todayHigh.coerceAtLeast(priceHigh)
        }
        if (todayLow != 0f) {
            priceLow = if (priceLow == 0f) todayLow else todayLow.coerceAtMost(priceLow)
        }

        val precision = CommonFunc.getRadixPointFactor(mWindCode)
        if (priceHigh.equals(priceLow) && priceHigh != 0f) {
            var dec = 2f
            if (precision > 0) {
                for (i in 0..precision) {
                    dec *= 0.1f
                }
            }
            priceHigh += dec
            priceLow -= dec
        }
        // 最大最小值根据昨收重新计算(有精度要求)
        val deltaPrice: Float
        if ((priceHigh - priceClose).compareTo(priceClose - priceLow) > 0) {
            deltaPrice = CommonFunc.addFloat(priceHigh, -priceClose)
            priceLow = CommonFunc.addFloat(priceClose, -deltaPrice)
        } else {
            deltaPrice = CommonFunc.addFloat(priceClose, -priceLow)
            priceHigh = CommonFunc.addFloat(priceClose, deltaPrice)
        }
        // 价格差
        priceDifference = priceHigh - priceLow

        val historyData = mHistoryData
        val mCurrentDataVec = if (historyData?.getTrendLineList() != null) {
            historyData.getTrendLineList()
        } else {
            Vector<Any>()
        }
        if (mCurrentDataVec.size == 0) {
            mCurrentDataVec.add(newTrendDataItem)
        } else {
            val lastItem = mCurrentDataVec.lastElement()
            if (lastItem is NewTrendDataItem) {
                if (newTrendDataItem.newValue == 0f) {
                    newTrendDataItem.newValue = lastItem.newValue
                }
                if (newTrendDataItem.tradeTime == 0
                        || lastItem.tradeTime / 100 == newTrendDataItem.tradeTime / 100) {
                    newTrendDataItem.deltaVolum += lastItem.deltaVolum
                    newTrendDataItem.deltaAmount += lastItem.deltaAmount
                    if (newTrendDataItem.tradeTime == 0) {
                        newTrendDataItem.tradeTime = lastItem.tradeTime
                    }
                    mCurrentDataVec[mCurrentDataVec.size - 1] = newTrendDataItem
                } else {
                    // 屏蔽错误的数据
                    val newTimes = getTradeMarket(newTrendDataItem.tradeTime,
                            false)
                    val lastTimes = getTradeMarket(lastItem.tradeTime, false)

                    if (newTimes > lastTimes) {
                        mCurrentDataVec.add(newTrendDataItem)
                    } else {
                        LogHelper.println("SystemOut", "屏蔽错误的数据:$newTimes:$lastTimes")
                    }
                }
            }
        }
        if (historyData != null) {
            historyData.trendLineList = mCurrentDataVec
            mHistoryData = historyData
        }
        refreshData(mCurrentDataVec)
    }


    private fun addAfterData() {
        val historyData = mHistoryData
        val mCurrentDataVec = if (historyData?.getTrendLineList() != null) {
            historyData.getTrendLineList()
        } else {
            Vector<Any>()
        }

        refreshData(mCurrentDataVec)
    }

    private fun refreshData(mCurrentDataVec: Vector<Any>) {
        val afterData = mAfterData?.bidDataList
        if (afterData != null && afterData.size > 0) {
            // 盘后数据
            val needAddLast = ArrayList<NewTrendDataItem>()
            if (mCurrentDataVec.size > 0) {
                // 盘后数据的第一根firstItem
                val firstItem: BidTrendDataItem = afterData[0]
                for (i in mCurrentDataVec.size - 1..0) { // 集合中的最后一根lastItem
                    val lastItem = mCurrentDataVec[i]
                    // 集合中的最后一根和盘后数据的第一根进行比较，若lastItem时间大于firstItem，则需要将盘后数据插入进来
                    if (lastItem is NewTrendDataItem && lastItem.tradeTime > firstItem.tradeTime) {
                        needAddLast.add(lastItem)
                        mCurrentDataVec.removeElementAt(i)
                    } else {
                        break
                    }
                }
            }
            for (item in afterData) {
                if (item.tradeTime > 0 && item.newValue > 0 && item.bidVolume > 0) {
                    val newTrendDataItem = NewTrendDataItem()
                    newTrendDataItem.tradeTime = item.tradeTime
                    newTrendDataItem.newValue = item.newValue
                    newTrendDataItem.totalVolume = item.bidVolume
                    mCurrentDataVec.add(newTrendDataItem)
                }
            }
            if (needAddLast.size > 0) {
                for (i in needAddLast.indices.reversed()) {
                    mCurrentDataVec.add(needAddLast[i])
                }
            }
        }

        //计算最大成交量
        for (i in 0 until mCurrentDataVec.size) {
            val item = mCurrentDataVec[i]
            if (item is NewTrendDataItem) {
                if (i == 0) {
                    maxVolume = maxVolume.coerceAtLeast(item.totalVolume)
                    maxAmount = maxAmount.coerceAtLeast(item.totalAmount)
                } else {
                    maxVolume = maxVolume.coerceAtLeast(item.deltaVolum.toFloat())
                    maxAmount = maxAmount.coerceAtLeast(item.deltaAmount)
                }
            }
        }
        val vec = trendItemComplete(mCurrentDataVec)
        val precision = CommonFunc.getRadixPointFactor(mWindCode)
        mTradeTrendKlineCallback?.onTrendCallback(marketTime, convertTime, timeDifference, priceDifference, priceClose,
                priceHigh, priceLow, maxVolume, maxAmount, precision, isHighTech, vec)
    }

    fun clean() {
        marketTime = mutableListOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
        convertTime = mutableListOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
        priceClose = 0f
        priceHigh = 0f
        priceLow = 0f
        maxVolume = 0f
        maxAmount = 0f
        mHistoryData = null
        mAfterData = null
        mRealQuoteItem = null
    }

    /**
     * 当日分钟补全
     */
    private fun trendItemComplete(trendList: Vector<Any>): Vector<Any> {
        val newTrendList = Vector<Any>() // 存储新的容器
        if (trendList.size == 0) {
            return newTrendList
        }
        var item: NewTrendDataItem? //  当前根
        var lastItem: NewTrendDataItem? = null // 上一根
        var currentTimes = 0 //  当前根
        var lastTimes = 0 // 上一根
        for (i in trendList.indices) {
            item = trendList[i] as NewTrendDataItem
            currentTimes = getTradeMarket(item.tradeTime, false).toInt()
            if (lastItem != null) {
                // 获取上一根的状态
                lastTimes = getTradeMarket(lastItem.tradeTime, false).toInt()
            }
            if (currentTimes > 0 && currentTimes - lastTimes > 1) {
                // 补全 从0开始
                val start = if (lastTimes < 0) 0 else lastTimes + 1
                var copyItem: NewTrendDataItem? = null
                // addcopy
                for (j in start until currentTimes) {
                    if (lastItem == null) {
                        lastItem = NewTrendDataItem()
                        lastItem.newValue = priceClose
                        lastItem.tradeTime = getTradeTime(marketTime[0].toInt() - 1)
                    }
                    copyItem = if (copyItem == null) {
                        lastItem.copy()
                    } else {
                        copyItem.copy()
                    }
                    if (lastTimes < 0) {
                        // 开盘时间开始
                        copyItem.tradeTime = getTradeTime(marketTime[0].toInt())
                    } else {
                        copyItem.tradeTime = addTradeTime(copyItem.tradeTime, 1)
                        copyItem.deltaAmount = 0f
                        copyItem.deltaVolum = 0
                        copyItem.totalAmount = 0f
                        copyItem.totalVolume = 0f
                    }
                    lastItem = copyItem.copy()
                    newTrendList.add(copyItem)
                }
            }
            if (currentTimes >= 0 && currentTimes >= lastTimes) {
                // 过滤盘前/盘后/盘中休盘的数据 && 过滤多余的数据(currentTimes < lastTimes 代表数据多余)
                lastItem = item.copy()
                newTrendList.add(item)
            }
        }
        if (currentTimes < lastTimes) {
            // 避免最后一根数据错误 被过滤掉之后 currentTimes未更新的问题
            currentTimes = lastTimes
        }
        return newTrendList
    }

    /**
     * 计算距离开市时间的分钟数/根据距离开市时间的分钟数反计算时间
     *
     * @param tradeTime
     * @param inverted
     * @return
     * @throws
     * @since 1.0.0
     */
    private fun getTradeMarket(tradeTime: Int, inverted: Boolean): Float {
        var tradeMarket: Float
        if (!inverted) {
            val time = SpeedTrendUtil.getTimeMinutes(tradeTime)
            if (marketTime[7] - marketTime[0] > 0 && (time < marketTime[0] || time > marketTime[7])) {
                // 收大于开 同时time小于开 代表未开盘 如A股
                // 收大于开 同时time大于收 代表盘后 如美股
                return (-2).toFloat()
            }
            tradeMarket = time - marketTime[0]
            if (tradeMarket < 0) {
                tradeMarket += 1440f
            }
            if (tradeMarket > convertTime[1] - marketTime[0]
                    && marketTime[1] != 0f) {
                if (tradeMarket < convertTime[2] - marketTime[0]
                        && marketTime[2] != 0f) {
                    return (-2).toFloat()
                }
                if (tradeMarket > convertTime[3] - marketTime[0]
                        && marketTime[3] != 0f) {
                    if (tradeMarket < convertTime[4] - marketTime[0]
                            && marketTime[4] != 0f) {
                        return (-2).toFloat()
                    }
                    tradeMarket -= if (tradeMarket > convertTime[5] - marketTime[0]
                            && marketTime[5] != 0f) {
                        if (tradeMarket < convertTime[6] - marketTime[0]
                                && marketTime[6] != 0f) {
                            return (-2).toFloat()
                        }
                        (convertTime[6] - convertTime[5]
                                + (convertTime[4] - convertTime[3])
                                + (convertTime[2] - convertTime[1]))
                    } else {
                        convertTime[2] - convertTime[1] +
                                convertTime[4] - convertTime[3]
                    }
                } else {
                    tradeMarket -= convertTime[2] - convertTime[1]
                }
            }
        } else {
            tradeMarket = marketTime[0] + tradeTime
            if (tradeMarket > convertTime[1] && marketTime[1] != 0f) {
                tradeMarket += convertTime[2] - convertTime[1]
                if (tradeMarket > convertTime[3] && marketTime[3] != 0f) {
                    tradeMarket += convertTime[4] - convertTime[3]
                    if (tradeMarket > convertTime[5] && marketTime[5] != 0f) {
                        tradeMarket += convertTime[6] - convertTime[5]
                    }
                }
            }
        }
        return tradeMarket
    }

    /**
     * 根据分钟数 获取 时间HHmmss
     *
     * @param minutes
     * @return
     */
    private fun getTradeTime(minutes: Int): Int {
        try {
            val time = SpeedTrendUtil.minuteToTime(minutes.toFloat())
            val sdf = SimpleDateFormat("HH:mm")
            val sdf1 = SimpleDateFormat("HHmmss")
            val s = sdf1.format(sdf.parse(time))
            return StringUtils.valueOfInteger(s, 0)
        } catch (e: ParseException) {
            e.printStackTrace()
        }
        return 0
    }

    /**
     * HHmmss 增加1分钟
     *
     * @param tradeTime
     * @param addMinute
     * @return
     */
    private fun addTradeTime(tradeTime: Int, addMinute: Int): Int {
        try {
            val sdf = SimpleDateFormat("HHmmss")
            val date = sdf.parse(CommonFunc.paddingZeroLeft(tradeTime.toString(), 6))
            date.time = date.time + addMinute * 60 * 1000
            // 如果在休盘时间 跨过休盘区域
            val times = StringUtils.valueOfInteger(sdf.format(date), 0)
            var minutes = SpeedTrendUtil.getTimeMinutes(times)
            if (minutes > marketTime[1] && minutes <= marketTime[2]) {
                minutes += (marketTime[2] - marketTime[1]).toInt()
            }
            return getTradeTime(minutes)
        } catch (e: ParseException) {
            e.printStackTrace()
        }
        return 0
    }
}