package wind.android.bussiness.trade.abstocktrade.presenter

import android.graphics.Color
import android.text.Spannable
import android.text.SpannableString
import android.text.style.ForegroundColorSpan
import cn.com.hh.trade.data.*
import cn.com.hh.trade.data.NSDPROCAPI.FUNID_ADVENTRUST
import eventbus.EventBusManager
import net.datamodel.network.CommonFunc
import net.datamodel.speed.SecType2
import net.datamodel.speed.WindCodeType
import net.util.TimeHelper
import network.NetManager
import util.language.Language
import wind.android.bussiness.trade.abstocktrade.*
import wind.android.bussiness.trade.abstocktrade.bean.AbTradeBusinessModel
import wind.android.bussiness.trade.abstocktrade.eventbus.StockTradeMsgEvent
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbBuySellResultListener
import wind.android.bussiness.trade.abstocktrade.view.IAStockBuySellView
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.constant.TradeUserAction
import wind.android.bussiness.trade.home.manager.TradeHelper
import wind.android.bussiness.trade.home.model.JSHoldStockValue
import wind.android.bussiness.trade.util.TradeAccountManager
import java.math.BigDecimal
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.max
import kotlin.math.min

/**
 * # **********************************************************************************************
 * # ClassName:      ATradeBuyPresenter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/19    16:08
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockBuySellPresenter(buyView: IAStockBuySellView, val bsFlag: Byte) : BaseTradePresenter<IAStockBuySellView>(buyView) {
    /**
     * 数据请求model层
     */
    private val manager: AbTradeManager<Array<TagAns_Fun1010?>?> by lazy { AbTradeManager<Array<TagAns_Fun1010?>?>() }
    /**
     * 港股通类型标识
     */
    @GggType
    private var ggtType: Int = NON_GGT_TYPE
    /**
     * 业务对象
     */
    private val businessModel: AbTradeBusinessModel by lazy { AbTradeBusinessModel() }
    /**
     * 委托方式
     */
    var delegateType: TradeDelegateType = TradeDelegateType.LIMIT_ORDER
        set(value) {
            field = value
            businessModel.delegateType = value
            selectDelegateType(value)
        }
    /**
     * 股东账号
     */
    var userCode: String? = null
    /**
     * 持仓请求回调
     */
    private val callback = object : OnAbBuySellResultListener<Array<TagAns_Fun1010?>?> {
        override fun onMaxCallback(obj: Any?) {
            if (obj is Array<*>) {
                val first = obj.firstOrNull { it is TagAns_Fun1014 } as? TagAns_Fun1014
                first?.let {
                    getView()?.onUpdateMaxVolume(it.nMaxStkQty, businessModel.maxVolumeUnit)
                }
            }
        }

        override fun onAssetInfo(data: Array<TagAns_Fun1023_0>?) {
        }

        override fun onHoldStock(data: Array<TagAns_Fun1023_1>?) {
            val list = data?.map {
                transformToHoldItem(it)
            }
            val view = getView()
            view?.let {
                it.hideLoading()
                it.onUpdateHoldList(TradeHelper.sortStockByMarketvalue(list))
            }
        }

        override fun onCallback(data: Array<TagAns_Fun1010?>?) {
            val view = getView() ?: return
            view.hideLoading()
            val firstItem = data?.firstOrNull()
            firstItem?.let {
                val text = "证券代码 : ${it.getString(it.chStockCode)}\n" +
                        "买卖方向 : ${if (bsFlag == BUY) "买入" else "卖出"}\n" +
                        "委托数量 : ${it.nOrderVolume}\n" +
                        "委  托  号 : ${it.getString(it.chOrderNumber)}"
                view.onTradeResult(true, text)
            } ?: view.onTradeResult(false, "交易失败")
        }

        override fun onError(error: Int, message: String?) {
            val view = getView() ?: return
            view.hideLoading()
            if (!message.isNullOrEmpty() && (message.contains(MMWT_ERROR1)
                            || message.contains(MMWT_ERROR2)) && advanceTrustTime()) {
                val tipMsg = getSpannableString(ADVANCE_DELEGATE_TIP1, 22, 28)
                view.onShowAdvanceDelegate(ADVANCE_DELEGATE_TIP_TYPE1, tipMsg)
            } else {
                view.errorMsg(TIP_ALERT, message)
            }
        }

        override fun onAdventrustBack(functionID: Int, obj: Any?) {
            val view = getView() ?: return
            view.hideLoading()
            if (functionID == FUNID_ADVENTRUST) {
                obj?.let {
                    val tipMsg = getSpannableString(ADVANCE_DELEGATE_TIP2, 14, 20)
                    view.onShowAdvanceDelegate(ADVANCE_DELEGATE_TIP_TYPE2, tipMsg)
                }
            }
        }

    }

    /**
     * 湘财支持预约委托 且在预约委托时间段 判断条件是周一到周六 18点到24点
     *
     * @return
     */
    private fun advanceTrustTime(): Boolean {
        if (TradeConstantData.selectedItem.bookingOrderSupport == 1) {
            val current = TimeHelper.getInstance().getDate("yyyy-MM-dd HH:mm:ss")
            val df = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault())
            try {
                val calendar = Calendar.getInstance()
                calendar.time = df.parse(current)
                if (calendar[Calendar.DAY_OF_WEEK] != Calendar.SUNDAY) {
                    val hour = calendar[Calendar.HOUR_OF_DAY]
                    return hour in 18 until 24
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        return false
    }

    private fun getSpannableString(tips: String, start: Int, end: Int): SpannableString {
        val spannableString = SpannableString(tips)
        val span = ForegroundColorSpan(Color.RED)
        spannableString.setSpan(span, start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        return spannableString
    }

    /**
     * 请求持仓列表数据
     *
     * @param ggtType 港股通
     */
    fun reqHoldListData(@GggType ggtType: Int = NON_GGT_TYPE) {
        this.ggtType = ggtType
        getView()?.showLoading()
        manager.queryHoldList(ggtType)
    }

    /**
     * 请求最大可买量
     *
     * @param windCode
     * @param orderPrice
     * @param delegateType
     * @param ggtType
     * @param lotSize
     */
    fun queryMaxVolume(windCode: String? = businessModel.windCode, orderPrice: Double = businessModel.orderPrice,
                       delegateType: TradeDelegateType = businessModel.delegateType,
                       @GggType ggtType: Int = NON_GGT_TYPE, lotSize: Int = 100) {
        val userCode = this.userCode
        //买的时候，如果委托价格为0不发送最大可买量请求，卖的时候，即使为0也发送请求
        val isSendReq = if (bsFlag == BUY) orderPrice != 0.0 else true
        if (!windCode.isNullOrEmpty() && !userCode.isNullOrEmpty() && isSendReq) {
            manager.queryMaxVolume(windCode, bsFlag, orderPrice,
                    lotSize.toFloat(), userCode, ggtType, delegateType.value)
        }
    }

    /**
     * 买卖操作
     */
    fun bsDelegateOperate() {
        val view = getView() ?: return
        if (!NetManager.isNetworkAvailable()) {
            view.errorMsg(TIP_TOAST, Language.NETWORK_OFF)
            return
        }
        // 按钮买入卖出
        if (bsFlag == BUY) {
            TradeConstantData.submitUserActionEx(
                    TradeUserAction.BUTTON_TRADE_BUY,
                    businessModel.windCode)
        } else {
            TradeConstantData.submitUserActionEx(
                    TradeUserAction.BUTTON_TRADE_SELL,
                    businessModel.windCode)
        }
        val tradeNet = TradeAccountManager.getInstance().tradeAccount
        if (tradeNet == null) {
            view.errorMsg(TIP_TOAST, "网络异常，请重新登录")
            return
        }

        if (businessModel.windCode.isNullOrEmpty() || businessModel.stockCode.isNullOrEmpty()) {
            view.errorMsg(TIP_ALERT, "您没有选择股票")
            return
        }
        if (businessModel.orderPrice == 0.00) {
            view.errorMsg(TIP_ALERT, "您没有选择价格")
            return
        }
        if (businessModel.orderAmount <= 0) {
            view.errorMsg(TIP_ALERT, "请输入合法成交数量")
            return
        }
        val marketType = TradeConstantData.getMarketType(businessModel.windCode)
        if (TradeConstantData.TYPE_HKSTOCK == TradeConstantData.selectedItem?.type
                && marketType != NSDPROCAPI.MARKET_HK) {
            view.errorMsg(TIP_ALERT, "非港股股票!")
            return
        } else if (TradeConstantData.TYPE_HSSTOCK == TradeConstantData.selectedItem?.type
                && marketType == -1) {
            view.errorMsg(TIP_ALERT, "非沪深股票!")
            return
        }
        val title = if (bsFlag == BUY) "买入确认" else "卖出确认"
        val governmentBond = businessModel.windCodeType == TradeWindCodeType.GOVERNMENT_BOND_SZ
                || businessModel.windCodeType == TradeWindCodeType.GOVERNMENT_BOND_SH
        val isSTIBCode = businessModel.windCodeType == TradeWindCodeType.STIB_CODE
        val tip = when {
            isSTIBCode && businessModel.delegateType == TradeDelegateType.STIB_TYPE
                    && businessModel.orderAmount > 1000000 -> TIP_HIGHTECH_AFTERMARKET
            isSTIBCode && businessModel.delegateType != TradeDelegateType.LIMIT_ORDER
                    && businessModel.orderAmount > 50000 -> TIP_HIGHTECH_MARKETORDER
            else -> ""
        }
        val key = when {
            governmentBond -> "证券\n价格\n数量\n金额"
            isSTIBCode && businessModel.delegateType != TradeDelegateType.STIB_TYPE
                    && businessModel.delegateType != TradeDelegateType.LIMIT_ORDER -> "证券名称\n证券代码\n保护限价\n委托数量\n委托金额\n委托类型"
            else -> "证券名称\n证券代码\n委托价格\n委托数量\n委托金额\n委托类型"
        }
        val moneyStr = CommonFunc.fixText(businessModel.orderAmount * businessModel.orderPrice, 2)
        val valueStr = if (governmentBond) "${businessModel.stockName}（${businessModel.stockCode}）" +
                "\n100元/张\n${businessModel.orderAmount.addThousandth()}张\n${(businessModel.orderAmount * 100).addThousandth()}元" else
            "${businessModel.stockName}\n${businessModel.stockCode}\n${(businessModel.orderPrice)}" +
                    "\n${businessModel.orderAmount}\n${moneyStr}\n${businessModel.delegateType.delegateName}"
        view.onShowTradeConfirm(title, tip, key, valueStr)
    }

    /**
     * 买卖委托请求
     *
     * @param ggtType
     * @param hkMark
     */
    @JvmOverloads
    fun bsDelegate(@GggType ggtType: Int = NON_GGT_TYPE, hkMark: Int = 0) {
        val windCode = businessModel.windCode
        val price = businessModel.orderPrice
        val orderAmount = businessModel.orderAmount
        val delegateTypeValue = businessModel.delegateType.value
        val userCode = this.userCode
        if (!windCode.isNullOrEmpty() && !userCode.isNullOrEmpty()) {
            getView()?.showLoading()
            manager.bsDelegate(windCode, bsFlag, price,
                    orderAmount, delegateTypeValue, hkMark.toByte(), userCode, ggtType)
        }
    }

    /**
     * 获取委托方式列表
     *
     * @return
     */
    fun getDelegateTypeData(): List<TradeDelegateType>? {
        return when (businessModel.delegateDataType) {
            TradeDelegateDataType.DELEGATE_TYPE_SH -> DELEGATE_TYPE_SH
            TradeDelegateDataType.DELEGATE_TYPE_SZ -> DELEGATE_TYPE_SZ
            TradeDelegateDataType.DELEGATE_TYPE_STIB -> DELEGATE_TYPE_STIB
            TradeDelegateDataType.DELEGATE_TYPE_STIB_RZRQ -> DELEGATE_TYPE_STIB_RZRQ
            TradeDelegateDataType.DELEGATE_TYPE_DEF -> null
        }
    }

    /**
     * 设置当前选中股票信息
     *
     * @param windCode
     * @param stockName
     */
    fun setWindCodeInfo(windCode: String?, stockName: String?, userCode: String? = null) {
        val view = getView()
        businessModel.resetFieldValue()
        windCode?.let {
            EventBusManager.getInstance().post(StockTradeMsgEvent(
                    StockTradeMsgEvent.USERCODE_FRAGMENT_ACTIVITY, userCode, it))
            businessModel.windCode = it
            businessModel.stockCode = it.takeWhile { s -> s != '.' }
            businessModel.stockName = stockName ?: WindCodeType.getStockInfo(it).shortName
            businessModel.pricePrecision = CommonFunc.getRadixPointFactor(it)
            val isSTIBCode = SecType2.isHighTech(WindCodeType.getWindSecType(it))
            val isGovernmentBondSH = COLLATERAL_CODES.any { item ->
                item == businessModel.windCode && item.endsWith("SH")
            }
            val isGovernmentBondSZ = COLLATERAL_CODES.any { item ->
                item == businessModel.windCode && item.endsWith("SZ")
            }
            val isBond = SecType2.isDebt(WindCodeType.getWindSecType(it))
            businessModel.windCodeType = when {
                isSTIBCode -> TradeWindCodeType.STIB_CODE
                isGovernmentBondSH -> TradeWindCodeType.GOVERNMENT_BOND_SH
                isGovernmentBondSZ -> TradeWindCodeType.GOVERNMENT_BOND_SZ
                isBond -> TradeWindCodeType.BOND
                else -> TradeWindCodeType.A_STOCK
            }
            businessModel.lotSize = when {
                isSTIBCode -> 1
                isGovernmentBondSH -> 1000
                isGovernmentBondSZ || isBond -> 10
                else -> 100
            }
            businessModel.maxVolumeUnit = when {
                isSTIBCode && it.startsWith("689") -> "份"
                isGovernmentBondSH || isGovernmentBondSZ || isBond -> "张"
                else -> "股"
            }
            businessModel.delegateDataType = when {
                isSTIBCode && !isFlag4RZRQ() -> TradeDelegateDataType.DELEGATE_TYPE_STIB
                isSTIBCode -> TradeDelegateDataType.DELEGATE_TYPE_STIB_RZRQ
                isGovernmentBondSH || isGovernmentBondSZ -> TradeDelegateDataType.DELEGATE_TYPE_DEF
                it.endsWith(".SH") -> TradeDelegateDataType.DELEGATE_TYPE_SH
                it.endsWith(".SZ") -> TradeDelegateDataType.DELEGATE_TYPE_SZ
                else -> TradeDelegateDataType.DELEGATE_TYPE_DEF
            }
            if (view != null) {
                view.onUpdateDelegateType(businessModel.delegateType.delegateName, "价 格", clickable = true,
                        isClearPrice = false, isChangeable = !isBond && !isGovernmentBondSH && !isGovernmentBondSZ)
                view.onUpdateOrderPrice("")
                view.onUpdateOrderAmount("")
                view.onUpdateMaxVolume(PLACEHOLDER_MAX_VOLUME, null)
                view.onUpdateHighLimitPrice(PLACEHOLDER_STR)
                view.onUpdateLowLimitPrice(PLACEHOLDER_STR)
                view.onUpdateCodeName(it, businessModel.stockCode!!, stockName ?: "")
            }
        }
    }

    /**
     * 提供设置股票名称方法， 解决外部传入windCode，没有stockName问题
     *
     * @param stockName
     */
    fun setStockName(stockName: String) {
        if (!businessModel.windCode.isNullOrEmpty() && businessModel.stockName.isNullOrEmpty()) {
            businessModel.stockName = stockName
        }
    }

    /**
     * 设置当前选中股票价格信息
     *
     * @param highLimitPrice
     * @param lowLimitPrice
     * @param currentPrice
     */
    fun setStockPriceInfo(highLimitPrice: Float, lowLimitPrice: Float, currentPrice: String?) {
        val view = getView()
        if (!businessModel.windCode.isNullOrEmpty()) {
            val curPrice = currentPrice?.let {
                try {
                    it.toDouble()
                } catch (e: Exception) {
                    0.00
                }
            } ?: 0.00
            businessModel.currentPrice = curPrice
            businessModel.highLimitPrice = highLimitPrice
            businessModel.lowLimitPrice = lowLimitPrice
            businessModel.pricePrecision = CommonFunc.getRadixPointFactor(businessModel.windCode)
            setNewOrderPrice(curPrice)
            view?.let {
                it.onUpdateHighLimitPrice(highLimitPrice.toString())
                it.onUpdateLowLimitPrice(lowLimitPrice.toString())
            }
        }
    }

    /**
     * 价格+
     *
     * @param currentPriceStr
     */
    fun addPrice(currentPriceStr: String) {
        if (businessModel.windCode.isNullOrEmpty() || businessModel.highLimitPrice == -1.0f ||
                !businessModel.priceChangeable) {
            return
        }
        val lengthPrecision = businessModel.pricePrecision
        val inputValue = try {
            currentPriceStr.toDouble()
        } catch (e: Exception) {
            0.00
        }
        val value = when (lengthPrecision) {
            1 -> if (businessModel.highLimitPrice > 0.0f)
                min(inputValue + 0.1, businessModel.highLimitPrice.toDouble()) else
                inputValue + 0.1
            3 -> if (businessModel.highLimitPrice > 0.0f)
                min(inputValue + 0.001, businessModel.highLimitPrice.toDouble()) else
                inputValue + 0.001
            else -> if (businessModel.highLimitPrice > 0.0f)
                min(inputValue + 0.01, businessModel.highLimitPrice.toDouble()) else
                inputValue + 0.01
        }
        val numberFormatValue = numberFormat(value, lengthPrecision)
        businessModel.orderPrice = numberFormatValue
        getView()?.onUpdateOrderPrice(numberFormatValue.priceDecimalFormat(businessModel.pricePrecision))
    }

    /**
     * 价格-
     *
     * @param currentPriceStr
     */
    fun minusPrice(currentPriceStr: String) {
        if (businessModel.windCode.isNullOrEmpty() || businessModel.lowLimitPrice == -1.0f ||
                !businessModel.priceChangeable) {
            return
        }
        val lengthPrecision = businessModel.pricePrecision
        val inputValue = try {
            currentPriceStr.toDouble()
        } catch (e: Exception) {
            0.00
        }
        val value = when (lengthPrecision) {
            1 -> if (businessModel.lowLimitPrice > 0.0f)
                max(inputValue - 0.1, businessModel.lowLimitPrice.toDouble()) else
                max(inputValue - 0.1, 0.0)
            3 -> if (businessModel.lowLimitPrice > 0.0f)
                max(inputValue - 0.001, businessModel.lowLimitPrice.toDouble()) else
                max(inputValue - 0.001, 0.000)
            else -> if (businessModel.lowLimitPrice > 0.0f)
                max(inputValue - 0.01, businessModel.lowLimitPrice.toDouble()) else
                max(inputValue - 0.01, 0.00)
        }
        val numberFormatValue = numberFormat(value, lengthPrecision)
        businessModel.orderPrice = numberFormatValue
        getView()?.onUpdateOrderPrice(numberFormatValue.priceDecimalFormat(businessModel.pricePrecision))
    }

    /**
     * 数量+
     *
     * @param currentAmount
     */
    fun addAmount(currentAmount: String) {
        if (businessModel.windCode.isNullOrEmpty()) {
            return
        }
        val inputValue = try {
            currentAmount.toInt()
        } catch (e: Exception) {
            0
        }
        //是否是科创板
        val stib = businessModel.windCodeType == TradeWindCodeType.STIB_CODE
        val addValue = inputValue + businessModel.lotSize
        val value = if (stib && addValue < businessModel.maxVolume && businessModel.maxVolume < 200)
            businessModel.maxVolume else if (stib && addValue < 200) 200 else addValue
        businessModel.orderAmount = value
        getView()?.onUpdateOrderAmount(value.toString())
    }

    /**
     * 数量-
     *
     * @param currentAmount
     */
    fun minusAmount(currentAmount: String) {
        if (businessModel.windCode.isNullOrEmpty()) {
            return
        }
        val inputValue = try {
            currentAmount.toInt()
        } catch (e: Exception) {
            0
        }
        //是否是科创板
        val stib = businessModel.windCodeType == TradeWindCodeType.STIB_CODE
        val minusValue = inputValue - businessModel.lotSize
        val value = if ((stib && minusValue < 200) || minusValue < 0) 0 else minusValue
        businessModel.orderAmount = value
        getView()?.onUpdateOrderAmount(value.toString())
    }

    /**
     * 设置数量
     *
     * @param currentAmount
     */
    fun setAmount(currentAmount: String) {
        if (businessModel.windCode.isNullOrEmpty()) {
            return
        }
        val inputValue = try {
            currentAmount.toInt()
        } catch (e: Exception) {
            0
        }
        businessModel.orderAmount = inputValue
    }

    /**
     * 设置委托价格
     *
     * @param price
     * @param priceChangeable
     */
    fun setNewOrderPrice(price: Double, priceChangeable: Boolean = businessModel.priceChangeable,
                         updateView: Boolean = true) {
        if (!businessModel.windCode.isNullOrEmpty() && priceChangeable) {
            val formatValue = numberFormat(price, businessModel.pricePrecision)
            businessModel.orderPrice = formatValue
            if (updateView) {
                getView()?.onUpdateOrderPrice(formatValue.priceDecimalFormat(businessModel.pricePrecision))
            } else {
                queryMaxVolume(businessModel.windCode!!, formatValue, businessModel.delegateType,
                        lotSize = businessModel.lotSize)
            }
        }
    }

    /**
     * 处理价格输入框为空的情况
     */
    fun setBlankPrice() {
        if (!businessModel.windCode.isNullOrEmpty()) {
            val price = if (businessModel.windCodeType == TradeWindCodeType.STIB_CODE
                    || businessModel.priceChangeable) 0.0 else businessModel.currentPrice
            val formatValue = numberFormat(price, businessModel.pricePrecision)
            if (businessModel.orderPrice != formatValue) {
                businessModel.orderPrice = formatValue
                val queryMaxVolumePrice = if (businessModel.windCodeType == TradeWindCodeType.STIB_CODE
                        && businessModel.delegateType != TradeDelegateType.LIMIT_ORDER
                        && businessModel.delegateType != TradeDelegateType.STIB_TYPE) businessModel.currentPrice else
                    formatValue
                queryMaxVolume(businessModel.windCode!!, queryMaxVolumePrice, businessModel.delegateType,
                        lotSize = businessModel.lotSize)
            }
        }
    }

    /**
     * 涨停价设为委托价格
     */
    fun setHighLimitPriceToOrder() {
        if (!businessModel.windCode.isNullOrEmpty() && businessModel.highLimitPrice != -1.0f
                && (businessModel.delegateType == TradeDelegateType.LIMIT_ORDER ||
                        businessModel.delegateType == TradeDelegateType.STIB_TYPE)) {
            val formatValue = numberFormat(businessModel.highLimitPrice.toDouble(), businessModel.pricePrecision)
            businessModel.orderPrice = formatValue
            getView()?.onUpdateOrderPrice(formatValue.priceDecimalFormat(businessModel.pricePrecision))
        }
    }

    /**
     * 跌停价设为委托价格
     */
    fun setLowLimitPriceToOrder() {
        if (!businessModel.windCode.isNullOrEmpty() && businessModel.lowLimitPrice != -1.0f
                && (businessModel.delegateType == TradeDelegateType.LIMIT_ORDER ||
                        businessModel.delegateType == TradeDelegateType.STIB_TYPE)) {
            val formatValue = numberFormat(businessModel.lowLimitPrice.toDouble(), businessModel.pricePrecision)
            businessModel.orderPrice = formatValue
            getView()?.onUpdateOrderPrice(formatValue.priceDecimalFormat(businessModel.pricePrecision))
        }
    }

    /**
     * 最大可买量设为委托数量
     */
    fun setMaxAmountToOrder() {
        if (!businessModel.windCode.isNullOrEmpty()) {
            businessModel.orderAmount = businessModel.maxVolume
            getView()?.onUpdateOrderAmount(businessModel.orderAmount.toString())
        }
    }

    /**
     * 设置最大可买/卖量，由外部调用，不在请求回调里直接设置，保证businessModel
     * 只在同一个线程里操作
     *
     * @param maxVolume 最大值
     */
    fun setMaxVolume(maxVolume: Int) {
        if (!businessModel.windCode.isNullOrEmpty()) {
            businessModel.maxVolume = maxVolume
        }
    }

    /**
     * 计算数量（全仓、半仓、1/3、1/4）
     *
     * @param rate 比例
     */
    fun calculateAmount(rate: Double) {
        if (!businessModel.windCode.isNullOrEmpty()) {
            val calculateValue = businessModel.maxVolume * rate
            val amountValue = if (rate == 1.0) {
                //全仓，直接用最大值，不做处理
                calculateValue.toInt()
            } else if (businessModel.windCodeType == TradeWindCodeType.STIB_CODE) {
                if (calculateValue < 200 && businessModel.maxVolume < 200) businessModel.maxVolume
                else if (calculateValue < 200) 200
                else calculateValue.toInt()
            } else {
                (calculateValue / businessModel.lotSize).toInt() * businessModel.lotSize
            }
            businessModel.orderAmount = amountValue
            getView()?.onUpdateOrderAmount(amountValue.toString())
        }
    }

    /**
     * 格式化数字
     *
     * @param number
     * @param count
     * @return
     */
    private fun numberFormat(number: Double, count: Int): Double {
        val bigDecimal = BigDecimal(number.toString())
        return bigDecimal.setScale(count, BigDecimal.ROUND_HALF_UP).toDouble()
    }

    /**
     * 注册数据请求监听
     */
    fun registerListener() {
        manager.registerListener(callback)
    }

    /**
     * 移除数据请求监听
     */
    fun unregisterListener() {
        manager.unregisterListener(callback)
    }

    /**
     * 转换成持仓列表item
     *
     * @param it 交易后台返回持仓列表item 类型
     * @return
     */
    private fun transformToHoldItem(it: TagAns_Fun1023_1): JSHoldStockValue {
        val holdValueItem = JSHoldStockValue()
        holdValueItem.moneyType = it.chMoneyType.toInt()
        holdValueItem.marketType = it.chMarketType.toInt()
        val parseWindCode = TradeHelper.parseWindCode(it.chStockCode,
                it.chMarketType, it.iResv1)
        holdValueItem.windCode = parseWindCode
        holdValueItem.stockName = it.getString(it.chStockName)
        holdValueItem.nStockQty = it.nStockQty.toString()
        holdValueItem.nCostPrice = CommonFunc.doubleFormat(
                it.nCostPrice * 0.0001, 3)
        holdValueItem.dIncome = it.dIncome.toString()
        holdValueItem.currencyMark = TradeHelper.getMoneyMark(it.chMoneyType.toInt())
        holdValueItem.dMktVal = it.dMktVal.toString()
        holdValueItem.nStockAvl = it.nStockAvl.toString()
        holdValueItem.nStockSell = it.nStockAvl
        holdValueItem.chUserCode = it.getString(it.chUserCode)
        val newPrice = if (it.nNewPrice <= 0 && it.nStockQty != 0)
            it.dMktVal / it.nStockQty else it.nNewPrice * 0.0001
        holdValueItem.nNewPrice = CommonFunc.doubleFormat(newPrice,
                CommonFunc.getRadixPointFactor(holdValueItem.windCode))
        holdValueItem.chUserCode = it.getString(it.chUserCode)
        return holdValueItem
    }

    /**
     * 是否融资融券
     *
     * @return
     */
    private fun isFlag4RZRQ(): Boolean {
        return TradeAccountManager.getInstance().tradeAccount?.getShareTradeAccountData()
                ?.flag4RZRQ ?: false
    }

    override fun detachView() {
        super.detachView()
        manager.unregisterListener(callback)
        TradeAccountManager.getInstance().tradeAccount?.setTradeAdventrustListener(null)
    }

    /**
     * 选择委托方式后需要设置的逻辑
     *
     * @param delegateType
     */
    private fun selectDelegateType(delegateType: TradeDelegateType) {
        val view = getView() ?: return
        if (!businessModel.windCode.isNullOrEmpty()) {
            when (delegateType) {
                TradeDelegateType.LIMIT_ORDER,
                TradeDelegateType.STIB_TYPE -> {
                    businessModel.priceChangeable = true
                    view.onUpdateDelegateType(businessModel.delegateType.delegateName, "价 格",
                            clickable = true, isClearPrice = false)
                    setNewOrderPrice(businessModel.currentPrice, true)
                }
                TradeDelegateType.BEST_PRICE_TO_OTHER,
                TradeDelegateType.BEST_PRICE_TO_SELF,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_LIMIT_ORDER,
                TradeDelegateType.DEAL_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.ALL_DEAL_OR_CANCEL -> {
                    val isStib = businessModel.delegateDataType == TradeDelegateDataType.DELEGATE_TYPE_STIB ||
                            businessModel.delegateDataType == TradeDelegateDataType.DELEGATE_TYPE_STIB_RZRQ
                    val hint = if (isStib) "输入保护限价" else "市价委托"
                    businessModel.priceChangeable = isStib
                    view.onUpdateDelegateType(businessModel.delegateType.delegateName, hint,
                            clickable = isStib, isClearPrice = true)
                }
            }
        }
    }

    /**
     * 价格是否可变
     *
     * @return
     */
    fun priceChangeable(): Boolean {
        return businessModel.priceChangeable
    }

    /**
     * 预约委托
     */
    fun advanceDelegate() {
        if (!businessModel.windCode.isNullOrEmpty()) {
            val marketType = TradeConstantData.getMarketType(businessModel.windCode).toByte()
            val accountType = TradeConstantData.getAccountType(businessModel.windCode, marketType)
            val userCode = this.userCode
            val stockCode = businessModel.stockCode
            val amount = businessModel.orderAmount
            val price = businessModel.orderPrice
            if (!userCode.isNullOrEmpty() && !stockCode.isNullOrEmpty()) {
                getView()?.showLoading()
                manager.bsAdventrust(marketType, userCode, stockCode, amount,
                        price, bsFlag, 0.toByte(), accountType)
            }
        }
    }

    companion object {
        val DELEGATE_TYPE_SH = listOf(TradeDelegateType.LIMIT_ORDER,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_LIMIT_ORDER)
        val DELEGATE_TYPE_STIB = listOf(TradeDelegateType.LIMIT_ORDER,
                TradeDelegateType.BEST_PRICE_TO_OTHER,
                TradeDelegateType.BEST_PRICE_TO_SELF,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_LIMIT_ORDER,
                TradeDelegateType.STIB_TYPE)
        val DELEGATE_TYPE_STIB_RZRQ = listOf(TradeDelegateType.LIMIT_ORDER,
                TradeDelegateType.BEST_PRICE_TO_OTHER,
                TradeDelegateType.BEST_PRICE_TO_SELF,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_LIMIT_ORDER)
        val DELEGATE_TYPE_SZ = listOf(TradeDelegateType.LIMIT_ORDER,
                TradeDelegateType.BEST_PRICE_TO_OTHER,
                TradeDelegateType.BEST_PRICE_TO_SELF,
                TradeDelegateType.DEAL_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.FIVE_IMMEDIATELY_SURPLUS_CANCEL,
                TradeDelegateType.ALL_DEAL_OR_CANCEL)

        val COLLATERAL_CODES = listOf("131810.SZ", "131811.SZ", "131800.SZ", "131809.SZ",
                "131801.SZ", "131802.SZ", "131803.SZ", "131805.SZ", "131806.SZ", "204001.SH",
                "204002.SH", "204003.SH", "204004.SH", "204007.SH", "204014.SH", "204028.SH",
                "204091.SH", "204182.SH")

        const val BUY = 'B'.toByte()
        const val SELL = 'S'.toByte()

        const val PLACEHOLDER_STR = "--"
        const val PLACEHOLDER_MAX_VOLUME = -1

        const val TIP_TOAST = 1
        const val TIP_ALERT = 2

        const val ADVANCE_DELEGATE_TIP_TYPE1 = 0
        const val ADVANCE_DELEGATE_TIP_TYPE2 = 1

        private const val TIP_HIGHTECH_AFTERMARKET = "提示：盘后固定价格委托数量超过上限100万股/笔，将委托失败"
        private const val TIP_HIGHTECH_MARKETORDER = "提示：市价委托数量超过上限5万股/笔，将委托失败"
        private const val MMWT_ERROR1 = "当前时间不允许委托"
        private const val MMWT_ERROR2 = "该功能禁止在目前状态下运行"
        private const val ADVANCE_DELEGATE_TIP1 = "您正在申请的隔日预约委托指令，发出后指令将在下一个交易日自动生效。"
        private const val ADVANCE_DELEGATE_TIP2 = "您申请的隔日预约委托指令将在下一个交易日自动生效。"
    }
}