package wind.android.bussiness.trade.abstocktrade.presenter

import cn.com.hh.trade.data.NSDPROCAPI
import cn.com.hh.trade.data.TagAns_Fun1025
import cn.com.hh.trade.data.TagReq_Fun1011
import wind.android.bussiness.trade.abstocktrade.GggType
import wind.android.bussiness.trade.abstocktrade.NON_GGT_TYPE
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbCancelResultListener
import wind.android.bussiness.trade.abstocktrade.transferMode1096
import wind.android.bussiness.trade.abstocktrade.view.IAStockCancelView

/**
 * # **********************************************************************************************
 * # ClassName:      AStockCanelPresenter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/21    17:19
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class AStockCancelPresenter(cancelView: IAStockCancelView<TagAns_Fun1025>) : BaseTradePresenter<IAStockCancelView<TagAns_Fun1025>>(cancelView) {
    /**
     * 数据请求model层
     */
    private val manager: AbTradeManager<Array<TagAns_Fun1025>?> by lazy { AbTradeManager<Array<TagAns_Fun1025>?>() }
    /**
     * 港股通类型标识
     */
    @GggType
    private var ggtType: Int = NON_GGT_TYPE
    /**
     * 请求回调
     */
    private val callback: OnAbCancelResultListener<Array<TagAns_Fun1025>?> by lazy {
        object : OnAbCancelResultListener<Array<TagAns_Fun1025>?> {
            override fun onCancelEntrust(obj: Any?) {
                getView()?.hideLoading()
                obj?.let { reqCancelableData(ggtType) }
            }

            override fun onAdventrustBack(functionID: Int, obj: Any?) {
                if (functionID == NSDPROCAPI.FUNID_ADVENTRUST_WITHDRAW) {
                    getView()?.hideLoading()
                    obj?.let { reqCancelableData(ggtType) }
                } else if (functionID == NSDPROCAPI.FUNID_GET_ADVENTRUST_WITHDRAW && obj is Array<*>) {
                    val view = getView()
                    view?.let {
                        it.onAddUpdateList(obj.map { item -> transferMode1096(item) }.toList())
                    }
                }
            }

            override fun onCallback(data: Array<TagAns_Fun1025>?) {
                val view = getView()
                view?.let {
                    it.hideLoading()
                    it.onUpdateList(data?.toList())
                    manager.queryAdventrustCancel()
                }
            }

            override fun onError(error: Int, message: String?) {
                val view = getView()
                view?.let {
                    it.hideLoading()
                    it.errorMsg(error, message)
                }
            }
        }
    }

    /**
     * 请求可撤单列表数据
     *
     * @param ggtType 港股通
     */
    fun reqCancelableData(@GggType ggtType: Int = NON_GGT_TYPE) {
        this.ggtType = ggtType
        getView()?.showLoading()
        manager.queryCancel(ggtType)
    }

    /**
     * 撤单（单个）
     *
     * @param itemModel 列表item类型
     * @param ggtType 港股通
     */
    fun cancelOrder(itemModel: TagAns_Fun1025) {
        getView()?.showLoading()
        manager.cancelEntrust(itemModel, ggtType)
    }

    /**
     * 批量撤单
     *
     * @param list List<TagAns_Fun1025>
     */
    fun cancelOrderAll(list: List<TagAns_Fun1025>) {
        val cancelListReq =
                list.filter { it.iResv2 != 1096.toLong() }.map {
                    val reqFun1011 = TagReq_Fun1011()
                    reqFun1011.chStockCode = it.chStockCode
                    reqFun1011.setChOrderNumber(it.chOrderNumber)
                    reqFun1011.setChOrderGroup(it.chOrderGroup)
                    reqFun1011.nOrderDate = it.nOrderDate
                    reqFun1011.nOrderTime = it.nOrderTime
                    reqFun1011.chMarketType = it.chMarketType
                    reqFun1011.chBSFlag = it.chBSFlag
                    reqFun1011.chExtFlag2 = it.chExtFlag2
                    reqFun1011.nOrderVolume = it.nOrderVolume
                    reqFun1011.nOrderPrice = it.nOrderPrice
                    reqFun1011.chExtFlag1 = it.chExtFlag1
                    reqFun1011.setChUserCode(it.chUserCode)
                    reqFun1011
                }
        getView()?.showLoading()
        manager.cancelEntrusts(cancelListReq)
        list.filter { it.iResv2 == 1096.toLong() }.forEach { manager.cancelAdventrust(it.chOrderNumber) }
    }

    fun registerListener() {
        manager.registerListener(callback)
    }

    fun unregisterListener() {
        manager.unregisterListener(callback)
    }

    override fun detachView() {
        super.detachView()
        manager.unregisterListener(callback)
    }
}