package wind.android.bussiness.trade.abstocktrade.presenter

import business.sky.SkyStockSearch
import database.orm.CommDao
import net.datamodel.speed.SecType2
import net.datamodel.speed.WindCodeType
import wind.android.bussiness.optionalstock.model.SearchRecord
import wind.android.bussiness.optionalstock.model.SearchRecord.FIELD_WIND_CODE_NAME
import wind.android.bussiness.optionalstock.model.SearchRecord.PROPERTY_RECORD_TYPE
import wind.android.bussiness.trade.abstocktrade.view.ITradeSearchView

/**
 * # **********************************************************************************************
 * # ClassName:      TradeSearchPresenter
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/26    19:45
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class TradeSearchPresenter(searchView: ITradeSearchView) : BaseTradePresenter<ITradeSearchView>(searchView) {
    @Volatile
    private var reqId = 0
    /**
     * 请求页码
     */
    @Volatile
    private var pageNo = 0
    /**
     * 缓存关键词，用于下一页搜索
     */
    private var keyWordCache: String = ""
    /**
     * 缓存搜索结果列表
     */
    private val cacheSearchList: ArrayList<SearchRecord> by lazy { ArrayList<SearchRecord>() }

    /**
     * 查询历史记录
     */
    fun queryHistory() {
        var historyList: MutableList<SearchRecord?>? = null
        try {
            historyList = CommDao.getInstance().queryWhere(SearchRecord::class.java)
                    .orderBy("_id", false)
                    .where()
                    .eq(PROPERTY_RECORD_TYPE, SearchRecord.TYPE_STOCK).or()
                    .eq(PROPERTY_RECORD_TYPE, SearchRecord.TYPE_FUND).query() as MutableList<SearchRecord?>?
        } catch (e: Exception) {
            e.printStackTrace()
        }
        getView()?.updateList(historyList, true)
    }

    /**
     * 搜索
     *
     * @param keyWord 关键词
     * @param nextPage 是否下一页
     */
    @JvmOverloads
    fun search(keyWord: String = keyWordCache, nextPage: Boolean = false) {
        if (keyWordCache != keyWord) {
            keyWordCache = keyWord
        }
        if (keyWord.isEmpty() || keyWord.isBlank()) {
            queryHistory()
            return
        }
        pageNo = if (nextPage) pageNo + 1 else 0
        reqId = SkyStockSearch.searchStockForATrade(pageNo, 20, keyWord.trim()) {
            if (reqId == it?.SerialNum && it.data?.isNullOrEmpty() != true) {
                val result = it.data.get(0) as? Array<String>
                if (pageNo == 0) {
                    cacheSearchList.clear()
                }
                val searchResultList = dealSearchResult(result)
                searchResultList?.let { list -> cacheSearchList.addAll(list) }
                getView()?.updateList(cacheSearchList, false)
            }
        }
        if (reqId == -1) {
            getView()?.errorMsg(1, "")
        }
    }

    /**
     * 保存历史记录
     *
     * @param item
     */
    fun saveRecord(item: SearchRecord) {
        if (!item.windcode.isNullOrEmpty() && !item.stockType.isNullOrEmpty()
                && !item.stockname.isNullOrEmpty()) {
            val type = try {
                item.stockType.toInt()
            } catch (e: NumberFormatException) {
                e.printStackTrace()
                SecType2.Unknown.toInt()
            }
            if (type != SecType2.Unknown.toInt() && type != SecType2.OLDNAME.toInt()) {
                WindCodeType.putCodeSectype(item.windcode, type, item.stockname)
            }
            val map = mapOf<String, String>(FIELD_WIND_CODE_NAME to item.windcode,
                    PROPERTY_RECORD_TYPE to item.recordType)
            val queryList = CommDao.getInstance().query(map, SearchRecord::class.java)
            queryList?.forEach { queryItem -> CommDao.getInstance().delete(queryItem, SearchRecord::class.java) }
            CommDao.getInstance().createOrUpdateByKey(item, SearchRecord::class.java)
        }
    }

    /**
     * 解析搜索结果
     *
     * @param result
     * @return
     */
    private fun dealSearchResult(result: Array<String>?): List<SearchRecord>? {
        return result?.map {
            val searchItem = SearchRecord()
            val splitList = it.split(" ")
            if (splitList.size > 2) {
                searchItem.windcode = splitList[0]
                searchItem.stockname = splitList[1]
                val parserCodeType = parserCodeType(splitList)
                if (parserCodeType != SecType2.Unknown) {
                    searchItem.stockType = parserCodeType.toString()
                }
                searchItem.recordType = when (parserCodeType) {
                    20.toByte(), 18.toByte(), 19.toByte(), 51.toByte() -> SearchRecord.TYPE_FUND
                    else -> SearchRecord.TYPE_STOCK
                }
            }
            searchItem
        }
    }

    /**
     * 解析股票类型
     *
     * @param temp
     * @return
     */
    private fun parserCodeType(temp: List<String>): Byte {
        val typeStr = temp[temp.size - 1]
        return try {
            when {
                typeStr.startsWith("-") -> SecType2.OLDNAME
                typeStr.length < 5 -> 0
                else -> typeStr.substring(0, if (typeStr.length == 5) 1 else 2).toByte()
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SecType2.Unknown
        }
    }
}