package wind.android.bussiness.trade.abstocktrade.view

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Build
import android.text.SpannableString
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import android.view.View.OnClickListener
import android.view.WindowManager
import android.view.inputmethod.EditorInfo
import android.widget.*
import base.RootWindActivity
import cn.com.hh.trade.data.TagAns_Fun1040
import cn.com.hh.trade.data.TagAns_Fun1043
import kotlinx.android.synthetic.main.view_bank_to_security.view.*
import ui.keyboard.UILoginKeyBoard
import ui.screen.UIScreen
import util.ToastTool
import wind.android.R
import wind.android.bussiness.optionalstock.preopt.view.AlertDialogEditText
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbBankResultListener
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.constant.TradeUserAction
import wind.android.f5.view.element.LandscapeKlineAdapter
import java.lang.reflect.Method
import java.nio.charset.Charset

/**
 * # **********************************************************************************************
 * # ClassName:      BankToSecurityView
 * # Description:    银证转证券
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class BankToSecurityView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr), View.OnClickListener {

    private val mAbTradeManager = AbTradeManager<Any>()

    private var mBankInfo = ArrayList<TagAns_Fun1040>()
    private var mBankInfoName = ArrayList<String>()

    private var mSelectBank = 0

    private lateinit var mPopupWindow: PopupWindow
    private lateinit var landscapeKlineAdapter: LandscapeKlineAdapter

    private val mOnAbBankResultListener by lazy {
        object : OnAbBankResultListener<Any> {
            override fun onBankInfo(obj: Any?) {
                if (obj is Array<*> && obj.size > 0) {
                    for (item in obj) {
                        if (item is TagAns_Fun1040 && item.chMoneyType != '1'.toByte() && item.chMoneyType != '2'.toByte()) {
                            mBankInfo.add(item)
                            mBankInfoName.add(String(item.chBankName, Charset.forName("GBK")))
                        }
                    }
                }
                post {
                    if (mBankInfoName.size > 0 && mSelectBank >= 0 && mSelectBank < mBankInfoName.size) {
                        bankNameTv.text = mBankInfoName[mSelectBank]
                    }
                }
            }

            override fun onBankMoney(obj: Any?) {
                post {
                    if (obj is Array<*> && obj.size > 0 && obj[0] is TagAns_Fun1043) {
                        val ans1043 = obj[0] as TagAns_Fun1043
                        clickToQuery.text = ans1043.dFundEffect.addThousandth()
                    }
                }
            }

            override fun onBankTransfer(obj: Any?) {
                ToastTool.showToast(R.string.transfer_finish)
                post {
                    bankPasswordET.setText("")
                    transferMoneyET.setText("")
                }
            }

            override fun onCallback(data: Any?) {

            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    private val mOnClickListener by lazy {
        OnClickListener {
            val password = it.tag.toString()
            if (password.isEmpty()) {
                ToastTool.showToast(resources.getString(R.string.hit_input_bankPassword))
            } else {
                val ansFun1040 = mBankInfo[mSelectBank]
                // 银行余额查询
                mAbTradeManager.queryBankMoney(ansFun1040, password)
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_bank_to_security, this)
        clickToQuery.setOnClickListener(this)
        transferToSecurity.setOnClickListener(this)
        RMB.setOnClickListener(this)
        bankNameTv.setOnClickListener(this)

        val spannableString = SpannableString(String.format(resources.getString(R.string.transferTip), SecurityToBankView.TIME))
        val colorSpan = ForegroundColorSpan(Color.parseColor("#DEB887"))
        val index = spannableString.indexOf(SecurityToBankView.TIME)
        spannableString.setSpan(colorSpan, index, index + SecurityToBankView.TIME.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        transferTip.text = spannableString

        bankPasswordET.imeOptions = EditorInfo.IME_ACTION_DONE
        hideSysSoftInput(bankPasswordET)
        bankPasswordET.setOnTouchListener { _, event ->
            UILoginKeyBoard.getKeyBoard()
                    .showMyKeyBoard(bankPasswordET, null, true)
            bankPasswordET.onTouchEvent(event)
        }
        transferMoneyET.imeOptions = EditorInfo.IME_ACTION_DONE
        hideSysSoftInput(transferMoneyET)
        transferMoneyET.setOnTouchListener { _, event ->
            UILoginKeyBoard.getKeyBoard()
                    .showMyKeyBoard(transferMoneyET, null, true)
            transferMoneyET.onTouchEvent(event)
        }
    }

    /**
     * 隐藏系统键盘
     *
     * @param accountCodeText
     */
    private fun hideSysSoftInput(accountCodeText: EditText) {
        if (Build.VERSION.SDK_INT > 10) {
            (context as Activity).window.setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN)
            val cls = EditText::class.java
            var setSoftInputShownOnFocus: Method
            try {
                setSoftInputShownOnFocus = cls.getMethod(
                        "setSoftInputShownOnFocus", Boolean::class.javaPrimitiveType)
                setSoftInputShownOnFocus.isAccessible = true
                setSoftInputShownOnFocus.invoke(accountCodeText, false)
            } catch (e: Exception) {
                try {
                    setSoftInputShownOnFocus = cls.getMethod(
                            "setShowSoftInputOnFocus", Boolean::class.javaPrimitiveType)
                    setSoftInputShownOnFocus.isAccessible = true
                    setSoftInputShownOnFocus.invoke(accountCodeText, false)
                } catch (e1: Exception) {
                }
            }
        }
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbBankResultListener)
        // 银行信息查询
        mAbTradeManager.queryBankInfo()
    }

    override fun hide() {
        mBankInfo.clear()
        mBankInfoName.clear()
        mAbTradeManager.unregisterListener(mOnAbBankResultListener)
        UILoginKeyBoard.getKeyBoard().closeMyKeyBoard()
    }

    override fun destroyView() {

    }

    override fun onClick(v: View?) {
        if (mBankInfo.size == 0) {
            ToastTool.showToast(resources.getString(R.string.tip_no_bank))
        } else {
            when (v?.id) {
                R.id.bankNameTv,
                R.id.RMB -> {
                    if (!this::mPopupWindow.isInitialized) {
                        initPopupWindow()
                    }
                    if (mPopupWindow.isShowing) {
                        mPopupWindow.dismiss()
                    } else {
                        mPopupWindow.showAsDropDown(bankNameTv, 0, 0)
                    }
                }
                R.id.clickToQuery -> {
                    showPwdDialog()
                }
                R.id.transferToSecurity -> {
                    TradeConstantData.submitUserActionEx(TradeUserAction.BUTTON_TRADE_TURN)
                    val password = bankPasswordET.text.toString()
                    val money = transferMoneyET.text.toString()
                    when {
                        password.isEmpty() -> {
                            ToastTool.showToast(resources.getString(R.string.hit_input_bankPassword))
                        }
                        money.isEmpty() -> {
                            ToastTool.showToast(resources.getString(R.string.hit_input_bankMoney))
                        }
                        else -> {
                            val ansFun1040 = mBankInfo[mSelectBank]
                            // 银证转账
                            mAbTradeManager.bankTransfer(true, ansFun1040, password, money.toDouble())
                        }
                    }
                }
            }
        }
    }

    private fun showPwdDialog() {
        context?.let {
            if (context is RootWindActivity) {
                UILoginKeyBoard.getKeyBoard().closeMyKeyBoard()
//                BankToSecurityDialog().setOnClickListener(mOnClickListener)
//                        .show((context as RootWindActivity).supportFragmentManager, "BankToSecurityDialog")
                val dialogEditText = AlertDialogEditText(context).builder()
                dialogEditText.setTitle(resources.getString(R.string.hit_input_bankPassword))
                        .setHint(resources.getString(R.string.hit_input_bankPassword), Gravity.CENTER)
                        .setCancelable(true).showCustomKeyboard()
                        .setPositiveButton(resources.getString(R.string.confirm)) { str ->
                            dialogEditText.dismiss()
                            when {
                                mBankInfo.size == 0 || mSelectBank >= mBankInfo.size -> {
                                    ToastTool.showToast(resources.getString(R.string.tip_no_bank))
                                }
                                str.isEmpty() -> {
                                    ToastTool.showToast(resources.getString(R.string.hit_input_bankPassword))
                                }
                                else -> {
                                    val ansFun1040 = mBankInfo[mSelectBank]
                                    // 银行余额查询
                                    mAbTradeManager.queryBankMoney(ansFun1040, str)
                                }
                            }
                        }
                        .setNegativeButton(null, null)
                dialogEditText.show()
            }
        }
    }

    private fun initPopupWindow() {
        val listView = ListView(context)
        landscapeKlineAdapter = LandscapeKlineAdapter(false)
        landscapeKlineAdapter.setData(mBankInfoName)
        listView.adapter = landscapeKlineAdapter
        listView.dividerHeight = 0
        listView.cacheColorHint = 0
        listView.selector = ColorDrawable(Color.TRANSPARENT)
        listView.onItemClickListener = AdapterView.OnItemClickListener { _, _, position, _ ->
            mPopupWindow.dismiss()
            mSelectBank = position
            if (mBankInfoName.size > 0 && mSelectBank >= 0 && mSelectBank < mBankInfoName.size) {
                bankNameTv.text = mBankInfoName[mSelectBank]
            }
        }
        mPopupWindow = PopupWindow(listView, UIScreen.screenWidth / 2,
                RelativeLayout.LayoutParams.WRAP_CONTENT, true)
        mPopupWindow.isTouchable = true
        mPopupWindow.setTouchInterceptor { _, _ ->
            false
            // 这里如果返回true的话，touch事件将被拦截
            // 拦截后 PopupWindow的onTouchEvent不被调用，这样点击外部区域无法dismiss
        }
        // 如果不设置PopupWindow的背景，无论是点击外部区域还是Back键都无法dismiss弹框
        mPopupWindow.setBackgroundDrawable(resources.getDrawable(
                R.drawable.transparent))
        mPopupWindow.setOnDismissListener { }
    }
}
