package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import cn.com.hh.trade.data.TagAns_CommItem
import cn.com.hh.trade.data.TagAns_Fun1030
import cn.com.hh.trade.data.TagAns_Fun1031
import kotlinx.android.synthetic.main.view_trade_history_query.view.*
import net.util.Assist
import net.util.TimeHelper
import ui.view.timepicker.TimePopupWindow
import util.DateUtil
import util.StringUtils
import util.ToastTool
import wind.android.R
import wind.android.bussiness.strategy.view.MoneySwitchView
import wind.android.bussiness.trade.abstocktrade.adapter.AStockQueryHisDelegateAdapter
import wind.android.bussiness.trade.abstocktrade.adapter.AStockQueryTransactionAdapter
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbResultListener
import wind.android.bussiness.trade.abstocktrade.transferMode
import wind.android.bussiness.trade.model.TradeDelegationModel
import wind.android.bussiness.trade.model.TradeDoneModel
import java.text.SimpleDateFormat

/**
 * # **********************************************************************************************
 * # ClassName:      HistoryQueryView
 * # Description:    历史交易
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class HistoryQueryView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr), View.OnClickListener {

    private val mAbTradeManager = AbTradeManager<Array<TagAns_CommItem>>()

    private val mDelegateAdapter by lazy { AStockQueryHisDelegateAdapter(context) }
    private val mTransactionAdapter by lazy { AStockQueryTransactionAdapter(context) }

    private var mStartDatePick = true

    private lateinit var mTimePopupWindow: TimePopupWindow

    companion object {
        const val FMT_STR_SHOW = "yyyy-MM-dd"
        const val FMT_STR_REQ = "yyyyMMdd"
        const val DIFF_DAY = 89
    }

    private val format4Show = SimpleDateFormat(FMT_STR_SHOW)

    private val mOnAbResultListener by lazy {
        object : OnAbResultListener<Array<TagAns_CommItem>> {
            override fun onCallback(data: Array<TagAns_CommItem>?) {
                val list = data?.let {
                    it.sortWith(compareBy({ item ->
                        when (item) {
                            is TagAns_Fun1030 -> item.nOrderDate
                            is TagAns_Fun1031 -> item.nMadeDate
                            else -> null
                        }
                    }, { item ->
                        when (item) {
                            is TagAns_Fun1030 -> item.nOrderTime
                            is TagAns_Fun1031 -> item.nMadeTime
                            else -> null
                        }
                    }))
                    it.reverse()
                    it.map { item ->
                        when (item) {
                            is TagAns_Fun1030 -> transferMode(item)
                            is TagAns_Fun1031 -> transferMode(item)
                            else -> item
                        }
                    }
                }?.toTypedArray()
                post {
                    if (!historySwitch.isSwitch) {
                        val delegate = list?.filterIsInstance<TradeDelegationModel>()?.toTypedArray()
                        listView.adapter = mDelegateAdapter
                        mDelegateAdapter.setHistory(true)
                        mDelegateAdapter.setData(delegate)
                        if (delegate.isNullOrEmpty()) {
                            tipInfo.text = resources.getString(R.string.trade_tip_delegate_list)
                            tipInfo.visibility = VISIBLE
                        } else {
                            tipInfo.visibility = GONE
                        }
                    } else {
                        val transaction = list?.filterIsInstance<TradeDoneModel>()?.toTypedArray()
                        listView.adapter = mTransactionAdapter
                        mTransactionAdapter.setHistory(true)
                        mTransactionAdapter.setData(transaction)
                        if (transaction.isNullOrEmpty()) {
                            tipInfo.text = resources.getString(R.string.trade_tip_transaction_list)
                            tipInfo.visibility = VISIBLE
                        } else {
                            tipInfo.visibility = GONE
                        }
                    }
                }
            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    private val mOnSwitchChangeListener by lazy {
        object : MoneySwitchView.OnSwitchChangeListener {
            override fun onSwitchChange(isSwitch: Boolean) {
                val start = Assist.getDateByFormat(startTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
                val end = Assist.getDateByFormat(endTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
                if (start.isNotEmpty() && end.isNotEmpty()) {
                    if (!historySwitch.isSwitch) {
                        field1.text = resources.getString(R.string.queryTime)
                        field2.text = resources.getString(R.string.queryHisBuySell)
                        field3.text = resources.getString(R.string.queryDelegatePrice)
                        field4.text = resources.getString(R.string.queryNumber)
                        mAbTradeManager.queryDelegateHis(start.toInt(), end.toInt(), ggType)
                    } else {
                        field1.text = resources.getString(R.string.queryTime)
                        field2.text = resources.getString(R.string.queryPrice)
                        field3.text = resources.getString(R.string.queryOverNumber)
                        field4.text = resources.getString(R.string.queryMoney)
                        mAbTradeManager.queryTransactionHis(start.toInt(), end.toInt(), ggType)
                    }
                }
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_trade_history_query, this)
        startTransferDate.setOnClickListener(this)
        endTransferDate.setOnClickListener(this)
        val today = TimeHelper.getInstance().getDate("yyyyMMdd").toInt()
        val nextDay = DateUtil.getDayDiff(today.toString(), "yyyyMMdd", -1).toInt()
        val preDay = DateUtil.getDayDiff(nextDay.toString(), "yyyyMMdd", -DIFF_DAY).toInt()
        val endDate = Assist.getDateByFormat(nextDay.toString(), FMT_STR_REQ, FMT_STR_SHOW)
        val startDate = Assist.getDateByFormat(preDay.toString(), FMT_STR_REQ, FMT_STR_SHOW)
        startTransferDate.text = startDate
        endTransferDate.text = endDate
        historySwitch.setSwitchText("委托", "成交")
        historySwitch.setSwitchTextSize(StringUtils.dipToPx(13f).toFloat())
        historySwitch.addOnSwitchChangeListener(mOnSwitchChangeListener)
        field1.text = resources.getString(R.string.queryTime)
        field2.text = resources.getString(R.string.queryHisBuySell)
        field3.text = resources.getString(R.string.queryDelegatePrice)
        field4.text = resources.getString(R.string.queryNumber)
        mTimePopupWindow = TimePopupWindow(context, TimePopupWindow.Type.YEAR_MONTH_DAY)
        mTimePopupWindow.setOnTimeSelectListener { _, date ->
            if (mStartDatePick) {
                startTransferDate.text = format4Show.format(date)
                val nextDayTime = DateUtil.getDayDiff(date, "yyyyMMdd", DIFF_DAY).toInt()
                val endDateTime = Assist.getDateByFormat(nextDayTime.toString(), FMT_STR_REQ, FMT_STR_SHOW)
                endTransferDate.text = endDateTime
            } else {
                endTransferDate.text = format4Show.format(date)
                val preDayTime = DateUtil.getDayDiff(date, "yyyyMMdd", -DIFF_DAY).toInt()
                val startDateTime = Assist.getDateByFormat(preDayTime.toString(), FMT_STR_REQ, FMT_STR_SHOW)
                startTransferDate.text = startDateTime
            }
            val start = Assist.getDateByFormat(startTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
            val end = Assist.getDateByFormat(endTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
            if (start.isNotEmpty() && end.isNotEmpty()) {
                if (!historySwitch.isSwitch) {
                    mAbTradeManager.queryDelegateHis(start.toInt(), end.toInt(), ggType)
                } else {
                    mAbTradeManager.queryTransactionHis(start.toInt(), end.toInt(), ggType)
                }
            } else {
                ToastTool.showToast(R.string.select_date)
            }
        }
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbResultListener)
        val start = Assist.getDateByFormat(startTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
        val end = Assist.getDateByFormat(endTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
        if (start.isNotEmpty() && end.isNotEmpty()) {
            if (!historySwitch.isSwitch) {
                mAbTradeManager.queryDelegateHis(start.toInt(), end.toInt(), ggType)
            } else {
                mAbTradeManager.queryTransactionHis(start.toInt(), end.toInt(), ggType)
            }
        }
    }

    override fun hide() {
        mAbTradeManager.unregisterListener(mOnAbResultListener)
    }

    override fun destroyView() {

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.startTransferDate,
            R.id.endTransferDate -> {
                mStartDatePick = v.id == R.id.startTransferDate
                mTimePopupWindow.setTitle(resources.getString(R.string.select_date))
                val showText = (if (mStartDatePick) startTransferDate else endTransferDate).text.toString()
                val date =
                        if (showText.isNotEmpty()) {
                            format4Show.parse(showText)
                        } else {
                            TimeHelper.getInstance().curDate
                        }
                mTimePopupWindow.showAtLocation(this, Gravity.BOTTOM, 0, 0, date)
            }

        }
    }
}
