package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.util.AttributeSet
import android.view.View
import cn.com.hh.trade.data.TagAns_Fun1023_0
import cn.com.hh.trade.data.TagAns_Fun1023_1
import kotlinx.android.synthetic.main.view_query_hold.view.*
import kotlinx.android.synthetic.main.view_transfer_query.view.listView
import net.datamodel.network.CommonFunc
import util.ToastTool
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.adapter.AStockBuyAdapter
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbAssetInfoResultListener
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.home.manager.TradeHelper
import wind.android.bussiness.trade.home.model.JSHoldStockValue
import wind.android.news.anews.StockUtil
import java.nio.charset.Charset

/**
 * # **********************************************************************************************
 * # ClassName:      QueryhHoldView
 * # Description:    持仓
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class QueryhHoldView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr), View.OnClickListener {

    private val mAbTradeManager = AbTradeManager<Any>()

    private val mAdapter by lazy { AStockBuyAdapter(context) }

    private var isHideAssetInfo = false

    private var mAssetInfo: TagAns_Fun1023_0? = null

    private val mOnAbAssetInfoResultListener by lazy {
        object : OnAbAssetInfoResultListener<Any> {
            override fun onAssetInfo(data: Array<TagAns_Fun1023_0>?) {
                mAssetInfo = data?.firstOrNull { it.chMoneyType.toInt() == TradeConstantData.CURRENCY_RMB }
                mAssetInfo?.run {
                    post {
                        val market = dMarketValue.addThousandth()
                        val currencySign = resources.getString(R.string.currencySign)
                        val totalText = "$currencySign $market"
                        value3.setTextColor(StockUtil.getChangeColor(dIncome.toFloat()))
                        total.text = totalText
                        value1.text = dStkValue.addThousandth()
                        value2.text = dFundAvl.addThousandth()
                        value3.text = dIncome.addThousandth()
                    }
                }

            }

            override fun onHoldStock(data: Array<TagAns_Fun1023_1>?) {
                val list = data?.map {
                    transformToHoldItem(it)
                }
                post {
                    val dataList = TradeHelper.sortStockByMarketvalue(list)
                    mAdapter.setData(dataList)
                    if (dataList.isEmpty()) {
                        tipInfo.visibility = VISIBLE
                    } else {
                        tipInfo.visibility = GONE
                    }
                }
            }

            override fun onCallback(data: Any?) {

            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message + error.toString())
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_query_hold, this)
        listView.adapter = mAdapter
        RMBTotal.setOnClickListener(this)
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbAssetInfoResultListener)
        mAbTradeManager.queryHoldList(ggType)
    }

    override fun hide() {
        mAbTradeManager.unregisterListener(mOnAbAssetInfoResultListener)
    }

    override fun destroyView() {

    }

    /**
     * 转换成持仓列表item
     *
     * @param it 交易后台返回持仓列表item 类型
     * @return
     */
    private fun transformToHoldItem(it: TagAns_Fun1023_1): JSHoldStockValue {
        val holdValueItem = JSHoldStockValue()
        holdValueItem.moneyType = it.chMoneyType.toInt()
        holdValueItem.marketType = it.chMarketType.toInt()
        val parseWindCode = TradeHelper.parseWindCode(it.chStockCode,
                it.chMarketType, it.iResv1)
        holdValueItem.windCode = parseWindCode
        holdValueItem.stockName = String(it.chStockName, Charset.forName("GBK"))
        holdValueItem.nStockQty = it.nStockQty.toString()
        holdValueItem.nCostPrice = CommonFunc.doubleFormat(
                it.nCostPrice * 0.0001, 3)
        holdValueItem.dIncome = it.dIncome.toString()
        holdValueItem.currencyMark = TradeHelper.getMoneyMark(it.chMoneyType.toInt())
        holdValueItem.dMktVal = it.dMktVal.toString()
        holdValueItem.nStockAvl = it.nStockAvl.toString()
        holdValueItem.nStockSell = it.nStockAvl
        val newPrice = if (it.nNewPrice <= 0 && it.nStockQty != 0)
            it.dMktVal / it.nStockQty else it.nNewPrice * 0.0001
        holdValueItem.nNewPrice = CommonFunc.doubleFormat(newPrice,
                CommonFunc.getRadixPointFactor(holdValueItem.windCode))
        holdValueItem.chUserCode = String(it.chUserCode, Charset.forName("GBK"))
        return holdValueItem
    }

    override fun onClick(v: View?) {
        if (v?.id == R.id.RMBTotal) {
            isHideAssetInfo = !isHideAssetInfo
            if (isHideAssetInfo) {
                val currencySign = resources.getString(R.string.currencySign)
                val totalText = "$currencySign ***"
                total.text = totalText
                value1.text = "***"
                value2.text = "***"
                value3.text = "***"
            } else {
                mAssetInfo?.run {
                    val market = CommonFunc.doubleFormat(dMarketValue, 2)
                    val currencySign = resources.getString(R.string.currencySign)
                    val totalText = "$currencySign $market"
                    value3.setTextColor(StockUtil.getChangeColor(dIncome.toFloat()))
                    total.text = totalText
                    value1.text = CommonFunc.doubleFormat(dStkValue, 2)
                    value2.text = CommonFunc.doubleFormat(dFundAvl, 2)
                    value3.text = CommonFunc.doubleFormat(dIncome, 2)
                }
            }

        }
    }
}
