package wind.android.bussiness.trade.abstocktrade.view

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Build
import android.text.SpannableString
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.View
import android.view.WindowManager
import android.view.inputmethod.EditorInfo
import android.widget.*
import cn.com.hh.trade.data.TagAns_Fun1023_0
import cn.com.hh.trade.data.TagAns_Fun1023_1
import cn.com.hh.trade.data.TagAns_Fun1040
import kotlinx.android.synthetic.main.view_security_to_bank.view.*
import ui.keyboard.UILoginKeyBoard
import ui.screen.UIScreen
import util.ToastTool
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.addThousandth
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbAssetInfoResultListener
import wind.android.bussiness.trade.abstocktrade.manager.OnAbBankResultListener
import wind.android.bussiness.trade.constant.TradeConstantData
import wind.android.bussiness.trade.constant.TradeUserAction
import wind.android.f5.view.element.LandscapeKlineAdapter
import java.lang.reflect.Method
import java.nio.charset.Charset


/**
 * # **********************************************************************************************
 * # ClassName:      SecurityToBankView
 * # Description:    证券转银行
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class SecurityToBankView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr), View.OnClickListener {

    private val mAbTradeManager = AbTradeManager<Any>()

    private var mBankInfo = ArrayList<TagAns_Fun1040>()
    private var mBankInfoName = ArrayList<String>()

    private var mSelectBank = 0

    private var allTransfer = 0.0

    private lateinit var mPopupWindow: PopupWindow
    private lateinit var landscapeKlineAdapter: LandscapeKlineAdapter

    companion object {
        const val TIME = "9:00-16:00"
    }

    private val mOnAbBankResultListener by lazy {
        object : OnAbBankResultListener<Any> {
            override fun onBankInfo(obj: Any?) {
                if (obj is Array<*> && obj.size > 0) {
                    for (item in obj) {
                        if (item is TagAns_Fun1040 && item.chMoneyType != '1'.toByte() && item.chMoneyType != '2'.toByte()) {
                            mBankInfo.add(item)
                            mBankInfoName.add(String(item.chBankName, Charset.forName("GBK")))
                        }
                    }
                }
                if (mBankInfoName.size > 0) {
                    post { bankNameTv.text = mBankInfoName[mSelectBank] }
                }
            }

            override fun onBankMoney(obj: Any?) {
            }

            override fun onBankTransfer(obj: Any?) {
                ToastTool.showToast(R.string.transfer_finish)
                post {
                    securityPasswordET.setText("")
                    transferMoneyET.setText("")
                }
            }

            override fun onCallback(data: Any?) {

            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    private val mOnAbAssetInfoResultListener by lazy {
        object : OnAbAssetInfoResultListener<Any> {
            override fun onAssetInfo(data: Array<TagAns_Fun1023_0>?) {
                val firstItem = data?.firstOrNull { it.chMoneyType.toInt() == TradeConstantData.CURRENCY_RMB }
                firstItem?.run {
                    allTransfer = dOther
                    post { transferMoneyET.hint = String.format(resources.getString(R.string.hit_can_transferOut), dOther.addThousandth()) }
                }

            }

            override fun onHoldStock(data: Array<TagAns_Fun1023_1>?) {

            }

            override fun onCallback(data: Any?) {

            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_security_to_bank, this)
        allTransferOut.setOnClickListener(this)
        transferToBank.setOnClickListener(this)
        RMB.setOnClickListener(this)
        bankNameTv.setOnClickListener(this)

        val spannableString = SpannableString(String.format(resources.getString(R.string.transferTip), SecurityToBankView.TIME))
        val colorSpan = ForegroundColorSpan(Color.parseColor("#DEB887"))
        val index = spannableString.indexOf(SecurityToBankView.TIME)
        spannableString.setSpan(colorSpan, index, index + SecurityToBankView.TIME.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        transferTip.text = spannableString

        securityPasswordET.imeOptions = EditorInfo.IME_ACTION_DONE
        hideSysSoftInput(securityPasswordET)
        securityPasswordET.setOnTouchListener { _, event ->
            UILoginKeyBoard.getKeyBoard()
                    .showMyKeyBoard(securityPasswordET, null, true)
            securityPasswordET.onTouchEvent(event)
        }
        transferMoneyET.imeOptions = EditorInfo.IME_ACTION_DONE
        hideSysSoftInput(transferMoneyET)
        transferMoneyET.setOnTouchListener { _, event ->
            UILoginKeyBoard.getKeyBoard()
                    .showMyKeyBoard(transferMoneyET, null, true)
            transferMoneyET.onTouchEvent(event)
        }
    }

    /**
     * 隐藏系统键盘
     *
     * @param accountCodeText
     */
    private fun hideSysSoftInput(accountCodeText: EditText) {
        if (Build.VERSION.SDK_INT > 10) {
            (context as Activity).window.setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN)
            val cls = EditText::class.java
            var setSoftInputShownOnFocus: Method
            try {
                setSoftInputShownOnFocus = cls.getMethod(
                        "setSoftInputShownOnFocus", Boolean::class.javaPrimitiveType)
                setSoftInputShownOnFocus.isAccessible = true
                setSoftInputShownOnFocus.invoke(accountCodeText, false)
            } catch (e: Exception) {
                try {
                    setSoftInputShownOnFocus = cls.getMethod(
                            "setShowSoftInputOnFocus", Boolean::class.javaPrimitiveType)
                    setSoftInputShownOnFocus.isAccessible = true
                    setSoftInputShownOnFocus.invoke(accountCodeText, false)
                } catch (e1: Exception) {
                }
            }
        }
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbBankResultListener)
        mAbTradeManager.registerListener(mOnAbAssetInfoResultListener)
        // 银行信息查询
        mAbTradeManager.queryBankInfo()
        mAbTradeManager.queryHoldList(ggType)
    }

    override fun hide() {
        mBankInfo.clear()
        mBankInfoName.clear()
        mAbTradeManager.unregisterListener(mOnAbBankResultListener)
        UILoginKeyBoard.getKeyBoard().closeMyKeyBoard()
    }

    override fun destroyView() {

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.bankNameTv,
            R.id.RMB -> {
                if (mBankInfo.size == 0) {
                    ToastTool.showToast(resources.getString(R.string.tip_no_bank))
                } else {
                    if (!this::mPopupWindow.isInitialized) {
                        initPopupWindow()
                    }
                    if (mPopupWindow.isShowing) {
                        mPopupWindow.dismiss()
                    } else {
                        mPopupWindow.showAsDropDown(bankNameTv, 0, 0)
                    }
                }
            }
            R.id.allTransferOut -> {
                transferMoneyET.setText(allTransfer.toString())
            }
            R.id.transferToBank -> {
                TradeConstantData.submitUserActionEx(TradeUserAction.BUTTON_TRADE_TURN)
                if (mBankInfo.size == 0) {
                    ToastTool.showToast(resources.getString(R.string.tip_no_bank))
                } else {
                    val password = securityPasswordET.text.toString()
                    val money = transferMoneyET.text.toString()
                    when {
                        password.isEmpty() -> {
                            ToastTool.showToast(resources.getString(R.string.hit_input_securityPassword))
                        }
                        money.isEmpty() -> {
                            ToastTool.showToast(resources.getString(R.string.hit_input_bankMoney))
                        }
                        else -> {
                            val ansFun1040 = mBankInfo[mSelectBank]
                            // 银证转账
                            mAbTradeManager.bankTransfer(false, ansFun1040, password, money.toDouble())
                        }
                    }
                }
            }
        }
    }

    private fun initPopupWindow() {
        val listView = ListView(context)
        landscapeKlineAdapter = LandscapeKlineAdapter(false)
        landscapeKlineAdapter.setData(mBankInfoName)
        listView.adapter = landscapeKlineAdapter
        listView.dividerHeight = 0
        listView.cacheColorHint = 0
        listView.selector = ColorDrawable(Color.TRANSPARENT)
        listView.onItemClickListener = AdapterView.OnItemClickListener { _, _, position, _ ->
            mPopupWindow.dismiss()
            mSelectBank = position
            bankNameTv.text = mBankInfoName[mSelectBank]
        }
        mPopupWindow = PopupWindow(listView, UIScreen.screenWidth / 2,
                RelativeLayout.LayoutParams.WRAP_CONTENT, true)
        mPopupWindow.isTouchable = true
        mPopupWindow.setTouchInterceptor { _, _ ->
            false
            // 这里如果返回true的话，touch事件将被拦截
            // 拦截后 PopupWindow的onTouchEvent不被调用，这样点击外部区域无法dismiss
        }
        // 如果不设置PopupWindow的背景，无论是点击外部区域还是Back键都无法dismiss弹框
        mPopupWindow.setBackgroundDrawable(resources.getDrawable(
                R.drawable.transparent))
        mPopupWindow.setOnDismissListener { }
    }
}
