package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.util.AttributeSet
import android.view.View
import cn.com.hh.trade.data.NSDPROCAPI
import cn.com.hh.trade.data.TagAns_Fun1024
import kotlinx.android.synthetic.main.view_today_delegate.view.*
import net.util.TimeHelper
import util.ToastTool
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.adapter.AStockQueryDelegateAdapter
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbCancelResultListener
import wind.android.bussiness.trade.abstocktrade.transferMode
import wind.android.bussiness.trade.abstocktrade.transferMode1095
import wind.android.bussiness.trade.constant.TradeConstantData
import java.text.SimpleDateFormat
import java.util.*

/**
 * # **********************************************************************************************
 * # ClassName:      TodayDelegateView
 * # Description:    当日委托
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class TodayDelegateView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr) {

    private val mAbTradeManager = AbTradeManager<Array<TagAns_Fun1024>>()

    private val mAdapter by lazy { AStockQueryDelegateAdapter(context) }

    private val mOnAbResultListener by lazy {
        object : OnAbCancelResultListener<Array<TagAns_Fun1024>> {
            override fun onCallback(data: Array<TagAns_Fun1024>?) {
                val list = data?.let {
                    it.map { item -> transferMode(item) }
                }?.toTypedArray()
                if (adventrustTime()) {
                    mAbTradeManager.queryAdventrustDelegate()
                }
                post {
                    mAdapter.setData(list)
                    if (list.isNullOrEmpty()) {
                        tipInfo.visibility = VISIBLE
                    } else {
                        tipInfo.visibility = GONE
                    }
                }
            }

            override fun onAdventrustBack(functionID: Int, obj: Any?) {
                if (functionID == NSDPROCAPI.FUNID_GET_ADVENTRUST && obj is Array<*>) {
                    val list = obj.let {
                        it.map { item -> transferMode1095(item) }
                    }.toTypedArray()
                    post {
                        mAdapter.addData(list)
                        if (!list.isNullOrEmpty()) {
                            tipInfo.visibility = GONE
                        }
                    }
                }
            }

            override fun onCancelEntrust(obj: Any?) {

            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_today_delegate, this)
        listView.adapter = mAdapter
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbResultListener)
        mAbTradeManager.queryDelegate(ggType)
    }

    override fun hide() {
        mAbTradeManager.unregisterListener(mOnAbResultListener)
    }

    override fun destroyView() {

    }

    private fun adventrustTime(): Boolean {
        // 湘财支持预约委托 且在预约委托时间段 判断条件是周一到周六 18点到24点
        if (TradeConstantData.selectedItem.bookingOrderSupport == 1) {
            val current = TimeHelper.getInstance().getDate("yyyy-MM-dd HH:mm:ss")
            val df = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
            try {
                val calendar = Calendar.getInstance()
                calendar.time = df.parse(current)
                if (calendar[Calendar.DAY_OF_WEEK] != Calendar.SUNDAY) {
                    val hour = calendar[Calendar.HOUR_OF_DAY]
                    if (hour in 18..23) {
                        return true
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        return false
    }
}
