package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.DashPathEffect
import android.graphics.Paint
import android.util.AttributeSet
import android.view.View
import net.datamodel.network.CommonFunc
import net.datamodel.speed.NewKLineDataItem
import ui.screen.UIScreen
import util.StringUtils
import wind.android.bussiness.trade.abstocktrade.manager.integrate.KlineIntegrate
import wind.android.news.anews.StockUtil
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:       TradeKlineView.kt
 * # Description:     k线view
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class TradeKlineView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : View(context, attrs, defStyleAttr) {
    /***************************绘制需要的数据，由外部传递****************************************/
    /**
     * 最高价、中间价、最低价、最大成交量
     */
    private var priceVolume = listOf(0f, 0f, 0f, 0f)
    /**
     * 起始日期、中间日期、终止日期
     */
    private var dateList = listOf(0, 0, 0)
    /**
     * K线数据
     */
    private var currentDataVec = Vector<Any>()
    /**
     * 单位
     */
    private var precision = 2
    /**
     * 时间区间 每个时间单位的长度
     */
    private var timePerDifference = 0f
    /**
     * 价格区间 每个价格单位的高度
     */
    private var pricePerDifference = 0f
    /**
     * 成交量区间 每个成交量单位的高度
     */
    private var volumePerDifference = 0f

    private var mPerWidth = 0f
    private var mPerHeight = 0f

    private var mTextHeight = 0f

    private val mBgPaint = Paint()
    private val mBgDaskPaint = Paint()
    private val mDashPathEffect = DashPathEffect(floatArrayOf(6f, 6f, 6f, 6f), 2f)

    private val mTextPaint = Paint()
    private val mKlinePaint = Paint()
    private val mVolumePaint = Paint()

    private val padding = StringUtils.dipToPx(3f).toFloat()

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val widthModel = MeasureSpec.getMode(widthMeasureSpec)
        val heightModel = MeasureSpec.getMode(heightMeasureSpec)
        var width = 0
        var height = 0
        if (widthModel == MeasureSpec.EXACTLY) {
            width = MeasureSpec.getSize(widthMeasureSpec)
        } else if (widthModel == MeasureSpec.AT_MOST) {
            width = MeasureSpec.getSize(widthMeasureSpec)
        }
        if (heightModel == MeasureSpec.EXACTLY) {
            height = MeasureSpec.getSize(heightMeasureSpec)
            if (height == 0) {
                height = (UIScreen.screenWidth * 11 / 12.0f).toInt()
            }
        } else if (heightModel == MeasureSpec.AT_MOST) {
            height = (UIScreen.screenWidth * 11 / 12.0f).toInt()
        }
        setMeasuredDimension(width, height)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mTextPaint.textSize = StringUtils.dipToPx(8f).toFloat()
        mTextHeight = getChatHeight(mTextPaint)
        // 宽度分为4份
        mPerWidth = width / 4f
        // 高度-字符显示区域  分为5份（其中1分成交量）
        mPerHeight = (height - mTextHeight - padding) / 5f

        // 画布
        mBgPaint.color = StockUtil.getChangeColor(0f)
        mBgDaskPaint.pathEffect = mDashPathEffect
        mBgDaskPaint.color = StockUtil.getChangeColor(0f)

        mKlinePaint.style = Paint.Style.FILL
        mKlinePaint.strokeWidth = StringUtils.dipToPx(0.8f).toFloat()
        mKlinePaint.isAntiAlias = true
        mVolumePaint.style = Paint.Style.FILL
        mVolumePaint.strokeWidth = StringUtils.dipToPx(0.8f).toFloat()
        mVolumePaint.isAntiAlias = true
    }

    fun setData(priceVolume: List<Float>, dateList: List<Int>, list: Vector<Any>, precision: Int) {
        this.precision = precision
        this.priceVolume = priceVolume
        this.dateList = dateList
        currentDataVec.clear()
        currentDataVec.addAll(list)

        // 计算每单位长度
        timePerDifference = (width - 2f) / KlineIntegrate.KLINE_COUNT
        pricePerDifference = mPerHeight * 4 / (priceVolume[0] - priceVolume[2])
        volumePerDifference = mPerHeight / priceVolume[3]
        postInvalidate()
    }

    fun clean() {
        this.priceVolume = listOf(0f, 0f, 0f, 0f)
        this.dateList = listOf(0, 0, 0)
        currentDataVec.clear()
        postInvalidate()
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        if (canvas != null) {
            // 绘制背景
            drawBg(canvas)
            // 绘制K线
            drawKline(canvas)
            // 绘制价格与时间
            drawText(canvas)
        }
    }

    /**
     * 绘制背景
     */
    private fun drawBg(canvas: Canvas) {
        // 实线
        val lines = floatArrayOf(1f, 1f, width.toFloat() - 1f, 1f,
                1f, mPerHeight * 4, width.toFloat() - 1f, mPerHeight * 4,
                1f, height.toFloat() - 1f, width.toFloat() - 1f, height.toFloat() - 1f,
                1f, 1f, 1f, height.toFloat() - 1f,
                width.toFloat() - 1f, 1f, width.toFloat() - 1f, height.toFloat() - 1f)
        canvas.drawLines(lines, mBgPaint)

        // 虚线
        val daskLines = floatArrayOf(1f, mPerHeight, width.toFloat() - 1f, mPerHeight,
                1f, mPerHeight * 2, width.toFloat() - 1f, mPerHeight * 2,
                1f, mPerHeight * 3, width.toFloat() - 1f, mPerHeight * 3,
                mPerWidth, 1f, mPerWidth, height.toFloat() - 1f,
                mPerWidth * 2, 1f, mPerWidth * 2, height.toFloat() - 1f,
                mPerWidth * 3, 1f, mPerWidth * 3, height.toFloat() - 1f)
        canvas.drawLines(daskLines, mBgDaskPaint)
    }

    /**
     * 绘制价格与时间
     */
    private fun drawText(canvas: Canvas) {
        mTextPaint.color = StockUtil.getChangeColor(0f)
        if (dateList[0] > 0) {
            val showStart = CommonFunc.FormartData(dateList[0].toString())
            canvas.drawText(showStart, padding, mPerHeight * 4 + mTextHeight, mTextPaint)
        }
        if (dateList[2] > 0) {
            val showEnd = CommonFunc.FormartData(dateList[2].toString())
            canvas.drawText(showEnd, width - padding - mTextPaint.measureText(showEnd), mPerHeight * 4 + mTextHeight, mTextPaint)
        }
        if (dateList[1] > 0) {
            val showMiddle = CommonFunc.FormartData(dateList[1].toString())
            canvas.drawText(showMiddle, (width - mTextPaint.measureText(showMiddle)) / 2, mPerHeight * 4 + mTextHeight, mTextPaint)
        }

        val middleShow = CommonFunc.doubleFormat(priceVolume[1].toDouble(), precision)
        val highShow = CommonFunc.doubleFormat(priceVolume[0].toDouble(), precision)
        val lowShow = CommonFunc.doubleFormat(priceVolume[2].toDouble(), precision)
        canvas.drawText(middleShow, padding, mPerHeight * 2 - padding, mTextPaint)

        mTextPaint.color = StockUtil.getChangeColor(1f)
        canvas.drawText(highShow, padding, mTextHeight, mTextPaint)

        mTextPaint.color = StockUtil.getChangeColor(-1f)
        canvas.drawText(lowShow, padding, mPerHeight * 4 - padding, mTextPaint)
    }

    /**
     * 绘制K线
     */
    private fun drawKline(canvas: Canvas) {
        if (currentDataVec.size > 0) {
            val startIndex =
                    if (currentDataVec.size >= KlineIntegrate.KLINE_COUNT) {
                        currentDataVec.size - KlineIntegrate.KLINE_COUNT
                    } else {
                        0
                    }
            var preClose = 0f
            for (i in startIndex until currentDataVec.size) {
                val item = currentDataVec[i]
                if (item is NewKLineDataItem) {
                    if (i == startIndex) {
                        preClose = item.open
                    } else {
                        val preItem = currentDataVec[i - 1]
                        if (preItem is NewKLineDataItem) {
                            preClose = preItem.close
                        }
                    }
                    val x = timePerDifference * (i - startIndex)
                    if (item.close >= item.open
                            || (item.close == item.open && item.close >= preClose)) {
                        mKlinePaint.color = StockUtil.getChangeColor(1f)
                        canvas.drawRect(x + 0.8f,
                                (priceVolume[0] - item.close) * pricePerDifference,
                                x + timePerDifference - 0.8f,
                                (priceVolume[0] - item.open) * pricePerDifference,
                                mKlinePaint)
                    } else {
                        mKlinePaint.color = StockUtil.getChangeColor(-1f)
                        canvas.drawRect(x + 0.8f,
                                (priceVolume[0] - item.open) * pricePerDifference,
                                x + timePerDifference - 0.8f,
                                (priceVolume[0] - item.close) * pricePerDifference,
                                mKlinePaint)
                    }
                    canvas.drawLine(x + timePerDifference / 2,
                            (priceVolume[0] - item.high) * pricePerDifference,
                            x + timePerDifference / 2,
                            (priceVolume[0] - item.low) * pricePerDifference,
                            mKlinePaint)


                    mVolumePaint.color = if (item.close >= preClose) {
                        StockUtil.getChangeColor(1f)
                    } else {
                        StockUtil.getChangeColor(-1f)
                    }
                    canvas.drawRect(x + 0.8f,
                            mPerHeight * 4 + mTextHeight + padding + (priceVolume[3] - item.volume) * volumePerDifference,
                            x + timePerDifference - 0.8f,
                            height.toFloat() - 1f,
                            mVolumePaint)
                }
            }
        }
    }

    /**
     * 用于字符高度计算
     *
     * @return
     * @exception
     * @since 1.0.0
     */
    private fun getChatHeight(paint: Paint): Float {
        val fm = paint.fontMetrics
        return fm.descent - fm.ascent
    }
}