package wind.android.bussiness.trade.abstocktrade.view

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Build
import android.os.Message
import android.support.v7.widget.AppCompatEditText
import android.support.v7.widget.LinearLayoutManager
import android.text.Editable
import android.text.InputType
import android.text.TextWatcher
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.widget.EditText
import android.widget.TextView
import base.ActivityHandler
import com.jcodecraeer.xrecyclerview.XRecyclerView
import ui.keyboard.UIKeyBoard
import util.ThemeUtils
import util.ToastTool
import wind.android.R
import wind.android.bussiness.optionalstock.model.SearchRecord
import wind.android.bussiness.optionalstock.preopt.view.RecyclerViewDivider
import wind.android.bussiness.trade.abstocktrade.adapter.TradeSearchAdapter
import wind.android.bussiness.trade.abstocktrade.presenter.TradeSearchPresenter
import wind.android.news2.util.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      TradeSearchDialog
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/11/26    17:35
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class TradeSearchDialog(context: Context) : Dialog(context, R.style.ActionSheetDialogStyle), XRecyclerView.LoadingListener,
        ITradeSearchView, ActivityHandler.ActivityHandlerListener, TextWatcher, DialogInterface.OnShowListener {

    /**
     * 输入框
     */
    private val searchEt: AppCompatEditText
    /**
     * 标题栏
     */
    private val listTitle: TextView
    /**
     * 列表
     */
    private val searchListView: XRecyclerView
    /**
     * 点击列表item回调事件（给外部
     */
    var onItemClickCallback: ((windCode: String?, stockName: String?) -> Unit)? = null
    /**
     * 列表adapter
     */
    private val adapter: TradeSearchAdapter by lazy { TradeSearchAdapter(context) }
    /**
     * presenter
     */
    private val presenter: TradeSearchPresenter by lazy { TradeSearchPresenter(this) }

    init {
        val view = LayoutInflater.from(context).inflate(R.layout.dialog_trade_search, null)
        val placeholderView = view.findViewById<View>(R.id.placeholder_view)
        placeholderView.setBackgroundResource(ThemeUtils.getResId(R.color.trade_search_dialog_bg_b,
                R.color.trade_search_dialog_bg_w))
        placeholderView.setOnClickListener { dismiss() }
        searchEt = view.findViewById(R.id.search_et)
        searchEt.setBackgroundResource(ThemeUtils.getResId(R.drawable.a_trade_search_edit_bg_b,
                R.drawable.a_trade_search_edit_bg_w))
        searchEt.addTextChangedListener(this)
        listTitle = view.findViewById(R.id.list_title)
        listTitle.text = "最近浏览"
        searchListView = view.findViewById<XRecyclerView>(R.id.search_list_view)
        searchListView.layoutManager = LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        searchListView.setPullRefreshEnabled(false)
        searchListView.setLoadingListener(this)
        searchListView.addItemDecoration(RecyclerViewDivider(context, LinearLayoutManager.HORIZONTAL,
                1, ThemeUtils.getColor(context.resources, R.color.color_line_setting_black,
                R.color.color_line_setting_white), true))
        searchListView.setFooterViewTextColor(StockUtil.getChangeColor(0f))
        searchListView.setOnTouchListener { _, _ ->
            closeKeyboard()
            false
        }
        adapter.onItemClickListener = {
            val list = adapter.data
            val searchRecord = if (it > -1 && it < list.size) list[it] else null
            searchRecord?.run {
                presenter.saveRecord(this)
                onItemClickCallback?.invoke(windcode, stockname) }
            dismiss()
        }
        searchListView.adapter = adapter
        setContentView(view)
        setOnShowListener(this)
        searchEt.setOnClickListener { showCustomKeyboard(searchEt) }
    }

    override fun show() {
        //获得当前窗体
        val window = window ?: return
        //重新设置
        val lp = window.attributes
        lp.gravity = Gravity.BOTTOM
        lp.width = WindowManager.LayoutParams.MATCH_PARENT
        lp.height = WindowManager.LayoutParams.MATCH_PARENT
        lp.flags = WindowManager.LayoutParams.FLAG_DIM_BEHIND
        window.attributes = lp
        super.show()
        presenter.queryHistory()
    }

    override fun dismiss() {
        super.dismiss()
        searchEt.removeTextChangedListener(this)
        closeKeyboard()
        UIKeyBoard.getKeyBoard().clear()
        presenter.detachView()
    }

    override fun onLoadMore() {
        presenter.search(nextPage = true)
    }

    override fun onRefresh() {
    }

    override fun updateList(list: List<SearchRecord?>?, history: Boolean) {
        val result: ArrayList<SearchRecord?> = ArrayList()
        list?.let { result.addAll(it) }
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_LIST
        msg.obj = list
        msg.arg1 = if (history) IS_HISTORY_LIST else IS_SEARCH_LIST
        sendMessage(msg)
    }

    override fun showLoading() {
    }

    override fun hideLoading() {
    }

    override fun errorMsg(errorType: Int, msg: String?) {
        val message = Message.obtain()
        message.what = MSG_UPDATE_LIST
        message.obj = msg
        sendMessage(message)
    }

    override fun handleMessage(msg: Message?) {
        when (msg?.what) {
            MSG_UPDATE_LIST -> {
                val list = msg.obj as? ArrayList<SearchRecord?>
                adapter.setData(list)
                searchListView.loadMoreComplete()
                val size = list?.size ?: 0
                if (size == 0 || size.rem(20) != 0) {
                    searchListView.setLoadingMoreEnabled(false)
                }
                listTitle.text = if (msg.arg1 == IS_SEARCH_LIST) "搜索结果" else "最近浏览"
            }

            MSG_ERROR -> {
                searchListView.loadMoreComplete()
                ToastTool.showToast(msg.obj as String)
            }
        }
    }

    /**
     * handler sendMessage
     *
     * @param msg
     */
    private fun sendMessage(msg: Message) {
        ActivityHandler.getInstance(this).sendMessage(msg)
    }

    /**
     * 展示自定义键盘
     *
     * @param editText
     */
    private fun showCustomKeyboard(editText: EditText) {
        //不展示系统软键盘
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            editText.showSoftInputOnFocus = false
        } else {
            try {
                val cls = EditText::class.java
                val method = cls.getMethod("setShowSoftInputOnFocus", Boolean::class.java)
                method.isAccessible = true
                method.invoke(editText, false)
            } catch (e: Exception) {
                e.printStackTrace()
                editText.inputType = InputType.TYPE_NULL
            }
        }
        if (!UIKeyBoard.getKeyBoard().isShownKeyBorad) {
            try {
                UIKeyBoard.getKeyBoard().showMyKeyBoard(
                        editText, null)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    /**
     * 关闭自定义键盘
     */
    private fun closeKeyboard() {
        try { // 关闭 自定义键盘
            if (UIKeyBoard.getKeyBoard().isShownKeyBorad) {
                UIKeyBoard.getKeyBoard().closeMyKeyBoard()
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
        }
    }

    override fun afterTextChanged(s: Editable?) {
        //监听输入，搜索股票
        s?.let {
            searchListView.setLoadingMoreEnabled(true)
            presenter.search(it.toString())
        }
    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
    }

    override fun onShow(dialog: DialogInterface?) {
        showCustomKeyboard(searchEt)
    }

    companion object {
        private const val MSG_UPDATE_LIST = 0x1
        private const val MSG_ERROR = 0x2
        private const val IS_HISTORY_LIST = 1
        private const val IS_SEARCH_LIST = 0
    }
}