package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.support.design.widget.TabLayout
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import cn.com.hh.trade.data.NSDPROCAPI
import wind.android.R
import wind.android.bussiness.strategy.limitupseizeup.view.TitleTabLayout
import wind.android.bussiness.trade.abstocktrade.manager.TradeTrendKlineControl
import wind.android.bussiness.trade.constant.TradeConstantData
import java.util.*

/**
 * # **********************************************************************************************
 * # ClassName:      TradeTrendLayout
 * # Description:    WindStockDev
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/12/3    14:13
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class TradeTrendLayout @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : LinearLayout(context, attrs, defStyleAttr), TabLayout.BaseOnTabSelectedListener<TabLayout.Tab> {
    private val titleTab: TitleTabLayout
    private var indexWindCode: String? = null
    private var currentTabIndex: Int = 0
    private val layoutParams: LayoutParams by lazy {
        LayoutParams(LayoutParams.MATCH_PARENT,
                LayoutParams.MATCH_PARENT)
    }

    private val tradeTrendView: TradeTrendView by lazy { TradeTrendView(context) }
    private val tradeKlineView: TradeKlineView by lazy { TradeKlineView(context) }
    private val tradeTrendKlineControl: TradeTrendKlineControl by lazy { TradeTrendKlineControl() }
    private val tradeTrendKlineCallback by lazy {
        object : TradeTrendKlineControl.TradeTrendKlineCallback {
            override fun onTrendCallback(marketTime: List<Float>, convertTime: List<Float>, timeDifference: Float,
                                         priceDifference: Float, preClose: Float, priceHigh: Float,
                                         priceLow: Float, maxVolume: Float, maxAmount: Float,
                                         precision: Int, isHighTech: Boolean, currentDataVec: Vector<Any>) {
                post {
                    tradeTrendView.setData(marketTime, convertTime, timeDifference, priceDifference, preClose,
                            priceHigh, priceLow, maxVolume, maxAmount, precision, isHighTech, currentDataVec)
                }
            }

            override fun onKlineCallback(priceVolume: List<Float>, dateList: List<Int>, list: Vector<Any>, precision: Int) {
                post { tradeKlineView.setData(priceVolume, dateList, list, precision) }
            }
        }
    }

    var windCode: String? = null
        set(value) {
            field = value
            value?.let {
                val marketType = TradeConstantData.getMarketType(it)
                val titleArray = when (marketType) {
                    NSDPROCAPI.MARKET_SZ -> resources.getStringArray(R.array.a_trade_trend_tab_sz)
                    else -> resources.getStringArray(R.array.a_trade_trend_tab_sh)
                }
                titleTab.setTabItems(titleArray, tabTitleTextSize = 16)
                indexWindCode = when (marketType) {
                    NSDPROCAPI.MARKET_SZ -> "399001.SZ"
                    else -> "000001.SH"
                }
                tradeTrendKlineControl.unsubTrend()
                tradeTrendKlineControl.unsubKline()
                tradeTrendKlineControl.clean()
                tradeTrendView.clean()
                tradeKlineView.clean()
                titleTab.setSelectTab(TREND_TAB)
            }
        }

    init {
        orientation = VERTICAL
        gravity = Gravity.CENTER_HORIZONTAL
        titleTab = TitleTabLayout(context)
        titleTab.tabGravity = TabLayout.GRAVITY_FILL
        titleTab.tabMode = TabLayout.MODE_FIXED
        titleTab.setTabRippleColorResource(R.color.transparent)
        titleTab.setTabItems(resources.getStringArray(R.array.a_trade_trend_tab_sh),
                tabTitleTextSize = 16)
        titleTab.addOnTabSelectedListener(this)
        addView(titleTab, 0, LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                resources.getDimension(R.dimen.dp_40).toInt()))
        titleTab.setSelectTab(TREND_TAB)
        tradeTrendKlineControl.setTradeTrendKlineCallback(tradeTrendKlineCallback)
    }

    override fun onTabReselected(p0: TabLayout.Tab?) {
    }

    override fun onTabUnselected(p0: TabLayout.Tab?) {
    }

    override fun onTabSelected(p0: TabLayout.Tab?) {
        val position = p0?.position ?: 0
        currentTabIndex = position
        val view: View? = getChildAt(1)
        view?.let { removeView(it) }
        when (position) {
            TREND_TAB -> {
                addView(tradeTrendView, 1, layoutParams)
                tradeTrendView.clean()
                tradeTrendKlineControl.clean()
                tradeTrendKlineControl.unsubTrend()
                tradeTrendKlineControl.unsubKline()
                windCode?.let { tradeTrendKlineControl.subTrend(it) }
            }
            KLINE_TAB -> {
                addView(tradeKlineView, 1, layoutParams)
                tradeTrendKlineControl.unsubTrend()
                tradeTrendKlineControl.unsubKline()
                windCode?.let { tradeTrendKlineControl.subKline(it) }
            }
            INDEX_TAB -> {
                addView(tradeTrendView, 1, layoutParams)
                tradeTrendView.clean()
                tradeTrendKlineControl.clean()
                tradeTrendKlineControl.unsubTrend()
                tradeTrendKlineControl.unsubKline()
                indexWindCode?.let { tradeTrendKlineControl.subTrend(it) }
            }
        }
    }

    fun onResume() {
        tradeTrendKlineControl.setTradeTrendKlineCallback(tradeTrendKlineCallback)
        when (currentTabIndex) {
            TREND_TAB -> windCode?.let { tradeTrendKlineControl.subTrend(it) }
            KLINE_TAB -> windCode?.let { tradeTrendKlineControl.subKline(it) }
            INDEX_TAB -> indexWindCode?.let { tradeTrendKlineControl.subTrend(it) }
        }
    }

    fun onPause() {
        tradeTrendKlineControl.unsubTrend()
        tradeTrendKlineControl.unsubKline()
        tradeTrendKlineControl.setTradeTrendKlineCallback(null)
    }

    companion object {
        const val TREND_TAB = 0
        const val KLINE_TAB = 1
        const val INDEX_TAB = 2
    }


}