package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.DashPathEffect
import android.graphics.Paint
import android.graphics.Path
import android.util.AttributeSet
import android.view.View
import net.datamodel.network.CommonFunc
import net.datamodel.speed.NewTrendDataItem
import ui.screen.UIScreen
import util.CommonValue
import util.StringUtils
import util.ThemeUtils
import wind.android.f5.view.element.SpeedTrendUtil
import wind.android.news.anews.StockUtil
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:       TradeTrendView.kt
 * # Description:     分时view
 * # @author:         ylrong
 * # @version:        Ver 1.0
 * # Create Date:     2019/11/18 日 18:55
 * # ***********************************************************************************************
 */
class TradeTrendView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : View(context, attrs, defStyleAttr) {

    /***************************绘制需要的数据，由外部传递****************************************/
    /**
     * 市场时间-open-Suspend-Resume-Suspend1-Resume1-Suspend2-Resume2-close
     * @see net.datamodel.network.Indicator.DI_MARKETOPENTIME
     * @see net.datamodel.network.Indicator.DI_MARKETSUSPENDTIME
     * @see net.datamodel.network.Indicator.DI_MARKETRESUMETIME
     * @see net.datamodel.network.Indicator.DI_MARKETCLOSETIME
     */
    private var marketTime = listOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
    private var convertTime = listOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
    /**
     * 昨收
     */
    private var preClose = 0f
    /**
     * 最高
     */
    private var priceHigh = 0f
    /**
     * 最低
     */
    private var priceLow = 0f
    /**
     * 成交量
     */
    private var maxVolume = 0f
    /**
     * 成交额
     */
    private var maxAmount = 0f
    /**
     * 单位
     */
    private var precision = 0

    private var isHighTech = false
    /**
     * 分时数据
     */
    private var currentDataVec = Vector<Any>()
    /**
     * 时间区间
     */
    private var timeDifference = 0f
    /**
     * 价格区间
     */
    private var priceDifference = 0f
    /**
     * 时间区间 每个时间单位的长度
     */
    private var timePerDifference = 0f
    /**
     * 价格区间 每个价格单位的高度
     */
    private var pricePerDifference = 0f
    /**
     * 成交量区间 每个成交量单位的高度
     */
    private var volumePerDifference = 0f

    private val mBgPaint = Paint()
    private val mBgDaskPaint = Paint()
    private val mDashPathEffect = DashPathEffect(floatArrayOf(6f, 6f, 6f, 6f), 2f)

    private val mHighTechPaint = Paint()

    private val mTextPaint = Paint()

    private val mTrendPaint = Paint()
    private val mTrendPath = Path()

    private val mAveragePaint = Paint()
    private val mAveragePath = Path()

    private val mVolumePaint = Paint()

    private var mPerWidth = 0f
    private var mPerHeight = 0f
    private var mTextHeight = 0f

    private val padding = StringUtils.dipToPx(3f).toFloat()

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val widthModel = MeasureSpec.getMode(widthMeasureSpec)
        val heightModel = MeasureSpec.getMode(heightMeasureSpec)
        var width = 0
        var height = 0
        if (widthModel == MeasureSpec.EXACTLY) {
            width = MeasureSpec.getSize(widthMeasureSpec)
        } else if (widthModel == MeasureSpec.AT_MOST) {
            width = MeasureSpec.getSize(widthMeasureSpec)
        }
        if (heightModel == MeasureSpec.EXACTLY) {
            height = MeasureSpec.getSize(heightMeasureSpec)
            if (height == 0) {
                height = (UIScreen.screenWidth * 11 / 12.0f).toInt()
            }
        } else if (heightModel == MeasureSpec.AT_MOST) {
            height = (UIScreen.screenWidth * 11 / 12.0f).toInt()
        }
        setMeasuredDimension(width, height)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mTextPaint.textSize = StringUtils.dipToPx(8f).toFloat()
        mTextHeight = getChatHeight(mTextPaint)
        // 宽度分为4份
        mPerWidth = width / 4f
        // 高度-字符显示区域  分为4份
        mPerHeight = (height - mTextHeight) / 4f

        // 画布
        mBgPaint.color = StockUtil.getChangeColor(0f)
        mBgDaskPaint.pathEffect = mDashPathEffect
        mBgDaskPaint.color = StockUtil.getChangeColor(0f)

        mTrendPaint.color = StockUtil.getChangeColor(0f)
        mTrendPaint.style = Paint.Style.STROKE
        mTrendPaint.strokeWidth = StringUtils.dipToPx(0.8f).toFloat()
        mTrendPaint.isAntiAlias = true

        mAveragePaint.color = ThemeUtils.getColor(-0x33cd, -0x6700)
        mAveragePaint.style = Paint.Style.STROKE
        mAveragePaint.strokeWidth = StringUtils.dipToPx(0.8f).toFloat()
        mAveragePaint.isAntiAlias = true
    }

    fun setData(marketTime: List<Float>, convertTime: List<Float>, timeDifference: Float,
                priceDifference: Float, preClose: Float, priceHigh: Float,
                priceLow: Float, maxVolume: Float, maxAmount: Float,
                precision: Int, isHighTech: Boolean, currentDataVec: Vector<Any>) {
        this.marketTime = marketTime
        this.convertTime = convertTime
        this.timeDifference = timeDifference
        this.priceDifference = priceDifference
        this.preClose = preClose
        this.priceHigh = priceHigh
        this.priceLow = priceLow
        this.maxVolume = maxVolume
        this.maxAmount = maxAmount
        this.precision = precision
        this.isHighTech = isHighTech
        this.currentDataVec.clear()
        this.currentDataVec.addAll(currentDataVec)

        // 计算每单位长度
        timePerDifference = (width - 2f) / timeDifference
        pricePerDifference = (height - 1f - mTextHeight) / priceDifference
        volumePerDifference = mPerHeight / maxVolume
        postInvalidate()
    }

    fun clean() {
        this.marketTime = listOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
        this.convertTime = listOf(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f)
        this.timeDifference = 0f
        this.priceDifference = 0f
        this.preClose = 0f
        this.priceHigh = 0f
        this.priceLow = 0f
        this.maxVolume = 0f
        this.maxAmount = 0f
        this.currentDataVec.clear()
        postInvalidate()
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        if (canvas != null) {
            // 绘制背景
            drawBg(canvas)
            // 盘后分时
            drawHighTech(canvas)
            // 绘制分时
            drawTrend(canvas)
            // 绘制价格与时间
            drawText(canvas)
        }
    }

    /**
     * 绘制背景
     */
    private fun drawBg(canvas: Canvas) {
        // 实线
        val lines = floatArrayOf(1f, 1f, width.toFloat() - 1f, 1f,
                1f, mPerHeight * 4, width.toFloat() - 1f, mPerHeight * 4,
                1f, 1f, 1f, mPerHeight * 4,
                width.toFloat() - 1f, 1f, width.toFloat() - 1f, mPerHeight * 4)
        canvas.drawLines(lines, mBgPaint)

        // 虚线
        val distance =
                if (isHighTech) {
                    width.toFloat() / (4 * 60 + 25) * 60
                } else {
                    mPerWidth
                }
        val daskLines = floatArrayOf(1f, mPerHeight, width.toFloat() - 1f, mPerHeight,
                1f, mPerHeight * 2, width.toFloat() - 1f, mPerHeight * 2,
                1f, mPerHeight * 3, width.toFloat() - 1f, mPerHeight * 3,
                distance, 1f, distance, mPerHeight * 4,
                distance * 2, 1f, distance * 2, mPerHeight * 4,
                distance * 3, 1f, distance * 3, mPerHeight * 4)
        canvas.drawLines(daskLines, mBgDaskPaint)
    }

    private fun drawHighTech(canvas: Canvas) {
        if (isHighTech && marketTime[7] == (15 * 60 + 30).toFloat()) {
            //科创板盘后交易的透明背景颜色绘制
            if (CommonValue.styleType == CommonValue.StyleType.STYLE_BLACK) {
                mHighTechPaint.color = -0xe3e0de
            } else {
                mHighTechPaint.color = -0x262627
            }
            mHighTechPaint.style = Paint.Style.FILL
            canvas.drawRect(width.toFloat() - timePerDifference * 25, 0f, width.toFloat(), mPerHeight * 4, mHighTechPaint)
        }
    }

    /**
     * 绘制价格与时间
     */
    private fun drawText(canvas: Canvas) {
        // 时间
        mTextPaint.color = StockUtil.getChangeColor(0f)
        if (marketTime[0] > 0) {
            val showOpen = SpeedTrendUtil.minuteToTime(marketTime[0])
            canvas.drawText(showOpen, padding, height.toFloat(), mTextPaint)
        }
        if (marketTime[7] > 0) {
            val showClose = SpeedTrendUtil.minuteToTime(marketTime[7])
            canvas.drawText(showClose, width - padding - mTextPaint.measureText(showClose), height.toFloat(), mTextPaint)
        }
        if (marketTime[1] > 0 && marketTime[2] > 0) {
            val showSuspend = SpeedTrendUtil.minuteToTime(marketTime[1])
            val showResume = SpeedTrendUtil.minuteToTime(marketTime[2])
            val distance = (marketTime[1] - marketTime[0]) * timePerDifference
            val show = "$showSuspend/$showResume"
            canvas.drawText(show, distance - mTextPaint.measureText(show) / 2, height.toFloat(), mTextPaint)
        }
        if (!isHighTech && marketTime[3] > 0 && marketTime[4] > 0) {
            val showSuspend1 = SpeedTrendUtil.minuteToTime(marketTime[3])
            val showResume1 = SpeedTrendUtil.minuteToTime(marketTime[4])
            val distance1 = (marketTime[3] - marketTime[0]) * timePerDifference
            val show1 = "$showSuspend1/$showResume1"
            canvas.drawText(show1, distance1 - mTextPaint.measureText(show1) / 2, height.toFloat(), mTextPaint)
        }

        // 价格
        val middleShow = CommonFunc.doubleFormat(preClose.toDouble(), precision)
        val highShow = CommonFunc.doubleFormat(priceHigh.toDouble(), precision)
        val lowShow = CommonFunc.doubleFormat(priceLow.toDouble(), precision)
        val middlePercent = "0.00%"
        val highPercent = CommonFunc.doubleFormat((priceHigh - preClose) / preClose * 100.toDouble(), 2) + "%"
        val lowPercent = CommonFunc.doubleFormat((priceLow - preClose) / preClose * 100.toDouble(), 2) + "%"
        canvas.drawText(middleShow, padding, mPerHeight * 2 - padding, mTextPaint)
        canvas.drawText(middlePercent, width - padding - mTextPaint.measureText(middlePercent), mPerHeight * 2 - padding, mTextPaint)

        mTextPaint.color = StockUtil.getChangeColor(1f)
        canvas.drawText(highShow, padding, mTextHeight, mTextPaint)
        canvas.drawText(highPercent, width - padding - mTextPaint.measureText(highPercent), mTextHeight, mTextPaint)

        mTextPaint.color = StockUtil.getChangeColor(-1f)
        canvas.drawText(lowShow, padding, mPerHeight * 4 - padding, mTextPaint)
        canvas.drawText(lowPercent, width - padding - mTextPaint.measureText(lowPercent), mPerHeight * 4 - padding, mTextPaint)
    }

    /**
     * 绘制分时
     */
    private fun drawTrend(canvas: Canvas) {
        mTrendPath.reset()
        mAveragePath.reset()
        if (currentDataVec.size > 0) {
            var prePrice = preClose
            for (item in currentDataVec) {
                if (item is NewTrendDataItem) {
                    val times = getTradeMarket(item.tradeTime, false)
                    var x = times * timePerDifference
                    x = when {
                        x < 0.5f -> {
                            0.5f
                        }
                        x > width - 1f -> {
                            width - 1f
                        }
                        else -> {
                            x
                        }
                    }
                    val y = pricePerDifference * (priceHigh - item.newValue)
                    if (mTrendPath.isEmpty) {
                        mTrendPath.moveTo(x, y)
                    } else {
                        mTrendPath.lineTo(x, y)
                    }
                    if (item.avgPrice > 0) {
                        val averageY = pricePerDifference * (priceHigh - item.avgPrice)
                        if (mAveragePath.isEmpty) {
                            mAveragePath.moveTo(x, averageY)
                        } else {
                            mAveragePath.lineTo(x, averageY)
                        }
                    }
                    val startY = mPerHeight * 3 + (maxVolume - item.deltaVolum) * volumePerDifference
                    mVolumePaint.color = StockUtil.getChangeColor(item.newValue - prePrice)
                    mVolumePaint.alpha = 175
                    canvas.drawLine(x, startY, x, mPerHeight * 4, mVolumePaint)
                    prePrice = item.newValue
                }
            }
        }
        if (!mTrendPath.isEmpty) {
            canvas.drawPath(mTrendPath, mTrendPaint)
        }
        if (!mAveragePath.isEmpty) {
            canvas.drawPath(mAveragePath, mAveragePaint)
        }
    }


    /**
     * 用于字符高度计算
     *
     * @return
     * @exception
     * @since 1.0.0
     */
    private fun getChatHeight(paint: Paint): Float {
        val fm = paint.fontMetrics
        return fm.descent - fm.ascent
    }

    /**
     * 计算距离开市时间的分钟数/根据距离开市时间的分钟数反计算时间
     *
     * @param tradeTime
     * @param inverted
     * @return
     * @throws
     * @since 1.0.0
     */
    private fun getTradeMarket(tradeTime: Int, inverted: Boolean): Float {
        var tradeMarket: Float
        if (!inverted) {
            val time = SpeedTrendUtil.getTimeMinutes(tradeTime)
            if (marketTime[7] - marketTime[0] > 0 && (time < marketTime[0] || time > marketTime[7])) {
                // 收大于开 同时time小于开 代表未开盘 如A股
                // 收大于开 同时time大于收 代表盘后 如美股
                return (-2).toFloat()
            }
            tradeMarket = time - marketTime[0]
            if (tradeMarket < 0) {
                tradeMarket += 1440f
            }
            if (tradeMarket > convertTime[1] - marketTime[0]
                    && marketTime[1] != 0f) {
                if (tradeMarket < convertTime[2] - marketTime[0]
                        && marketTime[2] != 0f) {
                    return (-2).toFloat()
                }
                if (tradeMarket > convertTime[3] - marketTime[0]
                        && marketTime[3] != 0f) {
                    if (tradeMarket < convertTime[4] - marketTime[0]
                            && marketTime[4] != 0f) {
                        return (-2).toFloat()
                    }
                    tradeMarket -= if (tradeMarket > convertTime[5] - marketTime[0]
                            && marketTime[5] != 0f) {
                        if (tradeMarket < convertTime[6] - marketTime[0]
                                && marketTime[6] != 0f) {
                            return (-2).toFloat()
                        }
                        (convertTime[6] - convertTime[5]
                                + (convertTime[4] - convertTime[3])
                                + (convertTime[2] - convertTime[1]))
                    } else {
                        convertTime[2] - convertTime[1] +
                                convertTime[4] - convertTime[3]
                    }
                } else {
                    tradeMarket -= convertTime[2] - convertTime[1]
                }
            }
        } else {
            tradeMarket = marketTime[0] + tradeTime
            if (tradeMarket > convertTime[1] && marketTime[1] != 0f) {
                tradeMarket += convertTime[2] - convertTime[1]
                if (tradeMarket > convertTime[3] && marketTime[3] != 0f) {
                    tradeMarket += convertTime[4] - convertTime[3]
                    if (tradeMarket > convertTime[5] && marketTime[5] != 0f) {
                        tradeMarket += convertTime[6] - convertTime[5]
                    }
                }
            }
        }
        return tradeMarket
    }
}