package wind.android.bussiness.trade.abstocktrade.view

import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import cn.com.hh.trade.data.TagAns_Fun1044
import kotlinx.android.synthetic.main.view_transfer_query.view.*
import net.util.Assist
import net.util.TimeHelper
import ui.view.timepicker.TimePopupWindow
import util.DateUtil
import util.ToastTool
import wind.android.R
import wind.android.bussiness.trade.abstocktrade.adapter.AStockTransferAdapter
import wind.android.bussiness.trade.abstocktrade.manager.AbTradeManager
import wind.android.bussiness.trade.abstocktrade.manager.OnAbBankResultListener
import java.text.SimpleDateFormat

/**
 * # **********************************************************************************************
 * # ClassName:      TransferQueryView
 * # Description:    转账查询
 * # Author:         ylrong
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    ylrong     2019/11/26    14:26
 * # Modifications:  initial
 * # **********************************************************************************************
 */
class TransferQueryView @kotlin.jvm.JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : BaseAStockTradeView(context, attrs, defStyleAttr), View.OnClickListener {

    private val mAbTradeManager = AbTradeManager<Any>()

    private val mAdapter by lazy { AStockTransferAdapter(context) }

    private var mStartDatePick = true

    private lateinit var mTimePopupWindow: TimePopupWindow

    companion object {
        const val FMT_STR_SHOW = "yyyy-MM-dd"
        const val FMT_STR_REQ = "yyyyMMdd"
        const val DIFF_DAY = 89
    }

    private val format4Show = SimpleDateFormat(FMT_STR_SHOW)

    private val mOnAbBankResultListener by lazy {
        object : OnAbBankResultListener<Any> {
            override fun onBankInfo(obj: Any?) {
            }

            override fun onBankMoney(obj: Any?) {
            }

            override fun onBankTransfer(obj: Any?) {


            }

            @Suppress("UNCHECKED_CAST")
            override fun onCallback(data: Any?) {
                post {
                    val fun1044 = data as? Array<TagAns_Fun1044>
                    mAdapter.setData(fun1044)
                    if (fun1044.isNullOrEmpty()) {
                        tipInfo.visibility = VISIBLE
                    } else {
                        tipInfo.visibility = GONE
                    }
                }
            }

            override fun onError(error: Int, message: String?) {
                ToastTool.showToast(message)
            }
        }
    }

    override fun initView() {
        View.inflate(context, R.layout.view_transfer_query, this)
        todayTransfer.setBackgroundResource(R.drawable.a_stock_buy_edit_bg)
        todayTransfer.setOnClickListener(this)
        todayTransfer.visibility = View.GONE
        startTransferDate.setOnClickListener(this)
        endTransferDate.setOnClickListener(this)
        val today = TimeHelper.getInstance().getDate("yyyyMMdd").toInt()
        val preDay = DateUtil.getDayDiff(today.toString(), "yyyyMMdd", -DIFF_DAY).toInt()
        val endDate = Assist.getDateByFormat(today.toString(), FMT_STR_REQ, FMT_STR_SHOW)
        val startDate = Assist.getDateByFormat(preDay.toString(), FMT_STR_REQ, FMT_STR_SHOW)
        startTransferDate.text = startDate
        endTransferDate.text = endDate
        listView.adapter = mAdapter
        mTimePopupWindow = TimePopupWindow(context, TimePopupWindow.Type.YEAR_MONTH_DAY)
        mTimePopupWindow.setOnTimeSelectListener { _, date ->
            if (mStartDatePick) {
                startTransferDate.text = format4Show.format(date)
                val nextDayTime = DateUtil.getDayDiff(date, "yyyyMMdd", DIFF_DAY).toInt()
                val endDateTime = Assist.getDateByFormat(nextDayTime.toString(), FMT_STR_REQ, FMT_STR_SHOW)
                endTransferDate.text = endDateTime
            } else {
                endTransferDate.text = format4Show.format(date)
                val preDayTime = DateUtil.getDayDiff(date, "yyyyMMdd", -DIFF_DAY).toInt()
                val startDateTime = Assist.getDateByFormat(preDayTime.toString(), FMT_STR_REQ, FMT_STR_SHOW)
                startTransferDate.text = startDateTime
            }
            val start = Assist.getDateByFormat(startTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
            val end = Assist.getDateByFormat(endTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
            if (start.isNotEmpty() && end.isNotEmpty()) {
                mAbTradeManager.queryBankFlow(start.toInt(), end.toInt())
            } else {
                ToastTool.showToast(R.string.select_date)
            }
        }
    }

    override fun show() {
        mAbTradeManager.registerListener(mOnAbBankResultListener)
        val start = Assist.getDateByFormat(startTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
        val end = Assist.getDateByFormat(endTransferDate.text.toString(), FMT_STR_SHOW, FMT_STR_REQ)
        if (start.isNotEmpty() && end.isNotEmpty()) {
            mAbTradeManager.queryBankFlow(start.toInt(), end.toInt())
        }
    }

    override fun hide() {
        mAbTradeManager.unregisterListener(mOnAbBankResultListener)
    }

    override fun destroyView() {

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.todayTransfer -> {
                val today = TimeHelper.getInstance().getDate("yyyyMMdd").toInt()
                mAbTradeManager.queryBankFlow(today, today)
                // 重置起止时间
                val date = Assist.getDateByFormat(today.toString(), FMT_STR_REQ, FMT_STR_SHOW)
                startTransferDate.text = date
                endTransferDate.text = date
            }
            R.id.startTransferDate,
            R.id.endTransferDate -> {
                mStartDatePick = v.id == R.id.startTransferDate
                mTimePopupWindow.setTitle(resources.getString(R.string.select_date))
                val showText = (if (mStartDatePick) startTransferDate else endTransferDate).text.toString()
                val date =
                        if (showText.isNotEmpty()) {
                            format4Show.parse(showText)
                        } else {
                            TimeHelper.getInstance().curDate
                        }
                mTimePopupWindow.showAtLocation(this, Gravity.BOTTOM, 0, 0, date)
            }

        }
    }
}
