package wind.android.f5.bussiness.investor.activity

import android.content.Intent
import android.os.Bundle
import android.os.Message
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import base.BaseActivity
import base.BaseNetHandler
import com.jcodecraeer.xrecyclerview.ProgressStyle
import com.jcodecraeer.xrecyclerview.XRecyclerView.LoadingListener
import kotlinx.android.synthetic.main.activity_investor_qa_list.*
import util.ThemeUtils
import wind.android.f5.R
import wind.android.f5.bussiness.investor.adapter.InvestorQaListAdapter
import wind.android.f5.bussiness.investor.model.InvestorQaModel
import wind.android.f5.bussiness.investor.presenter.InvestorQaListPresenter
import wind.android.f5.bussiness.investor.view.IInvestorQaView

/**
 * # ***********************************************************************************************
 * # ClassName:      InvestorQaListActivity$
 * # Function:       投资者问答列表
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/12$    10:14$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class InvestorQaListActivity: BaseActivity(), IInvestorQaView {

    companion object {
        const val DATA_TYPE_WIND_CODE = "wind_code"

        const val MSG_SET_DATA = 1001
        const val MSG_ADD_DATA = 1002
        const val MSG_REFRESH_COMPLETE = 1003
        const val MSG_LOAD_MORE_COMPLETE = 1004
    }

    private val mPresenter by lazy { InvestorQaListPresenter(this) }

    private val mAdapter by lazy { InvestorQaListAdapter(this) }

    private var mWindCode: String? = null

    private var mPageIndex = 1

    override fun onCreate(savedInstanceState: Bundle?) {
        ThemeUtils.setTheme(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_investor_qa_list)
        navigationBar.setTitle("投资者问答")
        initData()
        initView()
    }

    private fun initView() {
        val layoutManager = LinearLayoutManager(this)
        layoutManager.orientation = LinearLayoutManager.VERTICAL
        investor_list_view.layoutManager = layoutManager
        investor_list_view.adapter = mAdapter
        investor_list_view.setLoadingMoreEnabled(true)
        investor_list_view.setPullRefreshEnabled(true)
        investor_list_view.setRefreshProgressStyle(ProgressStyle.BallSpinFadeLoader)
        investor_list_view.setLoadingMoreProgressStyle(ProgressStyle.BallClipRotate)
        investor_list_view.setArrowImageView(R.drawable.classic_pull_arrow)
        investor_list_view.setFootViewText("加载中...", "无更多数据")
        investor_list_view.setFooterViewTextColor(ThemeUtils.getColor(resources,
                R.color.txt_color_black_2, R.color.txt_color_white_2))
        investor_list_view.setHeaderViewTextColor(ThemeUtils.getColor(resources,
                R.color.txt_color_black_2, R.color.txt_color_white_2))
        investor_list_view.setLoadingListener(object : LoadingListener {
            override fun onLoadMore() {
                mPageIndex++
                requestData()
            }

            override fun onRefresh() {
                mPageIndex = 1
                requestData()
            }
        })
        investor_list_view.refresh()
        mAdapter.setOnItemClickListener {
            val tag = it.tag
            if (tag is Int) {
                val item = mAdapter.getItem(tag)
                item?.let {
                    model ->
                    val intent = Intent()
                    intent.setClass(this, InvestorQaDetailActivity::class.java)
                    intent.putExtra(InvestorQaDetailActivity.DATA_TYPE_INVESTOR_QA_DATA, model)
                    startActivity(intent)
                }
            }
        }
    }

    private fun initData() {
        mWindCode = intent?.getStringExtra(DATA_TYPE_WIND_CODE)
    }

    private fun requestData() {
        mPresenter.requestInvestorQa(mWindCode, mPageIndex)
    }

    private fun showEmpty() {
        if (investor_list_view.visibility == View.GONE) {
            return
        }
        investor_list_view.visibility = View.GONE
        investor_list_empty.visibility = View.VISIBLE
    }

    private fun hideEmpty() {
        if (investor_list_view.visibility == View.VISIBLE) {
            return
        }
        investor_list_view.visibility = View.VISIBLE
        investor_list_empty.visibility = View.GONE
    }

    override fun onSkyStatusChange(status: Int, detailMsg: String?) {
        super.onSkyStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            // 断网重连
            if (mAdapter.itemCount == 0) {
                // 当前没有数据，重新请求
                requestData()
            }
        }
    }


    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when(msg?.what) {
            MSG_SET_DATA -> {
                if (msg.obj != null) {
                    val list = msg.obj as? MutableList<InvestorQaModel>
                    mAdapter.setData(list)
                }
                hideEmpty()
                investor_list_view.refreshComplete()
            }
            MSG_ADD_DATA -> {
                if (msg.obj != null) {
                    val list = msg.obj as? MutableList<InvestorQaModel>
                    mAdapter.addData(list)
                }
                investor_list_view.loadMoreComplete()
            }
            MSG_REFRESH_COMPLETE -> {
                investor_list_view.refreshComplete()
                if (mAdapter.itemCount == 0) {
                    showEmpty()
                }
            }
            MSG_LOAD_MORE_COMPLETE -> {
                investor_list_view.loadMoreComplete()
            }
        }
    }

    override fun setInvestorQaData(list: MutableList<InvestorQaModel>?) {
        val msg = Message.obtain()
        msg.what = MSG_SET_DATA
        msg.obj = list
        sendMessage(msg)
    }

    override fun addInvestorQaData(list: MutableList<InvestorQaModel>?) {
        val msg = Message.obtain()
        msg.what = MSG_ADD_DATA
        msg.obj = list
        sendMessage(msg)
    }

    override fun refreshComplete() {
        val msg = Message.obtain()
        msg.what = MSG_REFRESH_COMPLETE
        sendMessage(msg)
    }

    override fun loadMoreComplete() {
        val msg = Message.obtain()
        msg.what = MSG_LOAD_MORE_COMPLETE
        sendMessage(msg)
    }
}