package wind.android.f5.bussiness.investor.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import util.NewsTimeAssist
import wind.android.f5.R
import wind.android.f5.bussiness.investor.model.InvestorQaModel

/**
 * # ***********************************************************************************************
 * # ClassName:      InvestorQaListAdapter$
 * # Function:       投资者问答列表-适配器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/12$    11:07$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class InvestorQaListAdapter(private val context: Context): RecyclerView.Adapter<InvestorQaListAdapter.MyViewHolder>() {

    private val mLayoutInflater by lazy { LayoutInflater.from(context) }

    private val mData = mutableListOf<InvestorQaModel>()

    private var mListener: ((View) -> Unit)? = null

    fun setOnItemClickListener(listener: (View) -> Unit) {
        mListener = listener
    }

    fun setData(list: MutableList<InvestorQaModel>?) {
        mData.clear()
        list?.let {
            mData.addAll(list)
        }
        notifyDataSetChanged()
    }

    fun addData(list: MutableList<InvestorQaModel>?) {
        list?.let {
            mData.addAll(list)
        }
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, type: Int): MyViewHolder {
        val view = mLayoutInflater.inflate(R.layout.item_investor_qa, viewGroup, false)
        val viewHolder = MyViewHolder(view)
        viewHolder.itemLayout.setOnClickListener(mListener)
        return viewHolder
    }

    override fun getItemCount(): Int {
        return mData.size
    }

    fun getItem(position: Int): InvestorQaModel? {
        return mData.elementAtOrNull(position)
    }

    override fun onBindViewHolder(viewHolder: MyViewHolder, position: Int) {
        val item = getItem(position)
        viewHolder.itemLayout.tag = position
        viewHolder.investorQaTime.text = getShowTime(item?._source?.getPublishDateValue())
        viewHolder.investorQaTitle.text = item?._source?.getUnitValue() ?: ""
    }

    private fun getShowTime(time: String?): String {
        if (time.isNullOrBlank()) {
            return ""
        }
        return NewsTimeAssist.getInstance().getNewsTime4Show(time.trim())
    }

    class MyViewHolder(view: View): RecyclerView.ViewHolder(view) {
        val itemLayout: View = view.findViewById(R.id.item_investor_qa_layout)
        val investorQaTime: TextView = view.findViewById(R.id.investor_qa_date)
        val investorQaTitle: TextView = view.findViewById(R.id.investor_qa_title)
    }
}