package wind.android.f5.bussiness.investor.presenter

import com.alibaba.fastjson.JSON
import log.LogHelper
import net.data.network.SkyCallbackData
import net.listener.ISkyDataListenerEx
import net.util.TimeHelper
import util.NewsTimeAssist
import wind.android.f5.bussiness.investor.model.InvestorHitsModel
import wind.android.f5.bussiness.investor.model.InvestorQaRequestModel
import wind.android.f5.bussiness.investor.view.IInvestorQaView
import wind.android.f5.bussiness.news.presenter.SpeedF5NewsPresenter
import wind.android.f5.model.business.SkySubject
import wind.android.news.anews.SkyNewsResponse
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      InvestorQaListPresenter$
 * # Function:       投资者问答列表请求处理器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/12$    10:16$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class InvestorQaListPresenter(private val mView: IInvestorQaView) {

    fun requestInvestorQa(windCode: String?, pageIndex: Int) {
        if (windCode.isNullOrBlank()) {
            stopRefresh(pageIndex)
            return
        }
        val endDate = TimeHelper.getInstance().getTimeEx(System.currentTimeMillis(), NewsTimeAssist.OCEAN_TIME_FMT_STR)
        val c = Calendar.getInstance()
        // 请求一年内投资者问答数据
        c.add(Calendar.YEAR, -1)
        val startDate = TimeHelper.getInstance().getTimeEx(c.timeInMillis, NewsTimeAssist.OCEAN_TIME_FMT_STR)
        val model = InvestorQaRequestModel()
        model.indexName = "investor_qa"
        model.dimension = windCode
        model.pageIndex = pageIndex
        model.pageNum = SpeedF5NewsPresenter.PAGE_SIZE
        model.fileds = "unit,unitinfo"
        model.hfields = "unit,unitinfo"
        val requestInvestorQa = SkySubject.requestInvestorQa(model, startDate, endDate, object : ISkyDataListenerEx {
            override fun OnSkyCallback(data: SkyCallbackData?) {
                try {
                    val elementAt = data?.data?.elementAtOrNull(0)
                    if (elementAt is SkyNewsResponse) {
                        val source = String(elementAt.a_bytes)
                        val sourceObject = JSON.parseObject(source) ?: null
                        sourceObject?.let {
                            val hits = it.getJSONObject("hits")
                            if (hits != null) {
                                val s = hits.toJSONString()
                                val investorHitsModel = JSON.parseObject(s, InvestorHitsModel::class.java)
                                        ?: null
                                val list = investorHitsModel?.hits
                                if (pageIndex == 1) {
                                    mView.setInvestorQaData(list)
                                } else {
                                    mView.addInvestorQaData(list)
                                }
                                return
                            }
                        }
                    }
                    stopRefresh(pageIndex)
                } catch (e: Exception) {
                    stopRefresh(pageIndex)
                }
            }

            override fun OnSkyError(code: Int, serialNum: Int) {
                LogHelper.print("InvestorQaListPresenter", "OnSkyError code:$code")
                stopRefresh(pageIndex)
            }
        })

        if ((requestInvestorQa == 0 || requestInvestorQa == -1 || requestInvestorQa == -2) && pageIndex == 1) {
            LogHelper.print("InvestorQaListPresenter", "OnSkyError requestInvestorQa:$requestInvestorQa")
            stopRefresh(pageIndex)
        }
    }

    private fun stopRefresh(pageIndex: Int) {
        if (pageIndex == 1) {
            mView.refreshComplete()
        } else {
            mView.loadMoreComplete()
        }
    }
}