package wind.android.f5.bussiness.news.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import util.NewsTimeAssist
import util.SkinUtil
import util.ThemeUtils
import wind.android.f5.R
import wind.android.news.anews.NewsTitleModel

/**
 * # ***********************************************************************************************
 * # ClassName:      SpeedNewsAdapter$
 * # Function:       单券-新闻列表适配器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/1/22$    14:37$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class SpeedNewsAdapter(val context: Context): RecyclerView.Adapter<SpeedNewsAdapter.NewsViewHolder>() {

    private val mData = mutableListOf<NewsTitleModel>()

    private val mDragonColor by lazy {ThemeUtils.getColor(context.resources, R.color.color_btn_yellow,
            R.color.color_btn_yellow)}

    private val mInvestorColor by lazy {
        ThemeUtils.getColor(context.resources, R.color.color_btn_red,
                R.color.color_btn_red)
    }

    private val mNewsColor = SkinUtil.getColor("news_titlecolor", -0x2d2d2e)

    private var mClickListener: ((View) -> Unit)? = null

    fun setItemClickListener(listener: (View) -> Unit) {
        mClickListener = listener
    }

    fun setData(list: MutableList<NewsTitleModel>?) {
        mData.clear()
        list?.let {
            mData.addAll(it)
            notifyDataSetChanged()
        }
    }

    fun addData(list: MutableList<NewsTitleModel>?) {
        list?.let {
            mData.addAll(it)
            notifyDataSetChanged()
        }
    }

    fun addData(model: NewsTitleModel, index: Int) {
        if (index in 0..mData.size) {
            mData.add(index, model)
        }
        notifyDataSetChanged()
    }

    override fun getItemCount(): Int {
        return mData.size
    }

    fun getItem(position: Int): NewsTitleModel? {
        return mData.elementAtOrNull(position)
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): SpeedNewsAdapter.NewsViewHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.item_speed_news, viewGroup, false)
        val viewHolder = NewsViewHolder(view)
        mClickListener?.let {
            viewHolder.newsLayout.setOnClickListener(it)
        }
        return NewsViewHolder(view)
    }

    override fun onBindViewHolder(viewHolder: SpeedNewsAdapter.NewsViewHolder, position: Int) {
        val item = getItem(position)
        item?.let {
            viewHolder.newsLayout.tag = position
            when {
                it.isInDragonList -> dealDragon(it, viewHolder)
                it.isInvestor -> dealInvestor(it, viewHolder)
                else -> dealNews(it, viewHolder)
            }
        }
    }

    class NewsViewHolder(itemView: View): RecyclerView.ViewHolder(itemView) {
        val newsLayout: View = itemView.findViewById(R.id.item_news_layout)
        val dateText: TextView = itemView.findViewById(R.id.news_date)
        val titleText: TextView = itemView.findViewById(R.id.news_title)
        val typeText: TextView = itemView.findViewById(R.id.news_type)
    }

    private fun getShowTime(time: String?): String {
        if (time.isNullOrBlank()) {
            return ""
        }
        return NewsTimeAssist.getInstance().getNewsTime4Show(time.trim()) ?: ""
    }

    private fun dealDragon(model: NewsTitleModel, viewHolder: SpeedNewsAdapter.NewsViewHolder) {
        viewHolder.typeText.visibility = View.VISIBLE
        viewHolder.typeText.setBackgroundResource(R.drawable.bg_yellow)
        viewHolder.typeText.text = "龙虎榜"
        viewHolder.dateText.visibility = View.GONE
        viewHolder.titleText.text = model.title ?: ""
        viewHolder.titleText.setTextColor(mDragonColor)
    }

    private fun dealInvestor(model: NewsTitleModel, viewHolder: SpeedNewsAdapter.NewsViewHolder) {
        viewHolder.typeText.visibility = View.VISIBLE
        viewHolder.typeText.setBackgroundResource(R.drawable.bg_red)
        viewHolder.typeText.text = "投资者问答"
        viewHolder.dateText.visibility = View.GONE
        viewHolder.titleText.text = model.title ?: ""
        viewHolder.titleText.setTextColor(mInvestorColor)
    }

    private fun dealNews(model: NewsTitleModel, viewHolder: SpeedNewsAdapter.NewsViewHolder) {
        viewHolder.typeText.visibility = View.GONE
        viewHolder.dateText.visibility = View.VISIBLE
        viewHolder.dateText.text = getShowTime(model.newsTime)
        viewHolder.titleText.text = model.title ?: ""
        viewHolder.titleText.setTextColor(mNewsColor)
    }

}