package wind.android.f5.bussiness.news.fragment

import android.content.Intent
import android.os.Message
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import base.BaseNetHandler
import com.jcodecraeer.xrecyclerview.ProgressStyle
import com.jcodecraeer.xrecyclerview.XRecyclerView
import kotlinx.android.synthetic.main.fragment_speed_news.*
import log.BaseApplication
import net.datamodel.speed.WindCodeType
import util.CommonValue
import util.NewsActionStatisticsTool
import util.ThemeUtils
import wind.android.f5.R
import wind.android.f5.bussiness.investor.activity.InvestorQaListActivity
import wind.android.f5.bussiness.news.adapter.SpeedNewsAdapter
import wind.android.f5.bussiness.news.presenter.SpeedF5NewsPresenter
import wind.android.f5.bussiness.news.view.ISpeedNewsView
import wind.android.f5.util.HttpUrlUtils_F5
import wind.android.f5.view.bottom.fragment.BaseF5BottomFrag
import wind.android.news.NewsDetailActivity
import wind.android.news.NewsSession
import wind.android.news.anews.NewsConstants
import wind.android.news.anews.NewsDetilToNextModel
import wind.android.news.anews.NewsTitleModel

/**
 * # ***********************************************************************************************
 * # ClassName:      SpeedNewsFragment$
 * # Function:       F5-新闻列表
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/1/22$    10:54$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class F5NewsFragment: BaseF5BottomFrag(), ISpeedNewsView {

    companion object {
        const val MSG_SET_NEWS = 1000
        const val MSG_UPDATE_NEWS = 1001
        const val MSG_UPDATE_DRAGON_DATA = 1002
        const val MSG_UPDATE_INVESTOR = 1003
        const val MSG_LOAD_MORE_COMPLETE = 1005
    }

    private val mPresenter by lazy { SpeedF5NewsPresenter(this) }

    private val mAdapter by lazy { SpeedNewsAdapter(context!!) }

    private var mPageIndex = 1

    override fun getLayoutId(): Int {
        return R.layout.fragment_speed_news
    }

    override fun init() {
        speed_news_list_view.adapter = mAdapter
        speed_news_list_view.setRefreshProgressStyle(ProgressStyle.BallSpinFadeLoader)
        speed_news_list_view.setLoadingMoreProgressStyle(ProgressStyle.BallClipRotate)
        speed_news_list_view.setArrowImageView(R.drawable.classic_pull_arrow)
        speed_news_list_view.setFootViewText("加载中...", "无更多数据")
        speed_news_list_view.setFooterViewTextColor(ThemeUtils.getColor(resources,
                R.color.txt_color_black_2, R.color.txt_color_white_2))
        speed_news_list_view.setHeaderViewTextColor(ThemeUtils.getColor(resources,
                R.color.txt_color_black_2, R.color.txt_color_white_2))
        speed_news_list_view.setPullRefreshEnabled(false)
        speed_news_list_view.setLoadingMoreEnabled(true)
        val layoutManager = LinearLayoutManager(context)
        layoutManager.orientation = LinearLayoutManager.VERTICAL
        speed_news_list_view.layoutManager = layoutManager
        mAdapter.setItemClickListener{
            val tag = it.tag
            if (tag is Int) {
                val item = mAdapter.getItem(tag)
                when {
                    item?.isInDragonList ?: false -> {
                        jumpToDragon()
                    }
                    item?.isInvestor ?: false -> {
                        jumpToInvestorQA()
                    }
                    else -> {
                        jumpToNewsDetail(item, tag)
                    }
                }
            }
        }
        speed_news_list_view.setLoadingListener(object : XRecyclerView.LoadingListener{
            override fun onLoadMore() {
                mPageIndex++
                requestData()
            }

            override fun onRefresh() {
            }
        })
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mPresenter.onDestoryView()
    }

    override fun onFragHide() {

    }

    override fun onFragShow() {
        if (mAdapter.itemCount == 0) {
            // 当前没有数据，重新请求
            requestData()
        }
    }

    private fun showEmpty() {
        if (speed_news_list_view.visibility == View.GONE) {
            return
        }
        speed_news_list_view.visibility = View.GONE
        speed_news_empty_view.visibility = View.VISIBLE
    }

    private fun hideEmpty() {
        if (speed_news_list_view.visibility == View.VISIBLE) {
            return
        }
        speed_news_list_view.visibility = View.VISIBLE
        speed_news_empty_view.visibility = View.GONE
    }

    private fun requestData() {
        mPresenter.requestNews(mWindCode, mPageIndex)
    }

    override fun onSkyStatusChange(status: Int, detailMsg: String?) {
        super.onSkyStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            // 断网重连
            if (mAdapter.itemCount == 0) {
                // 当前没有数据，重新请求
                requestData()
            }
        }
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when(msg?.what) {
            MSG_SET_NEWS -> {
                msg.obj?.let {
                    val list = it as? MutableList<NewsTitleModel>
                    mAdapter.setData(list)
                }
                if (mAdapter.itemCount > 0) {
                    hideEmpty()
                } else {
                    showEmpty()
                }
            }
            MSG_UPDATE_NEWS -> {
                msg.obj?.let {
                    val list = it as? MutableList<NewsTitleModel>
                    mAdapter.addData(list)
                }
                speed_news_list_view.loadMoreComplete()
            }
            MSG_UPDATE_DRAGON_DATA -> {
                msg.obj?.let {
                    val model = it as NewsTitleModel
                    mAdapter.addData(model, 0)
                }
            }
            MSG_UPDATE_INVESTOR -> {
                msg.obj?.let {
                    val model = it as NewsTitleModel
                    mAdapter.addData(model, 0)
                }
            }
            MSG_LOAD_MORE_COMPLETE -> {
                speed_news_list_view.loadMoreComplete()
            }
        }
    }

    override fun updateNewsList(list: MutableList<NewsTitleModel>) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_NEWS
        msg.obj = list
        sendMessage(msg)
    }

    override fun setNewsList(list: MutableList<NewsTitleModel>) {
        val msg = Message.obtain()
        msg.what = MSG_SET_NEWS
        msg.obj = list
        sendMessage(msg)
    }

    override fun updateDragonData(model: NewsTitleModel) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_DRAGON_DATA
        msg.obj = model
        sendMessage(msg)
    }

    override fun updateInvestor(model: NewsTitleModel) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_INVESTOR
        msg.obj = model
        sendMessage(msg)
    }

    override fun loadMoreComplete() {
        val msg = Message.obtain()
        msg.what = MSG_LOAD_MORE_COMPLETE
        sendMessage(msg)
    }

    private fun jumpToDragon() {
        val intent = Intent("wind.android.ConfigSettingActivity")
        intent.setPackage(BaseApplication.getInstance().packageName)
        intent.putExtra("stock_commom_name", "龙虎榜")
        var skin = "white"
        if (CommonValue.styleType == CommonValue.StyleType.STYLE_BLACK) {
            skin = "black"
        }
        intent.putExtra(
                "stock_commom_url",
                String.format(
                        HttpUrlUtils_F5.FIND_TOP_TEN_JUMP_URL,
                        mWindCode, WindCodeType.getStockInfo(mWindCode).shortName, skin))
        startActivity(intent)
    }

    private fun jumpToNewsDetail(model: NewsTitleModel?, position: Int) {
        model?.let {
            item ->
            context?.let {
                val newsNextModel = NewsDetilToNextModel()
                newsNextModel.newsTitleId = item.itemId
                newsNextModel.newsRankId = 0
                newsNextModel.newsmodel = NewsDetilToNextModel.newsModel
                newsNextModel.preActivity = "OptionalStockActivity"
                val newsList = mutableListOf<NewsTitleModel>()
                newsList.add(item)
                NewsSession.getInstance().newsList = newsList
                val intent = Intent(it, NewsDetailActivity::class.java)
                intent.putExtra("NewsDetilToNextModel", newsNextModel)
                intent.putExtra("gateway", NewsActionStatisticsTool.Enter_By_F5)
                intent.setPackage(it.packageName)
                intent.putExtra(NewsDetailActivity.TYPE_COLUMN, NewsConstants.OCEAN_ID_F5)
                startActivity(intent)
            }
        }
    }

    private fun jumpToInvestorQA() {
        context?.let {
            val intent = Intent(it, InvestorQaListActivity::class.java)
            intent.setPackage(it.packageName)
            intent.putExtra(InvestorQaListActivity.DATA_TYPE_WIND_CODE, mWindCode)
            startActivity(intent)
        }

    }
}