package wind.android.f5.view.bottom.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseExpandableListAdapter
import android.widget.TextView
import log.LogHelper
import net.util.Assist
import wind.android.f5.R
import wind.android.news.anews.NewsTitleModel

/**
 * # ***********************************************************************************************
 * # ClassName:      F5BulletAdapter$
 * # Function:       单券-底部-公告列表-适配器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/14$    15:05$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class F5BulletAdapter(val context: Context): BaseExpandableListAdapter() {

    private val mGroupTitle = listOf("大事", "公告")

    private val mEventList = mutableListOf<NewsTitleModel>()

    private val mBulletList = mutableListOf<NewsTitleModel>()

    private val mLayoutInflater by lazy { LayoutInflater.from(context) }

    private var mMoreClickListener: ((View) -> Unit)? = null

    fun setEvent(list: MutableList<NewsTitleModel>?) {
        list?.let {
            mEventList.clear()
            mEventList.addAll(it)
        }
        LogHelper.print("F5BulletAdapter", "setEvent size:${mEventList.size}")
        notifyDataSetChanged()
    }

    fun setBullet(list: MutableList<NewsTitleModel>?) {
        list?.let {
            mBulletList.clear()
            mBulletList.addAll(it)
        }
        LogHelper.print("F5BulletAdapter", "setBullet size:${mBulletList.size}")
        notifyDataSetChanged()
    }

    fun updateBullet(list: MutableList<NewsTitleModel>?) {
        list?.let {
            mBulletList.addAll(it)
        }
        notifyDataSetChanged()
    }

    fun setMoreClickListener(listener: (View) -> Unit) {
        mMoreClickListener = listener
    }

    override fun getGroup(groupPosition: Int): String {
        return mGroupTitle.elementAtOrNull(groupPosition) ?: ""
    }

    override fun isChildSelectable(groupPosition: Int, childPosition: Int): Boolean {
        return true
    }

    override fun hasStableIds(): Boolean {
        return true
    }

    override fun getGroupView(groupPosition: Int, isExpanded: Boolean, convertView: View?, parent: ViewGroup?): View {
        val view: View
        val groupViewHolder: GroupViewHolder
        if (convertView == null) {
            view = mLayoutInflater.inflate(R.layout.item_f5_bullet_group, parent, false)
            groupViewHolder = GroupViewHolder(view)
            groupViewHolder.groupMore.setOnClickListener(mMoreClickListener)
            view.tag = groupViewHolder
        } else {
            groupViewHolder = convertView.tag as GroupViewHolder
            view = convertView
        }
        val group = getGroup(groupPosition)
        groupViewHolder.groupMore.tag = groupPosition
        groupViewHolder.groupTitleView.text = group
        return view
    }

    override fun getChildrenCount(groupPosition: Int): Int {
        return if (groupPosition == 0) {
            mEventList.size
        } else {
            mBulletList.size
        }
    }

    override fun getChild(groupPosition: Int, childPosition: Int): NewsTitleModel? {
        return if (groupPosition == 0) {
            mEventList.elementAtOrNull(childPosition)
        } else{
            mBulletList.elementAtOrNull(childPosition)
        }
    }

    override fun getGroupId(groupPosition: Int): Long {
        return groupPosition.toLong()
    }

    override fun getChildView(groupPosition: Int, childPosition: Int, isLastChild: Boolean, convertView: View?, parent: ViewGroup?): View {
        val view: View
        val childViewHolder: ChildViewHolder
        if (convertView == null) {
            view = mLayoutInflater.inflate(R.layout.item_f5_bullet_child, parent, false)
            childViewHolder = ChildViewHolder(view)
            view.tag = childViewHolder
        } else {
            childViewHolder = convertView.tag as ChildViewHolder
            view = convertView
        }
        val model = getChild(groupPosition, childPosition)
        childViewHolder.childDate.text = getShowTime(model)
        childViewHolder.childTitle.text = model?.title
        return view
    }

    override fun getChildId(groupPosition: Int, childPosition: Int): Long {
        return childPosition.toLong()
    }

    override fun getGroupCount(): Int {
        return mGroupTitle.size
    }

    private fun getShowTime(model: NewsTitleModel?): String? {
        return model?.let {
            if (it.type == NewsTitleModel.WMPUSH_EVENT) {
                Assist.getDateByFormat2(it.newsTime, "yyyy-MM-dd", "MM-dd")
            } else {
                Assist.getDateByFormat2(it.newsTime, "yyyy-MM-dd hh:mm:sss", "MM-dd")
            }
        }
    }

    class GroupViewHolder(view: View) {
        val groupTitleView: TextView = view.findViewById(R.id.bullet_title_group)
        val groupMore: TextView = view.findViewById(R.id.bullet_more_group)
    }

    class ChildViewHolder(view: View) {
        val childDate: TextView = view.findViewById(R.id.f5_bullet_date)
        val childTitle: TextView = view.findViewById(R.id.f5_bullet_title)
    }
}