package wind.android.f5.view.bottom.adapter

import android.graphics.Color
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import util.NewsTimeAssist
import util.SkinUtil
import wind.android.f5.R
import wind.android.news2.model.NewsBaseInfo
import wind.android.news2.util.StockUtil
import wind.android.news2.util.Tool

/**
 * # **********************************************************************************************
 * # ClassName:      F5IndexLiveAdapter
 * # Description:    WindStockDevNew
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2020/2/21    15:34
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class F5IndexLiveAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    /**
     * 数据源
     */
    private val data: ArrayList<NewsBaseInfo> by lazy { arrayListOf<NewsBaseInfo>() }
    /**
     * 点击事件
     */
    var onItemClickListener: ((Int) -> Unit)? = null

    /**
     * 设置数据并刷新列表
     *
     * @param list
     */
    fun setData(list: List<NewsBaseInfo>?) {
        data.clear()
        list?.let { data.addAll(it) }
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val view = LayoutInflater.from(viewGroup.context).inflate(R.layout.item_stock_index_news,
                viewGroup, false)
        return IndexLiveVH(view, onItemClickListener)
    }

    override fun getItemCount() = data.size

    override fun onBindViewHolder(p0: RecyclerView.ViewHolder, position: Int) {
        val viewHolder = p0 as IndexLiveVH
        val item = data.elementAtOrNull(position)
        item?.apply {
            viewHolder.indexNewsTime.text = getTime4Show(itemDate)
            viewHolder.indexNewsTitle.text = abstractStr
            val titleColor = if (StockUtil.isRead(itemId)) Color.GRAY else
                SkinUtil.getColor("news_titlecolor", 0XFFD2D2D2.toInt())
            viewHolder.indexNewsTitle.setTextColor(titleColor)
        }
        viewHolder.itemView.tag = position
    }

    /**
     * 更新已读标题
     *
     * @param viewHolder
     */
    fun updateItemRead(viewHolder: RecyclerView.ViewHolder?){
        val indexLiveVH = viewHolder as? IndexLiveVH
        indexLiveVH?.indexNewsTitle?.setTextColor(Color.GRAY)
    }

    /**
     * 当天，则显示HH:mm格式；否则显示MM-dd HH:mm格式
     */
    private fun getTime4Show(time: String?): String? {
        return if (!Tool.isCurrentDate(time)) {
            NewsTimeAssist.getInstance().getNewsTime4Show(time, NewsTimeAssist.DAY_AND_MINUTE_FORMAT)
        } else {
            NewsTimeAssist.getInstance().getNewsTime4Show(time, NewsTimeAssist.TIME_SHOW_FMT_STR)
        }
    }

    /**
     * ViewHolder
     */
    class IndexLiveVH(itemView: View, onItemClickListener: ((Int) -> Unit)?) : RecyclerView.ViewHolder(itemView) {
        val indexNewsTime: TextView by lazy { itemView.findViewById<TextView>(R.id.index_news_time) }
        val indexNewsTitle: TextView by lazy { itemView.findViewById<TextView>(R.id.index_news_title) }

        init {
            itemView.setOnClickListener {
                val position = it.tag as Int
                onItemClickListener?.invoke(position)
            }
        }
    }
}