package wind.android.f5.view.bottom.adapter

import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import net.datamodel.network.CommonFunc
import util.ThemeUtils
import wind.android.f5.R
import wind.android.f5.model.IndexComponetModel
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB0
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB1
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB2
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB3
import wind.android.news.anews.StockUtil

/**
 * # **********************************************************************************************
 * # ClassName:      F5IndexPlateAdapter
 * # Description:    大盘指数F5页面板块tab列表adapter
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2020/2/26    14:56
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class F5IndexPlateAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    /**
     * 数据源
     */
    private val data: ArrayList<IndexComponetModel> by lazy { arrayListOf<IndexComponetModel>() }
    /**
     * item点击事件
     */
    var onItemClickListener: OnItemClickListener? = null
    /**
     * 数据tab类型，控制数据展示类型
     */
    private var dataType = DATA_TAB0

    /**
     * 设置数据
     *
     * @param list
     */
    fun setData(list: List<IndexComponetModel>?, dataTabType: Int) {
        data.clear()
        list?.let { data.addAll(list) }
        dataType = dataTabType
    }

    /**
     * 设置点击事件
     *
     * @param listenerInit
     */
    fun setOnItemClickListenerFun(listenerInit: ItemClickWrap.() -> Unit) {
        val itemClickWrap = ItemClickWrap()
        itemClickWrap.listenerInit()
        onItemClickListener = itemClickWrap
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        if (viewType == VIEW_FOOTER_TYPE) {
            val view = LayoutInflater.from(viewGroup.context).inflate(R.layout.f5_index_foot, viewGroup, false)
            return IndexFooterVH(view, onItemClickListener)
        } else {
            val view = LayoutInflater.from(viewGroup.context).inflate(R.layout.f5_index_plate_item, viewGroup, false)
            return IndexPlateVH(view, onItemClickListener)
        }
    }

    override fun getItemCount() = data.size + 1

    override fun onBindViewHolder(vh: RecyclerView.ViewHolder, position: Int) {
        if (getItemViewType(position) == VIEW_ITEM_TYPE) {
            val viewHolder = vh as IndexPlateVH
            val item = data.elementAtOrNull(position)
            item?.apply {
                viewHolder.plateName.text = stockName
                when (dataType) {
                    DATA_TAB0, DATA_TAB1 -> {
                        val changeRateValue = "${CommonFunc.doubleFormat(changeRange.toDouble(), 2)}%"
                        viewHolder.changeRate.text = changeRateValue
                        viewHolder.changeRate.setTextColor(StockUtil.getChangeColor(changeRange))
                    }
                    DATA_TAB2, DATA_TAB3 -> {
                        val changeRateValue = "${CommonFunc.doubleFormat(change5Min.toDouble(), 2)}%"
                        viewHolder.changeRate.text = changeRateValue
                        viewHolder.changeRate.setTextColor(StockUtil.getChangeColor(change5Min))
                    }
                }
                viewHolder.itemView.tag = windCode
            }
        }
    }

    override fun getItemViewType(position: Int) = if (position == data.size)
        VIEW_FOOTER_TYPE else VIEW_ITEM_TYPE

    /**
     * 列表item ViewHolder
     */
    private class IndexPlateVH(itemView: View, itemClickListener: OnItemClickListener?) : RecyclerView.ViewHolder(itemView) {
        val plateName: TextView by lazy { itemView.findViewById<TextView>(R.id.plate_name) }
        val changeRate: TextView by lazy { itemView.findViewById<TextView>(R.id.change_rate) }

        init {
            // 设置 selector
            ThemeUtils.setViewBgByDrawableID(itemView,
                    R.drawable.list_selector_b, R.drawable.list_selector_w)
            itemView.setOnClickListener { itemClickListener?.onItemClick(it.tag as? String) }
        }
    }

    /**
     * Footer ViewHolder
     */
    private class IndexFooterVH(itemView: View, itemClickListener: OnItemClickListener?) : RecyclerView.ViewHolder(itemView) {
        init {
            itemView.setOnClickListener { itemClickListener?.onFooterClick() }
        }
    }

    /**
     * 回调接口
     */
    interface OnItemClickListener {
        fun onItemClick(windCode: String?)
        fun onFooterClick()
    }

    /**
     * kotlin高阶函数实现多方法回调
     */
    class ItemClickWrap : OnItemClickListener {
        private var itemClick: ((String?) -> Unit)? = null
        private var footerClick: (() -> Unit)? = null
        fun itemClickFun(itemClickFun: ((String?) -> Unit)?) {
            itemClick = itemClickFun
        }

        fun footerClickFun(footerClickFun: (() -> Unit)?) {
            footerClick = footerClickFun
        }

        override fun onItemClick(windCode: String?) {
            itemClick?.invoke(windCode)
        }

        override fun onFooterClick() {
            footerClick?.invoke()
        }
    }

    companion object {
        private const val VIEW_ITEM_TYPE = 0
        private const val VIEW_FOOTER_TYPE = 1
    }
}