package wind.android.f5.view.bottom.fragment

import android.content.Intent
import android.os.Message
import android.view.View
import android.widget.ExpandableListView
import android.widget.ExpandableListView.OnChildClickListener
import android.widget.ExpandableListView.OnGroupClickListener
import base.BaseNetHandler
import business.report.AttachInfo
import kotlinx.android.synthetic.main.fragment_f5_bullet.*
import net.download.DownloadFile
import ui.bell.listview.DragRefreshExpandableListView.DragRefreshListViewListener
import ui.bell.listview.ListViewFooter
import util.StringUtils
import wind.android.f5.R
import wind.android.f5.activity.MoreAnnouncementActivity
import wind.android.f5.activity.MoreEventActivity
import wind.android.f5.view.bottom.adapter.F5BulletAdapter
import wind.android.f5.view.bottom.presenter.F5BulletPresenter
import wind.android.f5.view.bottom.subview.IF5BulletView
import wind.android.news.NewsDetailActivity
import wind.android.news.NewsSession
import wind.android.news.anews.NewsConstants
import wind.android.news.anews.NewsDetilToNextModel
import wind.android.news.anews.NewsTitleModel
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      F5BulletFragment$
 * # Function:       单券-底部-公告列表
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/14$    13:55$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class F5BulletFragment: BaseF5BottomFrag(), DragRefreshListViewListener,
        OnChildClickListener, OnGroupClickListener, IF5BulletView {

    companion object {
        const val MSG_UPDATE_EVENT = 1001
        const val MSG_SET_BULLET = 1002
        const val MSG_UPDATE_BULLET = 1003
    }

    private val mAdapter by lazy { F5BulletAdapter(context!!) }

    private val mPresenter by lazy { F5BulletPresenter(this) }

    private var mPageIndex = 1

    override fun getLayoutId(): Int {
        return R.layout.fragment_f5_bullet
    }

    override fun init() {
        f5_bullet_list_view.setHeaderViewEnable(false)
        f5_bullet_list_view.setFooterViewState(ListViewFooter.STATE_HIDE)
        f5_bullet_list_view.setGroupIndicator(null)
        f5_bullet_list_view.setDragRefreshListViewListener(this)
        f5_bullet_list_view.setOnChildClickListener(this)
        f5_bullet_list_view.setOnGroupClickListener(this)
        f5_bullet_list_view.setAdapter(mAdapter)
        mAdapter.setMoreClickListener {
            val tag = it.tag
            if (tag is Int) {
                if (tag == 0) {
                    context?.let {
                        packageContext ->
                        val intent = Intent()
                        intent.setClass(packageContext, MoreEventActivity::class.java)
                        intent.putExtra("windcode", mWindCode)
                        startActivity(intent)
                    }
                } else {
                    context?.let {
                        packageContext ->
                        val intent = Intent()
                        intent.setClass(packageContext, MoreAnnouncementActivity::class.java)
                        intent.putExtra("windcode", mWindCode)
                        startActivity(intent)
                    }
                }
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mPresenter.onDestoryView()
    }

    private fun requestEvent() {
        mPresenter.requestEvent(mWindCode)
    }

    private fun requestBullet() {
        mPresenter.requestBullet(mWindCode, mPageIndex)
    }

    override fun onRefresh() {
    }

    override fun onLoadMore() {
        mPageIndex++
        requestBullet()
    }

    override fun onSkyStatusChange(status: Int, detailMsg: String?) {
        super.onSkyStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            // 断网重连
            val groupCount = mAdapter.groupCount
            var childCount = 0
            for (i in 0 until groupCount) {
                childCount += mAdapter.getChildrenCount(i)
            }
            if (childCount == 0) {
                // 当前没有数据，重新请求
                requestEvent()
            }
        }
    }

    override fun onChildClick(parent: ExpandableListView?, v: View?, groupPosition: Int, childPosition: Int, id: Long): Boolean {
        val newsTitleModel = mAdapter.getChild(groupPosition, childPosition)
        if (newsTitleModel?.type == NewsTitleModel.WMPUSH_EVENT) {
            jumpEventDetail(newsTitleModel)
        } else if (newsTitleModel?.type == NewsTitleModel.WMPUSH_BULLET) {
            jumpBulletDetail(newsTitleModel)
        }
        return true
    }

    override fun onGroupClick(parent: ExpandableListView?, v: View?, groupPosition: Int, id: Long): Boolean {
        return true
    }

    private fun jumpEventDetail(newsTitleModel: NewsTitleModel) {
        val model = NewsDetilToNextModel()
        StockUtil.addNewsID(newsTitleModel.newsId)
        model.newsTitleId = newsTitleModel.newsId
        model.preActivity = "_eventActivity"
        model.newsmodel = NewsDetilToNextModel.eventModel
        gotoDetail(newsTitleModel, model)
    }

    private fun jumpBulletDetail(newsTitleModel: NewsTitleModel) {
        val model = NewsDetilToNextModel()
        StockUtil.addNewsID(newsTitleModel.newsId)
        model.newsTitleId = newsTitleModel.newsId
        model.preActivity = "OptionalStockActivity"
        when {
            newsTitleModel.attechmentList?.isEmpty() != false -> mPresenter.getBulletinDetail(newsTitleModel)
            newsTitleModel.attechmentList.size == 1 -> downloadBullet(newsTitleModel.attechmentList)
            else -> {
                model.newsmodel = NewsDetilToNextModel.bulletModel
                gotoDetail(newsTitleModel, model)
            }
        }
    }

    private fun gotoDetail(newsTitleModel: NewsTitleModel, model: NewsDetilToNextModel) {
        context?.let {
            val intent = Intent(it, NewsDetailActivity::class.java)
            NewsSession.getInstance().newsList = mutableListOf(newsTitleModel)
            model.newsRankId = 0
            intent.putExtra("NewsDetilToNextModel", model)
            intent.setPackage(it.packageName)
            intent.putExtra(NewsDetailActivity.TYPE_COLUMN, NewsConstants.OCEAN_ID_OPTIONAL)
            startActivity(intent)
        }
    }

    override fun onFragHide() {

    }

    override fun onFragShow() {
        val groupCount = mAdapter.groupCount
        var childCount = 0
        for (i in 0 until groupCount) {
            childCount += mAdapter.getChildrenCount(i)
        }
        if (childCount == 0) {
            // 当前没有数据，重新请求
            requestEvent()
        }
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when(msg?.what) {
            MSG_UPDATE_EVENT -> {
                msg.obj?.let {
                    val list = it as? MutableList<NewsTitleModel>
                    mAdapter.setEvent(list)
                    val groupCount = mAdapter.groupCount
                    for (i in 0 until groupCount) {
                        f5_bullet_list_view.expandGroup(i)
                    }
                }
            }
            MSG_SET_BULLET -> {
                msg.obj?.let {
                    val list = it as? MutableList<NewsTitleModel>
                    mAdapter.setBullet(list)
                }
                if (msg.obj == null) {
                    f5_bullet_list_view.setFooterViewState(ListViewFooter.STATE_NORMAL,
                            StringUtils.getString(R.string.listview_footer_result))
                } else {
                    f5_bullet_list_view.setFooterViewState(ListViewFooter.STATE_NORMAL)
                }
                f5_bullet_list_view.refreshComplete(true, System.currentTimeMillis())
            }
            MSG_UPDATE_BULLET -> {
                msg.obj?.let {
                    val list = it as? MutableList<NewsTitleModel>
                    mAdapter.updateBullet(list)
                }
                if (msg.obj == null) {
                    f5_bullet_list_view.setFooterViewState(ListViewFooter.STATE_NORMAL,
                            StringUtils.getString(R.string.listview_footer_result))
                } else {
                    f5_bullet_list_view.setFooterViewState(ListViewFooter.STATE_NORMAL)
                }
            }
        }
    }

    override fun updateEvent(list: MutableList<NewsTitleModel>) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_EVENT
        msg.obj = list
        sendMessage(msg)
    }

    override fun setBullet(list: MutableList<NewsTitleModel>?) {
        val msg = Message.obtain()
        msg.what = MSG_SET_BULLET
        msg.obj = list
        sendMessage(msg)
    }

    override fun updateBullet(list: MutableList<NewsTitleModel>?) {
        val msg = Message.obtain()
        msg.what = MSG_UPDATE_BULLET
        msg.obj = list
        sendMessage(msg)
    }

    override fun doBulletJump(newsTitleModel: NewsTitleModel) {
        val model = NewsDetilToNextModel()
        model.newsTitleId = newsTitleModel.newsId
        model.preActivity = "OptionalStockActivity"
        model.newsmodel = NewsDetilToNextModel.bulletModel
        gotoDetail(newsTitleModel, model)
    }

    override fun doDownloadBullet(newsTitleModel: NewsTitleModel) {
        downloadBullet(newsTitleModel.attechmentList)
    }

    private fun downloadBullet(list: ArrayList<AttachInfo>) {
        val attachInfo = list.elementAtOrNull(0)
        val down = DownloadFile(activity)
        down.setAttchInfo(attachInfo, false)
    }
}