package wind.android.f5.view.bottom.fragment

import android.content.Intent
import android.graphics.Color
import android.os.Message
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import com.jcodecraeer.xrecyclerview.ProgressStyle
import com.jcodecraeer.xrecyclerview.XRecyclerView
import kotlinx.android.synthetic.main.f5_index_live_frag.*
import net.datamodel.network.RealQuoteItem
import net.datamodel.speed.ShortElfItem
import util.NewsActionStatisticsTool
import util.ThemeUtils
import wind.android.f5.R
import wind.android.f5.model.IndexComponetModel
import wind.android.f5.view.bottom.adapter.F5IndexLiveAdapter
import wind.android.f5.view.presenter.StockIndexPresenter
import wind.android.news2.NewsDetailActivity
import wind.android.news2.common.NewsConstants
import wind.android.news2.model.NewsBaseInfo
import wind.android.news2.util.StockUtil
import wind.android.news2.util.control.BaseViewControllerManager
import java.util.*

/**
 * # **********************************************************************************************
 * # ClassName:      F5IndexLiveFrag
 * # Description:    上证、深证、创业板 F5页面直播tab
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2020/2/21    14:14
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class F5IndexLiveFrag: BaseF5BottomFrag() {
    /**
     * 列表数据
     */
    private val data: ArrayList<NewsBaseInfo> by lazy { arrayListOf<NewsBaseInfo>() }
    /**
     * 页码
     */
    private var pageNum = 0
    /**
     * adapter
     */
    private val adapter: F5IndexLiveAdapter by lazy { F5IndexLiveAdapter() }
    /**
     * 请求管理器
     */
    private val manager: StockIndexPresenter by lazy {
        StockIndexPresenter(object : StockIndexPresenter.IRequestCallback{
            override fun onComponentBack(list: MutableList<IndexComponetModel>?, index: Int, zeroNotSub: Int) {
            }

            override fun onElfBack(elfList: MutableList<ShortElfItem>?) {
            }

            override fun onPlateBack(items: Vector<RealQuoteItem>?) {
            }

            override fun onNewsBack(list: MutableList<NewsBaseInfo>?) {
                val message = Message.obtain()
                message.what = UPDATE_LIST_MSG
                message.obj = list
                sendMessage(message)
            }

            override fun onNewsBackFailed() {
                val message = Message.obtain()
                message.what = ERROR_MSG
                sendMessage(message)
            }

        })
    }
    override fun init() {
        news_list_view.setLoadingMoreEnabled(true)
        news_list_view.setPullRefreshEnabled(false)
        news_list_view.setLoadingListener(object : XRecyclerView.LoadingListener{
            override fun onLoadMore() {
                manager.requestNews(pageNum++)
            }

            override fun onRefresh() {
            }
        })
        val layoutManager = LinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false)
        news_list_view.layoutManager = layoutManager
        news_list_view.setLoadingMoreProgressStyle(ProgressStyle.BallClipRotate)
        news_list_view.setFooterViewTextColor(ThemeUtils.getColor(Color.WHITE, Color.BLACK))
        adapter.onItemClickListener = {
            val newsItem = data.elementAtOrNull(it)
            if (newsItem != null) {
                val intent = Intent(activity, NewsDetailActivity::class.java)
                intent.putExtra(NewsDetailActivity.TYPE_NEWS, BaseViewControllerManager.TYPE_NEWS)
                intent.putExtra(NewsDetailActivity.TYPE_GATEWAY, NewsActionStatisticsTool.Enter_By_Info)
                if (!TextUtils.isEmpty(manager.topicCode)) {
                    intent.putExtra(NewsDetailActivity.TYPE_COLUMN, NewsConstants.OCEAN_ID_NEWS_INFO + "_" + manager.topicCode)
                }
                intent.putExtra(NewsDetailActivity.TYPE_NEWS_INFO, newsItem)
                StockUtil.addNewsID(newsItem.itemId)
                val viewHolder = news_list_view.findViewHolderForAdapterPosition(it + 1)
                adapter.updateItemRead(viewHolder)
                startActivity(intent)
            }
        }
        news_list_view.adapter = adapter
        pageNum = 0
        manager.requestNews(0)
    }

    override fun getLayoutId() = R.layout.f5_index_live_frag


    override fun onFragShow() {
    }

    override fun onFragHide() {
    }

    override fun handleMessage(msg: Message?) {
        super.handleMessage(msg)
        when(msg?.what) {
            UPDATE_LIST_MSG -> {
                val list = msg.obj as? List<NewsBaseInfo>
                list?.let { data.addAll(it) }
                adapter.setData(data)
                news_list_view.loadMoreComplete()
                val size = list?.size ?: 0
                if (size == 0 || size.rem(20) != 0) {
                    news_list_view.setLoadingMoreEnabled(false)
                }
            }
            ERROR_MSG -> {
                news_list_view.loadMoreComplete()
            }
        }
    }

    companion object{
        const val UPDATE_LIST_MSG = 0x1
        const val ERROR_MSG = 0x2
    }
}