package wind.android.f5.view.bottom.fragment

import android.content.Intent
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.SimpleItemAnimator
import android.widget.CompoundButton
import base.BaseNetHandler
import kotlinx.android.synthetic.main.f5_index_plate_frag.*
import session.F5Session
import wind.android.f5.R
import wind.android.f5.activity.SpeedDetailActivity
import wind.android.f5.model.IndexComponetModel
import wind.android.f5.view.bottom.adapter.F5IndexPlateAdapter
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB0
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB1
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB2
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.Companion.DATA_TAB3
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.F5IndexListDataReqType.*
import wind.android.f5.view.bottom.subview.index.IF5IndexSubView

/**
 * # **********************************************************************************************
 * # ClassName:      F5IndexPlateFrag
 * # Description:    指数F5页面板块tab
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2020/2/26    14:29
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class F5IndexPlateFrag : BaseF5BottomFrag(), CompoundButton.OnCheckedChangeListener, IF5IndexSubView<IndexComponetModel> {
    /**
     * 列表adapter
     */
    private val adapter: F5IndexPlateAdapter by lazy { F5IndexPlateAdapter() }
    /**
     * presenter
     */
    private val presenter: F5IndexComponentPresenter by lazy { F5IndexComponentPresenter(this) }
    /**
     * 标记列表类型（四个选择按钮选择的类型）
     */
    private var dataTabType = DATA_TAB0
    override fun init() {
        rise_top.setOnCheckedChangeListener(this)
        fall_top.setOnCheckedChangeListener(this)
        rise_5_top.setOnCheckedChangeListener(this)
        fall_5_top.setOnCheckedChangeListener(this)
        val linearLayoutManager = LinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false)
        list_view.layoutManager = linearLayoutManager
        val itemAnimator = list_view.itemAnimator
        if (itemAnimator is SimpleItemAnimator) {
            itemAnimator.supportsChangeAnimations = false
        }
        adapter.setOnItemClickListenerFun {
            itemClickFun {
                if (!it.isNullOrEmpty()) {
                    val intent = Intent(activity, SpeedDetailActivity::class.java)
                    intent.putExtra(F5Session.POSITION, 0)
                    F5Session.getInstance().f5WindCodes = arrayOf(it)
                    startActivity(intent)
                }
            }
            footerClickFun {
                gotoMorePage()
            }
        }
        list_view.adapter = adapter
        rise_top.isChecked = true
    }

    override fun getLayoutId(): Int = R.layout.f5_index_plate_frag

    override fun onFragShow() {
        presenter.sub()
    }

    override fun onFragHide() {
        presenter.unSub()
    }

    override fun onDestroy() {
        presenter.detachView()
        super.onDestroy()
    }
    /**
     * 跳转更多页面
     */
    private fun gotoMorePage() {
        val name: String
        val morePlateId: String
        val plateId = "a39901012c000000"
        if (dataTabType == DATA_TAB0) {
            name = "涨幅榜"
            morePlateId = "1032"
        } else if (dataTabType == DATA_TAB1) {
            name = "跌幅榜"
            morePlateId = "1033"
        } else if (dataTabType == DATA_TAB2) {
            name = "5分钟快速涨幅榜"
            morePlateId = "1034"
        } else {
            name = "5分钟快速跌幅榜"
            morePlateId = "1035"
        }
        val intent = Intent("wind.andorid.market.MARKET_DETAIL")
        intent.putExtra("title", name)
        intent.putExtra("morePlateId", morePlateId)
        intent.putExtra("plateId", plateId)
        intent.setPackage(activity!!.packageName)
        startActivity(intent)
    }

    override fun onCheckedChanged(buttonView: CompoundButton?, isChecked: Boolean) {
        when (buttonView?.id) {
            R.id.rise_top -> {
                if (isChecked) {
                    dataTabType = DATA_TAB0
                    presenter.reqListData(mWindCode, PLATE_RISE_TYPE)
                }
            }
            R.id.fall_top -> {
                if (isChecked) {
                    dataTabType = DATA_TAB1
                    presenter.reqListData(mWindCode, PLATE_FALL_TYPE)
                }
            }
            R.id.rise_5_top -> {
                if (isChecked) {
                    dataTabType = DATA_TAB2
                    presenter.reqListData(mWindCode, PLATE_RISE_5_TYPE)
                }
            }
            R.id.fall_5_top -> {
                if (isChecked) {
                    dataTabType = DATA_TAB3
                    presenter.reqListData(mWindCode, PLATE_FALL_5_TYPE)
                }
            }
        }
    }

    override fun updateListAll(listData: List<IndexComponetModel>?) {
        adapter.setData(listData, dataTabType)
        adapter.notifyDataSetChanged()
    }

    override fun updateListByPosition(position: Int) {
        adapter.notifyItemChanged(position)
    }

    override fun showProgress() {
        showProgressMum()
    }

    override fun hideProgress() {
        hideProgressMum()
    }

    override fun onError(errorMsg: String?) {
    }

    override fun onSpeedStatusChange(status: Int, detailMsg: String?) {
        super.onSpeedStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            val type = when(dataTabType) {
                DATA_TAB0 -> PLATE_RISE_TYPE
                DATA_TAB1 -> PLATE_FALL_TYPE
                DATA_TAB2 -> PLATE_RISE_5_TYPE
                DATA_TAB3 -> PLATE_FALL_5_TYPE
                else -> PLATE_RISE_TYPE
            }
            presenter.speedReconnect(mWindCode, type)
        }
    }
}