package wind.android.f5.view.bottom.presenter

import business.report.AttachInfo
import net.activity.BaseHandle
import net.protocol.model.Error
import net.protocol.model.Token
import net.util.Assist
import wind.android.f5.net.base.SpeedNewsConnection
import wind.android.f5.net.base.SpeedNewsConnection.OnDataReceivedListener
import wind.android.f5.net.news.NewsBulletionUtils
import wind.android.f5.view.base.CStockView
import wind.android.f5.view.bottom.subview.IF5BulletView
import wind.android.news.anews.EventResultModel
import wind.android.news.anews.NewsTitleModel
import java.util.ArrayList

/**
 * # ***********************************************************************************************
 * # ClassName:      F5BulletPresenter$
 * # Function:       公告页面-数据处理管理器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/14$    15:58$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class F5BulletPresenter(var mView: IF5BulletView? = null) {

    fun onDestoryView() {
        mView = null
    }

    fun requestEvent(windCode: String?) {
        if (windCode.isNullOrBlank()) {
            return
        }
        SpeedNewsConnection.getInstance().startRequestEventList(arrayOf(windCode),
                1, 3, object : OnDataReceivedListener<MutableList<EventResultModel>> {
            override fun onDateReceived(t: MutableList<EventResultModel>?) {
                val list = mutableListOf<NewsTitleModel>()
                t?.forEach {
                    val model = NewsTitleModel()
                    model.windCode = windCode
                    model.newsId = it.objectID
                    model.title = it.eventName
                    model.newsTime = it.occurDate
                    model.repeatcount = it.content
                    model.stockName = it.securityName
                    model.type = NewsTitleModel.WMPUSH_EVENT
                    if (model.newsTime != null && model.newsTime.length == 8) {
                        model.newsTime = Assist.getDateByFormat2(model.newsTime, "yyyyMMdd", "yyyy-MM-dd")
                    }
                    list.add(model)
                }
                mView?.updateEvent(list)
                requestBullet(windCode, 1)
            }

            override fun onError(e: Error?, mToken: Token<*>?, mActivity: BaseHandle?, errorData: Any?) {
                requestBullet(windCode, 1)
            }
        })
    }

    fun requestBullet(windCode: String?, pageIndex: Int) {
        if (windCode.isNullOrBlank()) {
            return
        }
        SpeedNewsConnection.getInstance().startRequestBullet(
                windCode, pageIndex, CStockView.PAGE_SIZE, object : OnDataReceivedListener<MutableList<NewsTitleModel>> {
            override fun onDateReceived(t: MutableList<NewsTitleModel>?) {
                t?.forEach {
                    it.type = NewsTitleModel.WMPUSH_BULLET
                }
                if (pageIndex == 1) {
                    mView?.setBullet(t)
                } else {
                    mView?.updateBullet(t)
                }
            }

            override fun onError(e: Error?, mToken: Token<*>?, mActivity: BaseHandle?, errorData: Any?) {

            }
        })
    }

    fun getBulletinDetail(newsTitleModel: NewsTitleModel) {
        NewsBulletionUtils.getNewsBulletions(newsTitleModel.newsId) {
            list: ArrayList<AttachInfo>? ->
            if (list?.size == 1) {
                newsTitleModel.attechmentList = list
                mView?.doDownloadBullet(newsTitleModel)
            } else {
                mView?.doBulletJump(newsTitleModel)
            }
        }
    }
}