package wind.android.f5.view.bottom.subview.finance.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.LinearLayout
import android.widget.TextView
import net.datamodel.network.CommonFunc
import wind.android.f5.R
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9ContentModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9ContentModel.Companion.TYPE_OF_CURRENCY
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9RawModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9TitleModel
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceF9DetailAdapter
 * # Function:       财务详情-列表适配器基类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/22$    16:52$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
open class BaseFinanceF9Adapter(val context: Context) : BaseAdapter() {

    companion object {
        const val TYPE_GROUP = 0
        const val TYPE_CHILD = 1
    }

    private var mNameList = mutableListOf<FinanceF9TitleModel>()

    private var mTitleView: View? = null

    private val mData = mutableListOf<FinanceF9RawModel>()

    private var mIsShowCompare = false

    private var mCurrency: String? = null

    fun setData(groups: List<FinanceF9TitleModel>?, list: MutableList<FinanceF9RawModel>?) {
        mData.clear()
        mNameList.clear()
        list?.let {
            mData.addAll(it)
        }
        groups?.let {
            mNameList.addAll(it)
        }
        notifyDataSetChanged()
    }

    fun setTitleView(view: View) {
        mTitleView = view
    }

    fun setCurrency(currency: String) {
        mCurrency = currency
    }

    private fun getTransverseCount(): Int {
        return mData.size
    }

    fun setShowCompare(isShowCompare: Boolean) {
        mIsShowCompare = isShowCompare
    }

    private fun getContentItem(raw: Int, line: Int): FinanceF9ContentModel? {
        return mData.elementAtOrNull(raw)?.mData?.elementAtOrNull(line)
    }

    private fun getDataType(position: Int): String {
        return mNameList.elementAtOrNull(position)?.mType ?: ""
    }

    private fun getExchangeRate(raw: Int): Double {
        if (mCurrency.isNullOrBlank()) {
            return 1.0
        }
        return mCurrency?.let {
            currency ->
            var result = 1.0
            val rawModel = mData.elementAtOrNull(raw)
            rawModel?.currencyList?.forEach {
                if (it.type.startsWith(currency)) {
                    result = it.value
                }
            }
            result
        } ?: 1.0
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val itemViewType = getItemViewType(position)
        val view: View
        val item = getItem(position)
        val inflater = LayoutInflater.from(context)
        if (itemViewType == TYPE_GROUP) {
            var hold: GroupViewHold
            if (convertView == null || convertView.tag !is GroupViewHold) {
                view = inflater.inflate(R.layout.item_finance_group, null)
                hold = GroupViewHold(view)
                view.tag = hold
            } else {
                view = convertView
            }
            hold = view.tag as GroupViewHold
            createGroupContentView(hold.groupContent, inflater, position)
            mTitleView?.let {
                val head = hold.groupContent.scrollX
                if (it.scrollX != head) {
                    hold.groupContent.scrollTo(it.scrollX, 0)
                }
            }
            item?.let { dealGroup(hold, it) }
        } else {
            var hold: ChildViewHold
            if (convertView == null || convertView.tag !is ChildViewHold) {
                view = inflater.inflate(R.layout.item_finance_child, null)
                hold = ChildViewHold(view)
                view.tag = hold
            } else {
                view = convertView
            }
            hold = view.tag as ChildViewHold
            createChildContentView(hold.childContent, inflater, position)
            item?.let { dealChild(hold, it) }
            mTitleView?.let {
                val head = hold.childContent.scrollX
                if (it.scrollX != head) {
                    hold.childContent.scrollTo(it.scrollX, 0)
                }
            }
        }
        return view
    }

    private fun createGroupContentView(parentView: ViewGroup, inflater: LayoutInflater, position: Int) {
        parentView.removeAllViews()
        while (getTransverseCount() > parentView.childCount) {
            val layout = inflater.inflate(R.layout.item_finance_content_group, null) ?: null
            layout?.let {
                parentView.addView(it)
            }
        }
        val childCount = parentView.childCount
        for (index in 0 until childCount) {
            val childAt = parentView.getChildAt(index) ?: null ?: continue
            val valueText = childAt.findViewById<TextView>(R.id.content_group_value)
            val model = getContentItem(index, position)
            val exchangeRate = getExchangeRate(index)
            val type = getDataType(position)
            val layoutParam = childAt.layoutParams
            layoutParam.width = context.resources.getDimensionPixelSize(R.dimen.dp_120)
            model?.let {
                if (it.type == TYPE_OF_CURRENCY) {
                    valueText.text = dealStr(it.mValue, true, type, exchangeRate)
                } else {
                    valueText.text = dealStr(it.mValue, true, type, 1.0)
                }
                if (it.srcField == TYPE_OF_CURRENCY && !mCurrency.isNullOrBlank()) {
                    // 显示最新的币种
                    valueText.text = mCurrency
                } else if (it.srcField == "rate") {
                    // 显示汇率
                    valueText.text = String.format("%.2f", exchangeRate)
                }
                if (mIsShowCompare) {
                    val compareText = childAt.findViewById<TextView>(R.id.compare_group_value)
                    val compareStr = getCompare(it)
                    val compare = dealCompare(compareStr, false)
                    compareText.visibility = View.VISIBLE
                    compareText.setTextColor(getTextColor(compareStr))
                    compareText.text = compare
                }
            }
        }
    }

    private fun createChildContentView(parentView: ViewGroup, inflater: LayoutInflater, position: Int) {
        parentView.removeAllViews()
        while (getTransverseCount() > parentView.childCount) {
            val layout = inflater.inflate(R.layout.item_finance_content_child, null) ?: null
            layout?.let {
                parentView.addView(it)
            }
        }
        val childCount = parentView.childCount
        for (index in 0 until childCount) {
            val childAt = parentView.getChildAt(index) ?: null ?: continue
            val valueText = childAt.findViewById<TextView>(R.id.content_child_value)
            val model = getContentItem(index, position)
            val exchangeRate = getExchangeRate(index)
            val type = getDataType(position)
            val layoutParam = childAt.layoutParams
            layoutParam.width = context.resources.getDimensionPixelSize(R.dimen.dp_120)
            model?.let {
                if (it.type == TYPE_OF_CURRENCY) {
                    valueText.text = dealStr(it.mValue, true, type, exchangeRate)
                } else {
                    valueText.text = dealStr(it.mValue, true, type, 1.0)
                }
                if (it.srcField == TYPE_OF_CURRENCY && !mCurrency.isNullOrBlank()) {
                    // 显示最新的币种
                    valueText.text = mCurrency
                } else if (it.srcField == "rate") {
                    // 显示汇率
                    valueText.text = String.format("%.2f", exchangeRate)
                }
                if (mIsShowCompare) {
                    val compareText = childAt.findViewById<TextView>(R.id.compare_child_value)
                    val compareStr = getCompare(it)
                    val compare = dealCompare(compareStr, true)
                    compareText.visibility = View.VISIBLE
                    compareText.setTextColor(getTextColor(compareStr))
                    compareText.text = compare
                }
            }
        }
    }

    protected open fun getCompare(model: FinanceF9ContentModel): String {
        return ""
    }

    private fun getTextColor(valueStr: String): Int {
        if (valueStr.isEmpty()) {
            return StockUtil.getChangeColor(0f)
        }
        try {
            val value = valueStr.toFloat()
            return StockUtil.getChangeColor(value)
        } catch (e: Exception) {
            e.printStackTrace()
        }

        return StockUtil.getChangeColor(0f)
    }

    private fun dealCompare(value: String, isChild: Boolean): String {
        val result = if (isChild) "--" else ""
        if (value.isEmpty()) {
            return result
        }
        return try {
            val toDouble = value.toDouble()
            CommonFunc.fixDoubleAndRound(toDouble, 2) + "%"
        } catch (e: Exception) {
            e.printStackTrace()
            result
        }
    }

    private fun dealGroup(viewHold: GroupViewHold, model: FinanceF9TitleModel) {
        viewHold.groupName.text = model.mTitle
    }

    private fun dealChild(viewHold: ChildViewHold, model: FinanceF9TitleModel) {
        viewHold.childName.text = model.mTitle
    }

    override fun getItemViewType(position: Int): Int {
        val item = getItem(position)
        val isGroup = item?.mIsGroup ?: false
        return if (isGroup) TYPE_GROUP else TYPE_CHILD
    }

    override fun getViewTypeCount(): Int {
        return 2
    }

    override fun getItem(position: Int): FinanceF9TitleModel? {
        return mNameList.elementAtOrNull(position)
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return mNameList.size
    }

    private fun canToFloat(type: String): Boolean {
        return when(type) {
            FinanceF9TitleModel.TYPE_CURRENCY -> true
            FinanceF9TitleModel.TYPE_LONG -> true
            FinanceF9TitleModel.TYPE_INTEGER -> true
            FinanceF9TitleModel.TYPE_DOUBLE -> true
            FinanceF9TitleModel.TYPE_FLOAT -> true
            else -> false
        }
    }

    private fun dealStr(value: String, isChild: Boolean, type: String, exchangeRate: Double): String {
        val defaultStr = if (isChild) "--" else ""
        if (value.isEmpty()) {
            return defaultStr
        }
        if (!canToFloat(type)) {
            return value
        }
        try {
            var float = value.toDouble()
            if (exchangeRate != 0.0) {
                float /= exchangeRate
            }
            val value1 = float / 100000000.0f
            if (Math.abs(value1) > 1) {
                return String.format("%.2f", value1) + "亿"
            }
            val value2 = float / 10000.0f
            if (Math.abs(value2) > 1) {
                return String.format("%.2f", value2) + "万"
            }
            return String.format("%.2f", float)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return value
    }

    class GroupViewHold(itemView: View) {
        var groupName: TextView = itemView.findViewById(R.id.group_name)
        var groupContent: LinearLayout = itemView.findViewById(R.id.scroll_content)
    }

    class ChildViewHold(itemView: View) {
        var childName: TextView = itemView.findViewById(R.id.child_name)
        var childContent: LinearLayout = itemView.findViewById(R.id.scroll_content)
    }

}