package wind.android.f5.view.bottom.subview.finance.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.LinearLayout
import android.widget.TextView
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceKeyIndexModel
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceDetailAdapter$
 * # Function:       财务详情-关键指标适配器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/22$    16:52$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class FinanceDetailAdapter(val context: Context) : BaseAdapter() {

    companion object {
        const val TYPE_CHILD = 0

        const val TYPE_GROUP = 1
    }

    private var mNameList = mutableListOf<GroupListModel>()

    private var mTitleView: View? = null

    private val mData = mutableListOf<MutableList<FinanceKeyIndexModel>>()

    private var mIsShowCompare = false

    fun hasGroupData(): Boolean {
        return mNameList.size > 0
    }

    fun setShowCompare(isShowCompare: Boolean) {
        mIsShowCompare = isShowCompare
    }

    fun setData(groups: List<GroupListModel>?, list: MutableList<MutableList<FinanceKeyIndexModel>>?) {
        mData.clear()
        mNameList.clear()
        list?.let {
            mData.addAll(it)
        }
        groups?.let {
            mNameList.addAll(it)
        }
        notifyDataSetChanged()
    }

    fun setTitleView(view: View) {
        mTitleView = view
    }

    private fun getTransverseCount(): Int {
        return mData.size
    }

    private fun getItem(raw: Int, line: Int): FinanceKeyIndexModel? {
        return mData.elementAtOrNull(raw)?.elementAtOrNull(line)
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
        val itemViewType = getItemViewType(position)
        val view: View
        val item = getItem(position)
        val inflater = LayoutInflater.from(context)
        if (itemViewType == TYPE_GROUP) {
            var hold: GroupViewHold
            if (convertView == null || convertView.tag !is GroupViewHold) {
                view = inflater.inflate(R.layout.item_finance_group, null)
                hold = GroupViewHold(view)
                view.tag = hold
            } else {
                view = convertView
            }
            hold = view.tag as GroupViewHold
            createGroupContentView(hold.groupContent, inflater, position)
            item?.let { dealGroup(hold, it) }
        } else {
            var hold: ChildViewHold
            if (convertView == null || convertView.tag !is ChildViewHold) {
                view = inflater.inflate(R.layout.item_finance_child, null)
                hold = ChildViewHold(view)
                view.tag = hold
            } else {
                view = convertView
            }
            hold = view.tag as ChildViewHold
            createChildContentView(hold.childContent, inflater, position)
            item?.let { dealChild(hold, it) }
        }
        return view
    }

    private fun createGroupContentView(parentView: ViewGroup, inflater: LayoutInflater, position: Int) {
        parentView.removeAllViews()
        while (getTransverseCount() > parentView.childCount) {
            val layout = inflater.inflate(R.layout.item_finance_content_group, null) ?: null
            layout?.let {
                parentView.addView(it)
            }
        }
        val childCount = parentView.childCount
        for (index in 0 until childCount) {
            val childAt = parentView.getChildAt(index) ?: null ?: continue
            val valueText = childAt.findViewById<TextView>(R.id.content_group_value)
            val model = getItem(index, position)
            val layoutParam = childAt.layoutParams  ?: null ?: continue
            layoutParam.width = context.resources.getDimensionPixelSize(R.dimen.dp_120)
            childAt.layoutParams = layoutParam
            model?.let {
                if (it.mValue.isNullOrEmpty()) {
                    valueText.text = ""
                } else {
                    valueText.text = (it.mValue + it.mUint)
                }
                if (mIsShowCompare) {
                    val compareText = childAt.findViewById<TextView>(R.id.compare_group_value)
                    val compare = if (it.mCompareValue.isEmpty()) "" else  it.mCompareValue
                    compareText.visibility = View.VISIBLE
                    compareText.setTextColor(getTextColor(compare))
                    compareText.text = compare
                }
            }
        }
        val head = parentView.scrollX
        mTitleView?.let {
            if (it.scrollX != head) {
                parentView.scrollTo(it.scrollX, 0)
            }
        }
    }

    private fun createChildContentView(parentView: ViewGroup, inflater: LayoutInflater, position: Int) {
        parentView.removeAllViews()
        while (getTransverseCount() > parentView.childCount) {
            val layout = inflater.inflate(R.layout.item_finance_content_child, null) ?: null
            layout?.let {
                parentView.addView(it)
            }
        }
        val childCount = parentView.childCount
        for (index in 0 until childCount) {
            val childAt = parentView.getChildAt(index) ?: null ?: continue
            val valueText = childAt.findViewById<TextView>(R.id.content_child_value)
            val model = getItem(index, position)
            val layoutParam = childAt.layoutParams ?: null ?: continue
            layoutParam.width = context.resources.getDimensionPixelSize(R.dimen.dp_120)
            childAt.layoutParams = layoutParam
            model?.let {
                if (it.mValue.isNullOrEmpty()) {
                    valueText.text = "--"
                } else {
                    valueText.text = (it.mValue + it.mUint)
                }
                if (mIsShowCompare) {
                    val compareText = childAt.findViewById<TextView>(R.id.compare_child_value)
                    val compare = if (it.mCompareValue.isEmpty()) "--" else  it.mCompareValue
                    compareText.visibility = View.VISIBLE
                    compareText.setTextColor(getTextColor(compare))
                    compareText.text = compare
                }
            }
        }
        val head = parentView.scrollX
        mTitleView?.let {
            if (it.scrollX != head) {
                parentView.scrollTo(it.scrollX, 0)
            }
        }
    }

    private fun getTextColor(valueStr: String): Int {
        if (valueStr.isEmpty() || !valueStr.endsWith("%")) {
            return StockUtil.getChangeColor(0f)
        }
        return try {
            val value = valueStr.substringBefore("%").toFloat()
            StockUtil.getChangeColor(value)
        } catch (e: Exception) {
            e.printStackTrace()
            StockUtil.getChangeColor(0f)
        }
    }

    private fun dealGroup(viewHold: GroupViewHold, model: GroupListModel) {
        viewHold.groupName.text = model.groupName
    }

    private fun dealChild(viewHold: ChildViewHold, model: GroupListModel) {
        viewHold.childName.text = model.groupName
    }

    override fun getItemViewType(position: Int): Int {
        val item = getItem(position)
        val isGroup = item?.isGroup ?: 0
        return if (isGroup == TYPE_GROUP) TYPE_GROUP else TYPE_CHILD
    }

    override fun getViewTypeCount(): Int {
        return 2
    }

    override fun getItem(position: Int): GroupListModel? {
        return mNameList.elementAtOrNull(position)
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }

    override fun getCount(): Int {
        return mNameList.size
    }

    class GroupViewHold(itemView: View) {
        var groupName: TextView = itemView.findViewById(R.id.group_name)
        var groupContent: LinearLayout = itemView.findViewById(R.id.scroll_content)
    }

    class ChildViewHold(itemView: View) {
        var childName: TextView = itemView.findViewById(R.id.child_name)
        var childContent: LinearLayout = itemView.findViewById(R.id.scroll_content)
    }


}