package wind.android.f5.view.bottom.subview.finance.manager

import org.xmlpull.v1.XmlPullParser
import org.xmlpull.v1.XmlPullParserException
import org.xmlpull.v1.XmlPullParserFactory
import wind.android.f5.view.bottom.subview.finance.model.*
import java.io.IOException
import java.io.StringReader

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceXmlAssist$
 * # Function:       财务详情-cloud报表解析器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/6$    13:40$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class FinanceXmlAssist {

    companion object {

        private val INVALID_ITEM_TITLES = listOf("核数师", "原始报表", "透视报表类型")

        @Throws(XmlPullParserException::class, IOException::class)
        fun parseFinanceReportXML(source: String): FinanceF9ReportModel {
            val detailModel = FinanceF9ReportModel()
            val factory = XmlPullParserFactory.newInstance()
            factory.isNamespaceAware = true
            val parser = factory.newPullParser()
            parser.setInput(StringReader(source))
            var eventType = parser.eventType
            val content = mutableListOf<FinanceF9ContentModel>()
            val tempContent = mutableListOf<FinanceF9ContentModel>()
            val currencyList = mutableListOf<FinanceCurrencyModel>()
            val tempCurrencyList = mutableListOf<FinanceCurrencyModel>()
            var reportType = ""
            var reportDate = ""
            var currency = ""
            var originalCurrency = ""
            while (eventType != XmlPullParser.END_DOCUMENT) {
                if (eventType == XmlPullParser.START_TAG) {
                    when(parser.name) {
                        "schema" -> {
                            val id = parser.getAttributeValue(null, "id") ?: ""
                            val title = parser.getAttributeValue(null, "caption") ?: ""
                            val visible = parser.getAttributeValue(null, "visible") ?: ""
                            val srcField = parser.getAttributeValue(null, "srcField") ?: ""
                            if (visible == "False" && srcField.endsWith("Rate")) {
                                // 该行数据会汇率
                                tempCurrencyList.add(FinanceCurrencyModel(id, srcField))
                            } else if (!INVALID_ITEM_TITLES.contains(title)) {
                                var isGroup = false
                                if (parser.getAttributeValue(null, "isGroup") != null) {
                                    if ("true" == parser.getAttributeValue(null, "isGroup")) {
                                        isGroup = true
                                    }
                                }
                                val type = parser.getAttributeValue(null, "type") ?: ""

                                val titleModel = FinanceF9TitleModel(title, isGroup, id, type)
                                detailModel.mLeftTitles.add(titleModel)
                                tempContent.add(FinanceF9ContentModel(id, type, srcField))
                            }
                        }
                        "r" -> {
                            reportType = parser.getAttributeValue(null, "reportType") ?: ""
                            reportDate = parser.getAttributeValue(null, "date") ?: ""
                            try {
                                tempContent.forEach {
                                    content.add(it.createClone())
                                }
                                tempCurrencyList.forEach {
                                    currencyList.add(it.createClone())
                                }
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }
                        }
                        "f" -> {
                            val id = parser.getAttributeValue(null, "id") ?: ""
                            currencyList.forEach {
                                if (id == it.id) {
                                    parser.next()
                                    val value= parser.text ?: ""
                                    try {
                                        it.value = value.toDouble()
                                    } catch (e: Exception) {
                                    }
                                }
                            }
                            content.forEach {
                                if (id == it.mId) {
                                    it.mMarketPercent = parser.getAttributeValue(null, "MP") ?: ""
                                    it.mAssetPercent = parser.getAttributeValue(null, "AP") ?: ""
                                    it.mCompare = parser.getAttributeValue(null, "CV") ?: ""
                                    it.mQuarterCompare = parser.getAttributeValue(null, "QCV") ?: ""
                                    parser.next()
                                    it.mValue = parser.text ?: ""
                                    if (it.srcField == "origionalCurrency") {
                                        // 获取原始币种
                                        originalCurrency = it.mValue
                                    } else if (it.srcField == "currency") {
                                        // 获取显示币种
                                        currency = it.mValue
                                    }
                                    return@forEach
                                }
                            }
                        }
                    }
                } else if (eventType == XmlPullParser.END_TAG) {
                    when(parser.name) {
                        "r" -> {
//                            while (content.size < detailModel.mLeftTitles.size) {
//                                val tempModel = FinanceF9ContentModel()
//                                content.add(tempModel)
//                            }
                            val tempList = mutableListOf<FinanceF9ContentModel>()
                            tempList.addAll(content)
                            if (tempList.size > 0) {
                                val model = tempList[0]
                                model.mValue = reportType
                            }
                            content.clear()
                            var title = ""
                            if (reportDate.length > 4) {
                                title = reportDate.substring(0, 4) + reportType
                            }
                            val reportModel = FinanceF9RawModel(reportType, reportDate, title)
                            reportModel.mData.addAll(tempList)
                            reportModel.currencyList.addAll(currencyList)
                            currencyList.clear()
                            detailModel.currency = currency
                            detailModel.originalCurrency = originalCurrency
                            detailModel.mAllList.add(reportModel)
                            detailModel.mTopTitles.add(title)
                        }
                    }
                }
                eventType = parser.next()
            }
            return detailModel
        }

        @Throws(XmlPullParserException::class, IOException::class)
        fun parseFinanceSeasonXML(source: String): FinanceF9SeasonModel {
            val detailModel = FinanceF9SeasonModel()
            val factory = XmlPullParserFactory.newInstance()
            factory.isNamespaceAware = true
            val parser = factory.newPullParser()
            parser.setInput(StringReader(source))
            var eventType = parser.eventType
            val content = mutableListOf<FinanceF9ContentModel>()
            val tempContent = mutableListOf<FinanceF9ContentModel>()
            val currencyList = mutableListOf<FinanceCurrencyModel>()
            val tempCurrencyList = mutableListOf<FinanceCurrencyModel>()
            var reportType = ""
            var reportDate = ""
            var currency = ""
            var originalCurrency = ""
            while (eventType != XmlPullParser.END_DOCUMENT) {
                if (eventType == XmlPullParser.START_TAG) {
                    when(parser.name) {
                        "schema" -> {
                            val id = parser.getAttributeValue(null, "id") ?: ""
                            val title = parser.getAttributeValue(null, "caption") ?: ""
                            val visible = parser.getAttributeValue(null, "visible") ?: ""
                            val srcField = parser.getAttributeValue(null, "srcField") ?: ""
                            if (visible == "False" && srcField.endsWith("Rate")) {
                                // 该行数据会汇率
                                tempCurrencyList.add(FinanceCurrencyModel(id, srcField))
                            } else if (!INVALID_ITEM_TITLES.contains(title)) {
                                var isGroup = false
                                if (parser.getAttributeValue(null, "isGroup") != null) {
                                    if ("true" == parser.getAttributeValue(null, "isGroup")) {
                                        isGroup = true
                                    }
                                }
                                val type = parser.getAttributeValue(null, "type") ?: ""

                                val titleModel = FinanceF9TitleModel(title, isGroup, id, type)
                                detailModel.mLeftTitles.add(titleModel)
                                tempContent.add(FinanceF9ContentModel(id, type, srcField))
                            }
                        }
                        "r" -> {
                            reportType = parser.getAttributeValue(null, "reportType") ?: ""
                            reportDate = parser.getAttributeValue(null, "date") ?: ""
                            try {
                                tempContent.forEach {
                                    content.add(it.createClone())
                                }
                                tempCurrencyList.forEach {
                                    currencyList.add(it.createClone())
                                }
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }
                        }
                        "f" -> {
                            val id = parser.getAttributeValue(null, "id") ?: ""
                            currencyList.forEach {
                                if (id == it.id) {
                                    parser.next()
                                    val value= parser.text ?: ""
                                    try {
                                        it.value = value.toDouble()
                                    } catch (e: Exception) {
                                    }
                                }
                            }
                            content.forEach {
                                if (id == it.mId) {
                                    it.mMarketPercent = parser.getAttributeValue(null, "MP") ?: ""
                                    it.mAssetPercent = parser.getAttributeValue(null, "AP") ?: ""
                                    val compare = parser.getAttributeValue(null, "CV") ?: ""
                                    it.mCompare = compare.filter {
                                        value ->
                                        value != ','
                                    }
                                    it.mQuarterCompare = parser.getAttributeValue(null, "QCV") ?: ""
                                    parser.next()
                                    it.mValue = parser.text ?: ""
                                    if (it.srcField == "origionalCurrency") {
                                        // 获取原始币种
                                        originalCurrency = it.mValue
                                    } else if (it.srcField == "currency") {
                                        // 获取显示币种
                                        currency = it.mValue
                                    }
                                    return@forEach
                                }
                            }
                        }
                    }
                } else if (eventType == XmlPullParser.END_TAG) {
                    when(parser.name) {
                        "r" -> {
//                            while (content.size < detailModel.mLeftTitles.size) {
//                                val tempModel = FinanceF9ContentModel()
//                                content.add(tempModel)
//                            }
                            val tempList = mutableListOf<FinanceF9ContentModel>()
                            tempList.addAll(content)
                            if (tempList.size > 0) {
                                val model = tempList[0]
                                model.mValue = reportType
                            }
                            content.clear()
                            var title = ""
                            if (reportDate.length > 4) {
                                title = reportDate.substring(0, 4) + reportType
                            }
                            val reportModel = FinanceF9RawModel(reportType, reportDate, title)
                            reportModel.mData.addAll(tempList)
                            reportModel.currencyList.addAll(currencyList)
                            currencyList.clear()
                            detailModel.currency = currency
                            detailModel.originalCurrency = originalCurrency
                            detailModel.mAllList.add(reportModel)
                            detailModel.mTopTitles.add(title)
                        }
                    }
                }
                eventType = parser.next()
            }
            return detailModel
        }
    }
}