package wind.android.f5.view.bottom.subview.finance.presenter

import android.content.res.Resources
import android.os.Message
import net.data.network.SkyCallbackData
import net.datamodel.network.CommonFunc
import net.listener.ISkyDataListenerEx
import net.util.TimeHelper
import util.ToastTool
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.StockFinanceData
import wind.android.f5.view.bottom.subview.finance.manager.FinanceDataManager
import wind.android.f5.view.bottom.subview.finance.manager.FinanceXmlAssist
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9RawModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9ReportModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9SeasonModel
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_ARD_TYPE_BALANCE_SHEET
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_ARD_TYPE_CASH
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_ARD_TYPE_PROFIT
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_GSD_TYPE_BALANCE_SHEET
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_GSD_TYPE_CASH
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_HK_GSD_TYPE_PROFIT
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_TYPE_BALANCE_SHEET
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_TYPE_CASH
import wind.android.f5.view.bottom.subview.finance.view.AbsFinanceView.Companion.PRESENTER_TYPE_PROFIT
import wind.android.f5.view.bottom.subview.finance.view.IFinanceKeyView
import java.nio.charset.Charset

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceProfitPresenter$
 * # Function:       财务详情Cloud报表presenter
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/20$    17:53$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class BaseFinanceF9Presenter(fragmentView : IFinanceKeyView, private var type: Int = 0)
    : BaseFinancePresenter<IFinanceKeyView>(fragmentView),
        ISkyDataListenerEx {

    companion object {

        const val DATE_FORMAT = "yyyyMMdd"

        const val FOUR_YEAR = 4 * 4
    }

    private var mNextReportDate: String? = null

    private var mNextTempReportDate: String? = null

    private var mReportRequestId = 0

    private var mNextSeasonDate: String? = null

    private var mNextTempSeasonDate: String? = null

    private var mSeasonRequestId = 0

    private val mManager by lazy { FinanceDataManager() }

    private var mReportModel: FinanceF9ReportModel? = null

    private var mSeasonModel: FinanceF9SeasonModel? = null

    override fun requestReportDescList(resources : Resources): MutableList<GroupListModel>? {
        return null
    }

    override fun requestReport(windCode : String?) {
        val reportSeason = getReportData(windCode)
        if (reportSeason.isNullOrBlank()) {
            getView()?.showEmptyView()
            return
        }
        val seasonList = CommonFunc.getSeasonDate(200, reportSeason, DATE_FORMAT) ?: null
        var startDate = ""
        var endDate = ""
        seasonList?.run {
            val size = seasonList.size
            if (size == 0) {
                getView()?.showEmptyView()
                return
            }
            if (size > FOUR_YEAR) {
                startDate = seasonList[FOUR_YEAR - 1]
                endDate = seasonList[0]
            } else {
                startDate = seasonList[size - 1]
                endDate = seasonList[0]
            }
            if (size > FOUR_YEAR + 1) {
                mNextTempReportDate = seasonList[FOUR_YEAR]
            }

        }
        mReportRequestId = requestReport(windCode, startDate, endDate)
        if (mReportRequestId == 0 || mReportRequestId == -1 || mReportRequestId == -2) {
            getView()?.showEmptyView()
        }
    }

    private fun requestReport(windCode: String?, startDate: String?, endDate: String?): Int {
        return when (type) {
            PRESENTER_TYPE_PROFIT -> mManager.requestABProfitDetail(windCode, startDate, endDate, this)
            PRESENTER_TYPE_BALANCE_SHEET -> mManager.requestABBalanceDetail(windCode, startDate, endDate, this)
            PRESENTER_TYPE_CASH -> mManager.requestABCashDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_GSD_TYPE_PROFIT -> mManager.requestHKGSDProfitDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_GSD_TYPE_BALANCE_SHEET -> mManager.requestHKGSDBalanceDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_GSD_TYPE_CASH -> mManager.requestHKGSDCashDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_ARD_TYPE_PROFIT -> mManager.requestHKARDProfitDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_ARD_TYPE_BALANCE_SHEET -> mManager.requestHKARDBalanceDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_ARD_TYPE_CASH -> mManager.requestHKARDCashDetail(windCode, startDate, endDate, this)
            else -> 0
        }
    }

    override fun requestSingleSeason(windCode: String?) {
        val reportSeason = getSeasonData(windCode)
        if (reportSeason.isNullOrBlank()) {
            getView()?.showEmptyView()
            return
        }
        val seasonList = CommonFunc.getSeasonDate(200, reportSeason, DATE_FORMAT) ?: null
        var startDate = ""
        var endDate = ""
        seasonList?.run {
            if (size == 0) {
                getView()?.showEmptyView()
                return
            }
            if (size > FOUR_YEAR) {
                startDate = this[FOUR_YEAR - 1]
                endDate = this[0]
            } else {
                startDate = this[size - 1]
                endDate = this[0]
            }
            if (size > FOUR_YEAR + 1) {
                mNextTempSeasonDate = this[FOUR_YEAR]
            }
        }
        mSeasonRequestId = requestSeason(windCode, startDate, endDate)
        if (mSeasonRequestId == 0 || mSeasonRequestId == -1 || mSeasonRequestId == -2) {
            getView()?.showEmptyView()
        }
    }

    private fun requestSeason(windCode: String?, startDate: String?, endDate: String?): Int {
        return when (type) {
            PRESENTER_TYPE_PROFIT -> mManager.requestABProfitSeasonDetail(windCode, startDate, endDate, this)
            PRESENTER_TYPE_CASH -> mManager.requestABCashSeasonDetail(windCode, startDate, endDate, this)
            PRESENTER_HK_GSD_TYPE_PROFIT -> mManager.requestHKGSDProfitSeason(windCode, startDate, endDate, this)
            PRESENTER_HK_GSD_TYPE_CASH -> mManager.requestHKGSDCashSeason(windCode, startDate, endDate, this)
            PRESENTER_HK_ARD_TYPE_PROFIT -> mManager.requestHKARDProfitSeason(windCode, startDate, endDate, this)
            PRESENTER_HK_ARD_TYPE_CASH -> mManager.requestHKARDCashSeason(windCode, startDate, endDate, this)
            else -> 0
        }
    }

    private fun getReportData(windCode : String?) : String? {
        if (mNextReportDate.isNullOrBlank()) {
            return StockFinanceData.reportSeasonMap[windCode]
                    ?: TimeHelper.getInstance().getDate(DATE_FORMAT)
        }
        return mNextReportDate
    }

    private fun getSeasonData(windCode : String?) : String? {
        if (mNextSeasonDate.isNullOrBlank()) {
            return StockFinanceData.reportSeasonMap[windCode]
                    ?: TimeHelper.getInstance().getDate(DATE_FORMAT)
        }
        return mNextSeasonDate
    }

    override fun OnSkyError(code: Int, serialNum: Int) {
        getView()?.showEmptyView()
    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        val requestId = data?.SerialNum ?: 0
        if (requestId == mReportRequestId) {
            try {
                val get = data?.data?.get(0)
                if (get == null) {
                    ToastTool.showToast(R.string.finance_load_no_data)
                    return
                }
                mNextReportDate = mNextTempReportDate
                if (get is ByteArray) {
                    val msgXml = String(get, Charset.forName("GB2312"))
                    val detailModel = FinanceXmlAssist.parseFinanceReportXML(msgXml)
                    dealReportData(detailModel)
                    val message = Message()
                    message.what = MSG_UPDATE_FINANCE_REPORT_LIST
                    message.obj = detailModel
                    sendMessage(message)
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        } else if (requestId == mSeasonRequestId) {
            try {
                val get = data?.data?.get(0)
                if (get == null) {
                    ToastTool.showToast(R.string.finance_load_no_data)
                    return
                }
                mNextSeasonDate = mNextTempSeasonDate
                if (get is ByteArray) {
                    val msgXml = String(get, Charset.forName("GB2312"))
                    val detailModel = FinanceXmlAssist.parseFinanceSeasonXML(msgXml)
                    dealSeasonData(detailModel)
                    val message = Message()
                    message.what = MSG_UPDATE_FINANCE_SEASON_LIST
                    message.obj = detailModel
                    sendMessage(message)
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    private fun dealReportData(detailModel: FinanceF9ReportModel) {
        detailModel.mAllList.forEach {
            when(it.mType) {
                FinanceF9RawModel.TYPE_YEAR_REPORT -> {
                    detailModel.mYearReportList.add(it)
                    detailModel.mYearReportTitle.add(it.mTitle)
                }
                FinanceF9RawModel.TYPE_MIDDLE_REPORT -> {
                    detailModel.mMiddleReportList.add(it)
                    detailModel.mMiddleReportTitle.add(it.mTitle)
                }
                FinanceF9RawModel.TYPE_FIRST_REPORT -> {
                    detailModel.mFirstReportList.add(it)
                    detailModel.mFirstReportTitle.add(it.mTitle)
                }
                FinanceF9RawModel.TYPE_THIRD_REPORT -> {
                    detailModel.mThirdReportList.add(it)
                    detailModel.mThirdReportTitle.add(it.mTitle)
                }
            }
        }
        mReportModel = mReportModel?.let {
            detailModel.mAllList.addAll(0, it.mAllList)
            detailModel.mYearReportList.addAll(0, it.mYearReportList)
            detailModel.mMiddleReportList.addAll(0, it.mMiddleReportList)
            detailModel.mFirstReportList.addAll(0, it.mFirstReportList)
            detailModel.mThirdReportList.addAll(0, it.mThirdReportList)
            detailModel.mTopTitles.addAll(0, it.mTopTitles)
            detailModel.mYearReportTitle.addAll(0, it.mYearReportTitle)
            detailModel.mMiddleReportTitle.addAll(0, it.mMiddleReportTitle)
            detailModel.mFirstReportTitle.addAll(0, it.mFirstReportTitle)
            detailModel.mThirdReportTitle.addAll(0, it.mThirdReportTitle)
            detailModel
        } ?: detailModel
    }

    private fun dealSeasonData(detailModel: FinanceF9SeasonModel) {
        detailModel.mAllList.forEach {
            when {
                it.mType.contains(FinanceF9RawModel.TYPE_FIRST_SEASON) -> {
                    detailModel.mFirstSeasonList.add(it)
                    detailModel.mFirstSeasonTitles.add(it.mTitle)
                }
                it.mType.contains(FinanceF9RawModel.TYPE_SECOND_SEASON) -> {
                    detailModel.mSecondSeasonList.add(it)
                    detailModel.mSecondSeasonTitles.add(it.mTitle)
                }
                it.mType.contains(FinanceF9RawModel.TYPE_THIRD_SEASON) -> {
                    detailModel.mThirdSeasonList.add(it)
                    detailModel.mThirdSeasonTitles.add(it.mTitle)
                }
                it.mType.contains(FinanceF9RawModel.TYPE_FOURTH_SEASON) -> {
                    detailModel.mFourthSeasonList.add(it)
                    detailModel.mFourthSeasonTitles.add(it.mTitle)
                }
            }
        }
        mSeasonModel = mSeasonModel?.let {
            detailModel.mAllList.addAll(0, it.mAllList)
            detailModel.mFirstSeasonList.addAll(0, it.mFirstSeasonList)
            detailModel.mSecondSeasonList.addAll(0, it.mSecondSeasonList)
            detailModel.mThirdSeasonList.addAll(0, it.mThirdSeasonList)
            detailModel.mFourthSeasonList.addAll(0, it.mFourthSeasonList)
            detailModel.mTopTitles.addAll(0, it.mTopTitles)
            detailModel.mFirstSeasonTitles.addAll(0, it.mFirstSeasonTitles)
            detailModel.mSecondSeasonTitles.addAll(0, it.mSecondSeasonTitles)
            detailModel.mThirdSeasonTitles.addAll(0, it.mThirdSeasonTitles)
            detailModel.mFourthSeasonTitles.addAll(0, it.mFourthSeasonTitles)
            detailModel
        } ?: detailModel
    }

}