package wind.android.f5.view.bottom.subview.finance.presenter

import android.content.res.Resources
import android.os.Message
import base.ActivityHandler
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9ReportModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9SeasonModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceReportDetailModel
import wind.android.f5.view.bottom.subview.finance.view.BaseFinanceView
import wind.android.f5.view.bottom.subview.finance.view.IFinanceKeyView
import java.lang.ref.WeakReference

/**
 * # ***********************************************************************************************
 * # ClassName:      BaseFinancePresenter$
 * # Function:       财务详情presenter基类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/20$    17:20$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
abstract class BaseFinancePresenter<T : BaseFinanceView>(view: T) : ActivityHandler.ActivityHandlerListener {

    companion object {
        /**
         * 更新关键指标页面
         */
        const val MSG_UPDATE_REPORT_LIST = 1001

        /**
         * 更新按报告期页面
         */
        const val MSG_UPDATE_FINANCE_REPORT_LIST = 1003

        /**
         * 更新按季度页面
         */
        const val MSG_UPDATE_FINANCE_SEASON_LIST = 1004

        /**
         * 一季报
         */
        const val TYPE_FIRST_REPORT = 1

        /**
         * 中报
         */
        const val TYPE_MIDDEL_REPORT = 2

        /**
         * 三季报
         */
        const val TYPE_THIRD_REPORT = 3

        /**
         * 年报
         */
        const val TYPE_YEAR_REPORT = 4
    }

    protected var mView:WeakReference<T> = WeakReference(view)

    fun getView() : T? {
        return mView.get()
    }

    fun detachView() {
        mView.clear()
    }

    override fun handleMessage(msg: Message?) {
        when(msg?.what) {
            MSG_UPDATE_REPORT_LIST -> {
                val model = msg.obj as? FinanceReportDetailModel ?: FinanceReportDetailModel()
                val view = getView()
                view?.let {
                    if (it is IFinanceKeyView) {
                        it.updateKeyIndexList(model)
                    }
                }
            }
            MSG_UPDATE_FINANCE_REPORT_LIST -> {
                val model = msg.obj as? FinanceF9ReportModel ?: FinanceF9ReportModel()
                val view = getView()
                view?.let {
                    if (it is IFinanceKeyView) {
                        it.updateFinanceReportList(model)
                    }
                }
            }
            MSG_UPDATE_FINANCE_SEASON_LIST -> {
                val model = msg.obj as? FinanceF9SeasonModel ?: FinanceF9SeasonModel()
                val view = getView()
                view?.let {
                    if (it is IFinanceKeyView) {
                        it.updateFinanceSeasonList(model)
                    }
                }
            }
        }
    }

    protected fun sendMessage(msg: Message) {
        ActivityHandler.getInstance(this).sendMessage(msg)
    }

    /**
     * 请求左侧描述信息
     */
    abstract fun requestReportDescList(resources : Resources): MutableList<GroupListModel>?

    /**
     * 请求包报告期数据
     */
    abstract fun requestReport(windCode : String?)

    /**
     * 请求单季报数据
     */
    abstract fun requestSingleSeason(windCode : String?)
}