package wind.android.f5.view.bottom.subview.finance.presenter

import android.content.res.Resources
import android.os.Message
import net.data.network.SkyCallbackData
import net.datamodel.network.CommonFunc
import net.listener.ISkyDataListenerEx
import util.NewsTimeAssist
import util.ToastTool
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.finance.manager.FinanceDataManager
import wind.android.f5.view.bottom.subview.finance.model.FinanceKeyIndexModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceReportDetailModel
import wind.android.f5.view.bottom.subview.finance.view.IFinanceKeyView
import wind.android.f5.view.bottom.subview.parse.StockFinanceXmlTool

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceProfitPresenter$
 * # Function:       港美股关键指标presenter
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/20$    17:53$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class FinanceHKKeyIndexPresenter(fragmentView : IFinanceKeyView) : BaseFinancePresenter<IFinanceKeyView>(fragmentView),
        ISkyDataListenerEx {

    companion object {

        const val REPORT_ALL_YEAR = "年报"

        const val REPORT_MIDDLE_YEAR = "中报"

        const val REPORT_THIRD_SEASON = "三季报"

        const val REPORT_FIRST_SEASON = "一季报"
    }

    private var mFirstLoad = false

    private var mReportRequestId = 0

    private val mManager by lazy { FinanceDataManager() }

    private var mPageIndex = 1

    override fun requestReportDescList(resources : Resources): MutableList<GroupListModel>? {
        val nameList = mutableListOf<GroupListModel>()
        StockFinanceXmlTool.conditionsXmlParse(resources, R.xml.keyindex_hk, nameList)
        return nameList
    }

    override fun requestReport(windCode : String?) {
        if (!mFirstLoad) {
            mPageIndex = 1
        } else {
            mPageIndex++
        }
        mReportRequestId = mManager.requestHKKeyDetail(windCode, mPageIndex, this@FinanceHKKeyIndexPresenter)
        if (mReportRequestId == 0 || mReportRequestId == -1 || mReportRequestId == -2) {
            // 请求失败，mPageIndex恢复
            mPageIndex--
            getView()?.showEmptyView()
        }
    }

    override fun requestSingleSeason(windCode: String?) {
    }

    override fun OnSkyError(code: Int, serialNum: Int) {
        getView()?.showEmptyView()
        // 请求失败，mPageIndex恢复
        mPageIndex--
    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        mFirstLoad = true
        val requestId = data?.SerialNum ?: 0
        if (requestId == mReportRequestId) {
            data?.data?.let {
                if (it.isEmpty()) {
                    ToastTool.showToast(R.string.finance_load_no_data)
                    return
                }
                // 请求成功，才更新报告期
                parseData(it)
            }
        }
    }

    private fun parseData(list: Any) {
        try {
            val dataList = list as? MutableList<MutableList<String?>?>
            if (dataList.isNullOrEmpty()) {
                ToastTool.showToast(R.string.finance_load_no_data)
                return
            }
            val data = mutableListOf<MutableList<FinanceKeyIndexModel>>()
            dataList.forEach continuing@{
                if (it == null) {
                    return@continuing
                }
                val firstValue = it.elementAtOrNull(1) ?: ""
                if (firstValue.indexOf("第") == -1) {
                    val tempList = mutableListOf<FinanceKeyIndexModel>()
                    val size = it.size
                    for (j in 0 until size) {
                        val value = it[j] ?: ""
                        val model = FinanceKeyIndexModel()
                        if (j == 2 || j == 9 || j == 13 || j == 17 || j == 21 || j == 25) {
                            tempList.add(FinanceKeyIndexModel("", ""))
                        }
                        if (j in 9..12 || j == 13 || j == 15 || j == 16 || j in 25..30)  {
                            val floatRate = paraStringToFloatRate(value)
                            if (!floatRate.isEmpty()) {
                                model.mUint = "%"
                                model.mValue = floatRate
                            }
                            tempList.add(model)
                        } else {
                            val len = 2
                            val temp = paraStringToFloat(value, len)
                            if (!temp.isEmpty()) {
                                val lastString = temp.last().toString()
                                if (isNumber(lastString)) {
                                    model.mValue = temp
                                    when (j) {
                                        in 2..8 -> model.mUint = "元"
                                        in 21..24 -> model.mUint = "次"
                                    }
                                } else {
                                    model.mValue = temp.substringBefore(lastString)
                                }
                            }
                            tempList.add(model)
                        }
                    }
                    data.add(tempList)
                }
            }
            dealReport(data, 2)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun isNumber(value: String): Boolean {
        if (value == "%" || value == "亿" || value == "万" || value == "天" || value == "元" || value == "次") {
            return false
        }
        return true
    }

    private fun dealReport(realData : MutableList<MutableList<FinanceKeyIndexModel>>, removeCount: Int) {
        val model = FinanceReportDetailModel()
        realData.forEach {
            var report = mutableListOf<FinanceKeyIndexModel>()
            if (!it.isNullOrEmpty()) {
                report.addAll(it)
                model.mReportList.add(it)
                val size = report.size
                if (size > 2) {
                    val date = report[0]
                    val year = NewsTimeAssist.getInstance().getNewsTime4Show(date.mValue, "yyyy")
                    val type = report[1]
                    val index = dealReport(type.mValue, year, model)
                    if (report.size > removeCount) {
                        report = report.subList(removeCount, size)
                    }
                    report.add(date)
                    report.add(FinanceKeyIndexModel(index.toString()))
                    insertReport(index, report, model)
                    model.mAllReportList.add(report)
                }
            }
        }
        doCompare(model.mYearReportList)
        doCompare(model.mMiddleReportList)
        doCompare(model.mFirstReportList)
        doCompare(model.mThirdReportList)
        val msgList = Message.obtain()
        msgList.what = MSG_UPDATE_REPORT_LIST
        msgList.obj = model
        sendMessage(msgList)
    }

    private fun doCompare(data: MutableList<MutableList<FinanceKeyIndexModel>>) {
        val size = data.size
        if (size > 1) {
            for (i in 0 until size - 1) {
                val currentList = data[i]
                val lastList = data[i + 1]
                if (getYear(currentList) - getYear(lastList) == 1) {
                    doCompare(currentList, lastList)
                }
            }
        }
    }

    private fun doCompare(currentData: MutableList<FinanceKeyIndexModel>, lastData: MutableList<FinanceKeyIndexModel>) {
        val currentSize = currentData.size
        val lastDataSize = lastData.size
        try {
            if (currentSize == lastDataSize) {
                for (i in 0 until currentSize) {
                    val currentModel = currentData[i]
                    val lastModel = lastData[i]
                    if (currentModel.mValue.isNullOrEmpty() || lastModel.mValue.isNullOrEmpty()) {
                        continue
                    }
                    val currentValue = currentModel.mValue?.toFloat() ?: 0f
                    val lastValue = lastModel.mValue?.toFloat() ?: 0f
                    if (lastValue != 0f) {
                        currentModel.mCompareValue = CommonFunc.paraFloatRate((currentValue - lastValue) / lastValue, 2)
                    }
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun getYear(data: MutableList<FinanceKeyIndexModel>): Int {
        val size = data.size
        try {
            if (size >= 2) {
                val keyIndexModel = data[size - 2]
                val year = NewsTimeAssist.getInstance().getNewsTime4Show(keyIndexModel.mValue, "yyyy") ?: ""
                return year.toInt()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return 0
    }

    private fun dealReport(source : String?, year: String, model : FinanceReportDetailModel) : Int {
        if (source.isNullOrEmpty()) {
            return 0
        }
        return try {
            val reportDate = when(source) {
                REPORT_FIRST_SEASON -> 1
                REPORT_MIDDLE_YEAR -> 2
                REPORT_THIRD_SEASON -> 3
                else -> 4
            }
            val title = year + source
            insertReportTitle(reportDate, title, model)
            model.mAllReportTitles.add(title)
            reportDate
        } catch (e: Exception) {
            e.printStackTrace()
            0
        }
    }

    private fun insertReport(reportDate : Int, data : MutableList<FinanceKeyIndexModel>, model: FinanceReportDetailModel) {
        if (data != null) {
            val newData = mutableListOf<FinanceKeyIndexModel>()
            newData.addAll(data)
            when (reportDate) {
                TYPE_FIRST_REPORT -> model.mFirstReportList.add(newData)
                TYPE_MIDDEL_REPORT -> model.mMiddleReportList.add(newData)
                TYPE_THIRD_REPORT -> model.mThirdReportList.add(newData)
                TYPE_YEAR_REPORT -> model.mYearReportList.add(newData)
            }
        }
    }

    private fun insertReportTitle(reportDate : Int, title : String, model: FinanceReportDetailModel) {
        when (reportDate) {
            TYPE_FIRST_REPORT -> model.mFirstReportTitles.add(title)
            TYPE_MIDDEL_REPORT -> model.mMiddleReportTitles.add(title)
            TYPE_THIRD_REPORT -> model.mThirdReportTitles.add(title)
            TYPE_YEAR_REPORT -> model.mYearReportTitles.add(title)
        }
    }

    private fun paraStringToFloatRate(text: String?): String {
        if (text.isNullOrEmpty()) {
            return ""
        }
        return try {
            val fr = text.toFloat()
            if (fr.toLong() == Long.MAX_VALUE) {
                ""
            } else {
                CommonFunc.doubleFormat(fr.toDouble(), 2)
            }
        } catch (e: Exception) {
            text
        }
    }

    private fun paraStringToFloat(text: String?, len: Int): String {
        if (text.isNullOrEmpty()) {
            return ""
        }
        return try {
            val fr = text.toFloat()
            if (fr.toLong() == Long.MAX_VALUE) {
                ""
            } else {
                CommonFunc.fixTextWithMark(fr.toDouble(), len)
            }
        } catch (e: Exception) {
            text
        }
    }

}