package wind.android.f5.view.bottom.subview.finance.presenter

import android.content.res.Resources
import android.os.Message
import net.data.network.SkyCallbackData
import net.datamodel.network.CommonFunc
import net.listener.ISkyDataListenerEx
import net.util.TimeHelper
import util.StringUtils
import util.ToastTool
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.StockFinanceData
import wind.android.f5.view.bottom.subview.finance.manager.FinanceDataManager
import wind.android.f5.view.bottom.subview.finance.model.FinanceKeyIndexModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceReportDetailModel
import wind.android.f5.view.bottom.subview.finance.view.IFinanceKeyView
import wind.android.f5.view.bottom.subview.parse.StockFinanceXmlTool

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceProfitPresenter$
 * # Function:       关键指标presenter
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/20$    17:53$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class FinanceKeyIndexPresenter(fragmentView : IFinanceKeyView) : BaseFinancePresenter<IFinanceKeyView>(fragmentView),
        ISkyDataListenerEx {

    companion object {

        const val DATE_FORMAT = "yyyyMMdd"

        const val FOUR_YEAR = 4 * 4

        const val LIST_SIZE = 54
    }

    private val mReportDesc by lazy { StringUtils.getStringArray(R.array.finance_report) }

    private val mReportSource = mutableListOf<MutableList<FinanceKeyIndexModel>>()

    private var mNextReportDate: String? = null

    private var mNextTempReportDate: String? = null

    private var mReportRequestId = 0

    private val mReportSeason = mutableListOf<String>()

    private val mManager by lazy { FinanceDataManager() }

    override fun requestReportDescList(resources : Resources): MutableList<GroupListModel>? {
        val nameList = mutableListOf<GroupListModel>()
        StockFinanceXmlTool.conditionsXmlParse(resources, R.xml.keyindex_ab, nameList)
        return nameList
    }

    override fun requestReport(windCode : String?) {
        mReportSeason.clear()
        val reportSeason = getReportData(windCode)
        if (reportSeason.isNullOrBlank()) {
            getView()?.showEmptyView()
            return
        }
        val seasonList = CommonFunc.getSeasonDate(200, reportSeason, DATE_FORMAT) ?: null
        seasonList?.run {
            if (size == 0) {
                getView()?.showEmptyView()
                return
            }
            val list = mutableListOf<String>()
            val realSize = Math.min(size, FOUR_YEAR)
            for (i in 0 until realSize) {
                val season = this[i]
                val quarter = CommonFunc.getReportSeasonIndex(season)
                if (season.length > 4) {
                    val reportSeasonStr = season.substring(0, 4) + "." + quarter
                    mReportSeason.add(reportSeasonStr)
                } else {
                    mReportSeason.add(season)
                }
                list.add(season)
            }
            if (size > FOUR_YEAR + 1) {
                mNextTempReportDate = this[FOUR_YEAR]
            }
            mReportRequestId = mManager.requestABKeyDetail(windCode, list, this@FinanceKeyIndexPresenter)
            if (mReportRequestId == 0 || mReportRequestId == -1 || mReportRequestId == -2) {
                getView()?.showEmptyView()
            }
        }
    }

    override fun requestSingleSeason(windCode: String?) {
    }

    private fun getReportData(windCode : String?): String? {
        if (mNextReportDate.isNullOrBlank()) {
            return StockFinanceData.reportSeasonMap[windCode]
                    ?: TimeHelper.getInstance().getDate(DATE_FORMAT)
        }
        return mNextReportDate
    }

    override fun OnSkyError(code: Int, serialNum: Int) {
        getView()?.showEmptyView()
    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        val requestId = data?.SerialNum ?: 0
        if (requestId == mReportRequestId) {
            data?.data?.let {
                if (it.isEmpty()) {
                    ToastTool.showToast(R.string.finance_load_no_data)
                    return
                }
                val any = it[0]
                // 请求成功，才更新报告期
                mNextReportDate = mNextTempReportDate
                // 删除前5位数据
                parseData(any)
            }
        }
    }

    private fun parseData(list: Any) {
        try {
            val dataList = list as? MutableList<String?>
            if (dataList == null || dataList.isEmpty()) {
                ToastTool.showToast(R.string.finance_load_no_data)
                return
            }
            val size = mReportSeason.size
            val data = mutableListOf<MutableList<FinanceKeyIndexModel>>()
            for (i in 0 until size) {
                val tempList = mutableListOf<FinanceKeyIndexModel>()
                tempList.add(FinanceKeyIndexModel("1", ""))
                tempList.add(FinanceKeyIndexModel(mReportSeason[i], ""))
                var offset = 0
                val builder = StringBuilder()
                for (j in 0 until 60) {
                    val index = 2 + offset + i * (LIST_SIZE)
                    val value = dataList.elementAtOrNull(index) ?: ""
                    builder.append(value)
                    val model = FinanceKeyIndexModel()
                    if (j == 0 || j == 12 || j == 20 || j == 28 || j == 36 || j == 47) {
                        tempList.add(model)
                    } else if ((j in 13..19) || j == 21 || j == 22 || (j in 24..27) || j == 33 || (j in 48..59)) {
                        val floatRate = paraStringToFloatRate(value)
                        if (!floatRate.isEmpty()) {
                            model.mUint = "%"
                            model.mValue = floatRate
                        }
                        tempList.add(model)
                        offset++
                    } else {
                        var len = 2
                        if (j in 1..11) {
                            len = 4
                        }
                        val temp = paraStringToFloat(value, len)
                        if (!temp.isEmpty()) {
                            val length = temp.length
                            val lastString = temp.substring(length - 1, length)
                            if (isNumber(lastString)) {
                                model.mValue = temp
                                when (j) {
                                    in 1..11 -> model.mUint = "元"
                                    in 37..41 -> model.mUint = "天"
                                    in 42..46 -> model.mUint = "次"
                                }
                            } else {
                                model.mValue = temp.substringBefore(lastString)
                            }
                        }
                        tempList.add(model)
                        offset++
                    }
                }
                if (!builder.toString().isEmpty()) {
                    data.add(tempList)
                }
            }
            dealReport(data, 2)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun isNumber(value: String): Boolean {
        if (value == "%" || value == "亿" || value == "万" || value == "天" || value == "元" || value == "次") {
            return false
        }
        return true
    }

    private fun dealReport(realData : MutableList<MutableList<FinanceKeyIndexModel>>, removeCount: Int) {
        val model = FinanceReportDetailModel()
        realData.addAll(0, mReportSource)
        mReportSource.clear()
        realData.forEach {
            val reports = it
            var report = mutableListOf<FinanceKeyIndexModel>()
            report.addAll(reports)
            model.mReportList.add(reports)
            mReportSource.add(reports)
            val size = report.size
            if (size > 2) {
                val date = report[1]
                val index = dealReport(date.mValue, model)
                if (report.size > removeCount) {
                    report = report.subList(removeCount, size)
                }
                report.add(date)
                report.add(FinanceKeyIndexModel())
                insertReport(index, report, model)
                model.mAllReportList.add(report)
            }
        }
        doCompare(model.mYearReportList)
        doCompare(model.mMiddleReportList)
        doCompare(model.mFirstReportList)
        doCompare(model.mThirdReportList)
        val msgList = Message.obtain()
        msgList.what = MSG_UPDATE_REPORT_LIST
        msgList.obj = model
        sendMessage(msgList)
    }

    private fun doCompare(data: MutableList<MutableList<FinanceKeyIndexModel>>) {
        val size = data.size
        if (size > 1) {
            for (i in 0 until size - 1) {
                val currentList = data[i]
                val lastList = data[i + 1]
                if (getYear(currentList) - getYear(lastList) == 1) {
                    doCompare(currentList, lastList)
                }
            }
        }
    }

    private fun doCompare(currentData: MutableList<FinanceKeyIndexModel>, lastData: MutableList<FinanceKeyIndexModel>) {
        val currentSize = currentData.size
        val lastDataSize = lastData.size
        try {
            if (currentSize == lastDataSize) {
                for (i in 0 until currentSize) {
                    val currentModel = currentData[i]
                    val lastModel = lastData[i]
                    if (currentModel.mValue.isNullOrEmpty() || lastModel.mValue.isNullOrEmpty()) {
                        continue
                    }
                    val currentValue = currentModel.mValue?.toFloat() ?: 0f
                    val lastValue = lastModel.mValue?.toFloat() ?: 0f
                    if (lastValue != 0f) {
                        currentModel.mCompareValue = CommonFunc.paraFloatRate((currentValue - lastValue) / lastValue, 2)
                    }
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun getYear(data: MutableList<FinanceKeyIndexModel>): Int {
        val size = data.size
        try {
            if (size >= 2) {
                val keyIndexModel = data[size - 2]
                val mValue = keyIndexModel.mValue
                mValue?.let {
                    if (it.contains(".")) {
                        return it.substringBefore(".").toInt()
                    }
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return 0
    }

    private fun parseReportTitle(index : Int) : String {
        mReportDesc?.let {
            if (index >= 0 && index < mReportDesc.size) {
                return mReportDesc.get(index)
            }
        }
        return ""
    }

    private fun dealReport(source : String?, model : FinanceReportDetailModel) : Int {
        if (source.isNullOrEmpty() || !source.contains(".")) {
            return 0
        }
        try {
            val reportDate = source.substringAfter(".").toInt()
            val reportStr = parseReportTitle(reportDate - 1)
            val year = source.substringBefore(".")
            val title = year + reportStr
            insertReportTitle(reportDate, title, model)
            model.mAllReportTitles.add(title)
            return reportDate
        } catch (e: Exception) {
            e.printStackTrace()
            return 0
        }
    }

    private fun insertReport(reportDate : Int, data : MutableList<FinanceKeyIndexModel>, model: FinanceReportDetailModel) {
        val newData = mutableListOf<FinanceKeyIndexModel>()
        newData.addAll(data)
        when (reportDate) {
            TYPE_FIRST_REPORT -> model.mFirstReportList.add(newData)
            TYPE_MIDDEL_REPORT -> model.mMiddleReportList.add(newData)
            TYPE_THIRD_REPORT -> model.mThirdReportList.add(newData)
            TYPE_YEAR_REPORT -> model.mYearReportList.add(newData)
        }
    }

    private fun insertReportTitle(reportDate : Int, title : String, model: FinanceReportDetailModel) {
        when (reportDate) {
            TYPE_FIRST_REPORT -> model.mFirstReportTitles.add(title)
            TYPE_MIDDEL_REPORT -> model.mMiddleReportTitles.add(title)
            TYPE_THIRD_REPORT -> model.mThirdReportTitles.add(title)
            TYPE_YEAR_REPORT -> model.mYearReportTitles.add(title)
        }
    }

    private fun paraStringToFloatRate(text: String?): String {
        if (text.isNullOrEmpty()) {
            return ""
        }
        return try {
            val fr = text.toFloat()
            if (fr.toLong() == Long.MAX_VALUE) {
                ""
            } else {
                CommonFunc.doubleFormat(fr.toDouble(), 2)
            }
        } catch (e: Exception) {
            text
        }
    }

    private fun paraStringToFloat(text: String?, len: Int): String {
        if (text.isNullOrEmpty()) {
            return ""
        }
        return try {
            val fr = text.toFloat()
            if (fr.toLong() == Long.MAX_VALUE) {
                ""
            } else {
                CommonFunc.fixTextWithMark(fr.toDouble(), len)
            }
        } catch (e: Exception) {
            text
        }
    }

}