package wind.android.f5.view.bottom.subview.finance.view

import android.content.Context
import android.support.design.widget.TabLayout
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.RelativeLayout
import android.widget.TextView
import kotlinx.android.synthetic.main.finance_report_layout.view.*
import ui.UIFixedScrollListView
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.finance.listener.IShowChangeListener
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9RawModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9TitleModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceKeyIndexModel
import wind.android.f5.view.bottom.subview.finance.presenter.*

/**
 * # ***********************************************************************************************
 * # ClassName:      AbsFinanceView$
 * # Function:       财务详情-视图基类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/28$    20:02$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
abstract class AbsFinanceView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : RelativeLayout(context, attrs, defStyleAttr), UIFixedScrollListView.OnHorizontalLoadListener, IFinanceKeyView {

    companion object {
        const val PRESENTER_TYPE_KEY_INDEX = 1

        const val PRESENTER_TYPE_PROFIT = 2

        const val PRESENTER_TYPE_BALANCE_SHEET = 3

        const val PRESENTER_TYPE_CASH = 4

        const val PRESENTER_HK_TYPE_KEY_INDEX = 5

        const val PRESENTER_HK_GSD_TYPE_PROFIT = 6

        const val PRESENTER_HK_GSD_TYPE_BALANCE_SHEET = 7

        const val PRESENTER_HK_GSD_TYPE_CASH = 8

        const val PRESENTER_HK_ARD_TYPE_PROFIT = 9

        const val PRESENTER_HK_ARD_TYPE_BALANCE_SHEET = 10

        const val PRESENTER_HK_ARD_TYPE_CASH = 11

    }

    protected var mWindCode = ""

    protected var mTabPosition = 0

    protected var mIsShowCompare = false

    var showListener: IShowChangeListener? = null

    protected var mPresenter : BaseFinancePresenter<IFinanceKeyView>? = null

    fun setWindCode(windCode: String) {
        mWindCode = windCode
    }

    fun createPresenter(type: Int) {
        mPresenter = when(type) {
            PRESENTER_TYPE_KEY_INDEX -> FinanceKeyIndexPresenter(this)
            PRESENTER_HK_TYPE_KEY_INDEX -> FinanceHKKeyIndexPresenter(this)
            PRESENTER_TYPE_PROFIT, PRESENTER_TYPE_BALANCE_SHEET, PRESENTER_TYPE_CASH,
            PRESENTER_HK_GSD_TYPE_PROFIT, PRESENTER_HK_GSD_TYPE_CASH,
            PRESENTER_HK_GSD_TYPE_BALANCE_SHEET, PRESENTER_HK_ARD_TYPE_PROFIT,
            PRESENTER_HK_ARD_TYPE_BALANCE_SHEET, PRESENTER_HK_ARD_TYPE_CASH-> BaseFinanceF9Presenter(this, type)
            else -> null
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        mPresenter?.detachView()
    }

    /**
     * 初始化比较按钮
     */
    protected fun initCompareBtn() {
        show_layout.setOnClickListener {
            val isChecked = !show_yoy.isChecked
            show_yoy.isChecked = isChecked
            title_layout.scrollTo(0, 0)
            mIsShowCompare = isChecked
            updateTitle(mTabPosition)
        }
    }

    /**
     * 初始化底部tab
     */
    protected fun initBottomTab() {
        val bottomTab = getBottomTab()
        bottomTab.forEach {
            finance_report_tab.addTab(finance_report_tab.newTab().setText(it))
        }
        finance_report_tab.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {
                mTabPosition = tab?.position ?: 0
                updateTitle(mTabPosition)
                title_layout.scrollTo(0, 0)
            }
        })
    }

    /**
     * 显示空页面
     */
    fun showEmpty() {
        finance_empty_view.visibility = View.VISIBLE
        list_view_detail.visibility = View.GONE
    }

    /**
     * 隐藏空页面
     */
    fun hideEmptyView() {
        if (finance_empty_view.visibility == View.VISIBLE) {
            finance_empty_view.visibility = View.GONE
            list_view_detail.visibility = View.VISIBLE
        }
    }

    /**
     * 创建标题tab
     */
    protected fun createTitle(list: MutableList<String>) {
        val from = LayoutInflater.from(context)
        title_layout.removeAllViews()
        list.forEach {
            val childContent = from.inflate(getItemLayoutId(), null)
            val valueText = childContent.findViewById<TextView>(R.id.title_value)
            valueText.text = it
            title_layout.addView(childContent)
        }
        var scrollWidth = 0
        for (index in 0 until title_layout.childCount) {
            val childAt = title_layout.getChildAt(index) ?: null ?: continue
            val layoutParam = childAt.layoutParams ?: null ?: continue
            layoutParam.width = context.resources.getDimensionPixelSize(R.dimen.dp_120)
            scrollWidth += layoutParam.width
            childAt.layoutParams = layoutParam
        }
        setTitleView(title_layout)
        // 计算横向滑动距离
        list_view_detail.maxScrollWidth = scrollWidth - title_layout.measuredWidth
        list_view_detail.titleView = title_layout
    }

    protected fun removeEmptyItem(groupList: MutableList<GroupListModel>, dataList: MutableList<MutableList<FinanceKeyIndexModel>>) {
        val size = groupList.size
        val removeList = mutableListOf<Int>()
        for (i in 0 until size) {
            if (groupList[i].isGroup == GroupListModel.TYPE_GROUP) {
                continue
            }
            val builder = StringBuilder()
            dataList.forEach {
                if (i >= it.size) {
                    return@forEach
                }
                builder.append(it[i].mValue ?: "")
            }
            if (builder.toString().isEmpty()) {
                removeList.add(i)
            }
        }
        val removeSize = removeList.size
        for ((offset, i) in (0 until removeSize).withIndex()) {
            val index = removeList[i] - offset
            if (groupList.size > index) {
                groupList.removeAt(index)
            }
            dataList.forEach {
                if (it.size > index) {
                    it.removeAt(index)
                }
            }
        }
    }

    protected fun removeEmptyItemF9(groups: MutableList<FinanceF9TitleModel>, list: MutableList<FinanceF9RawModel>) {
        val size = groups.size
        val removeList = mutableListOf<Int>()
        for (i in 0 until size) {
            if (groups[i].mIsGroup) {
                continue
            }
            val builder = StringBuilder()
            list.forEach {
                if (i >= it.mData.size) {
                    return@forEach
                }
                builder.append(it.mData[i].mValue)
            }
            if (builder.toString().isEmpty()) {
                removeList.add(i)
            }
        }
        val removeSize = removeList.size
        for ((offset, i) in (0 until removeSize).withIndex()) {
            val index = removeList[i] - offset
            if (groups.size > index) {
                groups.removeAt(index)
            }
            list.forEach {
                if (it.mData.size > index) {
                    it.mData.removeAt(index)
                }
            }
        }
    }

    /**
     * 获取标题item
     */
    abstract fun getItemLayoutId(): Int

    /**
     * 更新标题
     */
    abstract fun updateTitle(position : Int)

    /**
     * 获取底部tab内容
     */
    abstract fun getBottomTab(): List<String>

    /**
     * 将标题view传给adapter
     */
    abstract fun setTitleView(view: View)

}