package wind.android.f5.view.bottom.subview.finance.view

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import kotlinx.android.synthetic.main.finance_report_layout.view.*
import wind.android.f5.R
import wind.android.f5.view.bottom.subview.finance.adapter.FinanceReportF9Adapter
import wind.android.f5.view.bottom.subview.finance.model.*

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceReportView$
 * # Function:       报告期展示视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/28$    15:59$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class BaseFinanceF9ReportView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : AbsFinanceView(context, attrs, defStyleAttr), View.OnClickListener {

    companion object {
        const val INDEX_ALL_REPORT = 0
        const val INDEX_YEAR_REPORT = 1
        const val INDEX_MIDDLE_REPORT = 2
        const val INDEX_FIRST_REPORT = 3
        const val INDEX_THIRD_REPORT = 4
    }

    private val mAdapter by lazy { FinanceReportF9Adapter(context) }

    private var mModel: FinanceF9ReportModel? = null

    private var mIsShowCompareBtn = true

    init {
        LayoutInflater.from(context).inflate(R.layout.finance_report_layout, this)
        list_view_detail.setItemBackground(R.color.transparent)
        list_view_detail.adapter = mAdapter
        // 设置可滚动视图的id
        list_view_detail.setScrollId(R.id.scroll_content)
        list_view_detail.setOnHorizontalLoadListener(this)
        initBottomTab()
        initCompareBtn()
        show_desc.setOnClickListener(this)
        show_layout.visibility = View.VISIBLE
    }

    fun setDownBtnVisibility(visibility: Int) {
        down_btn.visibility = visibility
    }

    fun setShowCompare(isShow: Boolean) {
        mIsShowCompareBtn = isShow
        show_layout.visibility = if (mIsShowCompareBtn) View.VISIBLE else View.GONE
    }

    override fun getItemLayoutId(): Int {
        return R.layout.item_finance_title
    }

    override fun getBottomTab(): List<String> {
        return listOf("全部", "年报", "中报", "一季报", "三季报")
    }

    override fun updateTitle(position: Int) {
        when (position) {
            INDEX_ALL_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mTopTitles)
                    dealData(this.mLeftTitles, this.mAllList)
                    createTitle(title)
                }
            }
            INDEX_YEAR_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mYearReportTitle)
                    dealData(this.mLeftTitles, this.mYearReportList)
                    createTitle(title)
                }
            }
            INDEX_MIDDLE_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mMiddleReportTitle)
                    dealData(this.mLeftTitles, this.mMiddleReportList)
                    createTitle(title)
                }
            }
            INDEX_FIRST_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mFirstReportTitle)
                    dealData(this.mLeftTitles, this.mFirstReportList)
                    createTitle(title)
                }
            }
            INDEX_THIRD_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mThirdReportTitle)
                    dealData(this.mLeftTitles, this.mThirdReportList)
                    createTitle(title)
                }
            }
        }
    }

    private fun dealData(groups: MutableList<FinanceF9TitleModel>, list: MutableList<FinanceF9RawModel>) {
        val tempList = mutableListOf<FinanceF9RawModel>()
        list.forEach {
            val model =  FinanceF9RawModel()
            model.clone(it)
            tempList.add(model)
        }
        val tempGroup = mutableListOf<FinanceF9TitleModel>()
        groups.forEach {
            tempGroup.add(it)
        }
        removeEmptyItemF9(tempGroup, tempList)
        mAdapter.setData(tempGroup, tempList)
        if (tempList.isEmpty()) {
            showEmpty()
        } else {
            hideEmptyView()
        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.show_desc -> {
                showListener?.showSeason()
            }
        }
    }

    fun requestData() {
        if (mModel == null) {
            mPresenter?.requestReport(mWindCode)
        }
    }

    override fun onHorizontalLoadMore() {
        mPresenter?.requestReport(mWindCode)
    }

    override fun onHorizontalBackScroll() {
    }

    override fun setTitleView(view: View) {
        mAdapter.setTitleView(view)
    }

    override fun updateKeyIndexList(model: FinanceReportDetailModel) {
    }

    override fun updateFinanceReportList(model: FinanceF9ReportModel) {
        hideEmptyView()
        mModel = model
        mAdapter.setCurrency(model.originalCurrency)
        updateTitle(mTabPosition)
    }

    override fun updateFinanceSeasonList(model: FinanceF9SeasonModel) {
    }

    override fun showProgress() {
    }

    override fun hideProgress() {
    }

    override fun onError() {
    }

    override fun showEmptyView() {
        if (mModel == null) {
            super.showEmpty()
        }
    }

}