package wind.android.f5.view.bottom.subview.finance.view

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import kotlinx.android.synthetic.main.finance_report_layout.view.*
import wind.android.f5.R
import wind.android.f5.view.bottom.subview.finance.adapter.FinanceSeasonF9Adapter
import wind.android.f5.view.bottom.subview.finance.model.*

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceReportView$
 * # Function:       单季度展示视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/11/28$    15:59$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class BaseFinanceF9SeasonView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : AbsFinanceView(context, attrs, defStyleAttr), View.OnClickListener {

    companion object {
        const val INDEX_ALL_SEASON = 0
        const val INDEX_FIRST_SEASON = 1
        const val INDEX_SECOND_SEASON = 2
        const val INDEX_THIRD_SEASON = 3
        const val INDEX_FOURTH_SEASON = 4
    }

    private val mAdapter by lazy { FinanceSeasonF9Adapter(context) }

    private var mModel: FinanceF9SeasonModel? = null

    private var mIsShowCompareBtn = true

    init {
        LayoutInflater.from(context).inflate(R.layout.finance_report_layout, this)
        list_view_detail.setItemBackground(R.color.transparent)
        list_view_detail.adapter = mAdapter
        // 设置可滚动视图的id
        list_view_detail.setScrollId(R.id.scroll_content)
        list_view_detail.setOnHorizontalLoadListener(this)
        initBottomTab()
        initCompareBtn()
        show_desc.setText(R.string.finance_by_season)
        show_desc.setOnClickListener(this)
        show_layout.visibility = View.VISIBLE
        finance_data_compare.setText(R.string.finance_chain_ratio)
    }

    fun setShowCompare(isShow: Boolean) {
        mIsShowCompareBtn = isShow
        show_layout.visibility = if (mIsShowCompareBtn) View.VISIBLE else View.GONE
    }

    override fun getItemLayoutId(): Int {
        return R.layout.item_finance_title
    }

    override fun getBottomTab(): List<String> {
        return listOf("全部", "一季度", "二季度", "三季度", "四季度")
    }

    override fun updateTitle(position: Int) {
        when (position) {
            INDEX_ALL_SEASON -> {
                if (mIsShowCompareBtn) {
                    show_layout.visibility = View.VISIBLE
                }
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mTopTitles)
                    dealData(this.mLeftTitles, this.mAllList)
                    createTitle(title)
                }
            }
            INDEX_FIRST_SEASON -> {
                if (mIsShowCompareBtn) {
                    show_layout.visibility = View.GONE
                }
                mAdapter.setShowCompare(false)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mFirstSeasonTitles)
                    dealData(this.mLeftTitles, this.mFirstSeasonList)
                    createTitle(title)
                }
            }
            INDEX_SECOND_SEASON -> {
                if (mIsShowCompareBtn) {
                    show_layout.visibility = View.GONE
                }
                mAdapter.setShowCompare(false)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mSecondSeasonTitles)
                    dealData(this.mLeftTitles, this.mSecondSeasonList)
                    createTitle(title)
                }

            }
            INDEX_THIRD_SEASON -> {
                if (mIsShowCompareBtn) {
                    show_layout.visibility = View.GONE
                }
                mAdapter.setShowCompare(false)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mThirdSeasonTitles)
                    dealData(this.mLeftTitles, this.mThirdSeasonList)
                    createTitle(title)
                }
            }
            INDEX_FOURTH_SEASON -> {
                if (mIsShowCompareBtn) {
                    show_layout.visibility = View.GONE
                }
                mAdapter.setShowCompare(false)
                mModel?.run {
                    val title = mutableListOf<String>()
                    title.addAll(this.mFourthSeasonTitles)
                    dealData(this.mLeftTitles, this.mFourthSeasonList)
                    createTitle(title)
                }
            }
        }
    }

    private fun dealData(groups: MutableList<FinanceF9TitleModel>, list: MutableList<FinanceF9RawModel>) {
        val tempList = mutableListOf<FinanceF9RawModel>()
        list.forEach {
            val model =  FinanceF9RawModel()
            model.clone(it)
            tempList.add(model)
        }
        val tempGroup = mutableListOf<FinanceF9TitleModel>()
        groups.forEach {
            tempGroup.add(it)
        }
        removeEmptyItemF9(tempGroup, tempList)
        mAdapter.setData(tempGroup, tempList)
        if (tempList.isEmpty()) {
            showEmpty()
        } else {
            hideEmptyView()
        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.show_desc -> {
                showListener?.showReport()
            }
        }
    }

    fun requestData() {
        if (mModel == null) {
            mPresenter?.requestSingleSeason(mWindCode)
        }
    }

    override fun onHorizontalLoadMore() {
        mPresenter?.requestSingleSeason(mWindCode)
    }

    override fun onHorizontalBackScroll() {
    }

    override fun setTitleView(view: View) {
        mAdapter.setTitleView(view)
    }

    override fun updateKeyIndexList(model: FinanceReportDetailModel) {
    }

    override fun updateFinanceReportList(model: FinanceF9ReportModel) {
    }

    override fun updateFinanceSeasonList(model: FinanceF9SeasonModel) {
        hideEmptyView()
        mModel = model
        mAdapter.setCurrency(model.originalCurrency)
        updateTitle(mTabPosition)
    }

    override fun showProgress() {
    }

    override fun hideProgress() {
    }

    override fun onError() {
    }

    override fun showEmptyView() {
        if (mModel == null) {
            showEmpty()
        }
    }

}