package wind.android.f5.view.bottom.subview.finance.view

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import kotlinx.android.synthetic.main.finance_report_layout.view.*
import wind.android.f5.R
import wind.android.f5.model.GroupListModel
import wind.android.f5.view.bottom.subview.finance.adapter.FinanceDetailAdapter
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9ReportModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceF9SeasonModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceKeyIndexModel
import wind.android.f5.view.bottom.subview.finance.model.FinanceReportDetailModel

/**
 * # ***********************************************************************************************
 * # ClassName:      FinanceProfitReportView$
 * # Function:       关键指标视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/3$    10:25$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class FinanceKeyIndexReportView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : AbsFinanceView(context, attrs, defStyleAttr) {

    companion object {
        const val INDEX_ALL_REPORT = 0
        const val INDEX_YEAR_REPORT = 1
        const val INDEX_MIDDLE_REPORT = 2
        const val INDEX_FIRST_REPORT = 3
        const val INDEX_THIRD_REPORT = 4
    }

    private val mAdapter by lazy { FinanceDetailAdapter(context) }

    private var mModel: FinanceReportDetailModel? = null

    init {
        LayoutInflater.from(context).inflate(R.layout.finance_report_layout, this)
        list_view_detail.setItemBackground(R.color.transparent)
        down_btn.visibility = View.GONE
        list_view_detail.adapter = mAdapter
        // 设置可滚动视图的id
        list_view_detail.setScrollId(R.id.scroll_content)
        list_view_detail.setOnHorizontalLoadListener(this)
        initBottomTab()
        initCompareBtn()
        show_layout.visibility = View.VISIBLE
    }

    fun requestDesc() {
        if (!mAdapter.hasGroupData()) {
            mPresenter?.run {
                val descList = requestReportDescList(resources)
                descList?.let {
                    mAdapter.setData(it, mutableListOf())
                }
            }

        }
    }

    fun requestData() {
        if (mModel == null) {
            mPresenter?.requestReport(mWindCode)
        }
    }

    override fun getItemLayoutId(): Int {
        return R.layout.item_finance_title
    }

    override fun getBottomTab(): List<String> {
        return listOf("全部", "年报", "中报", "一季报", "三季报")
    }

    override fun onHorizontalLoadMore() {
        mPresenter?.requestReport(mWindCode)
    }

    override fun onHorizontalBackScroll() {
    }

    override fun updateTitle(position: Int) {
        when (position) {
            INDEX_ALL_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.let {
                    val title = mutableListOf<String>()
                    title.addAll(it.mAllReportTitles)
                    val descList = mPresenter?.requestReportDescList(resources)
                    descList?.run {
                        dealData(this, it.mAllReportList)
                        createTitle(title)
                    }
                }
            }
            INDEX_YEAR_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.let {
                    val title = mutableListOf<String>()
                    title.addAll(it.mYearReportTitles)
                    val descList = mPresenter?.requestReportDescList(resources)
                    descList?.run {
                        dealData(this, it.mYearReportList)
                        createTitle(title)
                    }
                }

            }
            INDEX_MIDDLE_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.let {
                    val title = mutableListOf<String>()
                    title.addAll(it.mMiddleReportTitles)
                    val descList = mPresenter?.requestReportDescList(resources)
                    descList?.run {
                        dealData(this, it.mMiddleReportList)
                        createTitle(title)
                    }
                }
            }
            INDEX_FIRST_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.let {
                    val title = mutableListOf<String>()
                    title.addAll(it.mFirstReportTitles)
                    val descList = mPresenter?.requestReportDescList(resources)
                    descList?.run {
                        dealData(this, it.mFirstReportList)
                        createTitle(title)
                    }
                }
            }
            INDEX_THIRD_REPORT -> {
                mAdapter.setShowCompare(mIsShowCompare)
                mModel?.let {
                    val title = mutableListOf<String>()
                    title.addAll(it.mThirdReportTitles)
                    val descList = mPresenter?.requestReportDescList(resources)
                    descList?.run {
                        dealData(this, it.mThirdReportList)
                        createTitle(title)
                    }
                }
            }
        }
    }

    private fun dealData(groups: MutableList<GroupListModel>, list: MutableList<MutableList<FinanceKeyIndexModel>>) {
        val tempList = mutableListOf<MutableList<FinanceKeyIndexModel>>()
        list.forEach {
            val itemList =  mutableListOf<FinanceKeyIndexModel>()
            itemList.addAll(it)
            tempList.add(itemList)
        }
        removeEmptyItem(groups, tempList)
        mAdapter.setData(groups, tempList)
    }

    fun updateList(model: FinanceReportDetailModel) {
        mModel = model
        updateTitle(mTabPosition)
    }

    override fun setTitleView(view: View) {
        mAdapter.setTitleView(view)
    }

    override fun updateKeyIndexList(model: FinanceReportDetailModel) {
        hideEmptyView()
        mModel = model
        updateTitle(mTabPosition)
    }

    override fun updateFinanceReportList(model: FinanceF9ReportModel) {
    }

    override fun updateFinanceSeasonList(model: FinanceF9SeasonModel) {
    }

    override fun showProgress() {
    }

    override fun hideProgress() {
    }

    override fun onError() {
    }

    override fun showEmptyView() {
        if (mModel == null) {
            showEmpty()
        }
    }
}