package wind.android.market.business.option.activity

import android.content.Intent
import android.os.Bundle
import android.support.design.widget.TabLayout
import android.util.SparseArray
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.widget.TextView
import base.BaseActivity
import base.BaseNetHandler
import eventinterface.TouchEventListener
import kotlinx.android.synthetic.main.activity_option_detail.*
import net.datamodel.network.RealQuoteItem
import session.F5Session
import ui.UINavigationBar
import util.ThemeUtils
import wind.android.f5.activity.SpeedDetailActivity
import wind.android.market.R
import wind.android.market.business.option.model.OptionMonthModel
import wind.android.market.business.option.model.OptionTitleSpeedModel
import wind.android.market.business.option.presenter.OptionDetailPresenter
import wind.android.market.business.option.view.IOptionDetailView
import wind.android.market.business.option.view.OptionRecyclerView
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      OptionDetailActivity$
 * # Function:       期权详情页面
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/20$    9:22$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class OptionDetailActivity: BaseActivity(), IOptionDetailView, TouchEventListener {

    companion object {
        const val DATA_TYPE_WIND_CODE = "wind_code"
        const val DATA_TYPE_CONTRACT_CODE = "contract_code"
        const val DATA_TYPE_SECURITY_NAME = "security_name"
        const val DATA_TYPE_EXCHANGE = "exchange"
    }

    private var mWindCode = ""

    private var mContractCode = ""

    private var mSecurityName = ""

    private var mExchange = ""

    private val mPresenter by lazy { OptionDetailPresenter(mWindCode, this) }

    private val mMonthList = mutableListOf<OptionMonthModel>()

    private val mViewList = SparseArray<OptionRecyclerView>()

    private var mCurrentView: OptionRecyclerView? = null

    private var mTitleModel: OptionTitleSpeedModel? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        ThemeUtils.setTheme(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_option_detail)
        initData()
        initView()
        requestData()
    }

    private fun initData() {
        mWindCode = intent?.getStringExtra(DATA_TYPE_WIND_CODE) ?: ""
        mSecurityName = intent?.getStringExtra(DATA_TYPE_SECURITY_NAME) ?: ""
        mExchange = intent?.getStringExtra(DATA_TYPE_EXCHANGE) ?: ""
        mContractCode = intent?.getStringExtra(DATA_TYPE_CONTRACT_CODE) ?: ""
    }

    /**
     * 请求数据
     */
    private fun requestData() {
        subTitle()
        requestMonth()
    }

    private fun requestMonth() {
        mPresenter.requestMonth(mContractCode, mExchange)
    }

    private fun subTitle() {
        mPresenter.subTitle()
    }

    override fun onResume() {
        super.onResume()
        mPresenter.onResume()
        mCurrentView?.subData()
    }

    override fun onPause() {
        super.onPause()
        mPresenter.onPause()
        mCurrentView?.unSubData()
    }

    private fun initView() {
        option_title_layout.setOnClickListener {
            if (!mWindCode.isEmpty()) {
                val intent = Intent(this, SpeedDetailActivity::class.java)
                intent.putExtra(F5Session.POSITION, 0)
                F5Session.getInstance().f5WindCodes = arrayOf(mWindCode)
                this.startActivity(intent)
            }
        }
        val titleView = LayoutInflater.from(this).inflate(R.layout.option_detail_title, null)
        val name = titleView.findViewById<TextView>(R.id.option_detail_name)
        name.text = mSecurityName
        val code = titleView.findViewById<TextView>(R.id.option_detail_code)
        code.text = dealWindCode(mWindCode)
        navigationBar.setTitleView(titleView)
        navigationBar.setListener(this)
        navigationBar.setRightView(R.drawable.icon_search, R.drawable.icon_search_focus)
        option_name.text = mSecurityName
        option_code.text = dealWindCode(mWindCode)
    }

    private fun dealWindCode(windCode: String): String {
        if (windCode.contains(".")) {
            return windCode.substringBefore(".")
        }
        return windCode
    }

    override fun setTitle(model: OptionTitleSpeedModel) {
        mTitleModel = model
        option_new_price.text = model.mNewPrice
        option_new_price.setTextColor(model.mTextColor)
        option_change.text = model.mChange
        option_change.setTextColor(model.mTextColor)
        option_change_range.text = model.mChangeRange
        option_change_range.setTextColor(model.mTextColor)
    }

    override fun updateTitle(item: RealQuoteItem) {
        val textColor = mTitleModel?.mTextColor ?: StockUtil.getChangeColor(0f)
        mPresenter.parseTitle(item, mTitleModel)
        option_new_price.text = mTitleModel?.mNewPrice ?: "--"
        option_new_price.setTextColor(textColor)
        option_change.text = mTitleModel?.mChange ?: "--"
        option_change.setTextColor(textColor)
        option_change_range.text = mTitleModel?.mChangeRange ?: "--"
        option_change_range.setTextColor(textColor)
    }

    override fun touchEvent(sender: View?, event: MotionEvent?) {
        if (sender?.tag  == UINavigationBar.RIGHT_BUTTON) {
            // 进入搜索
            val intent = Intent()
            intent.action = "wind.andorid.stock.STOCK_SEARCH"
            //原因子工程无法使用主工程的 数据；原来写法是：intent.putExtra(StockSearchFrag.LAST_ACTIVITY, CommSearchActivity2.SearchType.STOCK.val())
            intent.putExtra("last_activity", 3)
            intent.setPackage(packageName)
            startActivity(intent)
        }
    }

    override fun updateMonth(list: MutableList<OptionMonthModel>) {
        mMonthList.clear()
        mMonthList.addAll(list)
        option_month_tab.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener{
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {
                val position = tab?.position ?: 0
                if (position >= 0 && position < mMonthList.size) {
                    mCurrentView?.unSubData()
                    option_content.removeAllViews()
                    val view = getView(position)
                    option_content.addView(view)
                    view.requestData(mMonthList[position], mExchange)
                }
            }
        })
        list.forEach {
            val newTab = option_month_tab.newTab()
            newTab.text = it.mDateStr
            option_month_tab.addTab(newTab)
        }
    }

    override fun onSpeedStatusChange(status: Int, detailMsg: String?) {
        super.onSkyStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            mPresenter.unSubTitle()
            mPresenter.subTitle()
            mCurrentView?.unSubData()
            mCurrentView?.subData()
        }
    }

    override fun onSkyStatusChange(status: Int, detailMsg: String?) {
        super.onSkyStatusChange(status, detailMsg)
        if (status == BaseNetHandler.NetStatusListener.NET_CONNECTED) {
            val hasSubData = mCurrentView?.hasSubData() ?: false
            if (!hasSubData) {
                requestMonth()
            }
        }
    }

    private fun getView(posiotn: Int): OptionRecyclerView {
        var recyclerView = mViewList[posiotn] ?: null
        if (recyclerView == null) {
            recyclerView = OptionRecyclerView(this)
            mViewList.put(posiotn, recyclerView)
        }
        mCurrentView = recyclerView
        return recyclerView
    }

}