package wind.android.market.business.option.adapter

import android.content.Context
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import wind.android.market.business.option.viewholder.MirrorViewHolder

/**
 * # ***********************************************************************************************
 * # ClassName:      MirrorAdapter$
 * # Function:       镜像列表适配器-基类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/20$    9:54$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
abstract class BaseMirrorAdapter<T>(val mContext: Context): RecyclerView.Adapter<MirrorViewHolder>() {

    protected val mData: MutableList<T> = mutableListOf()

    private val mLeftViewList = mutableListOf<View>()
    private val mRightViewList = mutableListOf<View>()

    private var mMoveLeftX = -1

    private var mMoveRightX = -1

    protected var mLayoutManager: LinearLayoutManager? = null

    override fun onCreateViewHolder(p0: ViewGroup, viewType: Int): MirrorViewHolder {
        val view = LayoutInflater.from(mContext).inflate(getItemResId(), null)
        val viewHolder = MirrorViewHolder(view)
        initItemView(viewHolder)
        return viewHolder
    }

    fun setData(data: MutableList<T>?) {
        mData.clear()
        data?.let {
            mData.addAll(data)
        }
        notifyDataSetChanged()
    }

    fun setLayoutManager( layoutManager: LinearLayoutManager) {
        mLayoutManager = layoutManager
    }

    override fun getItemCount(): Int {
        return mData.size
    }

    override fun onBindViewHolder(viewHolder: MirrorViewHolder, position: Int) {
        val leftView = viewHolder.getView<View>(getLeftMoveId())
        if (mMoveLeftX >= 0) {
            leftView.scrollTo(mMoveLeftX, 0)
        }
        mLeftViewList.add(leftView)
        val rightView = viewHolder.getView<View>(getRightMoveId())
        if (mMoveRightX >= 0) {
            rightView.scrollTo(mMoveRightX, 0)
        }
        mRightViewList.add(rightView)
        val item = getItem(position)
        item?.run {
            bindData(viewHolder, position, this)
        }
    }

    fun getItem(position: Int): T? {
        if (position >= 0 && position < mData.size) {
            return mData[position]
        }
        return null
    }

    fun moveLeftItem(moveX: Int) {
        mMoveLeftX = moveX
        mLeftViewList.forEach {
            it.scrollTo(moveX, 0)
        }
    }

    fun moveRightItem(moveX: Int) {
        mMoveRightX = moveX
        mRightViewList.forEach {
            it.scrollTo(moveX, 0)
        }
    }

    protected open fun initItemView(viewHolder: MirrorViewHolder) {}

    abstract fun bindData(viewHolder: MirrorViewHolder, position: Int, data: T)

    abstract fun getItemResId(): Int

    abstract fun getLeftMoveId(): Int

    abstract fun getRightMoveId(): Int
}