package wind.android.market.business.option.adapter

import android.content.Context
import android.graphics.Color
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import net.datamodel.network.CommonFunc
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import ui.screen.UIScreen
import util.ThemeUtils
import wind.android.market.R
import wind.android.market.business.option.model.OptionContentModel
import wind.android.market.business.option.model.OptionDetailModel
import wind.android.market.business.option.viewholder.MirrorViewHolder
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      OptionMirrorAdapter$
 * # Function:       期权-镜像列表适配器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/20$    11:41$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class OptionMirrorAdapter(val context: Context): BaseMirrorAdapter<OptionContentModel>(context), View.OnClickListener {

    private var mLeftClickListener: ((View) -> Unit)? = null

    private var mRightClickListener: ((View) -> Unit)? = null

    /**
     * 更新期权列表
     */
    fun updateList(list: MutableList<RealQuoteItem>) {
        list.forEach {
            updateOptionDetail(it, mData)
        }
        notifyDataSetChanged()
    }

    fun setOnClickListener(leftListener: (View) -> Unit, rightListener: (View) -> Unit) {
        mLeftClickListener = leftListener
        mRightClickListener = rightListener
    }

    fun updateItem(item: RealQuoteItem) {
        updateOptionDetail(item, mData)
    }

    private fun updateOptionDetail(item: RealQuoteItem, data: MutableList<OptionContentModel>): Int {
        val windCode = item.WindCode
        val size = data.size
        for (i in 0 until size) {
            val contentModel = data[i]
            if (windCode == contentModel.leftModel.windCode) {
                parseOptionData(contentModel.leftModel, item)
                return i
            }
            if (windCode == contentModel.rightModel.windCode) {
                parseOptionData(contentModel.rightModel, item)
                return i
            }
        }
        return -1
    }

    private fun parseOptionData(model: OptionDetailModel, item: RealQuoteItem) {
        val size = item.indicators.size
        for (i in 0 until size) {
            when(item.indicators[i]) {
                Indicator.DI_NEWPRICE -> {
                    model.newPrice = item.value[i]
                }
                Indicator.DI_CHANGERANGE -> {
                    model.changeRange = item.value[i]
                }
                Indicator.DI_HIDDING_VOLATILITY -> {
                    model.hidingVolatility = item.value[i]
                }
                Indicator.DI_DELTA_VALUE -> {
                    model.delta = item.value[i]
                }
                Indicator.DI_GAMMA -> {
                    model.gamma = item.value[i]
                }
                Indicator.DI_VEGA_VALUE -> {
                    model.vega = item.value[i]
                }
                Indicator.DI_THETA_VALUE -> {
                    model.theta = item.value[i]
                }
                Indicator.DI_BUYPRICE1 -> {
                    model.buyPrice = item.value[i]
                }
                Indicator.DI_SELLPRICE1 -> {
                    model.sellPrice = item.value[i]
                }
                Indicator.DI_VOLUMN -> {
                    model.volume = item.value[i]
                }
                Indicator.DI_TOTALAMOUNT -> {
                    model.totalAmount = item.value[i]
                }
                Indicator.DI_POSITION -> {
                    model.position = item.value[i]
                }
                Indicator.DI_ACTUAL_LEVEL_TIMES -> {
                    model.actualLevelTimes = item.value[i]
                }
                Indicator.DI_INNER_PRICE -> {
                    model.innerPrice = item.value[i]
                }
                Indicator.DI_TIME_PRICE -> {
                    model.timesPrice = item.value[i]
                }
                Indicator.DI_PRICEUNIT -> {
                    model.priceUnit = item.value[i]
                }
            }
        }
    }

    override fun initItemView(viewHolder: MirrorViewHolder) {
        val strikeViewWidth = viewHolder.getView<TextView>(R.id.strike_price).layoutParams.width
        val width = (UIScreen.screenWidth * 0.235f).toInt()
        val leftLayout = viewHolder.getView<LinearLayout>(R.id.left_option_layout)
        leftLayout.setOnClickListener(this)
        viewHolder.getView<TextView>(R.id.time_price_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.inner_price_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.actual_level_times_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.position_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.total_amount_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.volume_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.sell_price_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.buy_price_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.theta_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.vega_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.gamma_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.delta_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.hiding_volatility_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.change_range_left).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.new_price_left).layoutParams.width = width
        leftLayout.scrollTo(width * 15 - (UIScreen.screenWidth * 0.5f - strikeViewWidth / 2).toInt(), 0)

        val rightLayout = viewHolder.getView<LinearLayout>(R.id.right_option_layout)
        rightLayout.setOnClickListener(this)
        viewHolder.getView<TextView>(R.id.new_price_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.change_range_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.hiding_volatility_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.delta_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.gamma_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.vega_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.theta_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.buy_price_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.sell_price_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.volume_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.total_amount_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.position_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.actual_level_times_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.inner_price_right).layoutParams.width = width
        viewHolder.getView<TextView>(R.id.time_price_right).layoutParams.width = width
    }

    override fun bindData(viewHolder: MirrorViewHolder, position: Int, data: OptionContentModel) {
        val leftModel = data.leftModel
        val leftPrecision = CommonFunc.getRadixPointFactor(leftModel.windCode)
        val leftPriceColor = StockUtil.getChangeColor(leftModel.changeRange)
        val leftLayout = viewHolder.getView<LinearLayout>(R.id.left_option_layout)
        leftLayout.tag = position
        val rightLayout = viewHolder.getView<LinearLayout>(R.id.right_option_layout)
        rightLayout.tag = position
        leftModel.run {
            viewHolder.getView<TextView>(R.id.time_price_left).text = CommonFunc.floatFormat(timesPrice.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.inner_price_left).text = CommonFunc.floatFormat(innerPrice.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.actual_level_times_left).text = CommonFunc.floatFormat(actualLevelTimes.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.position_left).text = CommonFunc.fixTText(position.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.total_amount_left).text = CommonFunc.fixTText(totalAmount.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.volume_left).text = CommonFunc.fixTText(volume.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.sell_price_left).text = CommonFunc.floatFormat(sellPrice.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.buy_price_left).text = CommonFunc.floatFormat(buyPrice.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.theta_left).text = CommonFunc.floatFormat(theta.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.vega_left).text = CommonFunc.floatFormat(vega.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.gamma_left).text = CommonFunc.floatFormat(gamma.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.delta_left).text = CommonFunc.floatFormat(delta.toDouble(), leftPrecision)
            viewHolder.getView<TextView>(R.id.hiding_volatility_left).text = (CommonFunc.fixTText(hidingVolatility.toDouble(), 2) + "%")
            viewHolder.getView<TextView>(R.id.change_range_left).setTextColor(leftPriceColor)
            viewHolder.getView<TextView>(R.id.change_range_left).text = (CommonFunc.fixTText(changeRange.toDouble(), 2) + "%")
            viewHolder.getView<TextView>(R.id.new_price_left).setTextColor(leftPriceColor)
            viewHolder.getView<TextView>(R.id.new_price_left).text = CommonFunc.floatFormat(newPrice.toDouble(), leftPrecision)
        }

        val strikePrice = data.strikePrice.toDouble()
        viewHolder.getView<TextView>(R.id.strike_price).text =  CommonFunc.floatFormat(strikePrice, leftPrecision)

        val rightModel = data.rightModel
        val rightPrecision = CommonFunc.getRadixPointFactor(rightModel.windCode)
        val rightPriceColor = StockUtil.getChangeColor(rightModel.changeRange)
        rightModel.run {
            viewHolder.getView<TextView>(R.id.new_price_right).text = CommonFunc.floatFormat(newPrice.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.new_price_right).setTextColor(rightPriceColor)
            viewHolder.getView<TextView>(R.id.change_range_right).text = (CommonFunc.fixTText(changeRange.toDouble(), 2) + "%")
            viewHolder.getView<TextView>(R.id.change_range_right).setTextColor(rightPriceColor)
            viewHolder.getView<TextView>(R.id.hiding_volatility_right).text = (CommonFunc.fixTText(hidingVolatility.toDouble(), 2) + "%")
            viewHolder.getView<TextView>(R.id.delta_right).text = CommonFunc.floatFormat(delta.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.gamma_right).text = CommonFunc.floatFormat(gamma.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.vega_right).text = CommonFunc.floatFormat(vega.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.theta_right).text = CommonFunc.floatFormat(theta.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.buy_price_right).text = CommonFunc.floatFormat(buyPrice.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.sell_price_right).text = CommonFunc.floatFormat(sellPrice.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.volume_right).text = CommonFunc.fixTText(volume.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.total_amount_right).text = CommonFunc.fixTText(totalAmount.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.position_right).text = CommonFunc.fixTText(position.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.actual_level_times_right).text = CommonFunc.floatFormat(actualLevelTimes.toDouble(), 2)
            viewHolder.getView<TextView>(R.id.inner_price_right).text = CommonFunc.floatFormat(innerPrice.toDouble(), rightPrecision)
            viewHolder.getView<TextView>(R.id.time_price_right).text = CommonFunc.floatFormat(timesPrice.toDouble(), rightPrecision)
        }
    }

    override fun getItemResId(): Int {
        return R.layout.item_mirror_option
    }

    override fun getLeftMoveId(): Int {
        return R.id.left_option_layout
    }

    override fun getRightMoveId(): Int {
        return R.id.right_option_layout
    }

    override fun onClick(v: View?) {
        when(v?.id) {
            R.id.left_option_layout -> {
                mLeftClickListener?.invoke(v)
            }
            R.id.right_option_layout -> {
                mRightClickListener?.invoke(v)
            }
        }
    }
}