package wind.android.market.business.option.presenter

import android.os.Message
import base.ActivityHandler
import net.data.network.SkyCallbackData
import net.listener.ISkyDataListenerEx
import wind.android.market.business.option.manager.OptionManager
import wind.android.market.business.option.model.MarketOptionModel
import wind.android.market.business.option.view.IOptionView

/**
 * # ***********************************************************************************************
 * # ClassName:      MarketOptionPresenter$
 * # Function:       行情-期权-数据管理类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/19$    9:37$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class MarketOptionPresenter(var mMainView: IOptionView) : ISkyDataListenerEx, ActivityHandler.ActivityHandlerListener {

    companion object {
        const val MSG_UPDATE_OPTION_LIST = 1001
    }

    private val mManager by lazy { createManager() }

    private var mRequestId = 0

    /**
     * 请求期权列表数据
     */
    fun requestOptionList() {
        mRequestId = mManager.requestOption(this)
    }

    private fun createManager(): OptionManager {
        return OptionManager()
    }

    override fun OnSkyError(code: Int, serialNum: Int) {
    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        data?.data?.run {
            val serialNum = data.SerialNum
            if (mRequestId == serialNum) {
                val source = this as? MutableList<MutableList<String?>>
                val dataList = mutableListOf<MarketOptionModel>()
                source?.forEach {
                    if (it.size == 5) {
                        val contractsName = it[0] ?: ""
                        val windCode = it[1] ?: ""
                        val contractsCode = it[2] ?: ""
                        val securityName = it[3] ?: ""
                        val exchange = it[4] ?: ""
                        if (!contractsName.contains("仿真")) {
                            val model = MarketOptionModel(contractsName, windCode, contractsCode, securityName, exchange)
                            dataList.add(model)
                        }
                    }
                }
                val msg = Message()
                msg.what = MSG_UPDATE_OPTION_LIST
                msg.obj = dataList
                sendMessage(msg)
            }
        }
    }

    private fun sendMessage(msg: Message) {
        ActivityHandler.getInstance(this).sendMessage(msg)
    }

    override fun handleMessage(msg: Message?) {
        msg?.run {
            when(this.what) {
                MSG_UPDATE_OPTION_LIST -> {
                    val data = this.obj as? MutableList<MarketOptionModel>
                    if (data != null) {
                        mMainView.updateOptionList(data)
                    }
                }
            }
        }

    }
}