package wind.android.market.business.option.view

import android.content.Context
import android.content.Intent
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import kotlinx.android.synthetic.main.market_option_layout.view.*
import ui.UIFixedScrollListView
import wind.android.market.R
import wind.android.market.business.option.activity.OptionDetailActivity
import wind.android.market.business.option.adapter.MarketOptionAdapter
import wind.android.market.business.option.model.MarketOptionModel
import wind.android.market.business.option.presenter.MarketOptionPresenter
import wind.android.market.parse.view.intf.ViewDataInterface
import wind.android.market.parse.view.model.BaseViewData

/**
 * # ***********************************************************************************************
 * # ClassName:      MarketOptionView$
 * # Function:       行情-期权页面
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/18$    15:57$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class MarketOptionView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        LinearLayout(context, attrs, defStyleAttr), ViewDataInterface<BaseViewData>, IOptionView, UIFixedScrollListView.FixedRowSelectListener {

    private val mPresenter by lazy { MarketOptionPresenter(this) }

    private val mAdapter by lazy { MarketOptionAdapter(context) }

    init {
        initView()
        requestData()
    }

    private fun initView() {
        LayoutInflater.from(context).inflate(R.layout.market_option_layout, this)
        this.orientation = LinearLayout.VERTICAL
        market_option_list.isScrollable = false
        market_option_list.adapter = mAdapter
        market_option_list.setAddRowSelectListener(this)
    }

    private fun requestData() {
        mPresenter.requestOptionList()
    }

    override fun setViewData(data: BaseViewData?) {
    }

    override fun getViewData(): BaseViewData? {
        return null
    }

    override fun updateOptionList(list: MutableList<MarketOptionModel>) {
        mAdapter.setData(list)
    }

    override fun onRowSelected(view: View?, index: Int) {
        val model = mAdapter.getItem(index)
        model?.let {
            val intent = Intent()
            intent.setClass(context, OptionDetailActivity::class.java)
            intent.putExtra(OptionDetailActivity.DATA_TYPE_WIND_CODE, it.mWindCode)
            intent.putExtra(OptionDetailActivity.DATA_TYPE_CONTRACT_CODE, it.contractsCode)
            intent.putExtra(OptionDetailActivity.DATA_TYPE_SECURITY_NAME, it.securityName)
            intent.putExtra(OptionDetailActivity.DATA_TYPE_EXCHANGE, it.exchange)
            context.startActivity(intent)
        }
    }

}