package wind.android.market.business.option.view

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.PointF
import android.support.v7.widget.LinearLayoutManager
import android.util.AttributeSet
import android.view.*
import android.widget.LinearLayout
import android.widget.Scroller
import kotlinx.android.synthetic.main.mirror_recycler_view_layout.view.*
import ui.screen.UIScreen
import wind.android.market.R
import wind.android.market.business.option.adapter.BaseMirrorAdapter

/**
 * # ***********************************************************************************************
 * # ClassName:      MirrorRecyclerView$
 * # Function:       镜像列表视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/20$    9:31$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
abstract class MirrorRecyclerView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0):
        LinearLayout(context, attrs, defStyleAttr) {

    companion object {
        /**
         * 滑动最小值
         */
        const val MOVE_MIN_VALUE = 5

        /**
         * 水平滑动
         */
        const val MOVE_DIRECTION_HORIZONTAL = 1

        /**
         * 竖直滑动
         */
        const val MOVE_DIRECTION_VERTICAL = 2
    }

    private lateinit var mAdapter:BaseMirrorAdapter<*>

    private var mTitleView: View? = null

    private var mLeftScrollView: View? = null

    private var mRightScrollView: View? = null

    private var mLeftScrollWidth = 0

    private var mRightScrollWidth = 0

    private val mLastP = PointF()

    private val mFirstP = PointF()

    private val mVelocityTracker: VelocityTracker by lazy { VelocityTracker.obtain() }

    /**
     * 计算滑动速度用
     */
    private var mMinimumVelocity: Int = 0

    private var mMaximumVelocity: Int = 0

    /**
     * 多点触摸只算第一根手指的速度
     */
    private var mPointerId: Int = 0

    private var mMoveMax = 0

    private var mScroller: Scroller

    private var mTouchLeft = false

    private var mTouchRight = false

    private var mMoveDirection = 0

    init {
        LayoutInflater.from(context).inflate(R.layout.mirror_recycler_view_layout, this.getRootGroup())
        this.orientation = LinearLayout.VERTICAL
        mScroller = Scroller(context)
        mMinimumVelocity = ViewConfiguration.get(context).scaledMinimumFlingVelocity
        mMaximumVelocity = ViewConfiguration.get(context).scaledMaximumFlingVelocity
    }

    abstract fun getRootGroup(): ViewGroup

    fun setAdapter(adapter: BaseMirrorAdapter<*>) {
        val layoutManager = LinearLayoutManager(context)
        layoutManager.orientation = LinearLayoutManager.VERTICAL
        mirror_recycler_view.adapter = adapter
        mirror_recycler_view.setItemViewCacheSize(20)
        val recycledViewPool = mirror_recycler_view.recycledViewPool
        recycledViewPool.setMaxRecycledViews(0, 20)
        mirror_recycler_view.setRecycledViewPool(recycledViewPool)
        mirror_recycler_view.layoutManager = layoutManager
        mAdapter = adapter
        mAdapter.setLayoutManager(layoutManager)
    }

    fun setTitleView(titleView: View, leftScrollView: View, rightScrollView: View) {
        mirror_title.addView(titleView)
        mTitleView = titleView
        mLeftScrollView = leftScrollView
        mRightScrollView = rightScrollView
    }

    fun setMoveMax(moveMax: Int) {
        this.mMoveMax = moveMax
    }

    @SuppressLint("ClickableViewAccessibility")
    override fun onTouchEvent(event: MotionEvent?): Boolean {
        var isConsume = false
        event?.let {
            val action = it.action
            mVelocityTracker.addMovement(it)
            when(action) {
                MotionEvent.ACTION_MOVE -> {
                    val startMoveX = it.x - mFirstP.x
                    val startMoveY = it.y - mFirstP.y
                    val absMoveX = Math.abs(startMoveX)
                    val absMoveY = Math.abs(startMoveY)
                    val rawX = it.rawX
                    val offsetX = (mLastP.x - (rawX)).toInt()
                    if (Math.abs(offsetX) > MOVE_MIN_VALUE && (absMoveX > absMoveY) && mMoveDirection == MOVE_DIRECTION_HORIZONTAL) {
                        if (isTouchLeft(mFirstP.x) && isTouchLeft(rawX)) {
                            mTouchLeft = true
                            if (Math.abs(offsetX) > 10) {
                                parent.requestDisallowInterceptTouchEvent(true)
                            }
                            val scrollX = mLeftScrollView?.scrollX ?: 0
                            val moveX = scrollX + offsetX
                            if (!(moveX > mMoveMax || moveX < 0)) {
                                mLeftScrollView?.scrollBy(offsetX, 0)
                                mRightScrollView?.scrollBy(-offsetX, 0)
                                mAdapter.moveLeftItem(mLeftScrollView?.scrollX ?: 0)
                                mAdapter.moveRightItem(mRightScrollView?.scrollX ?: 0)
                                mLastP.set(it.x, it.y)
                                isConsume = true
                            }
                        } else if (isTouchRight(mFirstP.x) && isTouchRight(rawX)) {
                            mTouchRight = true
                            if (Math.abs(offsetX) > 10) {
                                parent.requestDisallowInterceptTouchEvent(true)
                            }
                            val scrollX = mRightScrollView?.scrollX ?: 0
                            val moveX = scrollX + offsetX
                            if (!(moveX > mMoveMax || moveX < 0)) {
                                mLeftScrollView?.scrollBy(-offsetX, 0)
                                mRightScrollView?.scrollBy(offsetX, 0)
                                mAdapter.moveLeftItem(mLeftScrollView?.scrollX ?: 0)
                                mAdapter.moveRightItem(mRightScrollView?.scrollX ?: 0)
                                mLastP.set(it.x, it.y)
                                isConsume = true
                            }
                        }
                    }
                }
                MotionEvent.ACTION_UP -> {
                    if ((mTouchLeft || mTouchRight) && mMoveDirection == MOVE_DIRECTION_HORIZONTAL) {
                        mVelocityTracker.computeCurrentVelocity(1000, mMaximumVelocity.toFloat())
                        val xVelocity = mVelocityTracker.xVelocity.toInt()
                        if (Math.abs(xVelocity) > mMinimumVelocity) {
                            if (mTouchLeft) {
                                mScroller.fling(mLeftScrollView?.scrollX ?: 0, 0, -xVelocity, 0,
                                        0, mMoveMax, 0, 10)
                                postInvalidate()
                            } else if (mTouchRight) {
                                mScroller.fling(mRightScrollView?.scrollX ?: 0, 0, -xVelocity, 0,
                                        0, mMoveMax, 0, 10)
                                postInvalidate()
                            }
                        } else {
                            mAdapter.moveLeftItem(mLeftScrollView?.scrollX ?: 0)
                            mAdapter.moveRightItem(mRightScrollView?.scrollX ?: 0)
                        }

                    }
                    mVelocityTracker.clear()
                }
            }
        }
        if (isConsume) {
            return true
        }
        return super.onTouchEvent(event)
    }

    override fun onInterceptTouchEvent(ev: MotionEvent?): Boolean {
        when(ev?.action) {
            MotionEvent.ACTION_DOWN -> {
                mMoveDirection = 0
                if (!mScroller.isFinished) {
                    mScroller.abortAnimation()
                }
                mTouchLeft = false
                mTouchRight = false
                mLastP.set(ev.x, ev.y )
                mFirstP.set(ev.x, ev.y)
                mPointerId = ev.getPointerId(0)
            }
            MotionEvent.ACTION_MOVE -> {
                val offsetX = ev.x - mFirstP.x
                val offsetY = ev.y - mFirstP.y
                val absOffsetX = Math.abs(offsetX)
                val absOffsetY = Math.abs(offsetY)
                if (mMoveDirection == 0 && (absOffsetX > MOVE_MIN_VALUE || absOffsetY > MOVE_MIN_VALUE)) {
                    mMoveDirection = if (absOffsetX > absOffsetY) MOVE_DIRECTION_HORIZONTAL else MOVE_DIRECTION_VERTICAL
                }
                return MOVE_DIRECTION_HORIZONTAL == mMoveDirection
            }
        }
        return super.onInterceptTouchEvent(ev)
    }

    /**
     * 滑动关联，
     */
    override fun computeScroll() {
        super.computeScroll()
        if (mScroller.computeScrollOffset()) {
            val currX = mScroller.currX
            if (mTouchLeft) {
                mLeftScrollView?.scrollTo(currX, 0)
                mRightScrollView?.scrollTo(mMoveMax - currX, 0)
                mAdapter.moveLeftItem(currX)
                mAdapter.moveRightItem(mMoveMax - currX)
            } else if (mTouchRight) {
                mLeftScrollView?.scrollTo(mMoveMax - currX, 0)
                mRightScrollView?.scrollTo(currX, 0)
                mAdapter.moveLeftItem(mMoveMax - currX)
                mAdapter.moveRightItem(currX)
            }
            postInvalidate()
        }
    }

    /**
     * 判断是否左侧滑动
     */
    private fun isTouchLeft(x: Float): Boolean {
        if (mLeftScrollWidth == 0) {
            mLeftScrollWidth = mLeftScrollView?.measuredWidth ?: 0
        }
        if (x >= 0 && x <= mLeftScrollWidth) {
            return true
        }
        return false
    }

    /**
     * 判断是否右侧滑动
     */
    private fun isTouchRight(x: Float): Boolean {
        if (mRightScrollWidth == 0) {
            mRightScrollWidth = mRightScrollView?.measuredWidth ?: 0
        }
        if (x >= UIScreen.screenWidth - mRightScrollWidth && x <= UIScreen.screenWidth) {
            return true
        }
        return false
    }
}