package wind.android.market.business.option.view

import android.content.Context
import android.content.Intent
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.LinearLayout
import net.datamodel.network.RealQuoteItem
import session.F5Session
import ui.screen.UIScreen
import wind.android.f5.activity.SpeedDetailActivity
import wind.android.market.R
import wind.android.market.business.option.adapter.OptionMirrorAdapter
import wind.android.market.business.option.model.OptionContentModel
import wind.android.market.business.option.model.OptionMonthModel
import wind.android.market.business.option.presenter.OptionListViewPresenter

/**
 * # ***********************************************************************************************
 * # ClassName:      OptionRecyclerView$
 * # Function:       期权列表视图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/1/3$    14:15$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class OptionRecyclerView@JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0):
        MirrorRecyclerView(context, attrs, defStyleAttr), IOptionListView {

    private val mPresenter by lazy { OptionListViewPresenter(this) }

    private val mAdapter by lazy { OptionMirrorAdapter(context) }

    init {
        setAdapter(mAdapter)
        mAdapter.setOnClickListener({
            val tag = it.tag
            if (tag is Int) {
                val item = mAdapter.getItem(tag)
                val windCode = item?.leftModel?.windCode ?: ""
                if (!windCode.isEmpty()) {
                    val intent = Intent(context, SpeedDetailActivity::class.java)
                    intent.putExtra(F5Session.POSITION, 0)
                    F5Session.getInstance().f5WindCodes = arrayOf(windCode)
                    context.startActivity(intent)
                }
            }
        }, {
            val tag = it.tag
            if (tag is Int) {
                val item = mAdapter.getItem(tag)
                val windCode = item?.rightModel?.windCode ?: ""
                if (!windCode.isEmpty()) {
                    val intent = Intent(context, SpeedDetailActivity::class.java)
                    intent.putExtra(F5Session.POSITION, 0)
                    F5Session.getInstance().f5WindCodes = arrayOf(windCode)
                    context.startActivity(intent)
                }
            }
        })
        setOptionTitleView()
    }

    /**
     * 请求数据
     */
    fun requestData(model: OptionMonthModel, exchange: String) {
        mPresenter.requestOptionContent(model, exchange)
    }

    /**
     * 订阅期权数据
     */
    fun subData() {
        mPresenter.subOptionList()
    }

    /**
     * 取消订阅
     */
    fun unSubData() {
        mPresenter.unSubOptionList()
    }

    fun hasSubData(): Boolean {
        return mPresenter.hasSubData()
    }

    private fun setOptionTitleView() {
        val titleView = LayoutInflater.from(context).inflate(R.layout.option_title_layout, null)
        val leftOptionTitle = titleView.findViewById<LinearLayout>(R.id.left_option_title)
        val rightOptionTitle = titleView.findViewById<LinearLayout>(R.id.right_option_title)
        val strikeViewWidth = titleView.findViewById<LinearLayout>(R.id.option_desc).layoutParams.width
        setWidth(leftOptionTitle)
        setWidth(rightOptionTitle)
        val width = (UIScreen.screenWidth * 0.235f).toInt()
        leftOptionTitle.scrollTo(width * 15 - (UIScreen.screenWidth * 0.5f - strikeViewWidth / 2).toInt(), 0)
        setTitleView(titleView, leftOptionTitle, rightOptionTitle)
        setMoveMax(width * 15 - (UIScreen.screenWidth * 0.5f - strikeViewWidth / 2).toInt())
    }

    private fun setWidth(viewGroup: ViewGroup) {
        val childCount = viewGroup.childCount
        if (childCount == 0) {
            return
        }
        for (i in 0 until childCount) {
            viewGroup.getChildAt(i).layoutParams.width = (UIScreen.screenWidth * 0.235f).toInt()
        }
    }

    override fun getRootGroup(): ViewGroup {
        return this
    }

    override fun setOptionList(list: MutableList<OptionContentModel>) {
        mAdapter.setData(list)
    }

    override fun updateOptionList(list: MutableList<RealQuoteItem>) {
        mAdapter.updateList(list)
    }

    override fun updateOptionItem(item: RealQuoteItem) {
        mAdapter.updateItem(item)
    }

    override fun refreshOptionList() {
        mAdapter.notifyDataSetChanged()
    }
}